@@ -38,6 +38,7 @@
 #include "H5AbstractDs.h"
 #include "H5DataSet.h"
 #include "H5File.h"
+#include "H5Attribute.h"
 
 #ifndef H5_NO_NAMESPACE
 namespace H5 {
@@ -51,8 +52,6 @@ namespace H5 {
 // Function:	DataType overloaded constructor
 ///\brief	Creates a datatype using an existing datatype's id
 ///\param	existing_id - IN: Id of the existing datatype
-///\param	predefined  - IN: Indicates whether or not this datatype is
-///		a predefined datatype; default to \c false
 // Description
 //		Constructor creates a copy of an existing DataType using
 //		its id.  The argument "predefined" is default to false;
@@ -62,7 +61,10 @@ namespace H5 {
 //		- BMR 5/2004
 // Programmer	Binh-Minh Ribler - 2000
 //--------------------------------------------------------------------------
-DataType::DataType(const hid_t existing_id) : H5Object(existing_id) {}
+DataType::DataType(const hid_t existing_id) : H5Object()
+{
+    id = existing_id;
+}
 
 //--------------------------------------------------------------------------
 // Function:	DataType overloaded constructor
@@ -84,17 +86,74 @@ DataType::DataType( const H5T_class_t type_class, size_t size ) : H5Object()
 
 //--------------------------------------------------------------------------
 // Function:	DataType overload constructor - dereference
-///\brief	Given a reference to some object, returns that datatype
-///\param       obj - IN: Location reference object is in
+///\brief	Given a reference, ref, to an hdf5 group, creates a
+///		DataType object
+///\param       obj - IN: Specifying location referenced object is in
 ///\param	ref - IN: Reference pointer
-///\parDescription
-///		\c obj can be DataSet, Group, H5File, or named DataType, that 
+///\param	ref_type - IN: Reference type - default to H5R_OBJECT
+///\exception	H5::ReferenceException
+///\par Description
+///		\c obj can be DataSet, Group, or named DataType, that
 ///		is a datatype that has been named by DataType::commit.
 // Programmer	Binh-Minh Ribler - Oct, 2006
+// Modification
+//	Jul, 2008
+//		Added for application convenience.
 //--------------------------------------------------------------------------
-DataType::DataType(IdComponent& obj, void* ref) : H5Object()
+DataType::DataType(H5Object& obj, const void* ref, H5R_type_t ref_type) : H5Object()
 {
-   IdComponent::dereference(obj, ref);
+    try {
+	id = p_dereference(obj.getId(), ref, ref_type);
+    } catch (ReferenceException deref_error) {
+	throw ReferenceException("DataType constructor - located by an H5Object",
+		deref_error.getDetailMsg());
+    }
+}
+
+//--------------------------------------------------------------------------
+// Function:	DataType overload constructor - dereference
+///\brief	Given a reference, ref, to an hdf5 group, creates a
+///		DataType object
+///\param       h5file - IN: Location referenced object is in
+///\param	ref - IN: Reference pointer
+///\param	ref_type - IN: Reference type - default to H5R_OBJECT
+///\exception	H5::ReferenceException
+// Programmer	Binh-Minh Ribler - Oct, 2006
+// Modification
+//	Jul, 2008
+//		Added for application convenience.
+//--------------------------------------------------------------------------
+DataType::DataType(H5File& h5file, const void* ref, H5R_type_t ref_type) : H5Object()
+{
+    try {
+	id = p_dereference(h5file.getId(), ref, ref_type);
+    } catch (ReferenceException deref_error) {
+	throw ReferenceException("DataType constructor - located by an H5File",
+		deref_error.getDetailMsg());
+    }
+}
+
+//--------------------------------------------------------------------------
+// Function:	DataType overload constructor - dereference
+///\brief	Given a reference, ref, to an hdf5 group, creates a
+///		DataType object
+///\param       attr - IN: Specifying location where the referenced object is in
+///\param	ref - IN: Reference pointer
+///\param	ref_type - IN: Reference type - default to H5R_OBJECT
+///\exception	H5::ReferenceException
+// Programmer	Binh-Minh Ribler - Oct, 2006
+// Modification
+//	Jul, 2008
+//		Added for application convenience.
+//--------------------------------------------------------------------------
+DataType::DataType(Attribute& attr, const void* ref, H5R_type_t ref_type) : H5Object()
+{
+    try {
+	id = p_dereference(attr.getId(), ref, ref_type);
+    } catch (ReferenceException deref_error) {
+	throw ReferenceException("DataType constructor - located by an Attribute",
+		deref_error.getDetailMsg());
+    }
 }
 
 //--------------------------------------------------------------------------
@@ -102,14 +161,18 @@ DataType::DataType(IdComponent& obj, void* ref) : H5Object()
 ///\brief	Default constructor: Creates a stub datatype
 // Programmer	Binh-Minh Ribler - 2000
 //--------------------------------------------------------------------------
-DataType::DataType() : H5Object() {}
+DataType::DataType() : H5Object(), id(0) {}
 
 //--------------------------------------------------------------------------
 // Function:	DataType copy constructor
 ///\brief	Copy constructor: makes a copy of the original DataType object.
 // Programmer	Binh-Minh Ribler - 2000
 //--------------------------------------------------------------------------
-DataType::DataType(const DataType& original) : H5Object(original) {}
+DataType::DataType(const DataType& original) : H5Object(original)
+{
+    id = original.getId();
+    incRefCount(); // increment number of references to this id
+}
 
 //--------------------------------------------------------------------------
 // Function:	DataType::copy
@@ -142,10 +205,10 @@ void DataType::copy( const DataType& like_type )
 //--------------------------------------------------------------------------
 // Function:	DataType::copy
 ///\brief	Copies the datatype of the given dataset to this datatype object
-///\param	dset - IN: Dataset 
+///\param	dset - IN: Dataset
 ///\exception	H5::DataTypeIException
 // Programmer	Binh-Minh Ribler - Jan, 2007
-///\parDescription
+///\par Description
 ///		The resulted dataset will be transient and modifiable.
 //--------------------------------------------------------------------------
 void DataType::copy(const DataSet& dset)
@@ -210,7 +273,7 @@ bool DataType::operator==(const DataType& compared_type ) const
 // Function:	DataType::p_commit (private)
 //\brief	Commits a transient datatype to a file, creating a new
 //		named datatype
-//\param	loc_id - IN: The id of either a file, group, dataset, named 
+//\param	loc_id - IN: The id of either a file, group, dataset, named
 //			 datatype, or attribute.
 //\param	name - IN: Name of the datatype
 //\exception	H5::DataTypeIException
@@ -452,7 +515,7 @@ DataType DataType::getSuper() const
 ///\exception	H5::DataTypeIException
 ///\par Description
 ///		For more information, please see:
-/// http://hdf.ncsa.uiuc.edu/HDF5/doc/RM_H5T.html#Datatype-Register
+/// http://www.hdfgroup.org/HDF5/doc/RM/RM_H5T.html#Datatype-Register
 // Programmer	Binh-Minh Ribler - 2000
 //--------------------------------------------------------------------------
 void DataType::registerFunc( H5T_pers_t pers, const char* name, const DataType& dest, H5T_conv_t func ) const
@@ -616,52 +679,6 @@ bool DataType::isVariableStr() const
    }
 }
 
-//--------------------------------------------------------------------------
-// Function:	DataType::Reference
-///\brief	Important!!! - This functions may not work correctly, it 
-///		will be removed in the near future.  Please use similar
-///		DataType::reference instead!
-// Programmer	Binh-Minh Ribler - May, 2004
-//--------------------------------------------------------------------------
-void* DataType::Reference(const char* name, DataSpace& dataspace, H5R_type_t ref_type) const
-{
-   try {
-      return(p_reference(name, dataspace.getId(), ref_type));
-   }
-   catch (IdComponentException E) {
-      throw DataTypeIException(inMemFunc("Reference"), E.getDetailMsg());
-   }
-}
-
-//--------------------------------------------------------------------------
-// Function:	DataType::Reference
-///\brief	Important!!! - This functions may not work correctly, it 
-///		will be removed in the near future.  Please use similar
-///		DataType::reference instead!
-// Programmer	Binh-Minh Ribler - May, 2004
-//--------------------------------------------------------------------------
-void* DataType::Reference(const char* name) const
-{
-   try {
-      return(p_reference(name, -1, H5R_OBJECT));
-   }
-   catch (IdComponentException E) {
-      throw DataTypeIException(inMemFunc("Reference"), E.getDetailMsg());
-   }
-}
-
-//--------------------------------------------------------------------------
-// Function:	DataType::Reference
-///\brief	Important!!! - This functions may not work correctly, it 
-///		will be removed in the near future.  Please use similar
-///		DataType::reference instead!
-// Programmer	Binh-Minh Ribler - May, 2004
-//--------------------------------------------------------------------------
-void* DataType::Reference(const H5std_string& name) const
-{
-   return(Reference(name.c_str()));
-}
-
 #ifndef H5_NO_DEPRECATED_SYMBOLS
 //--------------------------------------------------------------------------
 // Function:	DataType::getObjType
@@ -707,6 +724,47 @@ DataSpace DataType::getRegion(void *ref, H5R_type_t ref_type) const
    }
 }
 
+//--------------------------------------------------------------------------
+// Function:    DataType::getId
+// Purpose:     Get the id of this attribute
+// Modification:
+//      May 2008 - BMR
+//              Class hierarchy is revised to address bugzilla 1068.  Class
+//              AbstractDS and Attribute are moved out of H5Object.  In
+//              addition, member IdComponent::id is moved into subclasses, and
+//              IdComponent::getId now becomes pure virtual function.
+// Programmer   Binh-Minh Ribler - May, 2008
+//--------------------------------------------------------------------------
+hid_t DataType::getId() const
+{
+   return(id);
+}
+
+//--------------------------------------------------------------------------
+// Function:    DataType::p_setId
+///\brief       Sets the identifier of this object to a new value.
+///
+///\exception   H5::IdComponentException when the attempt to close the HDF5
+///             object fails
+// Description:
+//              The underlaying reference counting in the C library ensures
+//              that the current valid id of this object is properly closed.
+//              Then the object's id is reset to the new id.
+// Programmer   Binh-Minh Ribler - 2000
+//--------------------------------------------------------------------------
+void DataType::p_setId(const hid_t new_id)
+{
+    // handling references to this old id
+    try {
+        close();
+    }
+    catch (Exception close_error) {
+        throw DataTypeIException(inMemFunc("p_setId"), close_error.getDetailMsg());
+    }
+   // reset object's id to the given id
+   id = new_id;
+}
+
 //--------------------------------------------------------------------------
 // Function:	DataType::close
 ///\brief	Closes the datatype if it is not a predefined type.
@@ -723,7 +781,7 @@ void DataType::close()
 	{
 	    throw DataTypeIException(inMemFunc("close"), "H5Tclose failed");
 	}
-	// reset the id because the datatype that it represents is now closed
+	// reset the id
 	id = 0;
     }
 }
@@ -740,14 +798,12 @@ void DataType::close()
 //--------------------------------------------------------------------------
 DataType::~DataType()
 {
-    try {
-	close();
-    }
-    catch (Exception close_error) {
-	cerr << inMemFunc("~DataType - ") << close_error.getDetailMsg() << endl;
-    }
+	try {
+	    close();
+	} catch (Exception close_error) {
+	    cerr << inMemFunc("~DataType - ") << close_error.getDetailMsg() << endl;
+	}
 }
-
 #ifndef H5_NO_NAMESPACE
 } // end namespace
 #endif
