@@ -36,7 +36,6 @@
 #endif  // H5_NO_STD
 #endif
 
-#include "testhdf5.h"	// C test header file
 #include "H5Cpp.h"	// C++ API header file
 
 #ifndef H5_NO_NAMESPACE
@@ -44,6 +43,7 @@
 #endif
 
 #include "h5cpputil.h"	// C++ utilility header file
+#include "H5srcdir.h"	// srcdir querying header file
 
 const H5std_string    TESTFILE("th5s.h5");
 const H5std_string    DATAFILE("th5s1.h5");
@@ -110,6 +110,9 @@ int space5_data = 7;
  *		       passed to verify_val to 'long' as well.  If problems
  *		       arises later, this will have to be specificly handled
  *		       with a special routine.
+ *     April 12, 2011: Raymond Lu
+ *                     Starting from the 1.8.7 release, we allow dimension
+ *                     size to be zero.  So I took out the test against it.
  *-------------------------------------------------------------------------
  */
 static void test_h5s_basic()
@@ -121,7 +124,7 @@ static void test_h5s_basic()
     hsize_t		tmax[4];
 
     // Output message about test being performed
-    SUBTEST("Testing Dataspace Manipulation");
+    SUBTEST("Dataspace Manipulation");
 
     try {
 	// Create simple dataspace sid1
@@ -210,28 +213,6 @@ static void test_h5s_basic()
     // CHECK_I(ret, "H5Fclose");  // leave this here, later, fake a failure
 		// in the p_close see how this will handle it. - BMR
 
-	// Verify that incorrect dimensions don't work
-	dims1[0] = 0;
-	try {
-	    DataSpace wrongdim_ds (SPACE1_RANK, dims1);
-
-	    // Should FAIL but didn't, so throw an invalid action exception
-	    throw InvalidActionException("DataSpace constructor", "Attempted to use incorrect dimensions");
-	}
-	catch( DataSpaceIException E ) // catching use of incorrect dimensions
-	{} // do nothing, exception expected
-
-	// Another incorrect dimension case
-	DataSpace sid3 (H5S_SIMPLE);
-	try {
-	    sid3.setExtentSimple( SPACE1_RANK, dims1 );
-
-	    // Should FAIL but didn't, so throw an invalid action exception
-	    throw InvalidActionException("DataSpace::setExtentSimple", "Attempted to use incorrect dimensions");
-	}
-	catch (DataSpaceIException E) // catching use of incorrect dimensions
-	{} // do nothing, exception expected
-
 	PASSED();
     }	// end of try block
 
@@ -270,7 +251,7 @@ static void test_h5s_basic()
 static void test_h5s_scalar_write()
 {
     // Output message about test being performed
-    SUBTEST("Testing Scalar Dataspace Writing");
+    SUBTEST("Scalar Dataspace Writing");
 
     try {
 	// Create file
@@ -300,7 +281,7 @@ static void test_h5s_scalar_write()
 	verify_val(ext_type, H5S_SCALAR, "DataSpace::getSimpleExtentType", __LINE__, __FILE__);
 
 	// Create and write a dataset
-	DataSet dataset = fid1.createDataSet("Dataset1", PredType::NATIVE_UINT,sid1); 
+	DataSet dataset = fid1.createDataSet("Dataset1", PredType::NATIVE_UINT,sid1);
 	dataset.write(&space3_data, PredType::NATIVE_UINT);
 
 	PASSED();
@@ -336,7 +317,7 @@ static void test_h5s_scalar_read()
     hsize_t		tdims[4];	// Dimension array to test with
 
     // Output message about test being performed
-    SUBTEST("Testing Scalar Dataspace Reading");
+    SUBTEST("Scalar Dataspace Reading");
 
     try {
 	// Create file
@@ -396,7 +377,7 @@ static void test_h5s_scalar_read()
 static void test_h5s_null()
 {
     // Output message about test being performed
-    SUBTEST("Testing Null Dataspace Writing");
+    SUBTEST("Null Dataspace Writing");
 
     try {
 	// Create file
@@ -451,7 +432,7 @@ static void test_h5s_null()
 static void test_h5s_compound_scalar_write()
 {
     // Output message about test being performed
-    SUBTEST("Testing Compound Dataspace Writing");
+    SUBTEST("Compound Dataspace Writing");
 
     try {
 	// Create file
@@ -526,7 +507,7 @@ static void test_h5s_compound_scalar_read()
     hsize_t		tdims[4];	// Dimension array to test with
 
     // Output message about test being performed
-    SUBTEST("Testing Compound Dataspace Reading");
+    SUBTEST("Compound Dataspace Reading");
     try {
 	// Create file
 	H5File fid1(DATAFILE, H5F_ACC_RDWR);
@@ -594,6 +575,7 @@ extern "C"
 void test_h5s()
 {
     // Output message about test being performed
+    //MESSAGE("Testing Dataspaces\n");
     MESSAGE(5, ("Testing Dataspaces\n"));
 
     test_h5s_basic();		// Test basic H5S code
