@@ -33,7 +33,6 @@
 #endif  // H5_NO_STD
 #endif
 
-#include "testhdf5.h"   // C test header file
 #include "H5Cpp.h"      // C++ API header file
 
 #ifndef H5_NO_NAMESPACE
@@ -121,7 +120,7 @@ static void test_classes()
 	}
 	PASSED();
     }   // end of try block
-    catch (Exception E) { 
+    catch (Exception E) {
 	issue_fail_msg("test_classes", __LINE__, __FILE__, E.getCDetailMsg());
     }
 }
@@ -178,8 +177,8 @@ static void test_copy()
         another_int_type = new_int_type;
 
 	PASSED();
-    } 
-    catch (Exception E) { 
+    }
+    catch (Exception E) {
 	issue_fail_msg("test_copy", __LINE__, __FILE__, E.getCDetailMsg());
     }
 }
@@ -191,12 +190,12 @@ static void test_copy()
  * Purpose:	Tests query functions of compound and enumeration types.
  *
  * Return:	Success: 	0
- * 	
+ *
  *		Failure:	number of errors
  *
  * Programmer:	Binh-Minh Ribler (use C version)
  *		January, 2007
- *  
+ *
  * Modifications:
  *
  *-------------------------------------------------------------------------
@@ -289,7 +288,7 @@ static void test_query()
         issue_fail_msg("test_query", __LINE__, __FILE__, E.getCDetailMsg());
     }
 }   // test_query
- 
+
 
 /*-------------------------------------------------------------------------
  * Function:	test_transient
@@ -311,7 +310,7 @@ const char* filename1 = "dtypes1.h5";
 static void test_transient ()
 {
     static hsize_t	ds_size[2] = {10, 20};
-    
+
     SUBTEST("Transient datatypes");
     try {
 
@@ -345,7 +344,7 @@ static void test_transient ()
 	} catch (DataTypeIException err) {}
 	itype.close();
 
-	// Get a copy of the dataset's datatype by applying DataType::copy() 
+	// Get a copy of the dataset's datatype by applying DataType::copy()
 	// to the dataset. The resulted datatype should be modifiable.
 	itype.copy(dset);
 	itype.setPrecision(256);
@@ -392,8 +391,8 @@ static void test_named ()
     static hsize_t	ds_size[2] = {10, 20};
     hsize_t		i;
     unsigned 		attr_data[10][20];
-    char		filename[1024];
-    
+    DataType           *ds_type = NULL;
+
     SUBTEST("Named datatypes");
     try {
 	// Create the file.
@@ -416,7 +415,7 @@ static void test_named ()
         itype.commit(file, "native-int");
 
 	// Check that it is committed.
-	if (itype.committed() <= 0)
+	if (itype.committed() == false)
 	    cerr << "IntType::committed() returned false" << endl;
 
 	// We should not be able to modify a type after it has been committed.
@@ -437,7 +436,7 @@ static void test_named ()
 
 	// It should be possible to define an attribute for the named type
 	Attribute attr1 = itype.createAttribute("attr1", PredType::NATIVE_UCHAR, space);
-	for (i=0; i<ds_size[0]*ds_size[1]; i++) 
+	for (i=0; i<ds_size[0]*ds_size[1]; i++)
 	    attr_data[0][i] = (int)i;/*tricky*/
 	attr1.write(PredType::NATIVE_UINT, attr_data);
 	attr1.close();
@@ -454,7 +453,7 @@ static void test_named ()
     /*
      * Close the committed type and reopen it.  It should return a named type.
 * This had something to do with the way IntType was returned and assigned
-and caused itype.committed not working correctly.  So, use another_type for 
+and caused itype.committed not working correctly.  So, use another_type for
 now.
 	itype.close();
 	itype = file.openIntType("native-int");
@@ -468,14 +467,15 @@ now.
 	// Create a dataset that uses the named type, then get the dataset's
 	// datatype and make sure it's a named type.
 	DataSet dset = file.createDataSet("dset1", another_type, space);
-	DataType *ds_type = new DataType(dset.getDataType());
+	ds_type = new DataType(dset.getDataType());
 	iscommitted = ds_type->committed();
 	if (!iscommitted)
 	    throw InvalidActionException("IntType::committed()", "1 Dataset type should be named type!");
 	dset.close();
 	ds_type->close();
+        delete ds_type;
 
-	// Reopen the dataset and its type, then make sure the type is 
+	// Reopen the dataset and its type, then make sure the type is
 	// a named type.
 	dset = file.openDataSet("dset1");
 	ds_type = new DataType(dset.getDataType());
@@ -489,6 +489,7 @@ now.
 	dset = file.createDataSet("dset2", *ds_type, space);
 	ds_type->close();
 	dset.close();
+        delete ds_type;
 
 	// Reopen the second dataset and make sure the type is shared
 	dset = file.openDataSet("dset2");
@@ -497,7 +498,7 @@ now.
 	if (!iscommitted)
 	    throw InvalidActionException("DataType::iscommitted()", "Dataset type should be named type!");
 	ds_type->close();
-    
+
 	// Get the dataset datatype by applying DataType::copy() to the
 	// dataset. The resulted datatype should be modifiable.
 	IntType copied_type;
@@ -515,6 +516,9 @@ now.
     catch (Exception E) {
         issue_fail_msg("test_named", __LINE__, __FILE__, E.getCDetailMsg());
     }
+
+    if(ds_type)
+        delete ds_type;
 }   // test_named
 
 
@@ -529,6 +533,7 @@ extern "C"
 void test_types()
 {
     // Output message about test being performed
+    //MESSAGE("Testing Generic Data Types\n");
     MESSAGE(5, ("Testing Generic Data Types\n"));
 
     // Test basic datatypes
