@@ -1,24 +1,27 @@
-!****h* fortran/src/H5f90global.f90
+!****h* ROBODoc/H5GLOBAL
 !
 ! NAME
-!   H5GLOBAL
-!  
-! FUNCTION
-!   This module is used to pass C stubs for H5 Fortran APIs. The C stubs are 
-!   packed into arrays in H5_f.c and these arrays are then passed to Fortran.
-!   This module then uses EQUIVALENCE to assign elements of the arrays to
-!   Fortran equivalent C stubs.
+!  MODULE H5GLOBAL
+!
+! FILE
+!  src/fortran/H5f90global.f90
+!
+! PURPOSE
+!  This module is used to pass C stubs for H5 Fortran APIs. The C stubs are
+!  packed into arrays in H5_f.c and these arrays are then passed to Fortran.
+!  This module then uses EQUIVALENCE to assign elements of the arrays to
+!  Fortran equivalent C stubs.
 !
 ! NOTES
-!   The size of the C arrays in H5_f.c have to match the values of the variables 
-!   declared as PARAMETER, hence if the size of an array in H5_f.c is changed
-!   then the PARAMETER of that corresponding array in Fortran must also be changed.
+!  The size of the C arrays in H5_f.c has to match the values of the variables
+!  declared as PARAMETER, hence if the size of an array in H5_f.c is changed
+!  then the PARAMETER of that corresponding array in Fortran must also be changed.
 !
 ! USES
-!   H5FORTRAN_TYPES - This module is generated at run time. See 
-!                     H5fortran_types.f90 for further informaton.
+!  H5FORTRAN_TYPES 	 - This module is generated at run time. See
+!
 ! COPYRIGHT
-! * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
+! * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 !   Copyright by The HDF Group.                                               *
 !   Copyright by the Board of Trustees of the University of Illinois.         *
 !   All rights reserved.                                                      *
@@ -31,7 +34,7 @@
 !   is linked from the top-level documents page.  It can also be found at     *
 !   http://hdfgroup.org/HDF5/doc/Copyright.html.  If you do not have          *
 !   access to either file, you may request a copy from help@hdfgroup.org.     *
-! * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
+! * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 !
 ! AUTHOR
 !  Elena Pourmal
@@ -43,8 +46,8 @@ MODULE H5GLOBAL
 
   ! Definitions for reference datatypes.
   ! If you change the value of these parameters, do not forget to change corresponding
-  ! values in the H5f90.h file. 
-  INTEGER, PARAMETER :: REF_REG_BUF_LEN = 3 
+  ! values in the H5f90.h file.
+  INTEGER, PARAMETER :: REF_REG_BUF_LEN = 3
 
   ! Parameters used in the function 'h5kind_to_type' located in H5_ff.f90.
   ! The flag is used to tell the function whether the kind input variable
@@ -61,17 +64,17 @@ MODULE H5GLOBAL
      INTEGER, DIMENSION(1:REF_REG_BUF_LEN) :: ref
   END TYPE hdset_reg_ref_t_f
 
-  INTEGER, PARAMETER :: PREDEF_TYPES_LEN = 13 ! Do not forget to change this
+  INTEGER, PARAMETER :: PREDEF_TYPES_LEN = 17 ! Do not forget to change this
                                               ! value when new predefined
                                               ! datatypes are added
 
-  ! Do not forget to change the following line when new predefined 
+  ! Do not forget to change the following line when new predefined
   ! floating data types are added
   INTEGER, PARAMETER :: FLOATING_TYPES_LEN = 4
 
-  ! Do not forget to change the following line when new predefined 
+  ! Do not forget to change the following line when new predefined
   ! integer data types are added
-  INTEGER, PARAMETER :: INTEGER_TYPES_LEN = 17
+  INTEGER, PARAMETER :: INTEGER_TYPES_LEN = 27
 
   INTEGER(HID_T) H5T_NATIVE_INTEGER_1, &
        H5T_NATIVE_INTEGER_2, &
@@ -106,7 +109,21 @@ MODULE H5GLOBAL
        H5T_STD_U32LE,   &
        H5T_STD_U64BE,   &
        H5T_STD_U64LE,   &
-       H5T_STRING
+       H5T_STRING,      &
+       H5T_STD_B8BE,    &
+       H5T_STD_B8LE,    &
+       H5T_STD_B16BE,   &
+       H5T_STD_B16LE,   &
+       H5T_STD_B32BE,   &
+       H5T_STD_B32LE,   &
+       H5T_STD_B64BE,   &
+       H5T_STD_B64LE,   &
+       H5T_NATIVE_B8,   &
+       H5T_NATIVE_B16,  &
+       H5T_NATIVE_B32,  &
+       H5T_NATIVE_B64,  &
+       H5T_FORTRAN_S1,  &
+       H5T_C_S1
 
 
   INTEGER(HID_T), DIMENSION(PREDEF_TYPES_LEN) :: predef_types
@@ -123,7 +140,10 @@ MODULE H5GLOBAL
   EQUIVALENCE (predef_types(11), H5T_NATIVE_REAL_4)
   EQUIVALENCE (predef_types(12), H5T_NATIVE_REAL_8)
   EQUIVALENCE (predef_types(13), H5T_NATIVE_REAL_16)
-
+  EQUIVALENCE (predef_types(14), H5T_NATIVE_B8 )
+  EQUIVALENCE (predef_types(15), H5T_NATIVE_B16)
+  EQUIVALENCE (predef_types(16), H5T_NATIVE_B32)
+  EQUIVALENCE (predef_types(17), H5T_NATIVE_B64)  
 
   INTEGER(HID_T), DIMENSION(FLOATING_TYPES_LEN) :: floating_types
   EQUIVALENCE (floating_types(1), H5T_IEEE_F32BE )
@@ -149,6 +169,16 @@ MODULE H5GLOBAL
   EQUIVALENCE (integer_types(15), H5T_STD_U64BE)
   EQUIVALENCE (integer_types(16), H5T_STD_U64LE)
   EQUIVALENCE (integer_types(17), H5T_STRING)
+  EQUIVALENCE (integer_types(18), H5T_STD_B8BE)
+  EQUIVALENCE (integer_types(19), H5T_STD_B8LE)
+  EQUIVALENCE (integer_types(20), H5T_STD_B16BE)
+  EQUIVALENCE (integer_types(21), H5T_STD_B16LE)
+  EQUIVALENCE (integer_types(22), H5T_STD_B32BE)
+  EQUIVALENCE (integer_types(23), H5T_STD_B32LE)
+  EQUIVALENCE (integer_types(24), H5T_STD_B64BE)
+  EQUIVALENCE (integer_types(25), H5T_STD_B64LE)
+  EQUIVALENCE (integer_types(26), H5T_FORTRAN_S1)
+  EQUIVALENCE (integer_types(27), H5T_C_S1)
 
 
   !      COMMON /PREDEFINED_TYPES/ H5T_NATIVE_INTEGER, &
@@ -169,7 +199,7 @@ MODULE H5GLOBAL
   !DEC$if defined(BUILD_HDF5_DLL)
   !DEC$ATTRIBUTES DLLEXPORT :: /FLOATING_TYPES/
   !DEC$endif
-  COMMON /FLOATING_TYPES/ floating_types 
+  COMMON /FLOATING_TYPES/ floating_types
   !
   !      COMMON /INTEGER_TYPES/ H5T_STD_I8BE,  &
   !                             H5T_STD_I8LE,    &
@@ -195,7 +225,7 @@ MODULE H5GLOBAL
   ! Fortran flags
   !
   !
-  ! H5F flags (DO NOT FORGET TO UPDATE WHEN NEW FLAGS ARE ADDEDD !)
+  ! H5F flags (DO NOT FORGET TO UPDATE WHEN NEW FLAGS ARE ADDED !)
   !
   ! H5F flags declaration
   !
@@ -206,7 +236,7 @@ MODULE H5GLOBAL
   !DEC$endif
   COMMON /H5F_FLAGS/ H5F_flags
 
-  INTEGER :: H5F_ACC_RDWR_F 
+  INTEGER :: H5F_ACC_RDWR_F
   INTEGER :: H5F_ACC_RDONLY_F
   INTEGER :: H5F_ACC_TRUNC_F
   INTEGER :: H5F_ACC_EXCL_F
@@ -224,6 +254,7 @@ MODULE H5GLOBAL
   INTEGER :: H5F_OBJ_ALL_F
   INTEGER :: H5F_LIBVER_EARLIEST_F
   INTEGER :: H5F_LIBVER_LATEST_F
+  INTEGER :: H5F_UNLIMITED_F
 
   EQUIVALENCE(H5F_flags(1), H5F_ACC_RDWR_F)
   EQUIVALENCE(H5F_flags(2), H5F_ACC_RDONLY_F)
@@ -243,6 +274,7 @@ MODULE H5GLOBAL
   EQUIVALENCE(H5F_flags(16), H5F_OBJ_ALL_F)
   EQUIVALENCE(H5F_flags(17), H5F_LIBVER_EARLIEST_F)
   EQUIVALENCE(H5F_flags(18), H5F_LIBVER_LATEST_F)
+  EQUIVALENCE(H5F_flags(19), H5F_UNLIMITED_F)
   !
   ! H5generic flags declaration
   !
@@ -260,7 +292,7 @@ MODULE H5GLOBAL
   INTEGER :: H5_ITER_UNKNOWN_F
   INTEGER :: H5_ITER_INC_F
   INTEGER :: H5_ITER_DEC_F
-  INTEGER :: H5_ITER_NATIVE_F 
+  INTEGER :: H5_ITER_NATIVE_F
   INTEGER :: H5_ITER_N_F
 
   EQUIVALENCE(H5generic_flags(1), H5_INDEX_UNKNOWN_F)
@@ -268,7 +300,7 @@ MODULE H5GLOBAL
   EQUIVALENCE(H5generic_flags(3), H5_INDEX_CRT_ORDER_F)
   EQUIVALENCE(H5generic_flags(4), H5_INDEX_N_F)
   EQUIVALENCE(H5generic_flags(5), H5_ITER_UNKNOWN_F)
-  EQUIVALENCE(H5generic_flags(6), H5_ITER_INC_F)     
+  EQUIVALENCE(H5generic_flags(6), H5_ITER_INC_F)
   EQUIVALENCE(H5generic_flags(7), H5_ITER_DEC_F)
   EQUIVALENCE(H5generic_flags(8), H5_ITER_NATIVE_F)
   EQUIVALENCE(H5generic_flags(9), H5_ITER_N_F)
@@ -301,7 +333,7 @@ MODULE H5GLOBAL
   EQUIVALENCE(H5G_flags(1), H5G_UNKNOWN_F)
   EQUIVALENCE(H5G_flags(2), H5G_GROUP_F)
   EQUIVALENCE(H5G_flags(3), H5G_DATASET_F)
-  EQUIVALENCE(H5G_flags(4), H5G_TYPE_F) 
+  EQUIVALENCE(H5G_flags(4), H5G_TYPE_F)
   ! XXX: Fix problems with H5G_LINK_F! - QAK ! these are really H5L values -MSB-
   EQUIVALENCE(H5G_flags(5), H5G_LINK_F)
   EQUIVALENCE(H5G_flags(6), H5G_LINK_ERROR_F)
@@ -309,10 +341,10 @@ MODULE H5GLOBAL
   EQUIVALENCE(H5G_flags(8), H5G_LINK_SOFT_F)
   ! XXX
 
-  EQUIVALENCE(H5G_flags(9), H5G_STORAGE_TYPE_UNKNOWN_F ) 
-  EQUIVALENCE(H5G_flags(10), H5G_STORAGE_TYPE_SYMBOL_TABLE_F) 
-  EQUIVALENCE(H5G_flags(11), H5G_STORAGE_TYPE_COMPACT_F)  
-  EQUIVALENCE(H5G_flags(12), H5G_STORAGE_TYPE_DENSE_F) 
+  EQUIVALENCE(H5G_flags(9), H5G_STORAGE_TYPE_UNKNOWN_F )
+  EQUIVALENCE(H5G_flags(10), H5G_STORAGE_TYPE_SYMBOL_TABLE_F)
+  EQUIVALENCE(H5G_flags(11), H5G_STORAGE_TYPE_COMPACT_F)
+  EQUIVALENCE(H5G_flags(12), H5G_STORAGE_TYPE_DENSE_F)
   !
   ! H5D flags declaration
   !
@@ -324,7 +356,7 @@ MODULE H5GLOBAL
   !DEC$endif
   COMMON /H5D_FLAGS/ H5D_flags
 
-  INTEGER :: H5D_COMPACT_F  
+  INTEGER :: H5D_COMPACT_F
   INTEGER :: H5D_CONTIGUOUS_F
   INTEGER :: H5D_CHUNKED_F
 
@@ -348,7 +380,7 @@ MODULE H5GLOBAL
   INTEGER :: H5D_FILL_VALUE_DEFAULT_F
   INTEGER :: H5D_FILL_VALUE_USER_DEFINED_F
 
-! shortened "_DEFAULT" to "_DFLT" to satisfy the limit of 31 
+! shortened "_DEFAULT" to "_DFLT" to satisfy the limit of 31
 ! characters for variable names in Fortran.
 
   INTEGER :: H5D_CHUNK_CACHE_NSLOTS_DFLT_F
@@ -377,12 +409,38 @@ MODULE H5GLOBAL
   EQUIVALENCE(H5D_flags(16), H5D_FILL_VALUE_ERROR_F)
   EQUIVALENCE(H5D_flags(17), H5D_FILL_VALUE_UNDEFINED_F)
   EQUIVALENCE(H5D_flags(18), H5D_FILL_VALUE_DEFAULT_F)
-  EQUIVALENCE(H5D_flags(19), H5D_FILL_VALUE_USER_DEFINED_F)  
+  EQUIVALENCE(H5D_flags(19), H5D_FILL_VALUE_USER_DEFINED_F)
 
   EQUIVALENCE(H5D_flags(20), H5D_CHUNK_CACHE_NSLOTS_DFLT_F)
   EQUIVALENCE(H5D_flags(21), H5D_CHUNK_CACHE_NBYTES_DFLT_F)
   EQUIVALENCE(H5D_flags(22), H5D_CHUNK_CACHE_W0_DFLT_F)
 
+  !
+  ! H5E flags declaration
+  !
+  INTEGER, PARAMETER :: H5E_FLAGS_LEN = 4
+  INTEGER H5E_flags(H5E_FLAGS_LEN)
+  INTEGER, PARAMETER :: H5E_HID_FLAGS_LEN = 1
+  INTEGER(HID_T) H5E_hid_flags(H5E_HID_FLAGS_LEN)
+  !DEC$if defined(BUILD_HDF5_DLL)
+  !DEC$ATTRIBUTES DLLEXPORT :: /H5E_FLAGS/
+  !DEC$ATTRIBUTES DLLEXPORT :: /H5E_HID_FLAGS/
+  !DEC$endif
+  COMMON /H5E_FLAGS/ H5E_flags
+  COMMON /H5E_HID_FLAGS/ H5E_hid_flags
+
+  INTEGER(HID_T) :: H5E_DEFAULT_F
+  INTEGER :: H5E_MAJOR_F
+  INTEGER :: H5E_MINOR_F
+  INTEGER :: H5E_WALK_UPWARD_F
+  INTEGER :: H5E_WALK_DOWNWARD_F
+
+  EQUIVALENCE(H5E_hid_flags(1), H5E_DEFAULT_F)
+  EQUIVALENCE(H5E_flags(1)    , H5E_MAJOR_F)
+  EQUIVALENCE(H5E_flags(2)    , H5E_MINOR_F)
+  EQUIVALENCE(H5E_flags(3)    , H5E_WALK_UPWARD_F)
+  EQUIVALENCE(H5E_flags(4)    , H5E_WALK_DOWNWARD_F)
+
   !
   ! H5FD flags declaration
   !
@@ -393,7 +451,7 @@ MODULE H5GLOBAL
   !DEC$endif
   COMMON /H5FD_FLAGS/ H5FD_flags
 
-  INTEGER :: H5FD_MPIO_INDEPENDENT_F 
+  INTEGER :: H5FD_MPIO_INDEPENDENT_F
   INTEGER :: H5FD_MPIO_COLLECTIVE_F
   INTEGER :: H5FD_MEM_NOLIST_F
   INTEGER :: H5FD_MEM_DEFAULT_F
@@ -473,10 +531,10 @@ MODULE H5GLOBAL
   INTEGER, PARAMETER :: H5L_FLAGS_LEN = 6
   INTEGER :: H5L_flags(H5L_FLAGS_LEN)
 
-  INTEGER :: H5L_LINK_F
-  INTEGER :: H5L_LINK_ERROR_F
-  INTEGER :: H5L_LINK_HARD_F
-  INTEGER :: H5L_LINK_SOFT_F
+  INTEGER :: H5L_TYPE_ERROR_F
+  INTEGER :: H5L_TYPE_HARD_F
+  INTEGER :: H5L_TYPE_SOFT_F
+  INTEGER :: H5L_TYPE_EXTERNAL_F
   INTEGER :: H5L_SAME_LOC_F
   INTEGER :: H5L_LINK_CLASS_T_VERS_F
 
@@ -485,10 +543,10 @@ MODULE H5GLOBAL
   !DEC$endif
   COMMON /H5L_FLAGS/ H5L_flags
 
-  EQUIVALENCE(H5L_flags(1), H5L_LINK_F)
-  EQUIVALENCE(H5L_flags(2), H5L_LINK_ERROR_F)
-  EQUIVALENCE(H5L_flags(3), H5L_LINK_HARD_F)
-  EQUIVALENCE(H5L_flags(4), H5L_LINK_SOFT_F)
+  EQUIVALENCE(H5L_flags(1), H5L_TYPE_ERROR_F)
+  EQUIVALENCE(H5L_flags(2), H5L_TYPE_HARD_F)
+  EQUIVALENCE(H5L_flags(3), H5L_TYPE_SOFT_F)
+  EQUIVALENCE(H5L_flags(4), H5L_TYPE_EXTERNAL_F)
   EQUIVALENCE(H5L_flags(5), H5L_SAME_LOC_F)
   EQUIVALENCE(H5L_flags(6), H5L_LINK_CLASS_T_VERS_F)
   !
@@ -501,11 +559,11 @@ MODULE H5GLOBAL
   !DEC$endif
   COMMON /H5O_FLAGS/ H5O_flags
 
-  INTEGER :: H5O_COPY_SHALLOW_HIERARCHY_F ! * THESE VARIABLES DO
-  INTEGER :: H5O_COPY_EXPAND_SOFT_LINK_F ! NOT MATCH THE C VARIABLE
-  INTEGER :: H5O_COPY_EXPAND_EXT_LINK_F !  IN ORDER
-  INTEGER :: H5O_COPY_EXPAND_REFERENCE_F ! TO STAY UNDER THE
-  INTEGER :: H5O_COPY_WITHOUT_ATTR_FLAG_F 
+  INTEGER :: H5O_COPY_SHALLOW_HIERARCHY_F ! *** THESE VARIABLES DO
+  INTEGER :: H5O_COPY_EXPAND_SOFT_LINK_F  ! NOT MATCH THE C VARIABLE
+  INTEGER :: H5O_COPY_EXPAND_EXT_LINK_F   ! IN ORDER
+  INTEGER :: H5O_COPY_EXPAND_REFERENCE_F  ! TO STAY UNDER THE
+  INTEGER :: H5O_COPY_WITHOUT_ATTR_FLAG_F
   INTEGER :: H5O_COPY_PRESERVE_NULL_FLAG_F
   INTEGER :: H5O_COPY_ALL_F
   INTEGER :: H5O_SHMESG_NONE_FLAG_F
@@ -518,7 +576,7 @@ MODULE H5GLOBAL
   INTEGER :: H5O_HDR_CHUNK0_SIZE_F
   INTEGER :: H5O_HDR_ATTR_CRT_ORDER_TRACK_F ! 32 CHARACTER
   INTEGER :: H5O_HDR_ATTR_CRT_ORDER_INDEX_F ! VARIABLE
-  INTEGER :: H5O_HDR_ATTR_STORE_PHASE_CHA_F !  LENGTH *
+  INTEGER :: H5O_HDR_ATTR_STORE_PHASE_CHA_F ! LENGTH ***
   INTEGER :: H5O_HDR_STORE_TIMES_F
   INTEGER :: H5O_HDR_ALL_FLAGS_F
   INTEGER :: H5O_SHMESG_MAX_NINDEXES_F
@@ -543,27 +601,27 @@ MODULE H5GLOBAL
   EQUIVALENCE(h5o_flags(17) , H5O_HDR_ATTR_CRT_ORDER_INDEX_F)
   EQUIVALENCE(h5o_flags(18) , H5O_HDR_ATTR_STORE_PHASE_CHA_F)
   EQUIVALENCE(h5o_flags(19) , H5O_HDR_STORE_TIMES_F)
-  EQUIVALENCE(h5o_flags(20) , H5O_HDR_ALL_FLAGS_F) 
+  EQUIVALENCE(h5o_flags(20) , H5O_HDR_ALL_FLAGS_F)
   EQUIVALENCE(h5o_flags(21) , H5O_SHMESG_MAX_NINDEXES_F)
   EQUIVALENCE(h5o_flags(22) , H5O_SHMESG_MAX_LIST_SIZE_F)
 
   !
   ! H5P flags declaration
   !
-  INTEGER, PARAMETER :: H5P_FLAGS_LEN = 18 
+  INTEGER, PARAMETER :: H5P_FLAGS_LEN = 18
   INTEGER(HID_T) H5P_flags(H5P_FLAGS_LEN)
   !DEC$if defined(BUILD_HDF5_DLL)
   !DEC$ATTRIBUTES DLLEXPORT :: /H5P_FLAGS/
   !DEC$endif
   COMMON /H5P_FLAGS/ H5P_flags
 
-  INTEGER(HID_T) :: H5P_FILE_CREATE_F 
-  INTEGER(HID_T) :: H5P_FILE_ACCESS_F 
+  INTEGER(HID_T) :: H5P_FILE_CREATE_F
+  INTEGER(HID_T) :: H5P_FILE_ACCESS_F
   INTEGER(HID_T) :: H5P_DATASET_CREATE_F
-  INTEGER(HID_T) :: H5P_DATASET_XFER_F 
-  INTEGER(HID_T) :: H5P_FILE_MOUNT_F 
-  INTEGER(HID_T) :: H5P_DEFAULT_F 
-  INTEGER(HID_T) :: H5P_ROOT_F  
+  INTEGER(HID_T) :: H5P_DATASET_XFER_F
+  INTEGER(HID_T) :: H5P_FILE_MOUNT_F
+  INTEGER(HID_T) :: H5P_DEFAULT_F
+  INTEGER(HID_T) :: H5P_ROOT_F
   INTEGER(HID_T) :: H5P_OBJECT_CREATE_F
   INTEGER(HID_T) :: H5P_DATASET_ACCESS_F
   INTEGER(HID_T) :: H5P_GROUP_CREATE_F
@@ -598,14 +656,14 @@ MODULE H5GLOBAL
   !
   ! H5P intgere flags declaration
   !
-  INTEGER, PARAMETER :: H5P_FLAGS_INT_LEN = 2 
+  INTEGER, PARAMETER :: H5P_FLAGS_INT_LEN = 2
   INTEGER  H5P_flags_int(H5P_FLAGS_INT_LEN)
   !DEC$if defined(BUILD_HDF5_DLL)
   !DEC$ATTRIBUTES DLLEXPORT :: /H5P_FLAGS_INT/
   !DEC$endif
   COMMON /H5P_FLAGS_INT/ H5P_flags_int
-  INTEGER :: H5P_CRT_ORDER_INDEXED_F 
-  INTEGER :: H5P_CRT_ORDER_TRACKED_F 
+  INTEGER :: H5P_CRT_ORDER_INDEXED_F
+  INTEGER :: H5P_CRT_ORDER_TRACKED_F
   EQUIVALENCE(H5P_flags_int(1), H5P_CRT_ORDER_INDEXED_F)
   EQUIVALENCE(H5P_flags_int(2), H5P_CRT_ORDER_TRACKED_F)
   !
@@ -627,16 +685,16 @@ MODULE H5GLOBAL
   !
   ! H5S flags declaration
   !
-  INTEGER, PARAMETER :: H5S_FLAGS_LEN = 20 
+  INTEGER, PARAMETER :: H5S_FLAGS_LEN = 20
   INTEGER H5S_flags(H5S_FLAGS_LEN)
   !DEC$if defined(BUILD_HDF5_DLL)
   !DEC$ATTRIBUTES DLLEXPORT :: /H5S_FLAGS/
   !DEC$endif
   COMMON /H5S_FLAGS/ H5S_flags
 
-  INTEGER :: H5S_SCALAR_F 
-  INTEGER :: H5S_SIMPLE_F 
-  INTEGER :: H5S_NULL_F 
+  INTEGER :: H5S_SCALAR_F
+  INTEGER :: H5S_SIMPLE_F
+  INTEGER :: H5S_NULL_F
 
   INTEGER :: H5S_UNLIMITED_F
   INTEGER :: H5S_ALL_F
@@ -644,13 +702,13 @@ MODULE H5GLOBAL
   INTEGER :: H5S_SELECT_NOOP_F
   INTEGER :: H5S_SELECT_SET_F
   INTEGER :: H5S_SELECT_OR_F
-  INTEGER :: H5S_SELECT_AND_F 
-  INTEGER :: H5S_SELECT_XOR_F 
-  INTEGER :: H5S_SELECT_NOTB_F 
-  INTEGER :: H5S_SELECT_NOTA_F 
-  INTEGER :: H5S_SELECT_APPEND_F 
-  INTEGER :: H5S_SELECT_PREPEND_F 
-  INTEGER :: H5S_SELECT_INVALID_F 
+  INTEGER :: H5S_SELECT_AND_F
+  INTEGER :: H5S_SELECT_XOR_F
+  INTEGER :: H5S_SELECT_NOTB_F
+  INTEGER :: H5S_SELECT_NOTA_F
+  INTEGER :: H5S_SELECT_APPEND_F
+  INTEGER :: H5S_SELECT_PREPEND_F
+  INTEGER :: H5S_SELECT_INVALID_F
 
 
   INTEGER :: H5S_SEL_ERROR_F
@@ -668,12 +726,12 @@ MODULE H5GLOBAL
   EQUIVALENCE(H5S_flags(7), H5S_ALL_F)
 
   EQUIVALENCE(H5S_flags(8), H5S_SELECT_NOOP_F)
-  EQUIVALENCE(H5S_flags(9), H5S_SELECT_AND_F) 
+  EQUIVALENCE(H5S_flags(9), H5S_SELECT_AND_F)
   EQUIVALENCE(H5S_flags(10), H5S_SELECT_XOR_F)
   EQUIVALENCE(H5S_flags(11), H5S_SELECT_NOTB_F)
   EQUIVALENCE(H5S_flags(12), H5S_SELECT_NOTA_F)
-  EQUIVALENCE(H5S_flags(13), H5S_SELECT_APPEND_F) 
-  EQUIVALENCE(H5S_flags(14), H5S_SELECT_PREPEND_F) 
+  EQUIVALENCE(H5S_flags(13), H5S_SELECT_APPEND_F)
+  EQUIVALENCE(H5S_flags(14), H5S_SELECT_PREPEND_F)
   EQUIVALENCE(H5S_flags(15), H5S_SELECT_INVALID_F)
 
 
@@ -682,51 +740,51 @@ MODULE H5GLOBAL
   EQUIVALENCE(H5S_flags(18), H5S_SEL_POINTS_F)
   EQUIVALENCE(H5S_flags(19), H5S_SEL_HYPERSLABS_F)
   EQUIVALENCE(H5S_flags(20), H5S_SEL_ALL_F)
-
-
   !
   ! H5T flags declaration
   !
-  INTEGER, PARAMETER :: H5T_FLAGS_LEN = 33
+  INTEGER, PARAMETER :: H5T_FLAGS_LEN = 35
   INTEGER H5T_flags(H5T_FLAGS_LEN)
   !DEC$if defined(BUILD_HDF5_DLL)
   !DEC$ATTRIBUTES DLLEXPORT :: /H5T_FLAGS/
   !DEC$endif
   COMMON /H5T_FLAGS/ H5T_flags
 
-  INTEGER ::  H5T_NO_CLASS_F 
-  INTEGER ::  H5T_INTEGER_F 
-  INTEGER ::  H5T_FLOAT_F  
-  INTEGER ::  H5T_TIME_F 
-  INTEGER ::  H5T_STRING_F 
-  INTEGER ::  H5T_BITFIELD_F
-  INTEGER ::  H5T_OPAQUE_F 
-  INTEGER ::  H5T_COMPOUND_F 
-  INTEGER ::  H5T_REFERENCE_F
-  INTEGER ::  H5T_ENUM_F 
-  INTEGER ::  H5T_VLEN_F
-  INTEGER ::  H5T_ARRAY_F
-  INTEGER ::  H5T_ORDER_LE_F 
-  INTEGER ::  H5T_ORDER_BE_F
-  INTEGER ::  H5T_ORDER_VAX_F
-  INTEGER ::  H5T_PAD_ZERO_F
-  INTEGER ::  H5T_PAD_ONE_F
-  INTEGER ::  H5T_PAD_BACKGROUND_F
-  INTEGER ::  H5T_PAD_ERROR_F    
-  INTEGER ::  H5T_SGN_NONE_F   
-  INTEGER ::  H5T_SGN_2_F     
-  INTEGER ::  H5T_SGN_ERROR_F
-  INTEGER ::  H5T_NORM_IMPLIED_F
-  INTEGER ::  H5T_NORM_MSBSET_F
-  INTEGER ::  H5T_NORM_NONE_F 
-  INTEGER ::  H5T_CSET_ASCII_F
-  INTEGER ::  H5T_CSET_UTF8_F
-  INTEGER ::  H5T_STR_NULLTERM_F 
-  INTEGER ::  H5T_STR_NULLPAD_F 
-  INTEGER ::  H5T_STR_SPACEPAD_F
-  INTEGER ::  H5T_STR_ERROR_F
-  INTEGER ::  H5T_DIR_ASCEND_F
-  INTEGER ::  H5T_DIR_DESCEND_F
+  INTEGER :: H5T_NO_CLASS_F
+  INTEGER :: H5T_INTEGER_F
+  INTEGER :: H5T_FLOAT_F
+  INTEGER :: H5T_TIME_F
+  INTEGER :: H5T_STRING_F
+  INTEGER :: H5T_BITFIELD_F
+  INTEGER :: H5T_OPAQUE_F
+  INTEGER :: H5T_COMPOUND_F
+  INTEGER :: H5T_REFERENCE_F
+  INTEGER :: H5T_ENUM_F
+  INTEGER :: H5T_VLEN_F
+  INTEGER :: H5T_ARRAY_F
+  INTEGER :: H5T_ORDER_LE_F
+  INTEGER :: H5T_ORDER_BE_F
+  INTEGER :: H5T_ORDER_VAX_F
+  INTEGER :: H5T_ORDER_NONE_F
+  INTEGER :: H5T_ORDER_MIXED_F
+  INTEGER :: H5T_PAD_ZERO_F
+  INTEGER :: H5T_PAD_ONE_F
+  INTEGER :: H5T_PAD_BACKGROUND_F
+  INTEGER :: H5T_PAD_ERROR_F
+  INTEGER :: H5T_SGN_NONE_F
+  INTEGER :: H5T_SGN_2_F
+  INTEGER :: H5T_SGN_ERROR_F
+  INTEGER :: H5T_NORM_IMPLIED_F
+  INTEGER :: H5T_NORM_MSBSET_F
+  INTEGER :: H5T_NORM_NONE_F
+  INTEGER :: H5T_CSET_ASCII_F
+  INTEGER :: H5T_CSET_UTF8_F
+  INTEGER :: H5T_STR_NULLTERM_F
+  INTEGER :: H5T_STR_NULLPAD_F
+  INTEGER :: H5T_STR_SPACEPAD_F
+  INTEGER :: H5T_STR_ERROR_F
+  INTEGER :: H5T_DIR_ASCEND_F
+  INTEGER :: H5T_DIR_DESCEND_F
 
   EQUIVALENCE(H5T_flags(1), H5T_NO_CLASS_F)
   EQUIVALENCE(H5T_flags(2), H5T_INTEGER_F)
@@ -740,45 +798,46 @@ MODULE H5GLOBAL
   EQUIVALENCE(H5T_flags(10), H5T_ENUM_F)
   EQUIVALENCE(H5T_flags(11), H5T_ORDER_LE_F)
   EQUIVALENCE(H5T_flags(12), H5T_ORDER_BE_F)
-  EQUIVALENCE(H5T_flags(13), H5T_ORDER_VAX_F)
-  EQUIVALENCE(H5T_flags(14), H5T_PAD_ZERO_F)
-  EQUIVALENCE(H5T_flags(15), H5T_PAD_ONE_F)
-  EQUIVALENCE(H5T_flags(16), H5T_PAD_BACKGROUND_F)
-  EQUIVALENCE(H5T_flags(17), H5T_PAD_ERROR_F)
-  EQUIVALENCE(H5T_flags(18), H5T_SGN_NONE_F)
-  EQUIVALENCE(H5T_flags(19), H5T_SGN_2_F)
-  EQUIVALENCE(H5T_flags(20), H5T_SGN_ERROR_F)
-  EQUIVALENCE(H5T_flags(21), H5T_NORM_IMPLIED_F)
-  EQUIVALENCE(H5T_flags(22), H5T_NORM_MSBSET_F)
-  EQUIVALENCE(H5T_flags(23), H5T_NORM_NONE_F)
-  EQUIVALENCE(H5T_flags(24), H5T_CSET_ASCII_F)
-  EQUIVALENCE(H5T_flags(25), H5T_CSET_UTF8_F)
-  EQUIVALENCE(H5T_flags(26), H5T_STR_NULLTERM_F)
-  EQUIVALENCE(H5T_flags(27), H5T_STR_NULLPAD_F)
-  EQUIVALENCE(H5T_flags(28), H5T_STR_SPACEPAD_F)
-  EQUIVALENCE(H5T_flags(29), H5T_STR_ERROR_F)
-  EQUIVALENCE(H5T_flags(30), H5T_VLEN_F)
-  EQUIVALENCE(H5T_flags(31), H5T_ARRAY_F)
-  EQUIVALENCE(H5T_flags(32), H5T_DIR_ASCEND_F)
-  EQUIVALENCE(H5T_flags(33), H5T_DIR_DESCEND_F)
-
+  EQUIVALENCE(H5T_flags(13), H5T_ORDER_MIXED_F)
+  EQUIVALENCE(H5T_flags(14), H5T_ORDER_VAX_F)
+  EQUIVALENCE(H5T_flags(15), H5T_ORDER_NONE_F)
+  EQUIVALENCE(H5T_flags(16), H5T_PAD_ZERO_F)
+  EQUIVALENCE(H5T_flags(17), H5T_PAD_ONE_F)
+  EQUIVALENCE(H5T_flags(18), H5T_PAD_BACKGROUND_F)
+  EQUIVALENCE(H5T_flags(19), H5T_PAD_ERROR_F)
+  EQUIVALENCE(H5T_flags(20), H5T_SGN_NONE_F)
+  EQUIVALENCE(H5T_flags(21), H5T_SGN_2_F)
+  EQUIVALENCE(H5T_flags(22), H5T_SGN_ERROR_F)
+  EQUIVALENCE(H5T_flags(23), H5T_NORM_IMPLIED_F)
+  EQUIVALENCE(H5T_flags(24), H5T_NORM_MSBSET_F)
+  EQUIVALENCE(H5T_flags(25), H5T_NORM_NONE_F)
+  EQUIVALENCE(H5T_flags(26), H5T_CSET_ASCII_F)
+  EQUIVALENCE(H5T_flags(27), H5T_CSET_UTF8_F)
+  EQUIVALENCE(H5T_flags(28), H5T_STR_NULLTERM_F)
+  EQUIVALENCE(H5T_flags(29), H5T_STR_NULLPAD_F)
+  EQUIVALENCE(H5T_flags(30), H5T_STR_SPACEPAD_F)
+  EQUIVALENCE(H5T_flags(31), H5T_STR_ERROR_F)
+  EQUIVALENCE(H5T_flags(32), H5T_VLEN_F)
+  EQUIVALENCE(H5T_flags(33), H5T_ARRAY_F)
+  EQUIVALENCE(H5T_flags(34), H5T_DIR_ASCEND_F)
+  EQUIVALENCE(H5T_flags(35), H5T_DIR_DESCEND_F)
   !
   ! H5Z flags declaration
   !
-  INTEGER, PARAMETER :: H5Z_FLAGS_LEN = 14
+  INTEGER, PARAMETER :: H5Z_FLAGS_LEN = 20
   INTEGER H5Z_flags(H5Z_FLAGS_LEN)
   !DEC$if defined(BUILD_HDF5_DLL)
   !DEC$ATTRIBUTES DLLEXPORT :: /H5Z_FLAGS/
   !DEC$endif
   COMMON /H5Z_FLAGS/ H5Z_flags
 
-  INTEGER :: H5Z_FILTER_ERROR_F 
-  INTEGER :: H5Z_FILTER_NONE_F 
+  INTEGER :: H5Z_FILTER_ERROR_F
+  INTEGER :: H5Z_FILTER_NONE_F
   INTEGER :: H5Z_FILTER_ALL_F
-  INTEGER :: H5Z_FILTER_DEFLATE_F 
-  INTEGER :: H5Z_FILTER_SHUFFLE_F 
-  INTEGER :: H5Z_FILTER_FLETCHER32_F 
-  INTEGER :: H5Z_FILTER_SZIP_F 
+  INTEGER :: H5Z_FILTER_DEFLATE_F
+  INTEGER :: H5Z_FILTER_SHUFFLE_F
+  INTEGER :: H5Z_FILTER_FLETCHER32_F
+  INTEGER :: H5Z_FILTER_SZIP_F
   INTEGER :: H5Z_ERROR_EDC_F
   INTEGER :: H5Z_DISABLE_EDC_F
   INTEGER :: H5Z_ENABLE_EDC_F
@@ -786,6 +845,12 @@ MODULE H5GLOBAL
   INTEGER :: H5Z_FLAG_OPTIONAL_F
   INTEGER :: H5Z_FILTER_ENCODE_ENABLED_F
   INTEGER :: H5Z_FILTER_DECODE_ENABLED_F
+  INTEGER :: H5Z_FILTER_NBIT_F
+  INTEGER :: H5Z_FILTER_SCALEOFFSET_F
+  INTEGER :: H5Z_SO_FLOAT_DSCALE_F
+  INTEGER :: H5Z_SO_FLOAT_ESCALE_F
+  INTEGER :: H5Z_SO_INT_F
+  INTEGER :: H5Z_SO_INT_MINBITS_DEFAULT_F
 
   EQUIVALENCE(H5Z_flags(1), H5Z_FILTER_ERROR_F)
   EQUIVALENCE(H5Z_flags(2), H5Z_FILTER_NONE_F)
@@ -801,13 +866,18 @@ MODULE H5GLOBAL
   EQUIVALENCE(H5Z_flags(12), H5Z_FILTER_ENCODE_ENABLED_F)
   EQUIVALENCE(H5Z_flags(13), H5Z_FILTER_DECODE_ENABLED_F)
   EQUIVALENCE(H5Z_flags(14), H5Z_FILTER_ALL_F)
-
+  EQUIVALENCE(H5Z_flags(15), H5Z_FILTER_NBIT_F)
+  EQUIVALENCE(H5Z_flags(16), H5Z_FILTER_SCALEOFFSET_F)
+  EQUIVALENCE(H5Z_flags(17), H5Z_SO_FLOAT_DSCALE_F)
+  EQUIVALENCE(H5Z_flags(18), H5Z_SO_FLOAT_ESCALE_F)
+  EQUIVALENCE(H5Z_flags(19), H5Z_SO_INT_F)
+  EQUIVALENCE(H5Z_flags(20), H5Z_SO_INT_MINBITS_DEFAULT_F)
 
   !
   ! H5 Library flags declaration
   !
   INTEGER, PARAMETER :: H5LIB_FLAGS_LEN =  2
-  INTEGER :: H5LIB_flags(H5LIB_FLAGS_LEN) 
+  INTEGER :: H5LIB_flags(H5LIB_FLAGS_LEN)
   !DEC$if defined(BUILD_HDF5_DLL)
   !DEC$ATTRIBUTES DLLEXPORT :: /H5LIB_FLAGS/
   !DEC$endif
