@@ -1,4 +1,13 @@
-! * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
+!****h* root/fortran/test/tH5A.f90
+!
+! NAME
+!  tH5A.f90
+!
+! FUNCTION
+!  Basic testing of Fortran H5A APIs.
+!
+! COPYRIGHT
+! * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 !   Copyright by The HDF Group.                                               *
 !   Copyright by the Board of Trustees of the University of Illinois.         *
 !   All rights reserved.                                                      *
@@ -11,23 +20,29 @@
 !   is linked from the top-level documents page.  It can also be found at     *
 !   http://hdfgroup.org/HDF5/doc/Copyright.html.  If you do not have          *
 !   access to either file, you may request a copy from help@hdfgroup.org.     *
-! * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
+! * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 !
+! CONTAINS SUBROUTINES
+!  attribute_test 
+!
+!
+!*****
+
     SUBROUTINE attribute_test(cleanup, total_error)
 
-!   This subroutine tests following functionalities: 
+!   This subroutine tests following functionalities:
 !   h5acreate_f,  h5awrite_f, h5aclose_f,h5aread_f, h5aopen_name_f,
 !   h5aget_name_f,h5aget_space_f, h5aget_type_f,
-! 
+!
+
+     USE HDF5 ! This module contains all necessary modules
 
-     USE HDF5 ! This module contains all necessary modules 
-        
      IMPLICIT NONE
      LOGICAL, INTENT(IN)  :: cleanup
-     INTEGER, INTENT(OUT) :: total_error 
+     INTEGER, INTENT(OUT) :: total_error
 
      CHARACTER(LEN=5), PARAMETER :: filename = "atest"    !File name
-     CHARACTER(LEN=80) :: fix_filename 
+     CHARACTER(LEN=80) :: fix_filename
      CHARACTER(LEN=9), PARAMETER :: dsetname = "atestdset"        !Dataset name
      CHARACTER(LEN=11), PARAMETER :: aname = "attr_string"   !String Attribute name
      CHARACTER(LEN=14), PARAMETER :: aname2 = "attr_character"!Character Attribute name
@@ -35,7 +50,7 @@
      CHARACTER(LEN=9), PARAMETER :: aname4 = "attr_real"      !Real Attribute name
      CHARACTER(LEN=12), PARAMETER :: aname5 = "attr_integer"  !Integer Attribute name
      CHARACTER(LEN=9), PARAMETER :: aname6 = "attr_null"     !Null Attribute name
-    
+
      !
      !data space rank and dimensions
      !
@@ -45,46 +60,44 @@
 
 
 
-     INTEGER(HID_T) :: file_id       ! File identifier 
-     INTEGER(HID_T) :: dset_id       ! Dataset identifier 
-     INTEGER(HID_T) :: dataspace     ! Dataspace identifier for dataset 
+     INTEGER(HID_T) :: file_id       ! File identifier
+     INTEGER(HID_T) :: dset_id       ! Dataset identifier
+     INTEGER(HID_T) :: dataspace     ! Dataspace identifier for dataset
 
-     INTEGER(HID_T) :: attr_id        !String Attribute identifier 
-     INTEGER(HID_T) :: attr2_id       !Character Attribute identifier 
-     INTEGER(HID_T) :: attr3_id       !Double Attribute identifier 
-     INTEGER(HID_T) :: attr4_id       !Real Attribute identifier 
-     INTEGER(HID_T) :: attr5_id       !Integer Attribute identifier 
-     INTEGER(HID_T) :: attr6_id       !Null Attribute identifier 
-     INTEGER(HID_T) :: aspace_id      !String Attribute Dataspace identifier 
-     INTEGER(HID_T) :: aspace2_id     !Character Attribute Dataspace identifier 
-     INTEGER(HID_T) :: aspace3_id     !Double Attribute Dataspace identifier 
-     INTEGER(HID_T) :: aspace4_id     !Real Attribute Dataspace identifier 
-     INTEGER(HID_T) :: aspace5_id     !Integer Attribute Dataspace identifier 
-     INTEGER(HID_T) :: aspace6_id     !Null Attribute Dataspace identifier 
-     INTEGER(HID_T) :: atype_id       !String Attribute Datatype identifier 
-     INTEGER(HID_T) :: atype2_id      !Character Attribute Datatype identifier 
-     INTEGER(HID_T) :: atype3_id      !Double Attribute Datatype identifier 
-     INTEGER(HID_T) :: atype4_id      !Real Attribute Datatype identifier 
-     INTEGER(HID_T) :: atype5_id      !Integer Attribute Datatype identifier 
+     INTEGER(HID_T) :: attr_id        !String Attribute identifier
+     INTEGER(HID_T) :: attr2_id       !Character Attribute identifier
+     INTEGER(HID_T) :: attr3_id       !Double Attribute identifier
+     INTEGER(HID_T) :: attr4_id       !Real Attribute identifier
+     INTEGER(HID_T) :: attr5_id       !Integer Attribute identifier
+     INTEGER(HID_T) :: attr6_id       !Null Attribute identifier
+     INTEGER(HID_T) :: aspace_id      !String Attribute Dataspace identifier
+     INTEGER(HID_T) :: aspace2_id     !Character Attribute Dataspace identifier
+     INTEGER(HID_T) :: aspace6_id     !Null Attribute Dataspace identifier
+     INTEGER(HID_T) :: atype_id       !String Attribute Datatype identifier
+     INTEGER(HID_T) :: atype2_id      !Character Attribute Datatype identifier
+     INTEGER(HID_T) :: atype3_id      !Double Attribute Datatype identifier
+     INTEGER(HID_T) :: atype4_id      !Real Attribute Datatype identifier
+     INTEGER(HID_T) :: atype5_id      !Integer Attribute Datatype identifier
      INTEGER(HSIZE_T), DIMENSION(1) :: adims = (/2/) ! Attribute dimension
      INTEGER(HSIZE_T), DIMENSION(1) :: adims2 = (/1/) ! Attribute dimension
      INTEGER     ::   arank = 1                      ! Attribure rank
      INTEGER(SIZE_T) :: attrlen    ! Length of the attribute string
 
-     INTEGER(HID_T) :: attr_space     !Returned String Attribute Space identifier 
-     INTEGER(HID_T) :: attr2_space    !Returned other Attribute Space identifier 
+     INTEGER(HID_T) :: attr_space     !Returned String Attribute Space identifier
+     INTEGER(HID_T) :: attr2_space    !Returned other Attribute Space identifier
      INTEGER(HID_T) :: attr_type      !Returned Attribute Datatype identifier
      INTEGER(HID_T) :: attr2_type      !Returned CHARACTER Attribute Datatype identifier
      INTEGER(HID_T) :: attr3_type      !Returned DOUBLE Attribute Datatype identifier
      INTEGER(HID_T) :: attr4_type      !Returned REAL Attribute Datatype identifier
      INTEGER(HID_T) :: attr5_type      !Returned INTEGER Attribute Datatype identifier
      INTEGER(HID_T) :: attr6_type      !Returned NULL Attribute Datatype identifier
-     INTEGER        :: num_attrs      !number of attributes 
+     INTEGER        :: num_attrs      !number of attributes
+     INTEGER(HSIZE_T) :: attr_storage   ! attributes storage requirements .MSB.
      CHARACTER(LEN=256) :: attr_name    !buffer to put attr_name
      INTEGER(SIZE_T)    ::  name_size = 80 !attribute name length
 
      CHARACTER(LEN=35), DIMENSION(2) ::  attr_data  ! String attribute data
-     CHARACTER(LEN=35), DIMENSION(2) ::  aread_data ! Buffer to put read back 
+     CHARACTER(LEN=35), DIMENSION(2) ::  aread_data ! Buffer to put read back
                                                ! string attr data
      CHARACTER ::  attr_character_data = 'A'
      DOUBLE PRECISION,  DIMENSION(1) ::  attr_double_data = 3.459
@@ -92,7 +105,7 @@
      INTEGER,      DIMENSION(1) ::  attr_integer_data = 5
      INTEGER(HSIZE_T), DIMENSION(7) :: data_dims
 
-     
+
      CHARACTER :: aread_character_data ! variable to put read back Character attr data
      INTEGER, DIMENSION(1)  :: aread_integer_data ! variable to put read back integer attr data
      INTEGER, DIMENSION(1)  :: aread_null_data = 7 ! variable to put read back null attr data
@@ -100,56 +113,56 @@
      REAL, DIMENSION(1)  :: aread_real_data ! variable to put read back real attr data
 
      !
-     !general purpose integer 
-     !         
+     !general purpose integer
+     !
      INTEGER     ::   i, j
      INTEGER     ::   error ! Error flag
-     
-     ! 
+
+     !
      !The dimensions for the dataset.
      !
      INTEGER(HSIZE_T), DIMENSION(2) :: dims = (/NX,NY/)
 
      !
-     !data buffers 
-     !         
-     INTEGER, DIMENSION(NX,NY) :: data_in, data_out
+     !data buffers
+     !
+     INTEGER, DIMENSION(NX,NY) :: data_in
 
 
      !
      !Initialize data_in buffer
      !
-     do i = 1, NX
-        do j = 1, NY
+     DO i = 1, NX
+        DO j = 1, NY
            data_in(i,j) =  (i-1) + (j-1)
-        end do
-     end do
+        END DO
+     END DO
      !
      ! Initialize attribute's data
      !
      attr_data(1) = 'Dataset character attribute'
-     attr_data(2) = 'Some other string here     ' 
-     attrlen = len(attr_data(1)) 
-    
+     attr_data(2) = 'Some other string here     '
+     attrlen = LEN(attr_data(1))
+
      !
      ! Create the file.
      !
           CALL h5_fixname_f(filename, fix_filename, H5P_DEFAULT_F, error)
-          if (error .ne. 0) then
-              write(*,*) "Cannot modify file name"
-              stop
-          endif
+          IF (error .NE. 0) THEN
+              WRITE(*,*) "Cannot modify file name"
+              STOP
+          ENDIF
      CALL h5fcreate_f(fix_filename, H5F_ACC_TRUNC_F, file_id, error)
      CALL check("h5fcreate_f",error,total_error)
 
      !
-     !Create data space for the dataset. 
+     !Create data space for the dataset.
      !
      CALL h5screate_simple_f(RANK, dims, dataspace, error)
      CALL check("h5screate_simple_f",error,total_error)
 
      !
-     ! create dataset in the file. 
+     ! create dataset in the file.
      !
      CALL h5dcreate_f(file_id, dsetname, H5T_NATIVE_INTEGER, dataspace, &
                dset_id, error)
@@ -164,17 +177,17 @@
      CALL check("h5dwrite_f",error,total_error)
 
      !
-     ! Create scalar data space for the String attribute. 
+     ! Create scalar data space for the String attribute.
      !
      CALL h5screate_simple_f(arank, adims, aspace_id, error)
      CALL check("h5screate_simple_f",error,total_error)
      !
-     ! Create scalar data space for all other attributes. 
+     ! Create scalar data space for all other attributes.
      !
      CALL h5screate_simple_f(arank, adims2, aspace2_id, error)
      CALL check("h5screate_simple_f",error,total_error)
      !
-     ! Create null data space for null attributes. 
+     ! Create null data space for null attributes.
      !
      CALL h5screate_f(H5S_NULL_F, aspace6_id, error)
      CALL check("h5screate_f",error,total_error)
@@ -224,7 +237,7 @@
      CALL h5acreate_f(dset_id, aname2, atype2_id, aspace2_id, &
                       attr2_id, error)
      CALL check("h5acreate_f",error,total_error)
-     
+
 
      !
      ! Create dataset DOUBLE attribute.
@@ -247,10 +260,12 @@
      !
      ! Create dataset NULL attribute of INTEGER.
      !
+
      CALL h5acreate_f(dset_id, aname6, atype5_id, aspace6_id, &
                       attr6_id, error)
+
      CALL check("h5acreate_f",error,total_error)
-    
+
      !
      ! Write the String attribute data.
      !
@@ -265,20 +280,20 @@
      !
      ! Write the DOUBLE attribute data.
      !
-     data_dims(1) = 1 
+     data_dims(1) = 1
      CALL h5awrite_f(attr3_id, atype3_id, attr_double_data, data_dims, error)
      CALL check("h5awrite_f",error,total_error)
      !
      ! Write the Real attribute data.
      !
-     data_dims(1) = 1 
+     data_dims(1) = 1
      CALL h5awrite_f(attr4_id, atype4_id, attr_real_data, data_dims, error)
      CALL check("h5awrite_f",error,total_error)
 
      !
      ! Write the Integer attribute data.
      !
-     data_dims(1) = 1 
+     data_dims(1) = 1
      CALL h5awrite_f(attr5_id, atype5_id, attr_integer_data, data_dims, error)
      CALL check("h5awrite_f",error,total_error)
 
@@ -287,9 +302,32 @@
      !
      CALL h5awrite_f(attr6_id, atype5_id, attr_integer_data, data_dims, error)
      CALL check("h5awrite_f",error,total_error)
-   
+
      !
-     ! Close the attribute. 
+     ! check the amount of storage that is required for the specified attribute .MSB.
+     !
+     CALL h5aget_storage_size_f(attr_id, attr_storage, error)
+     CALL check("h5aget_storage_size_f",error,total_error)
+!     CALL VERIFY("h5aget_storage_size_f",attr_storage,*SizeOf(attr_storage),total_error)
+     CALL h5aget_storage_size_f(attr2_id, attr_storage, error)
+     CALL check("h5aget_storage_size_f",error,total_error)
+!     CALL verify("h5aget_storage_size_f",attr_storage,1,total_error)
+     CALL h5aget_storage_size_f(attr3_id, attr_storage, error)
+     CALL check("h5aget_storage_size_f",error,total_error)
+!     CALL verify("h5aget_storage_size_f",attr_storage,8,total_error)
+     CALL h5aget_storage_size_f(attr4_id, attr_storage, error)
+     CALL check("h5aget_storage_size_f",error,total_error)
+!     CALL verify("h5aget_storage_size_f",attr_storage,4,total_error)
+     CALL h5aget_storage_size_f(attr5_id, attr_storage, error)
+     CALL check("h5aget_storage_size_f",error,total_error)
+!     CALL verify("h5aget_storage_size_f",attr_storage,4,total_error)
+     CALL h5aget_storage_size_f(attr6_id, attr_storage, error)
+     CALL check("h5aget_storage_size_f",error,total_error)
+!     CALL verify("h5aget_storage_size_f",attr_storage,0,total_error)
+
+
+     !
+     ! Close the attribute.
      !
      CALL h5aclose_f(attr_id, error)
      CALL check("h5aclose_f",error,total_error)
@@ -371,7 +409,7 @@
      !
      CALL h5aopen_name_f(dset_id, aname5, attr5_id, error)
      CALL check("h5aopen_idx_f",error,total_error)
-     
+
      !
      !open the NULL attrbute by name
      !
@@ -383,13 +421,13 @@
      !
      CALL h5aget_name_f(attr5_id, name_size, attr_name, error)
      CALL check("h5aget_name_f",error,total_error)
-     if (attr_name(1:12) .ne. aname5) then
+     IF (attr_name(1:12) .NE. aname5) THEN
        total_error = total_error + 1
-     end if
-     if (error .ne. 12) then
+     END IF
+     IF (error .NE. 12) THEN
        total_error = total_error + 1
-     end if
-       
+     END IF
+
      !
      !get the STRING attrbute space
      !
@@ -426,7 +464,7 @@
      !
      CALL h5aget_type_f(attr5_id, attr5_type, error)
      CALL check("h5aget_type_f",error,total_error)
-     
+
      !
      !get the null attrbute datatype
      !
@@ -438,10 +476,10 @@
      !
      CALL h5aget_num_attrs_f(dset_id, num_attrs, error)
      CALL check("h5aget_num_attrs_f",error,total_error)
-     if (num_attrs .ne. 6) then
-       write(*,*) "got number of attributes wrong", num_attrs
+     IF (num_attrs .NE. 6) THEN
+       WRITE(*,*) "got number of attributes wrong", num_attrs
        total_error = total_error +1
-     end if
+     END IF
 
      !
      !set the read back data type's size
@@ -458,63 +496,63 @@
      CALL h5aread_f(attr_id, atype_id, aread_data, data_dims, error)
      CALL check("h5aread_f",error,total_error)
 
-     if ( (aread_data(1) .ne. attr_data(1)) .or. (aread_data(2) .ne. attr_data(2)) ) then
-         write(*,*) "Read back string attrbute is wrong", aread_data(1), aread_data(2)
-         total_error = total_error + 1  
-     end if
-     
+     IF ( (aread_data(1) .NE. attr_data(1)) .OR. (aread_data(2) .NE. attr_data(2)) ) THEN
+         WRITE(*,*) "Read back string attrbute is wrong", aread_data(1), aread_data(2)
+         total_error = total_error + 1
+     END IF
+
      !
      !read the CHARACTER attribute data back to memory
      !
      CALL h5aread_f(attr2_id, H5T_NATIVE_CHARACTER, aread_character_data, data_dims, error)
      CALL check("h5aread_f",error,total_error)
-     if (aread_character_data .ne. 'A' ) then
-         write(*,*) "Read back character attrbute is wrong ",aread_character_data
-         total_error = total_error + 1  
-     end if
+     IF (aread_character_data .NE. 'A' ) THEN
+         WRITE(*,*) "Read back character attrbute is wrong ",aread_character_data
+         total_error = total_error + 1
+     END IF
      !
      !read the double attribute data back to memory
      !
-     data_dims(1) = 1 
+     data_dims(1) = 1
      CALL h5aread_f(attr3_id, H5T_NATIVE_DOUBLE, aread_double_data, data_dims, error)
      CALL check("h5aread_f",error,total_error)
-     if (aread_double_data(1) .ne. 3.459 ) then
-         write(*,*) "Read back double attrbute is wrong", aread_double_data(1)
+     IF (aread_double_data(1) .NE. 3.459 ) THEN
+         WRITE(*,*) "Read back double attrbute is wrong", aread_double_data(1)
          total_error = total_error + 1
-     end if  
+     END IF
      !
      !read the real attribute data back to memory
      !
-     data_dims(1) = 1 
+     data_dims(1) = 1
      CALL h5aread_f(attr4_id, H5T_NATIVE_REAL, aread_real_data, data_dims, error)
      CALL check("h5aread_f",error,total_error)
-     if (aread_real_data(1) .ne. 4.0 ) then
-         write(*,*) "Read back real attrbute is wrong ", aread_real_data
-         total_error = total_error + 1 
-     end if 
+     IF (aread_real_data(1) .NE. 4.0 ) THEN
+         WRITE(*,*) "Read back real attrbute is wrong ", aread_real_data
+         total_error = total_error + 1
+     END IF
      !
      !read the Integer attribute data back to memory
      !
-     data_dims(1) = 1 
+     data_dims(1) = 1
      CALL h5aread_f(attr5_id, H5T_NATIVE_INTEGER, aread_integer_data, data_dims, error)
      CALL check("h5aread_f",error,total_error)
-     if (aread_integer_data(1) .ne. 5 ) then
-         write(*,*) "Read back integer attrbute is wrong ", aread_integer_data
-         total_error = total_error + 1 
-     end if 
+     IF (aread_integer_data(1) .NE. 5 ) THEN
+         WRITE(*,*) "Read back integer attrbute is wrong ", aread_integer_data
+         total_error = total_error + 1
+     END IF
      !
      !read the null attribute data. nothing can be read.
      !
-     data_dims(1) = 1 
+     data_dims(1) = 1
      CALL h5aread_f(attr6_id, H5T_NATIVE_INTEGER, aread_null_data, data_dims, error)
      CALL check("h5aread_f",error,total_error)
-     if (aread_null_data(1) .ne. 7 ) then
-         write(*,*) "Read back null attrbute is wrong ", aread_null_data
-         total_error = total_error + 1 
-     end if 
-    
+     IF (aread_null_data(1) .NE. 7 ) THEN
+         WRITE(*,*) "Read back null attrbute is wrong ", aread_null_data
+         total_error = total_error + 1
+     END IF
+
      !
-     ! Close the attribute. 
+     ! Close the attribute.
      !
      CALL h5aclose_f(attr_id, error)
      CALL check("h5aclose_f",error,total_error)
@@ -530,9 +568,9 @@
      CALL check("h5aclose_f",error,total_error)
 
      !
-     ! Delete the attribute from the Dataset. 
+     ! Delete the attribute from the Dataset.
      !
-     CALL h5adelete_f(dset_id, aname, error)     
+     CALL h5adelete_f(dset_id, aname, error)
      CALL check("h5adelete_f",error,total_error)
 
      !
@@ -540,10 +578,10 @@
      !
      CALL h5aget_num_attrs_f(dset_id, num_attrs, error)
      CALL check("h5aget_num_attrs_f",error,total_error)
-     if (num_attrs .ne. 5) then
-       write(*,*) "got number of attributes wrong", num_attrs
+     IF (num_attrs .NE. 5) THEN
+       WRITE(*,*) "got number of attributes wrong", num_attrs
        total_error = total_error +1
-     end if
+     END IF
 
 
 
@@ -568,13 +606,13 @@
      CALL h5tclose_f(attr6_type, error)
      CALL check("h5tclose_f",error,total_error)
 
-     !   
+     !
      ! End access to the dataset and release resources used by it.
-     ! 
+     !
      CALL h5dclose_f(dset_id, error)
      CALL check("h5dclose_f",error,total_error)
 
-     ! 
+     !
      ! Close the file.
      !
      CALL h5fclose_f(file_id, error)
@@ -582,7 +620,7 @@
      !
      ! Remove the file
      !
-     if (cleanup) call h5_cleanup_f(filename, H5P_DEFAULT_F, error)
+     IF (cleanup) CALL h5_cleanup_f(filename, H5P_DEFAULT_F, error)
 
      RETURN
      END SUBROUTINE attribute_test
