@@ -129,14 +129,15 @@ herr_t H5DSattach_scale(hid_t did,
     hssize_t   nelmts;
     hid_t      sid;          /* space ID */
     hid_t      tid = -1;     /* attribute type ID */
+    hid_t      ntid = -1;    /* attribute native type ID */
     hid_t      aid = -1;     /* attribute ID */
     int        rank;         /* rank of dataset */
-    hsize_t    *dims=NULL;   /* dimension of the "REFERENCE_LIST" array */
+    hsize_t    dims[1];      /* dimension of the "REFERENCE_LIST" array */
     ds_list_t  dsl;          /* attribute data in the DS pointing to the dataset */
-    ds_list_t  *dsbuf=NULL;  /* array of attribute data in the DS pointing to the dataset */
+    ds_list_t  *dsbuf = NULL; /* array of attribute data in the DS pointing to the dataset */
     hobj_ref_t ref_to_ds;    /* reference to the DS */
     hobj_ref_t ref_j;        /* iterator reference */
-    hvl_t      *buf=NULL;    /* VL buffer to store in the attribute */
+    hvl_t      *buf = NULL;  /* VL buffer to store in the attribute */
     hid_t      dsid_j;       /* DS dataset ID in DIMENSION_LIST */
     H5O_info_t oi1, oi2;
     H5I_type_t it1, it2;
@@ -148,10 +149,10 @@ herr_t H5DSattach_scale(hid_t did,
     * parameter checking
     *-------------------------------------------------------------------------
     */
-   
+
     if ((is_scale = H5DSis_scale(did)) < 0)
         return FAIL;
-  
+
     /* the dataset cannot be a DS dataset */
     if ( is_scale == 1)
         return FAIL;
@@ -237,11 +238,6 @@ herr_t H5DSattach_scale(hid_t did,
     */
     if (has_dimlist == 0)
     {
-        /* create one entry array */
-        dims = (hsize_t*) malloc (1 * sizeof (hsize_t));
-
-        if(dims == NULL)
-            return FAIL;
 
         dims[0] = rank;
 
@@ -258,8 +254,7 @@ herr_t H5DSattach_scale(hid_t did,
             goto out;
 
         /* allocate and initialize the VL */
-        buf = (hvl_t*)malloc((size_t)rank * sizeof(hvl_t));
-
+        buf = (hvl_t *)malloc((size_t)rank * sizeof(hvl_t));
         if(buf == NULL)
             goto out;
 
@@ -274,30 +269,21 @@ herr_t H5DSattach_scale(hid_t did,
         ((hobj_ref_t *)buf[idx].p)[0] = ref_to_ds;
 
         /* write the attribute with the reference */
-        if (H5Awrite(aid,tid,buf) < 0)
+        if(H5Awrite(aid, tid, buf) < 0)
             goto out;
 
         /* close */
-        if (H5Dvlen_reclaim(tid,sid,H5P_DEFAULT,buf) < 0)
+        if(H5Dvlen_reclaim(tid, sid, H5P_DEFAULT, buf) < 0)
             goto out;
-        if (H5Sclose(sid) < 0)
+        if(H5Sclose(sid) < 0)
             goto out;
-        if (H5Tclose(tid) < 0)
+        if(H5Tclose(tid) < 0)
             goto out;
-        if (H5Aclose(aid) < 0)
+        if(H5Aclose(aid) < 0)
             goto out;
 
-        if (dims)
-        {
-            free(dims);
-            dims = NULL;
-        }
-        if (buf)
-        {
-            free(buf);
-            buf = NULL;
-        }
-
+        free(buf);
+        buf = NULL;
     }
 
     /*-------------------------------------------------------------------------
@@ -305,8 +291,7 @@ herr_t H5DSattach_scale(hid_t did,
     *  and insert the new reference
     *-------------------------------------------------------------------------
     */
-
-    else if ( has_dimlist == 1 )
+    else if(has_dimlist == 1)
     {
         if((aid = H5Aopen(did, DIMENSION_LIST, H5P_DEFAULT)) < 0)
             goto out;
@@ -318,8 +303,7 @@ herr_t H5DSattach_scale(hid_t did,
             goto out;
 
         /* allocate and initialize the VL */
-        buf = (hvl_t*)malloc((size_t)rank * sizeof(hvl_t));
-
+        buf = (hvl_t *)malloc((size_t)rank * sizeof(hvl_t));
         if(buf == NULL)
             goto out;
 
@@ -329,13 +313,12 @@ herr_t H5DSattach_scale(hid_t did,
 
         /* check to avoid inserting duplicates. it is not FAIL, just do nothing */
         /* iterate all the REFs in this dimension IDX */
-        for (i=0; i<(int)buf[idx].len; i++)
-        {
+        for(i = 0; i < (int)buf[idx].len; i++) {
             /* get the reference */
             ref_j = ((hobj_ref_t *)buf[idx].p)[i];
 
             /* get the scale id for this REF */
-            if ((dsid_j = H5Rdereference(did,H5R_OBJECT,&ref_j)) < 0)
+            if((dsid_j = H5Rdereference(did,H5R_OBJECT,&ref_j)) < 0)
                 goto out;
 
             /* get info for DS in the parameter list */
@@ -351,48 +334,41 @@ herr_t H5DSattach_scale(hid_t did,
                 found_ds = 1;
 
             /* close the dereferenced dataset */
-            if (H5Dclose(dsid_j) < 0)
+            if(H5Dclose(dsid_j) < 0)
                 goto out;
-        }
+        } /* end for */
 
-        if (found_ds==0)
-        {
+        if(found_ds == 0) {
             /* we are adding one more DS to this dimension */
-            if ( buf[idx].len > 0 )
-            {
+            if(buf[idx].len > 0) {
                 buf[idx].len++;
                 len = buf[idx].len;
-                buf[idx].p = realloc( buf[idx].p, len * sizeof(hobj_ref_t));
-                ((hobj_ref_t *)buf[idx].p)[ len-1 ] = ref_to_ds;
-            }
-            else
-            {
+                buf[idx].p = realloc(buf[idx].p, len * sizeof(hobj_ref_t));
+                ((hobj_ref_t *)buf[idx].p)[len - 1] = ref_to_ds;
+            } /* end if */
+            else {
                 /* store the REF information in the index of the dataset that has the DS */
                 buf[idx].len = 1;
-                buf[idx].p = malloc( 1 * sizeof(hobj_ref_t));
+                buf[idx].p = malloc(sizeof(hobj_ref_t));
                 ((hobj_ref_t *)buf[idx].p)[0] = ref_to_ds;
-            }
-        }
+            } /* end else */
+        } /* end if */
 
         /* write the attribute with the new references */
-        if (H5Awrite(aid,tid,buf) < 0)
+        if(H5Awrite(aid, tid, buf) < 0)
             goto out;
 
         /* close */
-        if (H5Dvlen_reclaim(tid,sid,H5P_DEFAULT,buf) < 0)
+        if(H5Dvlen_reclaim(tid, sid, H5P_DEFAULT, buf) < 0)
             goto out;
-        if (H5Sclose(sid) < 0)
+        if(H5Sclose(sid) < 0)
             goto out;
-        if (H5Tclose(tid) < 0)
+        if(H5Tclose(tid) < 0)
             goto out;
-        if (H5Aclose(aid) < 0)
+        if(H5Aclose(aid) < 0)
             goto out;
-        if (buf)
-        {
-            free(buf);
-            buf = NULL;
-        }
-
+        free(buf);
+        buf = NULL;
     } /* has_dimlist */
 
     /*-------------------------------------------------------------------------
@@ -401,25 +377,18 @@ herr_t H5DSattach_scale(hid_t did,
     */
 
     /* try to find the attribute "REFERENCE_LIST" on the >>DS<< dataset */
-    if ((has_reflist = H5LT_find_attribute(dsid,REFERENCE_LIST)) < 0)
+    if((has_reflist = H5LT_find_attribute(dsid, REFERENCE_LIST)) < 0)
         goto out;
 
     /*-------------------------------------------------------------------------
     * it does not exist. we create the attribute and its reference data
     *-------------------------------------------------------------------------
     */
-    if (has_reflist == 0)
-    {
-        /* create one entry array */
-        dims = (hsize_t*) malloc (1 * sizeof (hsize_t));
-
-        if (dims == NULL)
-            goto out;
-
+    if(has_reflist == 0) {
         dims[0] = 1;
 
         /* space for the attribute */
-        if ((sid = H5Screate_simple(1,dims,NULL)) < 0)
+        if((sid = H5Screate_simple(1,dims,NULL)) < 0)
             goto out;
 
         /* create the compound datatype for the attribute "REFERENCE_LIST" */
@@ -452,27 +421,23 @@ herr_t H5DSattach_scale(hid_t did,
             goto out;
         if(H5Aclose(aid) < 0)
             goto out;
-
-        if(dims)
-        {
-            free(dims);
-            dims = NULL;
-        }
-    }
+    } /* end if */
 
     /*-------------------------------------------------------------------------
     * the "REFERENCE_LIST" array already exists, open it and extend it
     *-------------------------------------------------------------------------
     */
-
-    else if(has_reflist ==  1)
-    {
+    else if(has_reflist ==  1) {
         if((aid = H5Aopen(dsid, REFERENCE_LIST, H5P_DEFAULT)) < 0)
             goto out;
 
         if((tid = H5Aget_type(aid)) < 0)
             goto out;
 
+        /* get native type to read attribute REFERENCE_LIST */
+        if((ntid = H5DS_get_REFLIST_type()) < 0)
+            goto out;
+
         /* get and save the old reference(s) */
         if((sid = H5Aget_space(aid)) < 0)
             goto out;
@@ -483,17 +448,16 @@ herr_t H5DSattach_scale(hid_t did,
         nelmts++;
 
         dsbuf = (ds_list_t*) malloc((size_t)nelmts * sizeof(ds_list_t));
-
-        if (dsbuf == NULL)
+        if(dsbuf == NULL)
             goto out;
 
-        if (H5Aread(aid,tid,dsbuf) < 0)
+        if(H5Aread(aid, ntid, dsbuf) < 0)
             goto out;
 
         /* close */
-        if (H5Sclose(sid) < 0)
+        if(H5Sclose(sid) < 0)
             goto out;
-        if (H5Aclose(aid) < 0)
+        if(H5Aclose(aid) < 0)
             goto out;
 
         /*-------------------------------------------------------------------------
@@ -507,29 +471,20 @@ herr_t H5DSattach_scale(hid_t did,
 
         /* store the IDX information (index of the dataset that has the DS) */
         dsl.dim_idx = idx;
-        dsbuf[nelmts-1] = dsl;
+        dsbuf[nelmts - 1] = dsl;
 
         /* create a new data space for the new references array */
-        dims = (hsize_t *)malloc((size_t)nelmts * sizeof(hsize_t));
-        if(dims == NULL)
-            goto out;
         dims[0] = nelmts;
 
         if((sid = H5Screate_simple(1, dims, NULL)) < 0)
             goto out;
 
-        if (dims)
-        {
-            free(dims);
-            dims = NULL;
-        }
-
         /* create the attribute again with the changes of space */
         if((aid = H5Acreate2(dsid, REFERENCE_LIST, tid, sid, H5P_DEFAULT, H5P_DEFAULT)) < 0)
             goto out;
 
         /* write the attribute with the new references */
-        if(H5Awrite(aid, tid, dsbuf) < 0)
+        if(H5Awrite(aid, ntid, dsbuf) < 0)
             goto out;
 
         /* close */
@@ -539,13 +494,11 @@ herr_t H5DSattach_scale(hid_t did,
             goto out;
         if(H5Aclose(aid) < 0)
             goto out;
+        if (H5Tclose(ntid) < 0)
+            goto out;
 
-        if (dsbuf)
-        {
-            free(dsbuf);
-            dsbuf = NULL;
-        }
-
+        free(dsbuf);
+        dsbuf = NULL;
     } /* has_reflist */
 
     /*-------------------------------------------------------------------------
@@ -553,11 +506,10 @@ herr_t H5DSattach_scale(hid_t did,
     *-------------------------------------------------------------------------
     */
 
-    if ((is_ds=H5DSis_scale(dsid)) < 0)
+    if((is_ds=H5DSis_scale(dsid)) < 0)
         return FAIL;
 
-    if (is_ds == 0 )
-    {
+    if(is_ds == 0) {
         if (H5LT_set_attribute_string(dsid,"CLASS",DIMENSION_SCALE_CLASS) < 0)
             return FAIL;
     }
@@ -566,26 +518,15 @@ herr_t H5DSattach_scale(hid_t did,
 
     /* error zone */
 out:
-
-    if (dims)
-    {
-        free(dims);
-        dims = NULL;
-    }
-    if (buf)
-    {
+    if(buf)
         free(buf);
-        buf = NULL;
-    }
-    if (dsbuf)
-    {
+    if(dsbuf)
         free(dsbuf);
-        dsbuf = NULL;
-    }
 
     H5E_BEGIN_TRY {
         H5Sclose(sid);
         H5Aclose(aid);
+        H5Tclose(ntid);
         H5Tclose(tid);
     } H5E_END_TRY;
     return FAIL;
@@ -613,9 +554,11 @@ out:
 *
 * Date: December 20, 2004
 *
-* Comments:
+* Comments: 
 *
-* Modifications:
+* Modifications: Function didn't delete DIMENSION_LIST attribute, when 
+*                all dimension scales were detached from a dataset; added.
+*                                                           2010/05/13 EIP
 *
 *-------------------------------------------------------------------------
 */
@@ -631,50 +574,48 @@ herr_t H5DSdetach_scale(hid_t did,
     hid_t      did_i;        /* dataset ID in REFERENCE_LIST */
     hid_t      sid;          /* space ID */
     hid_t      tid = -1;     /* attribute type ID */
+    hid_t      ntid = -1;    /* attribute native type ID */
     hid_t      aid = -1;     /* attribute ID */
     int        rank;         /* rank of dataset */
     ds_list_t  *dsbuf = NULL;  /* array of attribute data in the DS pointing to the dataset */
-    ds_list_t  *dsbufn = NULL; /* array of attribute data in the DS pointing to the dataset */
-    hsize_t    *dims = NULL;   /* dimension of the "REFERENCE_LIST" array */
+    hsize_t    dims[1];      /* dimension of the "REFERENCE_LIST" array */
     hobj_ref_t ref;          /* reference to the DS */
     hvl_t      *buf = NULL;  /* VL buffer to store in the attribute */
-    unsigned   i, j, jj;
-    H5O_info_t oi1, oi2, oi3, oi4;
+    int        i;
+    size_t     j;
+    hssize_t   ii;
+    H5O_info_t did_oi, dsid_oi, tmp_oi;
     int        found_dset = 0, found_ds = 0;
-    H5I_type_t it1, it2;
+    int        have_ds = 0;
     htri_t     is_scale;
 
     /*-------------------------------------------------------------------------
     * parameter checking
     *-------------------------------------------------------------------------
     */
-   
-    if ((is_scale = H5DSis_scale(did)) < 0)
+
+    /* check for valid types of identifiers */
+
+    if(H5I_DATASET!=H5Iget_type(did) || H5I_DATASET!=H5Iget_type(dsid))
         return FAIL;
-  
+
+    if((is_scale = H5DSis_scale(did)) < 0)
+        return FAIL;
+
     /* the dataset cannot be a DS dataset */
-    if ( is_scale == 1)
+    if( is_scale == 1)
         return FAIL;
 
     /* get info for the dataset in the parameter list */
-    if(H5Oget_info(did, &oi1) < 0)
+    if(H5Oget_info(did, &did_oi) < 0)
         return FAIL;
 
     /* get info for the scale in the parameter list */
-    if(H5Oget_info(dsid, &oi2) < 0)
+    if(H5Oget_info(dsid, &dsid_oi) < 0)
         return FAIL;
 
     /* same object, not valid */
-    if(oi1.fileno == oi2.fileno && oi1.addr == oi2.addr)
-        return FAIL;
-
-    /* get ID type */
-    if ((it1 = H5Iget_type(did)) < 0)
-        return FAIL;
-    if ((it2 = H5Iget_type(dsid)) < 0)
-        return FAIL;
-
-    if (H5I_DATASET!=it1 || H5I_DATASET!=it2)
+    if(did_oi.fileno == dsid_oi.fileno && did_oi.addr == dsid_oi.addr)
         return FAIL;
 
 
@@ -732,13 +673,12 @@ herr_t H5DSdetach_scale(hid_t did,
         goto out;
 
     /* allocate and initialize the VL */
-    buf = (hvl_t*)malloc((size_t)rank * sizeof(hvl_t));
-
+    buf = (hvl_t *)malloc((size_t)rank * sizeof(hvl_t));
     if(buf == NULL)
         goto out;
 
     /* read */
-    if (H5Aread(aid,tid,buf) < 0)
+    if(H5Aread(aid, tid, buf) < 0)
         goto out;
 
     /* reset */
@@ -753,66 +693,77 @@ herr_t H5DSdetach_scale(hid_t did,
             if ((dsid_j = H5Rdereference(did,H5R_OBJECT,&ref)) < 0)
                 goto out;
 
-            /* get info for DS in the parameter list */
-            if(H5Oget_info(dsid, &oi1) < 0)
+            /* get info for this DS */
+            if(H5Oget_info(dsid_j, &tmp_oi) < 0)
                 goto out;
 
-            /* get info for this DS */
-            if(H5Oget_info(dsid_j, &oi2) < 0)
+            /* Close the dereferenced dataset */
+            if (H5Dclose(dsid_j) < 0)
                 goto out;
 
             /* same object, reset */
-            if(oi1.fileno == oi2.fileno && oi1.addr == oi2.addr) 
+            if(dsid_oi.fileno == tmp_oi.fileno && dsid_oi.addr == tmp_oi.addr)
             {
-                size_t len;
+                /* If there are more than one reference in the VL element 
+                   and the reference we found is not the last one,
+                   copy the last one to replace the found one since the order
+                   of the references doesn't matter according to the spec; 
+                   reduce the size of the VL element by 1; 
+                   if the length of the element becomes 0, free the pointer 
+                   and reset to NULL */
 
-                for(jj=j; jj<buf[idx].len-1; jj++)
-                {
-                    ((hobj_ref_t *)buf[idx].p)[jj] = ((hobj_ref_t *)buf[idx].p)[jj+1];
+                size_t len = buf[idx].len;
+
+                if(j < len - 1) 
+                    ((hobj_ref_t *)buf[idx].p)[j] = ((hobj_ref_t *)buf[idx].p)[len-1]; 
+                len = --buf[idx].len;
+                if(len == 0) {
+                    free(buf[idx].p); 
+                    buf[idx].p = NULL;
                 }
-               
-                buf[idx].len--;
-                len = buf[idx].len;
-                buf[idx].p = realloc( buf[idx].p, len * sizeof(hobj_ref_t));
-                
+                /* Since a reference to a dim. scale can be inserted only once, 
+                   we do not need to continue the search if it is found */
                 found_ds = 1;
-                
-                /* close the dereferenced dataset and break */
-                if (H5Dclose(dsid_j) < 0)
-                    goto out;
                 break;
             }
-
-            /* close the dereferenced dataset */
-            if (H5Dclose(dsid_j) < 0)
-                goto out;
         } /* j */
     } /* if */
 
-    /* write the attribute */
-    if (H5Awrite(aid,tid,buf) < 0)
-        goto out;
-
-    /* close */
-    if (H5Dvlen_reclaim(tid,sid,H5P_DEFAULT,buf) < 0)
-        goto out;
-    if (H5Sclose(sid) < 0)
-        goto out;
-    if (H5Tclose(tid) < 0)
-        goto out;
-    if (H5Aclose(aid) < 0)
-        goto out;
-
-    if (buf)
-    {
-        free(buf);
-        buf = NULL;
-    }
-
-    /* the scale must be present */
+    /* the scale must be present to continue */
     if(found_ds == 0)
         goto out;
 
+    /* Write the attribute, but check first, if we have any scales left,
+       because if not, we should delete the attribute according to the spec */ 
+    for(i = 0; i < rank; i++) {
+        if(buf[i].len > 0)  {
+            have_ds = 1;
+            break; 
+        }
+    }
+    if(have_ds) {        
+        if(H5Awrite(aid, tid, buf) < 0)
+            goto out; 
+    }
+    else {
+        if(H5Adelete(did, DIMENSION_LIST) < 0)
+            goto out;
+    }
+
+    /* close */
+    if(H5Dvlen_reclaim(tid, sid, H5P_DEFAULT, buf) < 0)
+        goto out;
+    if(H5Sclose(sid) < 0)
+        goto out;
+    if(H5Tclose(tid) < 0)
+        goto out;
+    if(H5Aclose(aid) < 0)
+        goto out;
+
+    free(buf);
+    buf = NULL;
+
+
     /*-------------------------------------------------------------------------
     * the "REFERENCE_LIST" array exists, update
     *-------------------------------------------------------------------------
@@ -824,6 +775,10 @@ herr_t H5DSdetach_scale(hid_t did,
     if((tid = H5Aget_type(aid)) < 0)
         goto out;
 
+    /* get native type to read attribute REFERENCE_LIST */
+    if((ntid = H5DS_get_REFLIST_type()) < 0)
+        goto out;
+
     /* get and save the old reference(s) */
     if((sid = H5Aget_space(aid)) < 0)
         goto out;
@@ -835,44 +790,37 @@ herr_t H5DSdetach_scale(hid_t did,
     if(dsbuf == NULL)
         goto out;
 
-    if (H5Aread(aid,tid,dsbuf) < 0)
+    if(H5Aread(aid, ntid, dsbuf) < 0)
         goto out;
 
-    for(i=0; i<nelmts; i++)
-    {
-        /* get the reference to the dataset */
-        ref = dsbuf[i].ref;
+    for(ii=0; ii<nelmts; ii++) {
+        /* First check if we have the same dimension index */
+        if((int)idx == dsbuf[ii].dim_idx) {
+            /* get the reference to the dataset */
+            ref = dsbuf[ii].ref;
 
-        /* get the dataset id */
-        if ((did_i = H5Rdereference(did,H5R_OBJECT,&ref)) < 0)
-            goto out;
-
-        /* get info for dataset in the parameter list */
-        if(H5Oget_info(did, &oi3) < 0)
-            goto out;
-
-        /* get info for this dataset */
-        if(H5Oget_info(did_i, &oi4) < 0)
-            goto out;
-
-        /* same object, reset. we want to detach only for this DIM */
-        if(oi3.fileno == oi4.fileno && oi3.addr == oi4.addr && (int)idx == dsbuf[i].dim_idx) {
-            for(jj=i; jj<nelmts-1; jj++)
-                dsbuf[jj] = dsbuf[jj+1];
-            nelmts--;
-            found_dset=1;
-
-            /* close the dereferenced dataset and break */
-            if (H5Dclose(did_i) < 0)
+            /* get the dataset id */
+            if ((did_i = H5Rdereference(did,H5R_OBJECT,&ref)) < 0)
                 goto out;
-            break;
-        } /* if */
 
-        /* close the dereferenced dataset */
-        if (H5Dclose(did_i) < 0)
-            goto out;
+            /* get info for this dataset */
+            if(H5Oget_info(did_i, &tmp_oi) < 0)
+                goto out;
 
-    } /* i */
+            /* close the dereferenced dataset */
+            if(H5Dclose(did_i) < 0)
+                goto out;
+
+            /* same object, reset. we want to detach only for this DIM */
+            if(did_oi.fileno == tmp_oi.fileno && did_oi.addr == tmp_oi.addr) {
+                /* copy the last one to replace the one which is found */
+                dsbuf[ii] = dsbuf[nelmts-1];
+                nelmts--;
+                found_dset=1;
+                break;
+            } /* if */
+        } /* if we have the same dimension index */
+    } /* ii */
 
     /* close space and attribute */
     if (H5Sclose(sid) < 0)
@@ -902,19 +850,8 @@ herr_t H5DSdetach_scale(hid_t did,
     if(nelmts)
     {
         /* create a new data space for the new references array */
-        dims = (hsize_t*)malloc((size_t)nelmts * sizeof (hsize_t));
-        if(dims == NULL)
-            goto out;
         dims[0] = nelmts;
 
-        dsbufn = (ds_list_t*) malloc((size_t)nelmts * sizeof(ds_list_t));
-        if(dsbufn == NULL)
-            goto out;
-
-        /* store the new information */
-        for(i = 0; i < nelmts; i++)
-            dsbufn[i] = dsbuf[i];
-
         if((sid = H5Screate_simple(1, dims, NULL)) < 0)
             goto out;
 
@@ -923,7 +860,7 @@ herr_t H5DSdetach_scale(hid_t did,
             goto out;
 
         /* write the new attribute with the new references */
-        if(H5Awrite(aid, tid, dsbufn) < 0)
+        if(H5Awrite(aid, ntid, dsbuf) < 0)
             goto out;
 
         /* close space and attribute */
@@ -936,23 +873,11 @@ herr_t H5DSdetach_scale(hid_t did,
     /* close type */
     if (H5Tclose(tid) < 0)
         goto out;
+    if (H5Tclose(ntid) < 0)
+        goto out;
 
-    if (dsbuf) 
-    {
-        free(dsbuf);
-        dsbuf=NULL;
-    }
-    if (dsbufn) 
-    {
-        free(dsbufn);
-        dsbufn=NULL;
-    }
-    if (dims) 
-    {
-        free(dims);
-        dims=NULL;
-    }
-
+    free(dsbuf);
+    dsbuf = NULL;
 
     return SUCCEED;
 
@@ -961,27 +886,22 @@ out:
     H5E_BEGIN_TRY {
         H5Sclose(sid);
         H5Aclose(aid);
+        H5Tclose(ntid);
         H5Tclose(tid);
-        
-        if (dsbuf) 
-        {
+
+        if(dsbuf) {
             free(dsbuf);
-            dsbuf=NULL;
+            dsbuf = NULL;
         }
-        if (dsbufn) 
-        {
-            free(dsbufn);
-            dsbufn=NULL;
-        }
-        if (dims) 
-        {
-            free(dims);
-            dims=NULL;
-        }
-        if (buf) 
-        {
+        if(buf) {
+            /* Failure occured before H5Dvlen_reclaim was called;
+               free the pointers allocated when we read data in */
+            for(i = 0; i < rank; i++) {
+                if(buf[i].p)
+                    free(buf[i].p);
+            }
             free(buf);
-            buf=NULL;
+            buf = NULL;
         }
     } H5E_END_TRY;
     return FAIL;
@@ -1024,11 +944,12 @@ htri_t H5DSis_attached(hid_t did,
     hssize_t   nelmts;
     hid_t      sid;          /* space ID */
     hid_t      tid = -1;     /* attribute type ID */
+    hid_t      ntid = -1;    /* attribute native type ID */
     hid_t      aid = -1;     /* attribute ID */
     int        rank;         /* rank of dataset */
-    ds_list_t  *dsbuf;       /* array of attribute data in the DS pointing to the dataset */
+    ds_list_t  *dsbuf = NULL;   /* array of attribute data in the DS pointing to the dataset */
     hobj_ref_t ref;          /* reference to the DS */
-    hvl_t      *buf;         /* VL buffer to store in the attribute */
+    hvl_t      *buf = NULL;  /* VL buffer to store in the attribute */
     hid_t      dsid_j;       /* DS dataset ID in DIMENSION_LIST */
     hid_t      did_i;        /* dataset ID in REFERENCE_LIST */
     H5O_info_t oi1, oi2, oi3, oi4;
@@ -1041,10 +962,10 @@ htri_t H5DSis_attached(hid_t did,
     * parameter checking
     *-------------------------------------------------------------------------
     */
-   
+
     if ((is_scale = H5DSis_scale(did)) < 0)
         return FAIL;
-  
+
     /* the dataset cannot be a DS dataset */
     if ( is_scale == 1)
         return FAIL;
@@ -1113,7 +1034,6 @@ htri_t H5DSis_attached(hid_t did,
 
         /* allocate and initialize the VL */
         buf = (hvl_t*)malloc((size_t)rank * sizeof(hvl_t));
-
         if(buf == NULL)
             goto out;
 
@@ -1159,11 +1079,8 @@ htri_t H5DSis_attached(hid_t did,
             goto out;
         if (H5Aclose(aid) < 0)
             goto out;
-        if (buf)
-        {
-            free(buf);
-            buf = NULL;
-        }
+        free(buf);
+        buf = NULL;
     } /* has_dimlist */
 
     /*-------------------------------------------------------------------------
@@ -1188,6 +1105,10 @@ htri_t H5DSis_attached(hid_t did,
         if((tid = H5Aget_type(aid)) < 0)
             goto out;
 
+        /* get native type to read REFERENCE_LIST attribute */
+        if((ntid = H5DS_get_REFLIST_type()) < 0)
+            goto out;
+
         /* get and save the old reference(s) */
         if((sid = H5Aget_space(aid)) < 0)
             goto out;
@@ -1200,7 +1121,7 @@ htri_t H5DSis_attached(hid_t did,
         if (dsbuf == NULL)
             goto out;
 
-        if (H5Aread(aid,tid,dsbuf) < 0)
+        if (H5Aread(aid,ntid,dsbuf) < 0) 
             goto out;
 
         /*-------------------------------------------------------------------------
@@ -1242,15 +1163,15 @@ htri_t H5DSis_attached(hid_t did,
         /* close */
         if (H5Sclose(sid) < 0)
             goto out;
+        if (H5Tclose(ntid) < 0)
+            goto out;
         if (H5Tclose(tid) < 0)
             goto out;
         if (H5Aclose(aid) < 0)
             goto out;
-        if (dsbuf)
-        {
-            free(dsbuf);
-            dsbuf = NULL;
-        }
+
+        free(dsbuf);
+        dsbuf = NULL;
     } /* has_reflist */
 
     if (found_ds && found_dset)
@@ -1264,15 +1185,14 @@ out:
         H5Sclose(sid);
         H5Aclose(aid);
         H5Tclose(tid);
+        H5Tclose(ntid);
     } H5E_END_TRY;
 
-    if (buf)
-    {
+    if (buf) {
         free(buf);
         buf = NULL;
     }
-    if (dsbuf)
-    {
+    if(dsbuf) {
         free(dsbuf);
         dsbuf = NULL;
     }
@@ -1293,8 +1213,8 @@ out:
 *
 *  hid_t DID;               IN: the dataset
 *  unsigned int DIM;        IN: the dimension of the dataset
-*  int *DS_IDX;             IN/OUT: on input the dimension scale index to start iterating, 
-*                               on output the next index to visit. If NULL, start at 
+*  int *DS_IDX;             IN/OUT: on input the dimension scale index to start iterating,
+*                               on output the next index to visit. If NULL, start at
 *                               the first position.
 *  H5DS_iterate_t VISITOR;  IN: the visitor function
 *  void *VISITOR_DATA;      IN: arbitrary data to pass to the visitor function.
@@ -1335,7 +1255,7 @@ herr_t H5DSiterate_scales(hid_t did,
     hid_t        sid;          /* space ID */
     hid_t        tid = -1;     /* attribute type ID */
     hid_t        aid = -1;     /* attribute ID */
-    hvl_t        *buf=NULL;    /* VL buffer to store in the attribute */
+    hvl_t        *buf = NULL;    /* VL buffer to store in the attribute */
     H5I_type_t   it;           /* ID type */
     herr_t       ret_value=0;
     int          j_idx;
@@ -1435,7 +1355,7 @@ herr_t H5DSiterate_scales(hid_t did,
                 if((ret_value=(visitor)(did,dim,scale_id,visitor_data))!=0)
                 {
                     /* break */
-                    
+
                     /* close the DS id */
                     if (H5Dclose(scale_id) < 0)
                         goto out;
@@ -1459,20 +1379,16 @@ herr_t H5DSiterate_scales(hid_t did,
             goto out;
         if (H5Aclose(aid) < 0)
             goto out;
-        if (buf)
-        {
-            free(buf);
-            buf = NULL;
-        }
 
+        free(buf);
+        buf = NULL;
     } /* if has_dimlist */
 
     return ret_value;
 
 out:
     H5E_BEGIN_TRY {
-        if (buf)
-        {
+        if(buf) {
             H5Dvlen_reclaim(tid,sid,H5P_DEFAULT,buf);
             free(buf);
         }
@@ -1480,6 +1396,7 @@ out:
         H5Aclose(aid);
         H5Tclose(tid);
     } H5E_END_TRY;
+
     return FAIL;
 }
 
@@ -1501,7 +1418,7 @@ out:
 *-------------------------------------------------------------------------
 */
 
-herr_t H5DSset_label(hid_t did, unsigned int idx, const char *label) 
+herr_t H5DSset_label(hid_t did, unsigned int idx, const char *label)
 {
     int        has_labels;
     hid_t      sid = -1;      /* space ID */
@@ -1553,7 +1470,7 @@ herr_t H5DSset_label(hid_t did, unsigned int idx, const char *label)
     *-------------------------------------------------------------------------
     */
 
-    if (has_labels == 0) 
+    if (has_labels == 0)
     {
         dims[0] = rank;
 
@@ -1606,7 +1523,7 @@ herr_t H5DSset_label(hid_t did, unsigned int idx, const char *label)
     *-------------------------------------------------------------------------
     */
 
-    else 
+    else
     {
         if ((aid = H5Aopen(did, DIMENSION_LABELS, H5P_DEFAULT)) < 0)
             goto out;
@@ -1626,7 +1543,7 @@ herr_t H5DSset_label(hid_t did, unsigned int idx, const char *label)
 
         /* free the ptr that will be replaced by label */
         if (buf[idx])
-            free(buf[idx]);
+            free((void *)buf[idx]);
 
         /* store the label information in the required index */
         buf[idx] = label;
@@ -1639,10 +1556,10 @@ herr_t H5DSset_label(hid_t did, unsigned int idx, const char *label)
         buf[idx] = NULL;
 
         /* free all the ptr's from the H5Aread() */
-        for (i = 0; i < (unsigned int) rank; i++) 
+        for (i = 0; i < (unsigned int) rank; i++)
         {
             if (buf[i])
-                free(buf[i]);
+                free((void *)buf[i]);
         }
 
         /* close */
@@ -1660,16 +1577,16 @@ herr_t H5DSset_label(hid_t did, unsigned int idx, const char *label)
     return SUCCEED;
 
     /* error zone */
-out: 
-    if (buf) 
+out:
+    if (buf)
     {
         if (buf[idx]) /* check if we errored during H5Awrite */
             buf[idx] = NULL; /* don't free label */
         /* free all the ptr's from the H5Aread() */
-        for (i = 0; i < (unsigned int) rank; i++) 
+        for (i = 0; i < (unsigned int) rank; i++)
         {
             if (buf[i])
-                free(buf[i]);
+                free((void *)buf[i]);
         }
         free(buf);
     }
@@ -1700,10 +1617,12 @@ out:
 * Comments:
 *
 * Modifications:
+*     JIRA HDFFV-7673: Added a check to see if the label name exists, 
+*     if not then returns zero. July 30, 2011. MSB
 *
 *-------------------------------------------------------------------------
 */
-ssize_t H5DSget_label(hid_t did, unsigned int idx, char *label, size_t size) 
+ssize_t H5DSget_label(hid_t did, unsigned int idx, char *label, size_t size)
 {
     int        has_labels;
     hid_t      sid = -1;     /* space ID */
@@ -1712,7 +1631,7 @@ ssize_t H5DSget_label(hid_t did, unsigned int idx, char *label, size_t size)
     int        rank;         /* rank of dataset */
     char       **buf = NULL; /* buffer to store in the attribute */
     H5I_type_t it;           /* ID type */
-    size_t     nbytes;
+    size_t     nbytes = 0;
     size_t     copy_len;
     int        i;
 
@@ -1752,12 +1671,12 @@ ssize_t H5DSget_label(hid_t did, unsigned int idx, char *label, size_t size)
         return FAIL;
 
     /* return 0 and NULL for label if no label found */
-    if (has_labels == 0) 
+    if (has_labels == 0)
     {
         if (label)
             label[0] = 0;
         return 0;
-    }   
+    }
 
     /*-------------------------------------------------------------------------
     * open the attribute and read label
@@ -1768,6 +1687,7 @@ ssize_t H5DSget_label(hid_t did, unsigned int idx, char *label, size_t size)
     if ((aid = H5Aopen(did, DIMENSION_LABELS, H5P_DEFAULT)) < 0)
         goto out;
 
+
     if ((tid = H5Aget_type(aid)) < 0)
         goto out;
 
@@ -1781,26 +1701,30 @@ ssize_t H5DSget_label(hid_t did, unsigned int idx, char *label, size_t size)
     if (H5Aread(aid, tid, buf) < 0)
         goto out;
 
-    /* get the real string length */
-    nbytes = strlen(buf[idx]);
-
-    /* compute the string length which will fit into the user's buffer */
-    copy_len = MIN(size-1, nbytes);
-
-    /* copy all/some of the name */
-    if (label) 
+    /* do only if the label name exists for the dimension */
+    if (buf[idx] != NULL)
     {
-        memcpy(label, buf[idx], copy_len);
+        /* get the real string length */
+        nbytes = strlen(buf[idx]);
+
+	/* compute the string length which will fit into the user's buffer */
+	copy_len = MIN(size-1, nbytes);
+
+	/* copy all/some of the name */
+	if (label)
+	  {
+	    memcpy(label, buf[idx], copy_len);
+	    
+	    /* terminate the string */
+	    label[copy_len] = '\0';
+	  }
 
-        /* terminate the string */
-        label[copy_len] = '\0';
     }
-
     /* free all the ptr's from the H5Aread() */
-    for (i = 0; i < rank; i++) 
+    for (i = 0; i < rank; i++)
     {
         if (buf[i])
-            free(buf[i]);
+	  free(buf[i]);
     }
 
     /* close */
@@ -1817,11 +1741,11 @@ ssize_t H5DSget_label(hid_t did, unsigned int idx, char *label, size_t size)
     return (ssize_t) nbytes;
 
     /* error zone */
-out: 
-    if (buf) 
+out:
+    if (buf)
     {
         /* free all the ptr's from the H5Aread() */
-        for (i = 0; i < rank; i++) 
+        for (i = 0; i < rank; i++)
         {
             if (buf[i])
                 free(buf[i]);
@@ -2076,7 +2000,7 @@ int H5DSget_num_scales(hid_t did,
     hid_t      tid = -1;     /* attribute type ID */
     hid_t      aid = -1;     /* attribute ID */
     int        rank;         /* rank of dataset */
-    hvl_t      *buf;         /* VL buffer to store in the attribute */
+    hvl_t      *buf = NULL;  /* VL buffer to store in the attribute */
     H5I_type_t it;           /* ID type */
     int        nscales;
 
@@ -2085,10 +2009,10 @@ int H5DSget_num_scales(hid_t did,
     *-------------------------------------------------------------------------
     */
     /* get ID type */
-    if ((it = H5Iget_type(did)) < 0)
+    if((it = H5Iget_type(did)) < 0)
         return FAIL;
 
-    if (H5I_DATASET!=it)
+    if(H5I_DATASET != it)
         return FAIL;
 
     /*-------------------------------------------------------------------------
@@ -2096,19 +2020,19 @@ int H5DSget_num_scales(hid_t did,
     *-------------------------------------------------------------------------
     */
     /* get dataset space */
-    if ((sid = H5Dget_space(did)) < 0)
+    if((sid = H5Dget_space(did)) < 0)
         return FAIL;
 
     /* get rank */
-    if ((rank=H5Sget_simple_extent_ndims(sid)) < 0)
+    if((rank = H5Sget_simple_extent_ndims(sid)) < 0)
         goto out;
 
     /* close dataset space */
-    if (H5Sclose(sid) < 0)
+    if(H5Sclose(sid) < 0)
         goto out;
 
     /* dimemsion index IDX range checking */
-    if (idx>=(unsigned int )rank)
+    if(idx >= (unsigned int )rank)
         return FAIL;
 
     /* try to find the attribute "DIMENSION_LIST" on the >>data<< dataset */
@@ -2123,9 +2047,7 @@ int H5DSget_num_scales(hid_t did,
     * the attribute exists, open it
     *-------------------------------------------------------------------------
     */
-
-    else
-    {
+    else {
         if((aid = H5Aopen(did, DIMENSION_LIST, H5P_DEFAULT)) < 0)
             goto out;
         if((tid = H5Aget_type(aid)) < 0)
@@ -2134,8 +2056,7 @@ int H5DSget_num_scales(hid_t did,
             goto out;
 
         /* allocate and initialize the VL */
-        buf = (hvl_t*)malloc((size_t)rank * sizeof(hvl_t));
-
+        buf = (hvl_t *)malloc((size_t)rank * sizeof(hvl_t));
         if(buf == NULL)
             goto out;
 
@@ -2143,23 +2064,19 @@ int H5DSget_num_scales(hid_t did,
         if(H5Aread(aid, tid, buf) < 0)
             goto out;
 
-        nscales=(int)buf[idx].len;
+        nscales = (int)buf[idx].len;
 
         /* close */
-        if (H5Dvlen_reclaim(tid,sid,H5P_DEFAULT,buf) < 0)
+        if(H5Dvlen_reclaim(tid, sid, H5P_DEFAULT, buf) < 0)
             goto out;
-        if (H5Sclose(sid) < 0)
+        if(H5Sclose(sid) < 0)
             goto out;
-        if (H5Tclose(tid) < 0)
+        if(H5Tclose(tid) < 0)
             goto out;
-        if (H5Aclose(aid) < 0)
+        if(H5Aclose(aid) < 0)
             goto out;
-        if (buf)
-        {
-            free(buf);
-            buf = NULL;
-        }
-
+        free(buf);
+        buf = NULL;
     } /* has_dimlist */
 
     return nscales;
@@ -2172,11 +2089,9 @@ out:
         H5Tclose(tid);
     } H5E_END_TRY;
 
-    if (buf)
-    {
+    if(buf)
         free(buf);
-        buf = NULL;
-    }
+
     return FAIL;
 }
 
@@ -2241,7 +2156,7 @@ herr_t H5DS_is_reserved(hid_t did)
 
     /* error zone */
 out:
-    H5E_BEGIN_TRY 
+    H5E_BEGIN_TRY
     {
         H5Tclose(tid);
         H5Aclose(aid);
@@ -2249,6 +2164,47 @@ out:
     return FAIL;
 }
 
+/*-------------------------------------------------------------------------
+* Function: H5DS_get_REFLIST_type
+*
+* Purpose: This is a helper function to return a native type for
+*          the REFERENCE_LIST attribute.
+*
+* Return: Type identifier on success and negative on failure
+*
+* Programmer: epourmal@hdfgroup.org
+*
+* Date: May 22, 2010
+*
+* Comments:
+*
+* Modifications:
+*
+*-------------------------------------------------------------------------
+*/
 
+hid_t H5DS_get_REFLIST_type(void)
+{
+    hid_t ntid_t = -1; 
+    
+    /* Build native type that corresponds to compound datatype
+       used to store ds_list_t structure in the REFERENCE_LIST
+       attribute */
 
+    if((ntid_t = H5Tcreate(H5T_COMPOUND, sizeof(ds_list_t))) < 0)
+        goto out;
+
+    if(H5Tinsert(ntid_t, "dataset", HOFFSET(ds_list_t,ref), H5T_STD_REF_OBJ) < 0)
+        goto out;
+
+    if(H5Tinsert(ntid_t, "dimension", HOFFSET(ds_list_t, dim_idx), H5T_NATIVE_INT) < 0)
+        goto out;
+
+    return ntid_t;
+out:
+    H5E_BEGIN_TRY {
+        H5Tclose(ntid_t);
+    } H5E_END_TRY;
+    return FAIL;
+}    
 
