@@ -16,6 +16,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include "h5hltest.h"
+#include "H5srcdir.h"
 #include "H5LTpublic.h"
 
 #define FILE_NAME "test_lite1.h5"
@@ -1302,6 +1303,12 @@ static int test_enums(void)
     if(type_class != H5T_ENUM)
         goto out;
 
+    /* Convert the variable before using it */
+    if(!H5Tequal(H5T_STD_I32LE, H5T_NATIVE_INT)) {
+        if(H5Tconvert(H5T_NATIVE_INT, H5T_STD_I32LE, 1, &value1, NULL, H5P_DEFAULT) < 0)
+            goto out;
+    }
+
     if(H5Tenum_nameof(dtype, &value1, name1, size)<0)
         goto out;
     if(strcmp(name1, "BLUE"))
@@ -1309,6 +1316,13 @@ static int test_enums(void)
 
     if(H5Tenum_valueof(dtype, name2, &value2)<0)
         goto out;
+
+    /* Convert the variable before comparing it */
+    if(!H5Tequal(H5T_STD_I32LE, H5T_NATIVE_INT)) {
+        if(H5Tconvert(H5T_NATIVE_INT, H5T_STD_I32LE, 1, &value2, NULL, H5P_DEFAULT) < 0)
+            goto out;
+    }
+
     if(value2 != 8)
         goto out;
 
@@ -1317,10 +1331,12 @@ static int test_enums(void)
     dt_str = (char*)calloc(str_len, sizeof(char));
     if(H5LTdtype_to_text(dtype, dt_str, H5LT_DDL, &str_len)<0)
         goto out;
-    /*if(strcmp(dt_str, "H5T_ENUM {\n      H5T_STD_I32LE;\n      \"RED\"            5;\n      \"GREEN\"          6;\n      \"BLUE\"           7;\n      \"WHITE\"          8;\n   }")) {
-    printf("dt=\n%s\n", dt_str);
-    goto out;
-    }*/
+    if(strcmp(dt_str, "H5T_ENUM {\n      H5T_STD_I32LE;\n      \"RED\"              5;\n      \"GREEN\"            6;\n      \"BLUE\"             7;\n      \"WHITE\"            8;\n   }")) {
+
+        printf("dt=\n%s\n", dt_str);
+        goto out;
+    }
+
     free(dt_str);
 
     if(H5Tclose(dtype)<0)
@@ -1357,16 +1373,16 @@ static int test_variables(void)
 
     if(H5Tis_variable_str(dtype))
         goto out;
-
+    
     if(H5Tclose(dtype)<0)
         goto out;
-
+    
     if((dtype = H5LTtext_to_dtype("H5T_VLEN { H5T_VLEN { H5T_STD_I32BE } }", H5LT_DDL))<0)
         goto out;
-
+    
     if(H5Tis_variable_str(dtype))
         goto out;
-
+    
     if(H5LTdtype_to_text(dtype, NULL, H5LT_DDL, &str_len)<0)
         goto out;
     dt_str = (char*)calloc(str_len, sizeof(char));
@@ -1377,10 +1393,10 @@ static int test_variables(void)
         goto out;
     }
     free(dt_str);
-
+    
     if(H5Tclose(dtype)<0)
         goto out;
-
+    
     PASSED();
     return 0;
 
@@ -1427,10 +1443,11 @@ static int test_arrays(void)
     dt_str = (char*)calloc(str_len, sizeof(char));
     if(H5LTdtype_to_text(dtype, dt_str, H5LT_DDL, &str_len)<0)
         goto out;
-    /*if(strcmp(dt_str, "H5T_ARRAY { [5][7][13] H5T_ARRAY { [17][19] H5T_COMPOUND { H5T_STD_I8BE \"arr_compound_1\"; H5T_STD_I32BE \"arr_compound_2\"; } } }")) {
-    printf("dt=\n%s\n", dt_str);
-    goto out;
-    }*/
+    if(strcmp(dt_str, "H5T_ARRAY {\n      [5][7][13] H5T_ARRAY {\n         [17][19] H5T_COMPOUND {\n            H5T_STD_I8BE \"arr_compound_1\" : 0;\n            H5T_STD_I32BE \"arr_compound_2\" : 1;\n         }\n      }\n   }")) {
+        printf("dt=\n%s\n", dt_str);
+        goto out;
+    }
+
     free(dt_str);
 
     if(H5Tclose(dtype)<0)
@@ -1509,6 +1526,111 @@ out:
     return -1;
 }
 
+/*-------------------------------------------------------------------------
+* subroutine for test_text_dtype(): test_compound_bug(). Test case for 
+* issue 7701.
+*-------------------------------------------------------------------------
+*/
+static int test_compound_bug(void)
+{
+    hid_t       dtype;
+    H5T_class_t type_class;
+    int         nmembs;
+    char*       memb_name = NULL;
+    char*       dt_str;
+    size_t      str_len;
+    char text[] = "H5T_COMPOUND { H5T_STD_I32LE \"state_________________________________________________________________________________\"; H5T_STD_I32LE \"desc_________________________________________________________________________________________\"; H5T_VLEN { H5T_COMPOUND { H5T_ENUM { H5T_STD_I16LE; \"ZERO\" 0; \"ONE\" 1; \"TWO\" 2;  \"THREE\" 3; } \"type____\"; H5T_STD_I32LE \"sub_______________________________________________________________________________________________________________\"; H5T_STRING { STRSIZE H5T_VARIABLE; STRPAD H5T_STR_SPACEPAD; CSET H5T_CSET_ASCII; CTYPE H5T_C_S1; } \"sub_desc\"; H5T_STD_I32LE \"final___________________________________________________________________________________________________\"; } } \"sub\"; }";
+    char text2[] =
+     "H5T_COMPOUND {\n"
+     "  H5T_STD_I16LE \"state___________________________"
+         "__________________________________________________"
+         "____\" : 0;\n"
+     "  H5T_STD_I16LE \"desc____________________________"
+         "__________________________________________________"
+         "___________\" : 2;\n"
+     "  H5T_VLEN { H5T_COMPOUND {\n"
+     "    H5T_ENUM { H5T_STD_I16LE; \"ZERO\" 0; \"ONE\" "
+         "1; \"TWO\" 2;  \"THREE\" 3; } \"type____\" : 0;\n"
+     "    H5T_STD_I32LE \"sub___________________________"
+         "__________________________________________________"
+         "__________________________________1\" : 4;\n"
+     "    H5T_STRING { STRSIZE H5T_VARIABLE; STRPAD H5T_"
+         "STR_SPACEPAD; CSET H5T_CSET_ASCII; CTYPE H5T_C_S1;"
+         " } \"sub_desc\" : 8;\n"
+     "    H5T_STD_I32LE \"final_________________________"
+         "__________________________________________________"
+         "________________________\" : 16;\n"
+     "  } } \"sub\" : 8;\n"
+     "}\n";
+
+    TESTING3("        text for compound type of bug fix");
+
+    if((dtype = H5LTtext_to_dtype(text, H5LT_DDL))<0)
+        goto out;
+
+    if((type_class = H5Tget_class(dtype))<0)
+        goto out;
+    if(type_class != H5T_COMPOUND)
+        goto out;
+
+    if((memb_name = H5Tget_member_name(dtype, 2)) == NULL)
+        goto out;
+    if(strcmp(memb_name, "sub"))
+        goto out;
+    free(memb_name);
+
+    if(H5LTdtype_to_text(dtype, NULL, H5LT_DDL, &str_len)<0)
+        goto out;
+
+    dt_str = (char*)calloc(str_len, sizeof(char));
+    if(H5LTdtype_to_text(dtype, dt_str, H5LT_DDL, &str_len)<0)
+        goto out;
+    free(dt_str);
+
+    if(H5Tclose(dtype)<0)
+        goto out;
+
+
+    /* Test similar datatype in another format */
+    if((dtype = H5LTtext_to_dtype(text2, H5LT_DDL))<0)
+        goto out;
+
+    if((type_class = H5Tget_class(dtype))<0)
+        goto out;
+    if(type_class != H5T_COMPOUND)
+        goto out;
+
+    if((nmembs = H5Tget_nmembers(dtype))<0)
+        goto out;
+    if(nmembs != 3)
+        goto out;
+
+    if((memb_name = H5Tget_member_name(dtype, 1)) == NULL)
+        goto out;
+    if(strcmp(memb_name, "desc_________________________________________________________________________________________"))
+        goto out;
+    free(memb_name);
+
+    if(H5LTdtype_to_text(dtype, NULL, H5LT_DDL, &str_len)<0)
+        goto out;
+
+    dt_str = (char*)calloc(str_len, sizeof(char));
+    if(H5LTdtype_to_text(dtype, dt_str, H5LT_DDL, &str_len)<0)
+        goto out;
+
+    free(dt_str);
+
+    if(H5Tclose(dtype)<0)
+        goto out;
+
+    PASSED();
+    return 0;
+
+out:
+    H5_FAILED();
+    return -1;
+}
+
 /*-------------------------------------------------------------------------
 * subroutine for test_text_dtype(): test_complicated_compound().
 *-------------------------------------------------------------------------
@@ -1518,17 +1640,16 @@ static int test_complicated_compound(void)
     hid_t   dtype;
     int     nmembs;
     H5T_class_t type_class;
-    char*   line=NULL;
-    FILE    *fp;
+    char   *line = NULL;
+    FILE   *fp = NULL;
     size_t  size = 1024;
-    char    *srcdir = getenv("srcdir"); /* the source directory */
+    char   *srcdir = getenv("srcdir"); /* the source directory */
     char    filename[1024]="";
 
     TESTING3("        text for complicated compound types");
 
     /* compose the name of the file to open, using the srcdir, if appropriate */
-    if(srcdir)
-    {
+    if(srcdir) {
         strcpy(filename, srcdir);
         strcat(filename, "/");
     }
@@ -1536,8 +1657,7 @@ static int test_complicated_compound(void)
 
     /* Open input file */
     fp = fopen(filename, "r");
-    if(fp == NULL)
-    {
+    if(fp == NULL) {
         printf( "Could not find file %s. Try set $srcdir \n", filename);
         goto out;
     }
@@ -1563,6 +1683,7 @@ static int test_complicated_compound(void)
     }
 
     fclose(fp);
+    fp = NULL;
 
     if((dtype = H5LTtext_to_dtype(line, H5LT_DDL))<0)
         goto out;
@@ -1585,6 +1706,12 @@ static int test_complicated_compound(void)
     return 0;
 
 out:
+
+    if(line)
+        free(line);
+    if(fp)
+        fclose(fp);
+
     H5_FAILED();
     return -1;
 }
@@ -1621,6 +1748,9 @@ static int test_text_dtype(void)
     if(test_compounds()<0)
         goto out;
 
+    if(test_compound_bug()<0)
+        goto out;
+
     if(test_complicated_compound()<0)
         goto out;
 
