@@ -21,7 +21,6 @@
  */
 
 #include "h5test.h"
-#include <sys/time.h>
 
 #ifdef H5_HAVE_PARALLEL
 #define MAINPROCESS	(!mpi_rank)	/* define process 0 as main process */
@@ -324,14 +323,15 @@ create_dsets(hid_t file)
     /*
      * Create a dataset using the default dataset creation properties.
      */
-    for(i=0; i<NUM_DSETS; i++) {
+    for(i = 0; i < NUM_DSETS; i++) {
 	sprintf(dset_name, "dataset %d", i);
-    	if((dataset = H5Dcreate(file, dset_name, H5T_NATIVE_DOUBLE,
-				space, H5P_DEFAULT)) < 0)
-    		goto error;
+    	if((dataset = H5Dcreate2(file, dset_name, H5T_NATIVE_DOUBLE, space,
+                H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT)) < 0)
+            goto error;
 
-    	if (H5Dclose(dataset) < 0) goto error;
-    }
+    	if(H5Dclose(dataset) < 0)
+            goto error;
+    } /* end for */
 
     return 0;
 
@@ -366,7 +366,7 @@ create_attrs_1(void)
     char	attr_name[128];
     int		i, j;
     p_time      attr_t  = {0, 0, 0, 1000000, 0, ""};
-    p_time      open_t  = {0, 0, 0, 1000000, 0, "H5Dopen"};
+    p_time      open_t  = {0, 0, 0, 1000000, 0, "H5Dopen2"};
     p_time      close_t = {0, 0, 0, 1000000, 0, ""};
 
 #ifdef H5_HAVE_PARALLEL
@@ -379,38 +379,42 @@ create_attrs_1(void)
     h5_fixname(FILENAME[0], fapl, filename, sizeof filename);
 
     if ((file=H5Fcreate(filename, H5F_ACC_TRUNC, H5P_DEFAULT,
-	fapl))<0)
+	fapl)) < 0)
 	goto error;
 
-    if(create_dsets(file)<0)
+    if(create_dsets(file) < 0)
 	goto error;
 
     /*
      * Create all(user specifies the number) attributes for each dataset
      */
-    for(i=0; i<NUM_DSETS; i++) {
+    for(i = 0; i < NUM_DSETS; i++) {
 	sprintf(dset_name, "dataset %d", i);
         open_t.start = retrieve_time();
-	if((dataset=H5Dopen(file, dset_name))<0)
+	if((dataset = H5Dopen2(file, dset_name, H5P_DEFAULT)) < 0)
 		goto error;
 	perf(&open_t, open_t.start, retrieve_time());
 
-	for(j=0; j<NUM_ATTRS; j++) {
-		sprintf(attr_name, "all attrs for each dset %d", j);
-	        attr_t.start = retrieve_time();
-    		if((attr = H5Acreate(dataset, attr_name, H5T_NATIVE_DOUBLE,
-			small_space, H5P_DEFAULT)) < 0)
-    			goto error;
-    		if (H5Aclose(attr) < 0) goto error;
-	 	perf(&attr_t, attr_t.start, retrieve_time());
-                if(flush_attr && H5Fflush(file, H5F_SCOPE_LOCAL)<0) goto error;
-    	}
+	for(j = 0; j < NUM_ATTRS; j++) {
+            sprintf(attr_name, "all attrs for each dset %d", j);
+            attr_t.start = retrieve_time();
+            if((attr = H5Acreate2(dataset, attr_name, H5T_NATIVE_DOUBLE,
+                    small_space, H5P_DEFAULT, H5P_DEFAULT)) < 0)
+                goto error;
+            if(H5Aclose(attr) < 0)
+                goto error;
+            perf(&attr_t, attr_t.start, retrieve_time());
+            if(flush_attr && H5Fflush(file, H5F_SCOPE_LOCAL) < 0)
+                goto error;
+    	} /* end for */
 
 	close_t.start = retrieve_time();
-    	if(H5Dclose(dataset)<0) goto error;
+    	if(H5Dclose(dataset) < 0)
+            goto error;
 	perf(&close_t, close_t.start, retrieve_time());
-        if(flush_dset && H5Fflush(file,  H5F_SCOPE_LOCAL) < 0) goto error;
-    }
+        if(flush_dset && H5Fflush(file,  H5F_SCOPE_LOCAL) < 0)
+            goto error;
+    } /* end for */
 
     if(facc_type == FACC_MPIO || facc_type == FACC_MPIPOSIX) {
 #ifdef H5_HAVE_PARALLEL
@@ -434,7 +438,7 @@ create_attrs_1(void)
         print_perf(open_t, close_t, attr_t);
     }
 
-    if (H5Fclose(file)<0) goto error;
+    if (H5Fclose(file) < 0) goto error;
 
     return 0;
 
@@ -468,7 +472,7 @@ create_attrs_2(void)
     char	attr_name[128];
     int		i, j;
     p_time      attr_t  = {0, 0, 0, 1000000, 0, ""};
-    p_time      create_t  = {0, 0, 0, 1000000, 0, "H5Dcreate"};
+    p_time      create_t  = {0, 0, 0, 1000000, 0, "H5Dcreate2"};
     p_time      close_t = {0, 0, 0, 1000000, 0, ""};
 
 #ifdef H5_HAVE_PARALLEL
@@ -480,36 +484,40 @@ create_attrs_2(void)
 
     h5_fixname(FILENAME[1], fapl, filename, sizeof filename);
 
-    if ((file=H5Fcreate(filename, H5F_ACC_TRUNC, H5P_DEFAULT, fapl))<0)
+    if ((file=H5Fcreate(filename, H5F_ACC_TRUNC, H5P_DEFAULT, fapl)) < 0)
 	goto error;
 
     /*
      * Create all(user specifies the number) attributes for each new dataset
      */
-    for(i=0; i<NUM_DSETS; i++) {
+    for(i = 0; i < NUM_DSETS; i++) {
 	sprintf(dset_name, "dataset %d", i);
         create_t.start = retrieve_time();
-   	if((dataset = H5Dcreate(file, dset_name, H5T_NATIVE_DOUBLE,
-				space, H5P_DEFAULT)) < 0)
-    		goto error;
+   	if((dataset = H5Dcreate2(file, dset_name, H5T_NATIVE_DOUBLE,
+                space, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT)) < 0)
+            goto error;
 	perf(&create_t, create_t.start, retrieve_time());
 
-	for(j=0; j<NUM_ATTRS; j++) {
-		sprintf(attr_name, "all attrs for each dset %d", j);
-	        attr_t.start = retrieve_time();
-    		if((attr = H5Acreate(dataset, attr_name, H5T_NATIVE_DOUBLE,
-			small_space, H5P_DEFAULT)) < 0)
-    			goto error;
-    		if (H5Aclose(attr) < 0) goto error;
-	 	perf(&attr_t, attr_t.start, retrieve_time());
-                if(flush_attr && H5Fflush(file,  H5F_SCOPE_LOCAL) < 0) goto error;
-	}
+	for(j = 0; j < NUM_ATTRS; j++) {
+            sprintf(attr_name, "all attrs for each dset %d", j);
+            attr_t.start = retrieve_time();
+            if((attr = H5Acreate2(dataset, attr_name, H5T_NATIVE_DOUBLE,
+                    small_space, H5P_DEFAULT, H5P_DEFAULT)) < 0)
+                goto error;
+            if(H5Aclose(attr) < 0)
+                goto error;
+            perf(&attr_t, attr_t.start, retrieve_time());
+            if(flush_attr && H5Fflush(file,  H5F_SCOPE_LOCAL) < 0)
+                goto error;
+	} /* end for */
 
 	close_t.start = retrieve_time();
-    	if(H5Dclose(dataset)<0) goto error;
+    	if(H5Dclose(dataset) < 0)
+            goto error;
 	perf(&close_t, close_t.start, retrieve_time());
-        if(flush_dset && H5Fflush(file,  H5F_SCOPE_LOCAL) < 0) goto error;
-    }
+        if(flush_dset && H5Fflush(file,  H5F_SCOPE_LOCAL) < 0)
+            goto error;
+    } /* end for */
 
     if(facc_type == FACC_MPIO || facc_type == FACC_MPIPOSIX) {
 #ifdef H5_HAVE_PARALLEL
@@ -534,7 +542,7 @@ create_attrs_2(void)
         print_perf(create_t, close_t, attr_t);
     }
 
-    if (H5Fclose(file)<0) goto error;
+    if (H5Fclose(file) < 0) goto error;
 
     return 0;
 
@@ -570,7 +578,7 @@ create_attrs_3(void)
     int		loop_num;
     int		i, j, k;
     p_time      attr_t  = {0, 0, 0, 1000000, 0, ""};
-    p_time      open_t  = {0, 0, 0, 1000000, 0, "H5Dopen"};
+    p_time      open_t  = {0, 0, 0, 1000000, 0, "H5Dopen2"};
     p_time      close_t = {0, 0, 0, 1000000, 0, ""};
 
 #ifdef H5_HAVE_PARALLEL
@@ -583,10 +591,10 @@ create_attrs_3(void)
     h5_fixname(FILENAME[2], fapl, filename, sizeof filename);
 
     if ((file=H5Fcreate(filename, H5F_ACC_TRUNC, H5P_DEFAULT,
-	fapl))<0)
+	fapl)) < 0)
 	goto error;
 
-    if(create_dsets(file)<0)
+    if(create_dsets(file) < 0)
 	goto error;
 
     /*
@@ -595,32 +603,35 @@ create_attrs_3(void)
      */
     loop_num = NUM_ATTRS/BATCH_ATTRS;
 
-    for(i=0; i<loop_num; i++) {
-    	for(j=0; j<NUM_DSETS; j++) {
-		sprintf(dset_name, "dataset %d", j);
-        	open_t.start = retrieve_time();
-   		if((dataset = H5Dopen(file, dset_name)) < 0)
-    			goto error;
-		perf(&open_t, open_t.start, retrieve_time());
+    for(i = 0; i < loop_num; i++) {
+    	for(j = 0; j < NUM_DSETS; j++) {
+            sprintf(dset_name, "dataset %d", j);
+            open_t.start = retrieve_time();
+            if((dataset = H5Dopen2(file, dset_name, H5P_DEFAULT)) < 0)
+                goto error;
+            perf(&open_t, open_t.start, retrieve_time());
 
-    		for(k=0; k<BATCH_ATTRS; k++) {
-			sprintf(attr_name, "some attrs for each dset %d %d",
-					i, k);
-	        	attr_t.start = retrieve_time();
-    			if((attr = H5Acreate(dataset, attr_name,
-				H5T_NATIVE_DOUBLE, small_space, H5P_DEFAULT))
-					< 0) goto error;
-    			if (H5Aclose(attr) < 0) goto error;
-	 		perf(&attr_t, attr_t.start, retrieve_time());
-                        if(flush_attr && H5Fflush(file,  H5F_SCOPE_LOCAL) < 0) goto error;
-    		}
+            for(k = 0; k < BATCH_ATTRS; k++) {
+                sprintf(attr_name, "some attrs for each dset %d %d", i, k);
+                attr_t.start = retrieve_time();
+                if((attr = H5Acreate2(dataset, attr_name, H5T_NATIVE_DOUBLE,
+                        small_space, H5P_DEFAULT, H5P_DEFAULT)) < 0)
+                    goto error;
+                if(H5Aclose(attr) < 0)
+                    goto error;
+                perf(&attr_t, attr_t.start, retrieve_time());
+                if(flush_attr && H5Fflush(file,  H5F_SCOPE_LOCAL) < 0)
+                    goto error;
+            } /* end for */
 
-		close_t.start = retrieve_time();
-    		if(H5Dclose(dataset)<0) goto error;
-		perf(&close_t, close_t.start, retrieve_time());
-                if(flush_dset && H5Fflush(file,  H5F_SCOPE_LOCAL) < 0) goto error;
-    	}
-    }
+            close_t.start = retrieve_time();
+            if(H5Dclose(dataset) < 0)
+                goto error;
+            perf(&close_t, close_t.start, retrieve_time());
+            if(flush_dset && H5Fflush(file,  H5F_SCOPE_LOCAL) < 0)
+                goto error;
+    	} /* end for */
+    } /* end for */
 
     if(facc_type == FACC_MPIO || facc_type == FACC_MPIPOSIX) {
 #ifdef H5_HAVE_PARALLEL
@@ -644,7 +655,7 @@ create_attrs_3(void)
         print_perf(open_t, close_t, attr_t);
     }
 
-    if (H5Fclose(file)<0) goto error;
+    if (H5Fclose(file) < 0) goto error;
 
     return 0;
 
@@ -671,7 +682,7 @@ double retrieve_time(void)
     if(facc_type == FACC_DEFAULT) {
 #endif /*H5_HAVE_PARALLEL*/
         struct timeval t;
-        gettimeofday(&t, NULL);
+        HDgettimeofday(&t, NULL);
         return ((double)t.tv_sec + (double)t.tv_usec / 1000000);
 #ifdef H5_HAVE_PARALLEL
     } else {
@@ -808,25 +819,21 @@ main(int argc, char **argv)
         if(facc_type == FACC_DEFAULT || facc_type == FACC_MPIO)
             H5Pset_fapl_mpio(fapl, MPI_COMM_WORLD, MPI_INFO_NULL);
         else if(facc_type == FACC_MPIPOSIX)
-#ifdef H5_WANT_H5_V1_4_COMPAT
-            H5Pset_fapl_mpiposix(fapl, MPI_COMM_WORLD);
-#else /* H5_WANT_H5_V1_4_COMPAT */
             H5Pset_fapl_mpiposix(fapl, MPI_COMM_WORLD, FALSE);
-#endif /* H5_WANT_H5_V1_4_COMPAT */
 #endif /*H5_HAVE_PARALLEL*/
     }
 
-    nerrors += create_dspace()<0 	?1:0;
+    nerrors += create_dspace() < 0 	?1:0;
 
     if((RUN_TEST & TEST_1) || !RUN_TEST)
-        nerrors += create_attrs_1()<0 	?1:0;
+        nerrors += create_attrs_1() < 0 	?1:0;
     if((RUN_TEST & TEST_2) || !RUN_TEST)
-        nerrors += create_attrs_2()<0 	?1:0;
+        nerrors += create_attrs_2() < 0 	?1:0;
     if(((RUN_TEST & TEST_3) || !RUN_TEST) && BATCH_ATTRS && NUM_ATTRS)
-        nerrors += create_attrs_3()<0 	?1:0;
+        nerrors += create_attrs_3() < 0 	?1:0;
 
-    if (H5Sclose(space)<0) goto error;
-    if (H5Sclose(small_space)<0) goto error;
+    if (H5Sclose(space) < 0) goto error;
+    if (H5Sclose(small_space) < 0) goto error;
 
     h5_cleanup(FILENAME, fapl);
 
