@@ -16,9 +16,14 @@
 #ifndef PIO_PERF_H__
 #define PIO_PERF_H__
 
-#include "pio_timer.h"
+#ifndef STANDALONE
 #include "H5private.h"
 #include "h5test.h"
+#include "h5tools_utils.h"
+#else
+#include "pio_standalone.h"
+#endif
+#include "pio_timer.h"
 
 /* setup the dataset no fill option if this is v1.5 or more */
 #if H5_VERS_MAJOR > 1 || H5_VERS_MINOR > 4
@@ -43,6 +48,7 @@ typedef struct parameters_ {
     size_t 	blk_size;       /* Block size                           */
     unsigned    interleaved;    /* Interleaved vs. contiguous blocks    */
     unsigned    collective;     /* Collective vs. independent I/O       */
+    unsigned    dim2d;          /* 1D vs. 2D                            */
     hsize_t 	h5_align;       /* HDF5 object alignment                */
     hsize_t 	h5_thresh;      /* HDF5 object alignment threshold      */
     int 	h5_use_chunks;  /* Make HDF5 dataset chunked            */
