@@ -33,6 +33,10 @@ typedef enum timer_type_ {
     HDF5_DATASET_CREATE,
     HDF5_MPI_WRITE,
     HDF5_MPI_READ,
+    HDF5_FILE_READ_OPEN,
+    HDF5_FILE_READ_CLOSE,
+    HDF5_FILE_WRITE_OPEN,
+    HDF5_FILE_WRITE_CLOSE,
     HDF5_FINE_WRITE_FIXED_DIMS,
     HDF5_FINE_READ_FIXED_DIMS,
     HDF5_GROSS_WRITE_FIXED_DIMS,
