@@ -21,7 +21,9 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <fcntl.h>
-#include <unistd.h>
+#ifdef H5_HAVE_UNISTD_H
+#	include <unistd.h>
+#endif
 #include <errno.h>
 
 #include "hdf5.h"
@@ -96,7 +98,7 @@ static herr_t do_write(results *res, file_descr *fd, parameters *parms, void *bu
 static herr_t do_read(results *res, file_descr *fd, parameters *parms, void *buffer);
 static herr_t dset_write(int local_dim, file_descr *fd, parameters *parms, void *buffer);
 static herr_t posix_buffer_write(int local_dim, file_descr *fd, parameters *parms, void *buffer);
-static herr_t dset_read(int localrank, file_descr *fd, parameters *parms, void *buffer);
+static herr_t dset_read(int localrank, file_descr *fd, parameters *parms, void *buffer, const char *buffer2);
 static herr_t posix_buffer_read(int local_dim, file_descr *fd, parameters *parms, void *buffer);
 static herr_t do_fopen(parameters *param, char *fname, file_descr *fd /*out*/,
     int flags);
@@ -128,7 +130,6 @@ static size_t      cont_size;      /* size of contiguous POSIX access */
 static hid_t       fapl;           /* file access list */
 static unsigned char *buf_p;       /* buffer pointer */
 static const char *multi_letters = "msbrglo"; /* string for multi driver */
-static char *buffer2=NULL;         /* buffer for data verification */
 
 /* HDF5 global variables */
 static hsize_t     h5count[MAX_DIMS];      /*selection count               */
@@ -210,10 +211,8 @@ do_sio(parameters param)
     }
 
     /* Allocate transfer buffer */
-    buffer2 = malloc(linear_buf_size);
     if ((buffer = malloc(linear_buf_size)) == NULL){
-        HDfprintf(stderr, "malloc for transfer buffer size (%Hd) failed\n",
-        (long long)(linear_buf_size));
+        HDfprintf(stderr, "malloc for transfer buffer size (%Hd) failed\n", (long long)(linear_buf_size));
         GOTOERROR(FAIL);
     }
 
@@ -227,7 +226,7 @@ do_sio(parameters param)
      */
     /* Open file for write */
 
-    strcpy(base_name, "#sio_tmp");
+    HDstrcpy(base_name, "#sio_tmp");
     sio_create_filename(iot, base_name, fname, sizeof(fname), &param);
 
     if (sio_debug_level > 0)
@@ -335,7 +334,7 @@ sio_create_filename(iotype iot, const char *base_name, char *fullname, size_t si
     }
 
     /* First use the environment variable and then try the constant */
-    prefix = getenv("HDF5_PREFIX");
+    prefix = HDgetenv("HDF5_PREFIX");
 
 #ifdef HDF5_PREFIX
     if (!prefix)
@@ -349,8 +348,8 @@ sio_create_filename(iotype iot, const char *base_name, char *fullname, size_t si
      * directory instead. */
     register char *user, *login, *subdir;
 
-    user = getenv("USER");
-    login = getenv("LOGIN");
+    user = HDgetenv("USER");
+    login = HDgetenv("LOGIN");
     subdir = (user ? user : login);
 
     if (subdir) {
@@ -363,24 +362,24 @@ sio_create_filename(iotype iot, const char *base_name, char *fullname, size_t si
         fullname[i] = subdir[j];
     } else {
         /* We didn't append the prefix yet */
-        strncpy(fullname, prefix, MIN(strlen(prefix), size));
+        HDstrncpy(fullname, prefix, MIN(HDstrlen(prefix), size));
     }
 
-    if ((strlen(fullname) + strlen(base_name) + 1) < size) {
+    if ((HDstrlen(fullname) + HDstrlen(base_name) + 1) < size) {
         /* Append the base_name with a slash first. Multiple slashes are
          * handled below. */
         h5_stat_t buf;
 
         if (HDstat(fullname, &buf) < 0)
         /* The directory doesn't exist just yet */
-        if (mkdir(fullname, (mode_t)0755) < 0 && errno != EEXIST) {
+        if (HDmkdir(fullname, 0755) < 0 && errno != EEXIST) {
             /* We couldn't make the "/tmp/${USER,LOGIN}" subdirectory.
              * Default to PREFIX's original prefix value. */
-            strcpy(fullname, prefix);
+            HDstrcpy(fullname, prefix);
         }
 
-        strcat(fullname, "/");
-        strcat(fullname, base_name);
+        HDstrcat(fullname, "/");
+        HDstrcat(fullname, base_name);
     } else {
         /* Buffer is too small */
         return NULL;
@@ -389,23 +388,23 @@ sio_create_filename(iotype iot, const char *base_name, char *fullname, size_t si
     /* Buffer is too small */
         return NULL;
     } else {
-        strcpy(fullname, base_name);
+        HDstrcpy(fullname, base_name);
     }
 
     /* Append a suffix */
     if (suffix) {
-        if (strlen(fullname) + strlen(suffix) >= size)
+        if (HDstrlen(fullname) + HDstrlen(suffix) >= size)
             return NULL;
 
-        strcat(fullname, suffix);
+        HDstrcat(fullname, suffix);
     }
 
     /* Remove any double slashes in the filename */
-    for (ptr = fullname, i = j = 0; ptr && i < size; i++, ptr++) {
-    if (*ptr != '/' || last != '/')
-        fullname[j++] = *ptr;
+    for (ptr = fullname, i = j = 0; ptr && (i < size); i++, ptr++) {
+        if (*ptr != '/' || last != '/')
+            fullname[j++] = *ptr;
 
-    last = *ptr;
+        last = *ptr;
     }
 
     return fullname;
@@ -537,14 +536,14 @@ do_write(results *res, file_descr *fd, parameters *parms, void *buffer)
             h5dset_space_id, H5P_DEFAULT, h5dcpl, H5P_DEFAULT);
 
         if (h5ds_id < 0) {
-            fprintf(stderr, "HDF5 Dataset Create failed\n");
+            HDfprintf(stderr, "HDF5 Dataset Create failed\n");
             GOTOERROR(FAIL);
         }
 
         hrc = H5Pclose(h5dcpl);
         /* verifying the close of the dcpl */
         if (hrc < 0) {
-            fprintf(stderr, "HDF5 Property List Close failed\n");
+            HDfprintf(stderr, "HDF5 Property List Close failed\n");
             GOTOERROR(FAIL);
         }
 
@@ -764,6 +763,7 @@ done:
 static herr_t
 do_read(results *res, file_descr *fd, parameters *parms, void *buffer)
 {
+    char        *buffer2 = NULL;       /* Buffer for data verification */
     int         ret_code = SUCCESS;
     char        dname[64];
     long        i;
@@ -775,14 +775,19 @@ do_read(results *res, file_descr *fd, parameters *parms, void *buffer)
     hsize_t     h5start[MAX_DIMS];      /*selection start               */
     int         rank;
 
-        /* Prepare buffer for verifying data */
-        for (i=0; i < linear_buf_size; i++)
-            buffer2[i]=i%128;
+    /* Allocate data verification buffer */
+    if(NULL == (buffer2 = (char *)malloc(linear_buf_size))) {
+        HDfprintf(stderr, "malloc for data verification buffer size (%Zu) failed\n", linear_buf_size);
+        GOTOERROR(FAIL);
+    } /* end if */
+
+    /* Prepare buffer for verifying data */
+    for(i = 0; i < linear_buf_size; i++)
+        buffer2[i] = i % 128;
 
     rank = parms->rank;
-    for (i=0; i<rank; i++) {
+    for(i = 0; i < rank; i++)
         h5offset[i] = offset[i] = 0;
-    }
 
     /* I/O Access specific setup */
     switch (parms->io_type) {
@@ -838,10 +843,10 @@ do_read(results *res, file_descr *fd, parameters *parms, void *buffer)
         break;
 
         case HDF5:
-        sprintf(dname, "Dataset_%ld", parms->num_bytes);
+        sprintf(dname, "Dataset_%ld", (long)parms->num_bytes);
         h5ds_id = H5Dopen2(fd->h5fd, dname, H5P_DEFAULT);
         if (h5ds_id < 0) {
-            fprintf(stderr, "HDF5 Dataset open failed\n");
+            HDfprintf(stderr, "HDF5 Dataset open failed\n");
             GOTOERROR(FAIL);
         }
 
@@ -851,7 +856,7 @@ do_read(results *res, file_descr *fd, parameters *parms, void *buffer)
 
     /* Start "raw data" read timer */
     set_time(res->timers, HDF5_RAW_READ_FIXED_DIMS, START);
-    hrc = dset_read(rank-1, fd, parms, buffer);
+    hrc = dset_read(rank-1, fd, parms, buffer, buffer2);
 
     if (hrc < 0) {
         fprintf(stderr, "Error in dataset read\n");
@@ -908,6 +913,10 @@ done:
     }
     }
 
+    /* release generic resources */
+    if(buffer2)
+        free(buffer2);
+
     return ret_code;
 }
 
@@ -919,7 +928,8 @@ done:
  * Modifications:
  */
 
-static herr_t dset_read(int local_dim, file_descr *fd, parameters *parms, void *buffer)
+static herr_t dset_read(int local_dim, file_descr *fd, parameters *parms,
+    void *buffer, const char *buffer2)
 {
     int cur_dim = order[local_dim]-1;
     int         ret_code = SUCCESS;
@@ -934,7 +944,7 @@ static herr_t dset_read(int local_dim, file_descr *fd, parameters *parms, void *
         /* if traverse in order array is incomplete, recurse */
         if (local_dim > 0){
 
-            ret_code = dset_read(local_dim-1, fd, parms, buffer);
+            ret_code = dset_read(local_dim-1, fd, parms, buffer, buffer2);
 
         /* otherwise, write buffer into dataset */
         }else{
@@ -1048,7 +1058,7 @@ do_fopen(parameters *param, char *fname, file_descr *fd /*out*/, int flags)
             fd->posixfd = POSIXOPEN(fname, O_RDONLY);
 
         if (fd->posixfd < 0 ) {
-            fprintf(stderr, "POSIX File Open failed(%s)\n", fname);
+            HDfprintf(stderr, "POSIX File Open failed(%s)\n", fname);
             GOTOERROR(FAIL);
         }
 
@@ -1129,7 +1139,7 @@ set_vfd(parameters *param)
         HDmemset(memb_name, 0, sizeof memb_name);
         HDmemset(memb_addr, 0, sizeof memb_addr);
 
-        assert(HDstrlen(multi_letters)==H5FD_MEM_NTYPES);
+        HDassert(HDstrlen(multi_letters)==H5FD_MEM_NTYPES);
         for (mt=H5FD_MEM_DEFAULT; mt<H5FD_MEM_NTYPES; H5_INC_ENUM(H5FD_mem_t,mt)) {
             memb_fapl[mt] = H5P_DEFAULT;
             sprintf(sv[mt], "%%s-%c.h5", multi_letters[mt]);
@@ -1220,7 +1230,7 @@ do_cleanupfile(iotype iot, char *filename)
     hid_t       driver;
 
     if (clean_file_g == -1)
-    clean_file_g = (getenv("HDF5_NOCLEANUP")==NULL) ? 1 : 0;
+    clean_file_g = (HDgetenv("HDF5_NOCLEANUP")==NULL) ? 1 : 0;
 
     if (clean_file_g){
 
@@ -1638,7 +1648,7 @@ gpfs_invalidate_file_cache(const char *filename)
         filename);
     fprintf(stderr, " errno=%d errorOffset=%d\n",
         errno, inv_cache_hint.hdr.errorOffset);
-    exit(1);
+    exit(EXIT_FAILURE);
     }
 
     /* Close the file */
@@ -1647,7 +1657,7 @@ gpfs_invalidate_file_cache(const char *filename)
         "could not close file '%s' after flushing file cache, ",
         filename);
     fprintf(stderr, "errno=%d\n", errno);
-    exit(1);
+    exit(EXIT_FAILURE);
     }
 }
 
