@@ -74,8 +74,7 @@
 /****************************/
 /* Define the shared attribute structure */
 typedef struct H5A_shared_t {
-    unsigned    version;    /* Version to encode attribute with */
-    hbool_t     initialized;/* Indicate whether the attribute has been modified */
+    uint8_t     version;    /* Version to encode attribute with */
 
     char        *name;      /* Attribute's name */
     H5T_cset_t  encoding;   /* Character encoding of attribute name */
@@ -163,14 +162,16 @@ typedef struct {
 typedef herr_t (*H5A_lib_iterate_t)(const H5A_t *attr, void *op_data);
 
 /* Describe kind of callback to make for each attribute */
-struct H5A_attr_iter_op_t {
-    enum {
+typedef enum H5A_attr_iter_op_type_t {
 #ifndef H5_NO_DEPRECATED_SYMBOLS
-        H5A_ATTR_OP_APP,                /* Application callback */
+    H5A_ATTR_OP_APP,                /* Application callback */
 #endif /* H5_NO_DEPRECATED_SYMBOLS */
-        H5A_ATTR_OP_APP2,               /* Revised application callback */
-        H5A_ATTR_OP_LIB                 /* Library internal callback */
-    } op_type;
+    H5A_ATTR_OP_APP2,               /* Revised application callback */
+    H5A_ATTR_OP_LIB                 /* Library internal callback */
+} H5A_attr_iter_op_type_t;
+
+typedef struct H5A_attr_iter_op_t {
+    H5A_attr_iter_op_type_t op_type;
     union {
 #ifndef H5_NO_DEPRECATED_SYMBOLS
         H5A_operator1_t app_op;         /* Application callback for each attribute */
@@ -178,7 +179,7 @@ struct H5A_attr_iter_op_t {
         H5A_operator2_t app_op2;        /* Revised application callback for each attribute */
         H5A_lib_iterate_t lib_op;       /* Library internal callback for each attribute */
     } u;
-};
+} H5A_attr_iter_op_t;
 
 
 /*****************************/
@@ -282,8 +283,6 @@ H5_DLL H5A_t *H5A_attr_copy_file(const H5A_t *attr_src, H5F_t *file_dst, hbool_t
     H5O_copy_t *cpy_info, hid_t dxpl_id);
 H5_DLL herr_t H5A_attr_post_copy_file(const H5O_loc_t *src_oloc, const H5A_t *mesg_src,
     H5O_loc_t *dst_oloc, const H5A_t *mesg_dst, hid_t dxpl_id, H5O_copy_t *cpy_info);
-H5_DLL herr_t H5A_dense_copy_file_all(H5F_t *file_src, H5O_ainfo_t *ainfo_src, H5F_t *file_dst, 
-    const H5O_ainfo_t *ainfo_dst, hbool_t *recompute_size, H5O_copy_t *cpy_info, hid_t dxpl_id);
 H5_DLL herr_t H5A_dense_post_copy_file_all(const H5O_loc_t *src_oloc, const H5O_ainfo_t * ainfo_src,
     H5O_loc_t *dst_oloc, H5O_ainfo_t *ainfo_dst, hid_t dxpl_id, H5O_copy_t *cpy_info);
 
