@@ -225,6 +225,7 @@ H5E_walk1_cb(int n, H5E_error1_t *err_desc, void *client_data)
     const char		*maj_str = "No major description";      /* Major error description */
     const char		*min_str = "No minor description";      /* Minor error description */
     unsigned            have_desc = 1;  /* Flag to indicate whether the error has a "real" description */
+    herr_t              ret_value = SUCCEED;
 
     FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5E_walk1_cb)
 
@@ -240,7 +241,11 @@ H5E_walk1_cb(int n, H5E_error1_t *err_desc, void *client_data)
     /* Get descriptions for the major and minor error numbers */
     maj_ptr = (H5E_msg_t *)H5I_object_verify(err_desc->maj_num, H5I_ERROR_MSG);
     min_ptr = (H5E_msg_t *)H5I_object_verify(err_desc->min_num, H5I_ERROR_MSG);
-    HDassert(maj_ptr && min_ptr);
+
+    /* Check for bad pointer(s), but can't issue error, just leave */
+    if(!maj_ptr || !min_ptr)
+        HGOTO_DONE(FAIL)
+
     if(maj_ptr->msg)
         maj_str = maj_ptr->msg;
     if(min_ptr->msg)
@@ -269,13 +274,13 @@ H5E_walk1_cb(int n, H5E_error1_t *err_desc, void *client_data)
 	    MPI_Initialized(&mpi_initialized);
 	    if(mpi_initialized) {
 	        MPI_Comm_rank(MPI_COMM_WORLD, &mpi_rank);
-	        fprintf (stream, "MPI-process %d", mpi_rank);
+	        fprintf(stream, "MPI-process %d", mpi_rank);
 	    } /* end if */
             else
 	        fprintf(stream, "thread 0");
         } /* end block */
 #elif defined(H5_HAVE_THREADSAFE)
-        fprintf(stream, "thread %lu", HDpthread_self_ulong());
+        fprintf(stream, "thread %lu", (unsigned long)HDpthread_self_ulong());
 #else
         fprintf(stream, "thread 0");
 #endif
@@ -294,7 +299,8 @@ H5E_walk1_cb(int n, H5E_error1_t *err_desc, void *client_data)
     fprintf(stream, "%*smajor: %s\n", (H5E_INDENT * 2), "", maj_str);
     fprintf(stream, "%*sminor: %s\n", (H5E_INDENT * 2), "", min_str);
 
-    FUNC_LEAVE_NOAPI(SUCCEED)
+done:
+    FUNC_LEAVE_NOAPI(ret_value)
 } /* end H5E_walk1_cb() */
 #endif /* H5_NO_DEPRECATED_SYMBOLS */
 
@@ -341,6 +347,7 @@ H5E_walk2_cb(unsigned n, const H5E_error2_t *err_desc, void *client_data)
     const char		*maj_str = "No major description";      /* Major error description */
     const char		*min_str = "No minor description";      /* Minor error description */
     unsigned            have_desc = 1;  /* Flag to indicate whether the error has a "real" description */
+    herr_t              ret_value = SUCCEED;
 
     FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5E_walk2_cb)
 
@@ -356,14 +363,19 @@ H5E_walk2_cb(unsigned n, const H5E_error2_t *err_desc, void *client_data)
     /* Get descriptions for the major and minor error numbers */
     maj_ptr = (H5E_msg_t *)H5I_object_verify(err_desc->maj_num, H5I_ERROR_MSG);
     min_ptr = (H5E_msg_t *)H5I_object_verify(err_desc->min_num, H5I_ERROR_MSG);
-    HDassert(maj_ptr && min_ptr);
+
+    /* Check for bad pointer(s), but can't issue error, just leave */
+    if(!maj_ptr || !min_ptr)
+        HGOTO_DONE(FAIL)
+
     if(maj_ptr->msg)
         maj_str = maj_ptr->msg;
     if(min_ptr->msg)
         min_str = min_ptr->msg;
 
-    /* Get error class info */
-    cls_ptr = maj_ptr->cls;
+    /* Get error class info.  Don't use the class of the major or minor error because
+     * they might be different. */
+    cls_ptr = (H5E_cls_t *)H5I_object_verify(err_desc->cls_id, H5I_ERROR_CLASS);
 
     /* Print error class header if new class */
     if(eprint->cls.lib_name == NULL || HDstrcmp(cls_ptr->lib_name, eprint->cls.lib_name)) {
@@ -391,7 +403,7 @@ H5E_walk2_cb(unsigned n, const H5E_error2_t *err_desc, void *client_data)
 	        fprintf(stream, "thread 0");
         } /* end block */
 #elif defined(H5_HAVE_THREADSAFE)
-        fprintf(stream, "thread %lu", HDpthread_self_ulong());
+        fprintf(stream, "thread %lu", (unsigned long)HDpthread_self_ulong());
 #else
         fprintf(stream, "thread 0");
 #endif
@@ -410,7 +422,8 @@ H5E_walk2_cb(unsigned n, const H5E_error2_t *err_desc, void *client_data)
     fprintf(stream, "%*smajor: %s\n", (H5E_INDENT * 2), "", maj_str);
     fprintf(stream, "%*sminor: %s\n", (H5E_INDENT * 2), "", min_str);
 
-    FUNC_LEAVE_NOAPI(SUCCEED)
+done:
+    FUNC_LEAVE_NOAPI(ret_value)
 } /* end H5E_walk2_cb() */
 
 
@@ -658,6 +671,109 @@ H5E_set_auto(H5E_t *estack, const H5E_auto_op_t *op, void *client_data)
     FUNC_LEAVE_NOAPI(SUCCEED)
 } /* end H5E_set_auto() */
 
+
+/*-------------------------------------------------------------------------
+ * Function:	H5E_printf_stack
+ *
+ * Purpose:	Printf-like wrapper around H5E_push_stack.
+ *
+ * Return:	Non-negative on success/Negative on failure
+ *
+ * Programmer:	Quincey Koziol
+ *		Tuesday, August 12, 2008
+ *
+ *-------------------------------------------------------------------------
+ */
+herr_t
+H5E_printf_stack(H5E_t *estack, const char *file, const char *func, unsigned line,
+    hid_t cls_id, hid_t maj_id, hid_t min_id, const char *fmt, ...)
+{
+    va_list     ap;                     /* Varargs info */
+#ifndef H5_HAVE_VASPRINTF
+    int         tmp_len;        /* Current size of description buffer */
+    int         desc_len;       /* Actual length of description when formatted */
+#endif /* H5_HAVE_VASPRINTF */
+    char        *tmp = NULL;      /* Buffer to place formatted description in */
+    hbool_t     va_started = FALSE; /* Whether the variable argument list is open */
+    herr_t	ret_value = SUCCEED;    /* Return value */
+
+    /*
+     * WARNING: We cannot call HERROR() from within this function or else we
+     *		could enter infinite recursion.  Furthermore, we also cannot
+     *		call any other HDF5 macro or function which might call
+     *		HERROR().  HERROR() is called by HRETURN_ERROR() which could
+     *		be called by FUNC_ENTER().
+     */
+    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5E_printf_stack)
+
+    /* Sanity check */
+    HDassert(cls_id > 0);
+    HDassert(maj_id > 0);
+    HDassert(min_id > 0);
+    HDassert(fmt);
+
+/* Note that the variable-argument parsing for the format is identical in
+ *      the H5Epush2() routine - correct errors and make changes in both
+ *      places. -QAK
+ */
+
+    /* Start the variable-argument parsing */
+    va_start(ap, fmt);
+    va_started = TRUE;
+
+#ifdef H5_HAVE_VASPRINTF
+    /* Use the vasprintf() routine, since it does what we're trying to do below */
+    if(HDvasprintf(&tmp, fmt, ap) < 0)
+        HGOTO_DONE(FAIL)
+#else /* H5_HAVE_VASPRINTF */
+    /* Allocate space for the formatted description buffer */
+    tmp_len = 128;
+    if(NULL == (tmp = H5MM_malloc((size_t)tmp_len)))
+        HGOTO_DONE(FAIL)
+
+    /* If the description doesn't fit into the initial buffer size, allocate more space and try again */
+    while((desc_len = HDvsnprintf(tmp, (size_t)tmp_len, fmt, ap))
+#ifdef H5_VSNPRINTF_WORKS
+            >
+#else /* H5_VSNPRINTF_WORKS */
+            >=
+#endif /* H5_VSNPRINTF_WORKS */
+            (tmp_len - 1)
+#ifndef H5_VSNPRINTF_WORKS
+            || (desc_len < 0)
+#endif /* H5_VSNPRINTF_WORKS */
+            ) {
+        /* shutdown & restart the va_list */
+        va_end(ap);
+        va_start(ap, fmt);
+
+        /* Release the previous description, it's too small */
+        H5MM_xfree(tmp);
+
+        /* Allocate a description of the appropriate length */
+#ifdef H5_VSNPRINTF_WORKS
+        tmp_len = desc_len + 1;
+#else /* H5_VSNPRINTF_WORKS */
+        tmp_len = 2 * tmp_len;
+#endif /* H5_VSNPRINTF_WORKS */
+        if(NULL == (tmp = H5MM_malloc((size_t)tmp_len)))
+            HGOTO_DONE(FAIL)
+    } /* end while */
+#endif /* H5_HAVE_VASPRINTF */
+
+    /* Push the error on the stack */
+    if(H5E_push_stack(estack, file, func, line, cls_id, maj_id, min_id, tmp) < 0)
+        HGOTO_DONE(FAIL)
+
+done:
+    if(va_started)
+        va_end(ap);
+    if(tmp)
+        H5MM_xfree(tmp);
+
+    FUNC_LEAVE_NOAPI(ret_value)
+} /* end H5E_printf_stack() */
+
 
 /*-------------------------------------------------------------------------
  * Function:	H5E_push_stack
@@ -778,11 +894,11 @@ H5E_clear_entries(H5E_t *estack, size_t nentries)
 
         /* Decrement the IDs to indicate that they are no longer used by this stack */
         /* (In reverse order that they were incremented, so that reference counts work well) */
-        if(H5I_dec_ref(error->min_num, FALSE) < 0)
+        if(H5I_dec_ref(error->min_num) < 0)
             HGOTO_ERROR(H5E_ERROR, H5E_CANTDEC, FAIL, "unable to decrement ref count on error message")
-        if(H5I_dec_ref(error->maj_num, FALSE) < 0)
+        if(H5I_dec_ref(error->maj_num) < 0)
             HGOTO_ERROR(H5E_ERROR, H5E_CANTDEC, FAIL, "unable to decrement ref count on error message")
-        if(H5I_dec_ref(error->cls_id, FALSE) < 0)
+        if(H5I_dec_ref(error->cls_id) < 0)
             HGOTO_ERROR(H5E_ERROR, H5E_CANTDEC, FAIL, "unable to decrement ref count on error class")
 
         /* Release strings */
@@ -897,18 +1013,20 @@ H5E_dump_api_stack(int is_api)
         H5E_t *estack = H5E_get_my_stack();
 
         HDassert(estack);
+
+#ifdef H5_NO_DEPRECATED_SYMBOLS
+            if(estack->auto_op.func2)
+                (void)((estack->auto_op.func2)(H5E_DEFAULT, estack->auto_data));
+#else /* H5_NO_DEPRECATED_SYMBOLS */ 
         if(estack->auto_op.vers == 1) {
-#ifndef H5_NO_DEPRECATED_SYMBOLS
-            if(estack->auto_op.u.func1)
-                (void)((estack->auto_op.u.func1)(estack->auto_data));
-#else /* H5_NO_DEPRECATED_SYMBOLS */
-            HDassert(0 && "version 1 error stack dump without deprecated symbols!");
-#endif /* H5_NO_DEPRECATED_SYMBOLS */
+            if(estack->auto_op.func1)
+                (void)((estack->auto_op.func1)(estack->auto_data));
         } /* end if */
         else {
-            if(estack->auto_op.u.func2)
-                (void)((estack->auto_op.u.func2)(H5E_DEFAULT, estack->auto_data));
+            if(estack->auto_op.func2)
+                (void)((estack->auto_op.func2)(H5E_DEFAULT, estack->auto_data));
         } /* end else */
+#endif /* H5_NO_DEPRECATED_SYMBOLS */
     } /* end if */
 
 done:
