@@ -21,6 +21,7 @@
 
 /* Packages needed by this file... */
 #include "H5private.h"		/* Generic Functions			*/
+#include "H5ACprivate.h"        /* Metadata cache                       */
 #include "H5Eprivate.h"		/* Error handling		  	*/
 #include "H5Fpkg.h"             /* File access				*/
 #include "H5Gprivate.h"		/* Groups				*/
@@ -627,3 +628,160 @@ done:
     FUNC_LEAVE_NOAPI(ret_value)
 } /* end H5F_mount_count_ids() */
 
+
+/*-------------------------------------------------------------------------
+ * Function:	H5F_flush_mounts_recurse
+ *
+ * Purpose:	Flush a mount hierarchy, recursively
+ *
+ * Return:	SUCCEED/FAIL
+ *
+ * Programmer:	Quincey Koziol
+ *              Fri, August 21, 2009
+ *
+ *-------------------------------------------------------------------------
+ */
+static herr_t
+H5F_flush_mounts_recurse(H5F_t *f, hid_t dxpl_id)
+{
+    unsigned	nerrors = 0;            /* Errors from recursive flushes */
+    unsigned    u;                      /* Index variable */
+    herr_t      ret_value = SUCCEED;    /* Return value */
+
+    FUNC_ENTER_NOAPI_NOINIT(H5F_flush_mounts_recurse)
+
+    /* Sanity check */
+    HDassert(f);
+
+    /* Flush all child files, not stopping for errors */
+    for(u = 0; u < f->shared->mtab.nmounts; u++)
+        if(H5F_flush_mounts_recurse(f->shared->mtab.child[u].file, dxpl_id) < 0)
+            nerrors++;
+
+    /* Call the "real" flush routine, for this file */
+    if(H5F_flush(f, dxpl_id, FALSE) < 0)
+        HGOTO_ERROR(H5E_FILE, H5E_CANTFLUSH, FAIL, "unable to flush file's cached information")
+
+    /* Check flush errors for children - errors are already on the stack */
+    if(nerrors)
+        HGOTO_ERROR(H5E_FILE, H5E_CANTFLUSH, FAIL, "unable to flush file's child mounts")
+
+done:
+    FUNC_LEAVE_NOAPI(ret_value)
+} /* end H5F_flush_mounts_recurse() */
+
+
+/*-------------------------------------------------------------------------
+ * Function:	H5F_flush_mounts
+ *
+ * Purpose:	Flush a mount hierarchy
+ *
+ * Return:	SUCCEED/FAIL
+ *
+ * Programmer:	Quincey Koziol
+ *              Fri, August 21, 2009
+ *
+ *-------------------------------------------------------------------------
+ */
+herr_t
+H5F_flush_mounts(H5F_t *f, hid_t dxpl_id)
+{
+    herr_t      ret_value = SUCCEED;       /* Return value */
+
+    FUNC_ENTER_NOAPI(H5F_flush_mounts, FAIL)
+
+    /* Sanity check */
+    HDassert(f);
+
+    /* Find the top file in the mount hierarchy */
+    while(f->parent)
+        f = f->parent;
+
+    /* Flush the mounted file hierarchy */
+    if(H5F_flush_mounts_recurse(f, dxpl_id) < 0)
+        HGOTO_ERROR(H5E_FILE, H5E_CANTFLUSH, FAIL, "unable to flush mounted file hierarchy")
+
+done:
+    FUNC_LEAVE_NOAPI(ret_value)
+} /* end H5F_flush_mounts() */
+
+
+/*-------------------------------------------------------------------------
+ * Function:	H5F_traverse_mount
+ *
+ * Purpose:	If LNK is a mount point then copy the entry for the root
+ *		group of the mounted file into LNK.
+ *
+ * Return:	Non-negative on success/Negative on failure
+ *
+ * Programmer:	Robb Matzke
+ *              Tuesday, October  6, 1998
+ *
+ *-------------------------------------------------------------------------
+ */
+herr_t
+H5F_traverse_mount(H5O_loc_t *oloc/*in,out*/)
+{
+    H5F_t *parent = oloc->file,         /* File of object */
+        *child = NULL;                  /* Child file */
+    unsigned	lt, rt, md = 0;         /* Binary search indices */
+    int cmp;
+    H5O_loc_t	*mnt_oloc = NULL;       /* Object location for mount points */
+    herr_t ret_value = SUCCEED;         /* Return value */
+
+    FUNC_ENTER_NOAPI(H5F_traverse_mount, FAIL)
+
+    /* Sanity check */
+    HDassert(oloc);
+
+    /*
+     * The loop is necessary because we might have file1 mounted at the root
+     * of file2, which is mounted somewhere in file3.
+     */
+    do {
+	/*
+	 * Use a binary search to find the potential mount point in the mount
+	 * table for the parent
+	 */
+	lt = 0;
+	rt = parent->shared->mtab.nmounts;
+	cmp = -1;
+	while(lt < rt && cmp) {
+	    md = (lt + rt) / 2;
+	    mnt_oloc = H5G_oloc(parent->shared->mtab.child[md].group);
+	    cmp = H5F_addr_cmp(oloc->addr, mnt_oloc->addr);
+	    if(cmp < 0)
+		rt = md;
+	    else
+		lt = md + 1;
+	} /* end while */
+
+        /* Copy root info over to ENT */
+        if(0 == cmp) {
+            /* Get the child file */
+            child = parent->shared->mtab.child[md].file;
+
+            /* Get the location for the root group in the child's file */
+            mnt_oloc = H5G_oloc(child->shared->root_grp);
+
+            /* Release the mount point */
+            if(H5O_loc_free(oloc) < 0)
+                HGOTO_ERROR(H5E_FILE, H5E_CANTFREE, FAIL, "unable to free object location")
+
+            /* Copy the entry for the root group */
+            if(H5O_loc_copy(oloc, mnt_oloc, H5_COPY_DEEP) < 0)
+                HGOTO_ERROR(H5E_FILE, H5E_CANTCOPY, FAIL, "unable to copy object location")
+
+            /* In case the shared root group info points to a different file handle
+             * than the child, modify oloc */
+            oloc->file = child;
+
+            /* Switch to child's file */
+            parent = child;
+        } /* end if */
+    } while(!cmp);
+
+done:
+    FUNC_LEAVE_NOAPI(ret_value)
+} /* end H5F_traverse_mount() */
+
