@@ -101,6 +101,58 @@ H5F_get_intent(const H5F_t *f)
     FUNC_LEAVE_NOAPI(f->intent)
 } /* end H5F_get_intent() */
 
+
+/*-------------------------------------------------------------------------
+ * Function:	H5F_get_open_name
+ *
+ * Purpose:	Retrieve the name used to open a file.
+ *
+ * Return:	Success:	The name of the file.
+ * 		Failure:	? (should not happen)
+ *
+ * Programmer:	Neil Fortner
+ *		December 15 2008
+ *
+ *-------------------------------------------------------------------------
+ */
+char *
+H5F_get_open_name(const H5F_t *f)
+{
+    /* Use FUNC_ENTER_NOAPI_NOINIT_NOFUNC here to avoid performance issues */
+    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5F_get_open_name)
+
+    HDassert(f);
+    HDassert(f->open_name);
+
+    FUNC_LEAVE_NOAPI(f->open_name)
+} /* end H5F_get_open_name() */
+
+
+/*-------------------------------------------------------------------------
+ * Function:	H5F_get_actual_name
+ *
+ * Purpose:	Retrieve the actual name of a file, after resolving symlinks, etc.
+ *
+ * Return:	Success:	The name of the file.
+ * 		Failure:	? (should not happen)
+ *
+ * Programmer:	Quincey Koziol
+ *		November 25 2009
+ *
+ *-------------------------------------------------------------------------
+ */
+char *
+H5F_get_actual_name(const H5F_t *f)
+{
+    /* Use FUNC_ENTER_NOAPI_NOINIT_NOFUNC here to avoid performance issues */
+    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5F_get_actual_name)
+
+    HDassert(f);
+    HDassert(f->actual_name);
+
+    FUNC_LEAVE_NOAPI(f->actual_name)
+} /* end H5F_get_actual_name() */
+
 
 /*-------------------------------------------------------------------------
  * Function:	H5F_get_extpath
@@ -113,8 +165,6 @@ H5F_get_intent(const H5F_t *f)
  *
  * Programmer:	Vailin Choi, April 2, 2008
  *
- * Modifications:
- *
  *-------------------------------------------------------------------------
  */
 char *
@@ -124,35 +174,151 @@ H5F_get_extpath(const H5F_t *f)
     FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5F_get_extpath)
 
     HDassert(f);
+    HDassert(f->extpath);
 
     FUNC_LEAVE_NOAPI(f->extpath)
 } /* end H5F_get_extpath() */
 
 
 /*-------------------------------------------------------------------------
- * Function:	H5F_get_name
+ * Function:	H5F_get_shared
  *
- * Purpose:	Retrieve the name of a file.
+ * Purpose:	Retrieve the file's 'shared' pointer
  *
- * Return:	Success:	The name of the file.
+ * Return:	'shared' on success/abort on failure (shouldn't fail)
  *
- * 		Failure:	? (should not happen)
- *
- * Programmer:	Neil Fortner
- *		December 15 2008
+ * Programmer:	Quincey Koziol, July 20, 2011
  *
  *-------------------------------------------------------------------------
  */
-char *
-H5F_get_name(const H5F_t *f)
+H5F_file_t *
+H5F_get_shared(const H5F_t *f)
 {
     /* Use FUNC_ENTER_NOAPI_NOINIT_NOFUNC here to avoid performance issues */
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5F_get_name)
+    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5F_get_shared)
 
     HDassert(f);
 
-    FUNC_LEAVE_NOAPI(f->name)
-} /* end H5F_get_name() */
+    FUNC_LEAVE_NOAPI(f->shared)
+} /* end H5F_get_shared() */
+
+
+/*-------------------------------------------------------------------------
+ * Function:	H5F_same_shared
+ *
+ * Purpose:	Determine if two files have the same shared file pointer
+ *
+ * Return:	TRUE/FALSE on success/abort on failure (shouldn't fail)
+ *
+ * Programmer:	Quincey Koziol, July 19, 2011
+ *
+ *-------------------------------------------------------------------------
+ */
+hbool_t
+H5F_same_shared(const H5F_t *f1, const H5F_t *f2)
+{
+    /* Use FUNC_ENTER_NOAPI_NOINIT_NOFUNC here to avoid performance issues */
+    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5F_same_shared)
+
+    HDassert(f1);
+    HDassert(f1->shared);
+    HDassert(f2);
+    HDassert(f2->shared);
+
+    FUNC_LEAVE_NOAPI(f1->shared == f2->shared)
+} /* end H5F_same_shared() */
+
+
+/*-------------------------------------------------------------------------
+ * Function:	H5F_get_nopen_objs
+ *
+ * Purpose:	Retrieve the file's 'nopen_objs' value
+ *
+ * Return:	'nopen_objs' on success/abort on failure (shouldn't fail)
+ *
+ * Programmer:	Quincey Koziol, July 20, 2011
+ *
+ *-------------------------------------------------------------------------
+ */
+unsigned
+H5F_get_nopen_objs(const H5F_t *f)
+{
+    /* Use FUNC_ENTER_NOAPI_NOINIT_NOFUNC here to avoid performance issues */
+    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5F_get_nopen_objs)
+
+    HDassert(f);
+
+    FUNC_LEAVE_NOAPI(f->nopen_objs)
+} /* end H5F_get_nopen_objs() */
+
+
+/*-------------------------------------------------------------------------
+ * Function:	H5F_get_file_id
+ *
+ * Purpose:	Retrieve the file's 'file_id' value
+ *
+ * Return:	'file_id' on success/abort on failure (shouldn't fail)
+ *
+ * Programmer:	Quincey Koziol, July 20, 2011
+ *
+ *-------------------------------------------------------------------------
+ */
+hid_t
+H5F_get_file_id(const H5F_t *f)
+{
+    /* Use FUNC_ENTER_NOAPI_NOINIT_NOFUNC here to avoid performance issues */
+    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5F_get_file_id)
+
+    HDassert(f);
+
+    FUNC_LEAVE_NOAPI(f->file_id)
+} /* end H5F_get_file_id() */
+
+
+/*-------------------------------------------------------------------------
+ * Function:	H5F_get_parent
+ *
+ * Purpose:	Retrieve the file's 'parent' pointer
+ *
+ * Return:	'parent' on success/abort on failure (shouldn't fail)
+ *
+ * Programmer:	Quincey Koziol, July 19, 2011
+ *
+ *-------------------------------------------------------------------------
+ */
+H5F_t *
+H5F_get_parent(const H5F_t *f)
+{
+    /* Use FUNC_ENTER_NOAPI_NOINIT_NOFUNC here to avoid performance issues */
+    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5F_get_parent)
+
+    HDassert(f);
+
+    FUNC_LEAVE_NOAPI(f->parent)
+} /* end H5F_get_parent() */
+
+
+/*-------------------------------------------------------------------------
+ * Function:	H5F_get_nmounts
+ *
+ * Purpose:	Retrieve the file's 'nmounts' value
+ *
+ * Return:	'nmounts' on success/abort on failure (shouldn't fail)
+ *
+ * Programmer:	Quincey Koziol, July 20, 2011
+ *
+ *-------------------------------------------------------------------------
+ */
+unsigned
+H5F_get_nmounts(const H5F_t *f)
+{
+    /* Use FUNC_ENTER_NOAPI_NOINIT_NOFUNC here to avoid performance issues */
+    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5F_get_nmounts)
+
+    HDassert(f);
+
+    FUNC_LEAVE_NOAPI(f->nmounts)
+} /* end H5F_get_nmounts() */
 
 
 /*-------------------------------------------------------------------------
@@ -197,7 +363,7 @@ H5F_get_fcpl(const H5F_t *f)
  *
  *-------------------------------------------------------------------------
  */
-size_t
+uint8_t
 H5F_sizeof_addr(const H5F_t *f)
 {
     /* Use FUNC_ENTER_NOAPI_NOINIT_NOFUNC here to avoid performance issues */
@@ -224,7 +390,7 @@ H5F_sizeof_addr(const H5F_t *f)
  *
  *-------------------------------------------------------------------------
  */
-size_t
+uint8_t
 H5F_sizeof_size(const H5F_t *f)
 {
     /* Use FUNC_ENTER_NOAPI_NOINIT_NOFUNC here to avoid performance issues */
@@ -236,6 +402,78 @@ H5F_sizeof_size(const H5F_t *f)
     FUNC_LEAVE_NOAPI(f->shared->sizeof_size)
 } /* H5F_sizeof_size() */
 
+
+/*-------------------------------------------------------------------------
+ * Function:	H5F_get_sohm_addr
+ *
+ * Purpose:	Retrieve the file's 'sohm_addr' value
+ *
+ * Return:	'sohm_addr' on success/abort on failure (shouldn't fail)
+ *
+ * Programmer:	Quincey Koziol, July 20, 2011
+ *
+ *-------------------------------------------------------------------------
+ */
+haddr_t
+H5F_get_sohm_addr(const H5F_t *f)
+{
+    /* Use FUNC_ENTER_NOAPI_NOINIT_NOFUNC here to avoid performance issues */
+    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5F_get_sohm_addr)
+
+    HDassert(f);
+    HDassert(f->shared);
+
+    FUNC_LEAVE_NOAPI(f->shared->sohm_addr)
+} /* end H5F_get_sohm_addr() */
+
+
+/*-------------------------------------------------------------------------
+ * Function:	H5F_get_sohm_vers
+ *
+ * Purpose:	Retrieve the file's 'sohm_vers' value
+ *
+ * Return:	'sohm_vers' on success/abort on failure (shouldn't fail)
+ *
+ * Programmer:	Quincey Koziol, July 20, 2011
+ *
+ *-------------------------------------------------------------------------
+ */
+unsigned
+H5F_get_sohm_vers(const H5F_t *f)
+{
+    /* Use FUNC_ENTER_NOAPI_NOINIT_NOFUNC here to avoid performance issues */
+    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5F_get_sohm_vers)
+
+    HDassert(f);
+    HDassert(f->shared);
+
+    FUNC_LEAVE_NOAPI(f->shared->sohm_vers)
+} /* end H5F_get_sohm_vers() */
+
+
+/*-------------------------------------------------------------------------
+ * Function:	H5F_get_sohm_nindexes
+ *
+ * Purpose:	Retrieve the file's 'sohm_nindexes' value
+ *
+ * Return:	'sohm_nindexes' on success/abort on failure (shouldn't fail)
+ *
+ * Programmer:	Quincey Koziol, July 20, 2011
+ *
+ *-------------------------------------------------------------------------
+ */
+unsigned
+H5F_get_sohm_nindexes(const H5F_t *f)
+{
+    /* Use FUNC_ENTER_NOAPI_NOINIT_NOFUNC here to avoid performance issues */
+    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5F_get_sohm_nindexes)
+
+    HDassert(f);
+    HDassert(f->shared);
+
+    FUNC_LEAVE_NOAPI(f->shared->sohm_nindexes)
+} /* end H5F_get_sohm_nindexes() */
+
 
 /*-------------------------------------------------------------------------
  * Function:	H5F_sym_leaf_k
@@ -263,8 +501,9 @@ H5F_sym_leaf_k(const H5F_t *f)
 
     HDassert(f);
     HDassert(f->shared);
+    HDassert(f->shared->sblock);
 
-    FUNC_LEAVE_NOAPI(f->shared->sym_leaf_k)
+    FUNC_LEAVE_NOAPI(f->shared->sblock->sym_leaf_k)
 } /* end H5F_sym_leaf_k() */
 
 
@@ -294,11 +533,36 @@ H5F_Kvalue(const H5F_t *f, const H5B_class_t *type)
 
     HDassert(f);
     HDassert(f->shared);
+    HDassert(f->shared->sblock);
     HDassert(type);
 
-    FUNC_LEAVE_NOAPI(f->shared->btree_k[type->id])
+    FUNC_LEAVE_NOAPI(f->shared->sblock->btree_k[type->id])
 } /* end H5F_Kvalue() */
 
+
+/*-------------------------------------------------------------------------
+ * Function:	H5F_get_nrefs
+ *
+ * Purpose:	Retrieve the file's 'nrefs' value
+ *
+ * Return:	'nrefs' on success/abort on failure (shouldn't fail)
+ *
+ * Programmer:	Quincey Koziol, July 20, 2011
+ *
+ *-------------------------------------------------------------------------
+ */
+unsigned
+H5F_get_nrefs(const H5F_t *f)
+{
+    /* Use FUNC_ENTER_NOAPI_NOINIT_NOFUNC here to avoid performance issues */
+    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5F_get_nrefs)
+
+    HDassert(f);
+    HDassert(f->shared);
+
+    FUNC_LEAVE_NOAPI(f->shared->nrefs)
+} /* end H5F_get_nrefs() */
+
 
 /*-------------------------------------------------------------------------
  * Function:	H5F_rdcc_nslots
@@ -415,8 +679,9 @@ H5F_get_base_addr(const H5F_t *f)
 
     HDassert(f);
     HDassert(f->shared);
+    HDassert(f->shared->sblock);
 
-    FUNC_LEAVE_NOAPI(f->shared->base_addr)
+    FUNC_LEAVE_NOAPI(f->shared->sblock->base_addr)
 } /* end H5F_get_base_addr() */
 
 
@@ -687,3 +952,124 @@ done:
     FUNC_LEAVE_NOAPI(ret_value)
 } /* end H5F_get_fileno() */
 
+
+/*-------------------------------------------------------------------------
+ * Function:	H5F_get_eoa
+ *
+ * Purpose:	Quick and dirty routine to retrieve the file's 'eoa' value
+ *
+ * Return:	Non-negative on success/Negative on failure
+ *
+ * Programmer:	Quincey Koziol <koziol@ncsa.uiuc.edu>
+ *		June 1, 2004
+ *
+ *-------------------------------------------------------------------------
+ */
+haddr_t
+H5F_get_eoa(const H5F_t *f, H5FD_mem_t type)
+{
+    haddr_t	ret_value;
+
+    FUNC_ENTER_NOAPI(H5F_get_eoa, HADDR_UNDEF)
+
+    HDassert(f);
+    HDassert(f->shared);
+
+    /* Dispatch to driver */
+    if(HADDR_UNDEF == (ret_value = H5FD_get_eoa(f->shared->lf, type)))
+	HGOTO_ERROR(H5E_VFL, H5E_CANTINIT, HADDR_UNDEF, "driver get_eoa request failed")
+
+done:
+    FUNC_LEAVE_NOAPI(ret_value)
+} /* end H5F_get_eoa() */
+
+
+/*-------------------------------------------------------------------------
+ * Function:    H5F_get_vfd_handle
+ *
+ * Purpose:     Returns a pointer to the file handle of the low-level file
+ *              driver.  This is the private function for H5Fget_vfd_handle.
+ *
+ * Return:      Success:        Non-negative.
+ *              Failure:        negative.
+ *
+ * Programmer:  Raymond Lu
+ *              Sep. 16, 2002
+ *
+ *-------------------------------------------------------------------------
+ */
+herr_t
+H5F_get_vfd_handle(const H5F_t *file, hid_t fapl, void **file_handle)
+{
+    herr_t ret_value = SUCCEED;         /* Return value */
+
+    FUNC_ENTER_NOAPI(H5F_get_vfd_handle, FAIL)
+
+    /* Sanity check */
+    HDassert(file);
+    HDassert(file_handle);
+
+    /* Get the VFD handle */
+    if(H5FD_get_vfd_handle(file->shared->lf, fapl, file_handle) < 0)
+        HGOTO_ERROR(H5E_FILE, H5E_CANTGET, FAIL, "can't get file handle for file driver")
+
+done:
+    FUNC_LEAVE_NOAPI(ret_value)
+} /* end H5F_get_vfd_handle() */
+
+
+/*-------------------------------------------------------------------------
+ * Function:	H5F_is_tmp_addr
+ *
+ * Purpose:	Quick and dirty routine to determine if an address is in
+ *		the 'temporary' file space.
+ *          (Mainly added to stop non-file routines from poking about in the
+ *          H5F_t data structure)
+ *
+ * Return:	TRUE/FALSE on success/abort on failure (shouldn't fail)
+ *
+ * Programmer:	Quincey Koziol <koziol@hdfgroup.org>
+ *		June 11, 2009
+ *
+ *-------------------------------------------------------------------------
+ */
+hbool_t
+H5F_is_tmp_addr(const H5F_t *f, haddr_t addr)
+{
+    /* Use FUNC_ENTER_NOAPI_NOINIT_NOFUNC here to avoid performance issues */
+    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5F_is_tmp_addr)
+
+    HDassert(f);
+    HDassert(f->shared);
+
+    FUNC_LEAVE_NOAPI(H5F_addr_le(f->shared->tmp_addr, addr))
+} /* end H5F_is_tmp_addr() */
+
+
+/*-------------------------------------------------------------------------
+ * Function:	H5F_use_tmp_space
+ *
+ * Purpose:	Quick and dirty routine to determine if using temporary
+ *		file space is allowed for this file.
+ *          (Mainly added to stop non-file routines from poking about in the
+ *          H5F_t data structure)
+ *
+ * Return:	TRUE/FALSE on success/abort on failure (shouldn't fail)
+ *
+ * Programmer:	Quincey Koziol <koziol@hdfgroup.org>
+ *		July  1, 2009
+ *
+ *-------------------------------------------------------------------------
+ */
+hbool_t
+H5F_use_tmp_space(const H5F_t *f)
+{
+    /* Use FUNC_ENTER_NOAPI_NOINIT_NOFUNC here to avoid performance issues */
+    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5F_use_tmp_space)
+
+    HDassert(f);
+    HDassert(f->shared);
+
+    FUNC_LEAVE_NOAPI(f->shared->use_tmp_space)
+} /* end H5F_use_tmp_space() */
+
