@@ -23,22 +23,36 @@
  *
  *-------------------------------------------------------------------------
  */
-#define H5F_PACKAGE		/*suppress error about including H5Fpkg	  */
+
+/****************/
+/* Module Setup */
+/****************/
+
 #define H5G_PACKAGE		/*suppress error about including H5Gpkg	  */
 
 
-/* Packages needed by this file... */
+/***********/
+/* Headers */
+/***********/
 #include "H5private.h"		/* Generic Functions			*/
 #include "H5Dprivate.h"		/* Datasets				*/
 #include "H5Eprivate.h"		/* Error handling		  	*/
-#include "H5Fpkg.h"		/* File access				*/
+#include "H5Fprivate.h"		/* File access				*/
 #include "H5FLprivate.h"	/* Free Lists                           */
 #include "H5Gpkg.h"		/* Groups		  		*/
 #include "H5Iprivate.h"		/* IDs			  		*/
 #include "H5Lprivate.h"		/* Links                                */
 #include "H5MMprivate.h"	/* Memory wrappers			*/
 
-/* Private typedefs */
+
+/****************/
+/* Local Macros */
+/****************/
+
+
+/******************/
+/* Local Typedefs */
+/******************/
 
 /* Struct used by change name callback function */
 typedef struct H5G_names_t {
@@ -60,17 +74,15 @@ typedef struct H5G_gnba_iter_t {
     char *path;                 /* Name of the object */
 } H5G_gnba_iter_t;
 
-/* Private macros */
+/********************/
+/* Package Typedefs */
+/********************/
 
-/* Local variables */
 
-/* Declare extern the PQ free list for the wrapped strings */
-H5FL_BLK_EXTERN(str_buf);
+/********************/
+/* Local Prototypes */
+/********************/
 
-/* Declare the free list to manage haddr_t's */
-H5FL_DEFINE_STATIC(haddr_t);
-
-/* PRIVATE PROTOTYPES */
 static htri_t H5G_common_path(const H5RS_str_t *fullpath_r, const H5RS_str_t *prefix_r);
 static H5RS_str_t *H5G_build_fullpath(const char *prefix, const char *name);
 #ifdef NOT_YET
@@ -80,6 +92,25 @@ static herr_t H5G_name_move_path(H5RS_str_t **path_r_ptr,
     const char *full_suffix, const char *src_path, const char *dst_path);
 static int H5G_name_replace_cb(void *obj_ptr, hid_t obj_id, void *key);
 
+
+/*********************/
+/* Package Variables */
+/*********************/
+
+/* Declare extern the PQ free list for the wrapped strings */
+H5FL_BLK_EXTERN(str_buf);
+
+
+/*****************************/
+/* Library Private Variables */
+/*****************************/
+
+
+/*******************/
+/* Local Variables */
+/*******************/
+
+
 
 /*-------------------------------------------------------------------------
  * Function: H5G_common_path
@@ -429,51 +460,59 @@ H5G_name_copy(H5G_name_t *dst, const H5G_name_t *src, H5_copy_depth_t depth)
  *-------------------------------------------------------------------------
  */
 ssize_t
-H5G_get_name(hid_t id, char *name/*out*/, size_t size, hid_t lapl_id,
-    hid_t dxpl_id)
+H5G_get_name(const H5G_loc_t *loc, char *name/*out*/, size_t size,
+    hbool_t *cached, hid_t lapl_id, hid_t dxpl_id)
 {
-    H5G_loc_t     loc;          /* Object location */
-    ssize_t       ret_value = FAIL;
+    ssize_t len = 0;            /* Length of object's name */
+    ssize_t ret_value;          /* Return value */
 
     FUNC_ENTER_NOAPI(H5G_get_name, FAIL)
 
-    /* get object location */
-    if(H5G_loc(id, &loc) >= 0) {
-        ssize_t len = 0;
+    /* Sanity check */
+    HDassert(loc);
 
-        /* If the user path is available and it's not "hidden", use it */
-        if(loc.path->user_path_r != NULL && loc.path->obj_hidden == 0) {
-            len = H5RS_len(loc.path->user_path_r);
+    /* If the user path is available and it's not "hidden", use it */
+    if(loc->path->user_path_r != NULL && loc->path->obj_hidden == 0) {
+        len = H5RS_len(loc->path->user_path_r);
 
-            if(name) {
-                HDstrncpy(name, H5RS_get_str(loc.path->user_path_r), MIN((size_t)(len + 1), size));
-                if((size_t)len >= size)
-                    name[size - 1] = '\0';
-            } /* end if */
+        if(name) {
+            HDstrncpy(name, H5RS_get_str(loc->path->user_path_r), MIN((size_t)(len + 1), size));
+            if((size_t)len >= size)
+                name[size - 1] = '\0';
         } /* end if */
-	else if(!loc.path->obj_hidden) {
-	    hid_t	  file;
 
-            /* Retrieve file ID for name search */
-	    if((file = H5I_get_file_id(id)) < 0)
-		HGOTO_ERROR(H5E_SYM, H5E_CANTGET, FAIL, "can't retrieve file ID")
-
-            /* Search for name of object */
-	    if((len = H5G_get_name_by_addr(file, lapl_id, dxpl_id, loc.oloc, name, size)) < 0) {
-                H5I_dec_ref(file);
-		HGOTO_ERROR(H5E_SYM, H5E_CANTGET, FAIL, "can't determine name")
-            } /* end if */
-	
-            /* Close file ID used for search */
-	    if(H5I_dec_ref(file) < 0)
-		HGOTO_ERROR(H5E_SYM, H5E_CANTCLOSEFILE, FAIL, "can't determine name")
-	} /* end else */
-
-        /* Set return value */
-        ret_value = len;
+        /* Indicate that the name is cached, if requested */
+        /* (Currently only used for testing - QAK, 2010/07/26) */
+        if(cached)
+            *cached = TRUE;
     } /* end if */
+    else if(!loc->path->obj_hidden) {
+        hid_t	  file;
 
-done: 
+        /* Retrieve file ID for name search */
+        if((file = H5F_get_id(loc->oloc->file, FALSE)) < 0)
+            HGOTO_ERROR(H5E_SYM, H5E_CANTGET, FAIL, "can't get file ID")
+
+        /* Search for name of object */
+        if((len = H5G_get_name_by_addr(file, lapl_id, dxpl_id, loc->oloc, name, size)) < 0) {
+            H5I_dec_ref(file);
+            HGOTO_ERROR(H5E_SYM, H5E_CANTGET, FAIL, "can't determine name")
+        } /* end if */
+
+        /* Close file ID used for search */
+        if(H5I_dec_ref(file) < 0)
+            HGOTO_ERROR(H5E_SYM, H5E_CANTCLOSEFILE, FAIL, "can't determine name")
+
+        /* Indicate that the name is _not_ cached, if requested */
+        /* (Currently only used for testing - QAK, 2010/07/26) */
+        if(cached)
+            *cached = FALSE;
+    } /* end else */
+
+    /* Set return value */
+    ret_value = len;
+
+done:
     FUNC_LEAVE_NOAPI(ret_value)
 } /* end H5G_get_name() */
 
@@ -581,7 +620,6 @@ H5G_name_move_path(H5RS_str_t **path_r_ptr, const char *full_suffix, const char
     if(full_suffix_len < path_len) {
         const char *dst_suffix;         /* Destination suffix that changes */
         const char *src_suffix;         /* Source suffix that changes */
-        const char *path_prefix;        /* Prefix for path */
         size_t path_prefix_len;         /* Length of path prefix */
         const char *path_prefix2;       /* 2nd prefix for path */
         size_t path_prefix2_len;        /* Length of 2nd path prefix */
@@ -592,7 +630,6 @@ H5G_name_move_path(H5RS_str_t **path_r_ptr, const char *full_suffix, const char
 
 
         /* Compute path prefix before full suffix*/
-        path_prefix = path;
         path_prefix_len = path_len - full_suffix_len;
 
         /* Determine the common prefix for src & dst paths */
@@ -692,6 +729,19 @@ H5G_name_replace_cb(void *obj_ptr, hid_t obj_id, void *key)
             obj_path = H5T_nameof((H5T_t *)obj_ptr);
             break;
 
+        case H5I_UNINIT:
+        case H5I_BADID:
+        case H5I_FILE:
+        case H5I_DATASPACE:
+        case H5I_ATTR:
+        case H5I_REFERENCE:
+        case H5I_VFL:
+        case H5I_GENPROP_CLS:
+        case H5I_GENPROP_LST:
+        case H5I_ERROR_CLASS:
+        case H5I_ERROR_MSG:
+        case H5I_ERROR_STACK:
+        case H5I_NTYPES:
         default:
             HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "unknown data object")
     } /* end switch */
@@ -703,30 +753,30 @@ H5G_name_replace_cb(void *obj_ptr, hid_t obj_id, void *key)
         HGOTO_DONE(SUCCEED)     /* No need to look at object, it's path is already invalid */
 
     /* Find the top file in object's mount hier. */
-    if(oloc->file->mtab.parent) {
+    if(H5F_PARENT(oloc->file)) {
         /* Check if object is in child file (for mount & unmount operations) */
-        if(names->dst_file && oloc->file->shared == names->dst_file->shared)
+        if(names->dst_file && H5F_SAME_SHARED(oloc->file, names->dst_file))
             obj_in_child = TRUE;
 
         /* Find the "top" file in the chain of mounted files */
-        top_obj_file = oloc->file->mtab.parent;
-        while(top_obj_file->mtab.parent != NULL) {
+        top_obj_file = H5F_PARENT(oloc->file);
+        while(H5F_PARENT(top_obj_file) != NULL) {
             /* Check if object is in child mount hier. (for mount & unmount operations) */
-            if(names->dst_file && top_obj_file->shared == names->dst_file->shared)
+            if(names->dst_file && H5F_SAME_SHARED(top_obj_file, names->dst_file))
                 obj_in_child = TRUE;
 
-            top_obj_file = top_obj_file->mtab.parent;
+            top_obj_file = H5F_PARENT(top_obj_file);
         } /* end while */
     } /* end if */
     else
         top_obj_file = oloc->file;
 
     /* Check if object is in top of child mount hier. (for mount & unmount operations) */
-    if(names->dst_file && top_obj_file->shared == names->dst_file->shared)
+    if(names->dst_file && H5F_SAME_SHARED(top_obj_file, names->dst_file))
         obj_in_child = TRUE;
 
     /* Check if the object is in same file mount hier. */
-    if(top_obj_file->shared != names->src_file->shared)
+    if(!H5F_SAME_SHARED(top_obj_file, names->src_file))
         HGOTO_DONE(SUCCEED)     /* No need to look at object, it's path is already invalid */
 
     switch(names->op) {
@@ -917,7 +967,7 @@ done:
  */
 herr_t
 H5G_name_replace(const H5O_link_t *lnk, H5G_names_op_t op, H5F_t *src_file,
-    H5RS_str_t *src_full_path_r, H5F_t *dst_file, H5RS_str_t *dst_full_path_r, 
+    H5RS_str_t *src_full_path_r, H5F_t *dst_file, H5RS_str_t *dst_full_path_r,
     hid_t dxpl_id)
 {
     herr_t ret_value = SUCCEED;
@@ -967,6 +1017,9 @@ H5G_name_replace(const H5O_link_t *lnk, H5G_names_op_t op, H5F_t *src_file,
                                 search_datatype = TRUE;
                                 break;
 
+                            case H5O_TYPE_UNKNOWN:
+                            case H5O_TYPE_NTYPES:
+                                /* Search and replace names through datatype IDs */
                             default:
                                 HGOTO_ERROR(H5E_SYM, H5E_BADTYPE, FAIL, "not valid object type")
                         } /* end switch */
@@ -978,6 +1031,9 @@ H5G_name_replace(const H5O_link_t *lnk, H5G_names_op_t op, H5F_t *src_file,
                     search_group = search_dataset = search_datatype = TRUE;
                     break;
 
+                case H5L_TYPE_ERROR:
+                case H5L_TYPE_EXTERNAL:
+                case H5L_TYPE_MAX:
                 default:  /* User-defined link */
                     /* Check for unknown library-defined link type */
                     if(lnk->type < H5L_TYPE_UD_MIN)
@@ -1000,8 +1056,8 @@ H5G_name_replace(const H5O_link_t *lnk, H5G_names_op_t op, H5F_t *src_file,
             H5G_names_t names;          /* Structure to hold operation information for callback */
 
             /* Find top file in src location's mount hierarchy */
-            while(src_file->mtab.parent)
-                src_file = src_file->mtab.parent;
+            while(H5F_PARENT(src_file))
+                src_file = H5F_PARENT(src_file);
 
             /* Set up common information for callback */
             names.src_file = src_file;
@@ -1012,15 +1068,15 @@ H5G_name_replace(const H5O_link_t *lnk, H5G_names_op_t op, H5F_t *src_file,
 
             /* Search through group IDs */
             if(search_group)
-                H5I_search(H5I_GROUP, H5G_name_replace_cb, &names);
+                H5I_search(H5I_GROUP, H5G_name_replace_cb, &names, FALSE);
 
             /* Search through dataset IDs */
             if(search_dataset)
-                H5I_search(H5I_DATASET, H5G_name_replace_cb, &names);
+                H5I_search(H5I_DATASET, H5G_name_replace_cb, &names, FALSE);
 
             /* Search through datatype IDs */
             if(search_datatype)
-                H5I_search(H5I_DATATYPE, H5G_name_replace_cb, &names);
+                H5I_search(H5I_DATATYPE, H5G_name_replace_cb, &names, FALSE);
         } /* end if */
     } /* end if */
 
@@ -1083,14 +1139,15 @@ H5G_get_name_by_addr_cb(hid_t gid, const char *path, const H5L_info_t *linfo,
         /* Check for object in same file (handles mounted files) */
         /* (re-verify address, in case we traversed a file mount) */
         if(udata->loc->addr == obj_loc.oloc->addr && udata->loc->file == obj_loc.oloc->file) {
-            udata->path = H5MM_strdup(path);
+            if(NULL == (udata->path = H5MM_strdup(path)))
+                HGOTO_ERROR(H5E_SYM, H5E_CANTALLOC, H5_ITER_ERROR, "can't duplicate path string")
 
             /* We found a match so we return immediately */
             HGOTO_DONE(H5_ITER_STOP)
         } /* end if */
     } /* end if */
-   
-done:    
+
+done:
     if(obj_found && H5G_loc_free(&obj_loc) < 0)
         HDONE_ERROR(H5E_SYM, H5E_CANTRELEASE, H5_ITER_ERROR, "can't free location")
 
@@ -1123,6 +1180,9 @@ H5G_get_name_by_addr(hid_t file, hid_t lapl_id, hid_t dxpl_id, const H5O_loc_t *
     herr_t status;              /* Status from iteration */
     ssize_t ret_value;          /* Return value */
 
+    /* Portably clear udata struct (before FUNC_ENTER) */
+    HDmemset(&udata, 0, sizeof(udata));
+
     FUNC_ENTER_NOAPI(H5G_get_name_by_addr, FAIL)
 
     /* Construct the link info for the file's root group */
@@ -1131,7 +1191,8 @@ H5G_get_name_by_addr(hid_t file, hid_t lapl_id, hid_t dxpl_id, const H5O_loc_t *
 
     /* Check for root group being the object looked for */
     if(root_loc.oloc->addr == loc->addr && root_loc.oloc->file == loc->file) {
-        udata.path = H5MM_strdup("");
+        if(NULL == (udata.path = H5MM_strdup("")))
+            HGOTO_ERROR(H5E_SYM, H5E_CANTALLOC, FAIL, "can't duplicate path string")
         found_obj = TRUE;
     } /* end if */
     else {
@@ -1140,7 +1201,7 @@ H5G_get_name_by_addr(hid_t file, hid_t lapl_id, hid_t dxpl_id, const H5O_loc_t *
         udata.lapl_id = lapl_id;
         udata.dxpl_id = dxpl_id;
         udata.path = NULL;
-        
+
         /* Visit all the links in the file */
         if((status = H5G_visit(file, "/", H5_INDEX_NAME, H5_ITER_NATIVE, H5G_get_name_by_addr_cb, &udata, lapl_id, dxpl_id)) < 0)
             HGOTO_ERROR(H5E_SYM, H5E_BADITER, FAIL, "group traversal failed while looking for object name")
@@ -1150,10 +1211,8 @@ H5G_get_name_by_addr(hid_t file, hid_t lapl_id, hid_t dxpl_id, const H5O_loc_t *
 
     /* Check for finding the object */
     if(found_obj) {
-        size_t full_path_len = HDstrlen(udata.path) + 1;        /* Length of path + 1 (for "/") */
-
         /* Set the length of the full path */
-        ret_value = full_path_len;
+        ret_value = (ssize_t)(HDstrlen(udata.path) + 1);        /* Length of path + 1 (for "/") */
 
         /* If there's a buffer provided, copy into it, up to the limit of its size */
         if(name) {
@@ -1162,15 +1221,15 @@ H5G_get_name_by_addr(hid_t file, hid_t lapl_id, hid_t dxpl_id, const H5O_loc_t *
 
             /* Append the rest of the path */
             /* (less one character, for the initial path separator) */
-            HDstrncat(name, udata.path, (size - 1));
+            HDstrncat(name, udata.path, (size - 2));
             if((size_t)ret_value >= size)
                 name[size - 1] = '\0';
         } /* end if */
     } /* end if */
     else
         ret_value = 0;
-   
-done:    
+
+done:
     /* Release resources */
     H5MM_xfree(udata.path);
 
