@@ -28,7 +28,6 @@
 /* Module Setup */
 /****************/
 
-#define H5F_PACKAGE		/*suppress error about including H5Fpkg	  */
 #define H5G_PACKAGE		/*suppress error about including H5Gpkg  */
 
 
@@ -37,7 +36,7 @@
 /***********/
 #include "H5private.h"		/* Generic Functions			*/
 #include "H5Eprivate.h"		/* Error handling		  	*/
-#include "H5Fpkg.h"		/* File access				*/
+#include "H5Fprivate.h"		/* File access				*/
 #include "H5Gpkg.h"		/* Groups		  		*/
 #include "H5HLprivate.h"	/* Local Heaps				*/
 #include "H5Iprivate.h"		/* IDs			  		*/
@@ -81,6 +80,7 @@ typedef struct {
     hid_t       dxpl_id;                /* DXPL during insertion */
 } H5G_obj_stab_it_ud1_t;
 
+
 /********************/
 /* Package Typedefs */
 /********************/
@@ -125,29 +125,90 @@ static herr_t H5G_obj_remove_update_linfo(H5O_loc_t *oloc, H5O_linfo_t *linfo,
  *-------------------------------------------------------------------------
  */
 herr_t
-H5G_obj_create(H5F_t *f, hid_t dxpl_id, const H5O_ginfo_t *ginfo,
-    const H5O_linfo_t *linfo, hid_t gcpl_id, H5O_loc_t *oloc/*out*/)
+H5G_obj_create(H5F_t *f, hid_t dxpl_id, H5G_obj_create_t *gcrt_info,
+    H5O_loc_t *oloc/*out*/)
 {
-    size_t hdr_size;                    /* Size of object header to request */
-    hbool_t use_latest_format;          /* Flag indicating the new group format should be used */
+    H5P_genplist_t  *gc_plist;          /* Group creation property list */
+    H5O_ginfo_t ginfo;                  /* Group info */
+    H5O_linfo_t linfo;                  /* Link info */
+    H5O_pline_t pline;                  /* Pipeline */
     herr_t ret_value = SUCCEED;         /* Return value */
 
     FUNC_ENTER_NOAPI(H5G_obj_create, FAIL)
 
+    /*
+     * Check arguments.
+     */
+    HDassert(f);
+    HDassert(oloc);
+
+    /* Get the property list */
+    if(NULL == (gc_plist = (H5P_genplist_t *)H5I_object(gcrt_info->gcpl_id)))
+        HGOTO_ERROR(H5E_SYM, H5E_BADTYPE, FAIL, "not a property list")
+
+    /* Get the group info property */
+    if(H5P_get(gc_plist, H5G_CRT_GROUP_INFO_NAME, &ginfo) < 0)
+        HGOTO_ERROR(H5E_SYM, H5E_CANTGET, FAIL, "can't get group info")
+
+    /* Get the link info property */
+    if(H5P_get(gc_plist, H5G_CRT_LINK_INFO_NAME, &linfo) < 0)
+        HGOTO_ERROR(H5E_SYM, H5E_CANTGET, FAIL, "can't get group info")
+
+    /* Get the pipeline property */
+    if(H5P_get(gc_plist, H5O_CRT_PIPELINE_NAME, &pline) < 0)
+        HGOTO_ERROR(H5E_SYM, H5E_CANTGET, FAIL, "can't get group info")
+
+    /* Call the "real" group creation routine now */
+    if(H5G_obj_create_real(f, dxpl_id, &ginfo, &linfo, &pline, gcrt_info, oloc) < 0)
+        HGOTO_ERROR(H5E_SYM, H5E_CANTCREATE, FAIL, "unable to create group")
+
+done:
+    FUNC_LEAVE_NOAPI(ret_value)
+} /* end H5G_obj_create() */
+
+
+/*-------------------------------------------------------------------------
+ * Function:    H5G_obj_create_real
+ *
+ * Purpose:     Create an object header for a group and update object location info
+ *
+ * Return:      Non-negative on success/Negative on failure
+ *
+ * Programmer:  Quincey Koziol
+ *              koziol@ncsa.uiuc.edu
+ *              Sep 29 2005
+ *
+ *-------------------------------------------------------------------------
+ */
+herr_t
+H5G_obj_create_real(H5F_t *f, hid_t dxpl_id, const H5O_ginfo_t *ginfo,
+    const H5O_linfo_t *linfo, const H5O_pline_t *pline,
+    H5G_obj_create_t *gcrt_info, H5O_loc_t *oloc/*out*/)
+{
+    size_t hdr_size;                    /* Size of object header to request */
+    hbool_t use_latest_format;          /* Flag indicating the new group format should be used */
+    hid_t gcpl_id = gcrt_info->gcpl_id; /* Group creation property list ID */
+    herr_t ret_value = SUCCEED;         /* Return value */
+
+    FUNC_ENTER_NOAPI(H5G_obj_create_real, FAIL)
+
     /*
      * Check arguments.
      */
     HDassert(f);
     HDassert(ginfo);
+    HDassert(linfo);
+    HDassert(pline);
     HDassert(oloc);
 
     /* Check for invalid access request */
-    if(0 == (f->intent & H5F_ACC_RDWR))
+    if(0 == (H5F_INTENT(f) & H5F_ACC_RDWR))
 	HGOTO_ERROR(H5E_CACHE, H5E_BADVALUE, FAIL, "no write intent on file")
 
     /* Check for using the latest version of the group format */
     /* (add more checks for creating "new format" groups when needed) */
-    if(H5F_USE_LATEST_FORMAT(f) || linfo->track_corder)
+    if(H5F_USE_LATEST_FORMAT(f) || linfo->track_corder
+            || (pline && pline->nused))
         use_latest_format = TRUE;
     else
         use_latest_format = FALSE;
@@ -164,6 +225,7 @@ H5G_obj_create(H5F_t *f, hid_t dxpl_id, const H5O_ginfo_t *ginfo,
         char null_char = '\0';              /* Character for creating null string */
         size_t ginfo_size;                  /* Size of the group info message */
         size_t linfo_size;                  /* Size of the link info message */
+        size_t pline_size = 0;              /* Size of the pipeline message */
         size_t link_size;                   /* Size of a link message */
 
         /* Calculate message size infomation, for creating group's object header */
@@ -173,6 +235,11 @@ H5G_obj_create(H5F_t *f, hid_t dxpl_id, const H5O_ginfo_t *ginfo,
         ginfo_size = H5O_msg_size_f(f, gcpl_id, H5O_GINFO_ID, ginfo, (size_t)0);
         HDassert(ginfo_size);
 
+        if(pline && pline->nused) {
+            pline_size = H5O_msg_size_f(f, gcpl_id, H5O_PLINE_ID, pline, (size_t)0);
+            HDassert(pline_size);
+        } /* end if */
+
         lnk.type = H5L_TYPE_HARD;
         lnk.corder = 0;
         lnk.corder_valid = linfo->track_corder;
@@ -184,28 +251,37 @@ H5G_obj_create(H5F_t *f, hid_t dxpl_id, const H5O_ginfo_t *ginfo,
         /* Compute size of header to use for creation */
         hdr_size = linfo_size +
                     ginfo_size +
+                    pline_size +
                     (ginfo->est_num_entries * link_size);
     } /* end if */
     else
-        hdr_size = 4 + 2 * H5F_SIZEOF_ADDR(f);
+        hdr_size = (size_t)(4 + 2 * H5F_SIZEOF_ADDR(f));
 
     /*
      * Create group's object header.  It has a zero link count
      * since nothing refers to it yet.	The link count will be
      * incremented if the object is added to the group directed graph.
      */
-    if(H5O_create(f, dxpl_id, hdr_size, gcpl_id, oloc/*out*/) < 0)
+    if(H5O_create(f, dxpl_id, hdr_size, (size_t)1, gcpl_id, oloc/*out*/) < 0)
 	HGOTO_ERROR(H5E_SYM, H5E_CANTINIT, FAIL, "can't create header")
 
     /* Check for format of group to create */
     if(use_latest_format) {
         /* Insert link info message */
-        if(H5O_msg_create(oloc, H5O_LINFO_ID, 0, 0, linfo, dxpl_id) < 0)
+        /* (Casting away const OK - QAK) */
+        if(H5O_msg_create(oloc, H5O_LINFO_ID, 0, H5O_UPDATE_TIME, (void *)linfo, dxpl_id) < 0)
             HGOTO_ERROR(H5E_SYM, H5E_CANTINIT, FAIL, "can't create message")
 
         /* Insert group info message */
-        if(H5O_msg_create(oloc, H5O_GINFO_ID, H5O_MSG_FLAG_CONSTANT, H5O_UPDATE_TIME, ginfo, dxpl_id) < 0)
+        /* (Casting away const OK - QAK) */
+        if(H5O_msg_create(oloc, H5O_GINFO_ID, H5O_MSG_FLAG_CONSTANT, 0, (void *)ginfo, dxpl_id) < 0)
             HGOTO_ERROR(H5E_SYM, H5E_CANTINIT, FAIL, "can't create message")
+
+        /* Insert pipeline message */
+        if(pline && pline->nused)
+            /* (Casting away const OK - QAK) */
+            if(H5O_msg_create(oloc, H5O_PLINE_ID, H5O_MSG_FLAG_CONSTANT, 0, (void *)pline, dxpl_id) < 0)
+                HGOTO_ERROR(H5E_SYM, H5E_CANTINIT, FAIL, "can't create message")
     } /* end if */
     else {
         H5O_stab_t	stab;		/* Symbol table message	*/
@@ -213,11 +289,16 @@ H5G_obj_create(H5F_t *f, hid_t dxpl_id, const H5O_ginfo_t *ginfo,
         /* The group doesn't currently have a 'stab' message, go create one */
         if(H5G_stab_create(oloc, dxpl_id, ginfo, &stab) < 0)
             HGOTO_ERROR(H5E_SYM, H5E_CANTINIT, FAIL, "unable to create symbol table")
+
+        /* Cache the symbol table information */
+        gcrt_info->cache_type = H5G_CACHED_STAB;
+        gcrt_info->cache.stab.btree_addr = stab.btree_addr;
+        gcrt_info->cache.stab.heap_addr = stab.heap_addr;
     } /* end else */
 
 done:
     FUNC_LEAVE_NOAPI(ret_value)
-} /* end H5G_obj_create() */
+} /* end H5G_obj_create_real() */
 
 
 /*-------------------------------------------------------------------------
@@ -238,6 +319,7 @@ done:
 htri_t
 H5G_obj_get_linfo(const H5O_loc_t *grp_oloc, H5O_linfo_t *linfo, hid_t dxpl_id)
 {
+    H5B2_t *bt2_name = NULL;            /* v2 B-tree handle for name index */
     htri_t ret_value;           /* Return value */
 
     FUNC_ENTER_NOAPI(H5G_obj_get_linfo, FAIL)
@@ -258,9 +340,13 @@ H5G_obj_get_linfo(const H5O_loc_t *grp_oloc, H5O_linfo_t *linfo, hid_t dxpl_id)
         if(linfo->nlinks == HSIZET_MAX) {
             /* Check if we are using "dense" link storage */
             if(H5F_addr_defined(linfo->fheap_addr)) {
+                /* Open the name index v2 B-tree */
+                if(NULL == (bt2_name = H5B2_open(grp_oloc->file, dxpl_id, linfo->name_bt2_addr, NULL)))
+                    HGOTO_ERROR(H5E_SYM, H5E_CANTOPENOBJ, FAIL, "unable to open v2 B-tree for name index")
+
                 /* Retrieve # of records in "name" B-tree */
                 /* (should be same # of records in all indices) */
-                if(H5B2_get_nrec(grp_oloc->file, dxpl_id, H5G_BT2_NAME, linfo->name_bt2_addr, &linfo->nlinks) < 0)
+                if(H5B2_get_nrec(bt2_name, &linfo->nlinks) < 0)
                     HGOTO_ERROR(H5E_SYM, H5E_CANTGET, FAIL, "can't retrieve # of records in index")
             } /* end if */
             else {
@@ -272,6 +358,10 @@ H5G_obj_get_linfo(const H5O_loc_t *grp_oloc, H5O_linfo_t *linfo, hid_t dxpl_id)
     } /* end if */
 
 done:
+    /* Release resources */
+    if(bt2_name && H5B2_close(bt2_name, dxpl_id) < 0)
+        HDONE_ERROR(H5E_SYM, H5E_CLOSEERROR, FAIL, "can't close v2 B-tree for name index")
+
     FUNC_LEAVE_NOAPI(ret_value)
 } /* end H5G_obj_get_linfo() */
 
@@ -340,7 +430,8 @@ H5G_obj_stab_to_new_cb(const H5O_link_t *lnk, void *_udata)
 
     /* Insert link into group */
     /* (Casting away const OK - QAK) */
-    if(H5G_obj_insert(udata->grp_oloc, lnk->name, (H5O_link_t *)lnk, FALSE, udata->dxpl_id) < 0)
+    if(H5G_obj_insert(udata->grp_oloc, lnk->name, (H5O_link_t *)lnk, FALSE,
+            H5O_TYPE_UNKNOWN, NULL, udata->dxpl_id) < 0)
         HGOTO_ERROR(H5E_SYM, H5E_CANTINSERT, H5_ITER_ERROR, "can't insert link into group")
 
 done:
@@ -367,8 +458,10 @@ done:
  */
 herr_t
 H5G_obj_insert(const H5O_loc_t *grp_oloc, const char *name, H5O_link_t *obj_lnk,
-    hbool_t adj_link, hid_t dxpl_id)
+    hbool_t adj_link, H5O_type_t obj_type, const void *crt_info, hid_t dxpl_id)
 {
+    H5O_pline_t tmp_pline;      /* Pipeline message */
+    H5O_pline_t *pline = NULL;  /* Pointer to pipeline message */
     H5O_linfo_t linfo;		/* Link info message */
     htri_t linfo_exists;        /* Whether the link info message exists */
     hbool_t use_old_format;     /* Whether to use 'old format' (symbol table) for insertions or not */
@@ -420,11 +513,21 @@ H5G_obj_insert(const H5O_loc_t *grp_oloc, const char *name, H5O_link_t *obj_lnk,
         else if(linfo.nlinks < ginfo.max_compact && link_msg_size < H5O_MESG_MAX_SIZE)
             use_new_dense = FALSE;
         else {
+            htri_t              pline_exists;   /* Whether the pipeline message exists */
             H5G_obj_oh_it_ud1_t	udata;          /* User data for iteration */
             H5O_mesg_operator_t op;             /* Message operator */
 
+            /* Get the pipeline message, if it exists */
+            if((pline_exists = H5O_msg_exists(grp_oloc, H5O_PLINE_ID, dxpl_id)) < 0)
+                HGOTO_ERROR(H5E_SYM, H5E_CANTGET, FAIL, "unable to read object header")
+            if(pline_exists) {
+                if(NULL == H5O_msg_read(grp_oloc, H5O_PLINE_ID, &tmp_pline, dxpl_id))
+                    HGOTO_ERROR(H5E_SYM, H5E_BADMESG, FAIL, "can't get link pipeline")
+                pline = &tmp_pline;
+            } /* end if */
+
             /* The group doesn't currently have "dense" storage for links */
-            if(H5G_dense_create(grp_oloc->file, dxpl_id, &linfo) < 0)
+            if(H5G_dense_create(grp_oloc->file, dxpl_id, &linfo, pline) < 0)
                 HGOTO_ERROR(H5E_SYM, H5E_CANTINIT, FAIL, "unable to create 'dense' form of new format group")
 
             /* Set up user data for object header message iteration */
@@ -478,7 +581,8 @@ H5G_obj_insert(const H5O_loc_t *grp_oloc, const char *name, H5O_link_t *obj_lnk,
              *  group is in the "new format" now and the link info should be
              *  set up, etc.
              */
-            if(H5G_obj_insert(grp_oloc, name, obj_lnk, adj_link, dxpl_id) < 0)
+            if(H5G_obj_insert(grp_oloc, name, obj_lnk, adj_link, obj_type,
+                    crt_info, dxpl_id) < 0)
                 HGOTO_ERROR(H5E_SYM, H5E_CANTINSERT, FAIL, "unable to insert link into group")
 
             /* Done with insertion now */
@@ -491,7 +595,8 @@ H5G_obj_insert(const H5O_loc_t *grp_oloc, const char *name, H5O_link_t *obj_lnk,
     /* Insert into symbol table or "dense" storage */
     if(use_old_format) {
         /* Insert into symbol table */
-        if(H5G_stab_insert(grp_oloc, name, obj_lnk, dxpl_id) < 0)
+        if(H5G_stab_insert(grp_oloc, name, obj_lnk, obj_type, crt_info, dxpl_id)
+                < 0)
             HGOTO_ERROR(H5E_SYM, H5E_CANTINSERT, FAIL, "unable to insert entry into symbol table")
     } /* end if */
     else {
@@ -529,6 +634,10 @@ H5G_obj_insert(const H5O_loc_t *grp_oloc, const char *name, H5O_link_t *obj_lnk,
     } /* end if */
 
 done:
+    /* Free any space used by the pipeline message */
+    if(pline && H5O_msg_reset(H5O_PLINE_ID, pline) < 0)
+        HDONE_ERROR(H5E_SYM, H5E_CANTFREE, FAIL, "can't release pipeline")
+
     FUNC_LEAVE_NOAPI(ret_value)
 } /* end H5G_obj_insert() */
 
@@ -805,9 +914,9 @@ H5G_obj_remove_update_linfo(H5O_loc_t *oloc, H5O_linfo_t *linfo, hid_t dxpl_id)
                 if(H5G_dense_build_table(oloc->file, dxpl_id, linfo, H5_INDEX_NAME, H5_ITER_NATIVE, &ltable) < 0)
                     HGOTO_ERROR(H5E_SYM, H5E_CANTNEXT, FAIL, "error iterating over links")
 
-                /* Get a pointer to the object header itself */
-                if((oh = H5O_protect(oloc, dxpl_id)) == NULL)
-                    HGOTO_ERROR(H5E_SYM, H5E_CANTINIT, FAIL, "unable to protect dataset object header")
+                /* Pin the object header */
+                if(NULL == (oh = H5O_pin(oloc, dxpl_id)))
+                    HGOTO_ERROR(H5E_SYM, H5E_CANTPIN, FAIL, "unable to pin group object header")
 
                 /* Inspect links in table for ones that can't be converted back
                  * into link message form (currently only links which can't fit
@@ -825,8 +934,8 @@ H5G_obj_remove_update_linfo(H5O_loc_t *oloc, H5O_linfo_t *linfo, hid_t dxpl_id)
                     for(u = 0; u < linfo->nlinks; u++)
                         if(H5O_msg_append_oh(oloc->file, dxpl_id, oh, H5O_LINK_ID, 0, H5O_UPDATE_TIME, &(ltable.lnks[u])) < 0) {
                             /* Release object header */
-                            if(H5O_unprotect(oloc, oh) < 0)
-                                HDONE_ERROR(H5E_DATASET, H5E_CANTINIT, FAIL, "unable to unprotect dataset object header")
+                            if(H5O_unpin(oh) < 0)
+                                HDONE_ERROR(H5E_SYM, H5E_CANTUNPIN, FAIL, "unable to unpin group object header")
 
                             HGOTO_ERROR(H5E_SYM, H5E_CANTINIT, FAIL, "can't create message")
                         } /* end if */
@@ -837,8 +946,8 @@ H5G_obj_remove_update_linfo(H5O_loc_t *oloc, H5O_linfo_t *linfo, hid_t dxpl_id)
                 } /* end if */
 
                 /* Release object header */
-                if(H5O_unprotect(oloc, oh) < 0)
-                    HDONE_ERROR(H5E_DATASET, H5E_CANTINIT, FAIL, "unable to unprotect dataset object header")
+                if(H5O_unpin(oh) < 0)
+                    HGOTO_ERROR(H5E_SYM, H5E_CANTUNPIN, FAIL, "unable to unpin group object header")
 
                 /* Free link table information */
                 if(H5G_link_release_table(&ltable) < 0)
