@@ -17,25 +17,39 @@
  *	       Friday, September 19, 1997
  *
  */
-#define H5F_PACKAGE		/*suppress error about including H5Fpkg	  */
+
+/****************/
+/* Module Setup */
+/****************/
+
 #define H5G_PACKAGE		/*suppress error about including H5Gpkg	  */
 
 
-/* Packages needed by this file... */
+/***********/
+/* Headers */
+/***********/
 #include "H5private.h"		/* Generic Functions			*/
 #include "H5Eprivate.h"		/* Error handling		  	*/
-#include "H5Fpkg.h"		/* File access				*/
+#include "H5Fprivate.h"		/* File access				*/
 #include "H5Gpkg.h"		/* Groups		  		*/
 #include "H5HLprivate.h"	/* Local Heaps				*/
 #include "H5MMprivate.h"	/* Memory management			*/
 
-/* Private typedefs */
+
+/****************/
+/* Local Macros */
+/****************/
+
+
+/******************/
+/* Local Typedefs */
+/******************/
+
 /* User data for finding link information from B-tree */
 typedef struct {
     /* downward */
-    H5F_t      *file;           /* Pointer to file for query */
     const char *name;           /* Name to search for */
-    H5HL_t     *heap;           /* Local heap for group */
+    H5HL_t *heap;               /* Local heap for group */
 
     /* upward */
     H5O_link_t *lnk;            /* Caller's link location */
@@ -45,7 +59,7 @@ typedef struct {
 typedef struct H5G_bt_it_gnbi_t {
     /* downward */
     H5G_bt_it_idx_common_t common; /* Common information for "by index" lookup  */
-    H5HL_t      *heap;          /*symbol table heap 			     */
+    H5HL_t *heap;               /*symbol table heap 			     */
 
     /* upward */
     char         *name;         /*member name to be returned                 */
@@ -56,6 +70,7 @@ typedef struct H5G_bt_it_gnbi_t {
 typedef struct H5G_bt_it_gtbi_t {
     /* downward */
     H5G_bt_it_idx_common_t common; /* Common information for "by index" lookup  */
+    H5F_t       *f;             /* Pointer to file that symbol table is in */
     hid_t       dxpl_id;        /* DXPL for operation */
 
     /* upward */
@@ -67,14 +82,38 @@ typedef struct H5G_bt_it_gtbi_t {
 typedef struct H5G_bt_it_lbi_t {
     /* downward */
     H5G_bt_it_idx_common_t common; /* Common information for "by index" lookup  */
-    H5HL_t      *heap;          /*symbol table heap 			     */
+    H5HL_t *heap;               /*symbol table heap 			     */
 
     /* upward */
     H5O_link_t *lnk;            /*link to be returned                        */
     hbool_t     found;      	/*whether we found the link                  */
 } H5G_bt_it_lbi_t;
 
-/* Private prototypes */
+
+/********************/
+/* Package Typedefs */
+/********************/
+
+
+/********************/
+/* Local Prototypes */
+/********************/
+
+
+/*********************/
+/* Package Variables */
+/*********************/
+
+
+/*****************************/
+/* Library Private Variables */
+/*****************************/
+
+
+/*******************/
+/* Local Variables */
+/*******************/
+
 
 
 /*-------------------------------------------------------------------------
@@ -99,9 +138,9 @@ typedef struct H5G_bt_it_lbi_t {
 herr_t
 H5G_stab_create_components(H5F_t *f, H5O_stab_t *stab, size_t size_hint, hid_t dxpl_id)
 {
-    H5HL_t      *heap = NULL;           /* Pointer to local heap */
-    size_t	name_offset;	        /* Offset of "" name	*/
-    herr_t      ret_value = SUCCEED;    /* Return value */
+    H5HL_t *heap = NULL;            /* Pointer to local heap */
+    size_t name_offset;	            /* Offset of "" name */
+    herr_t ret_value = SUCCEED;     /* Return value */
 
     FUNC_ENTER_NOAPI(H5G_stab_create_components, FAIL)
 
@@ -136,7 +175,7 @@ H5G_stab_create_components(H5F_t *f, H5O_stab_t *stab, size_t size_hint, hid_t d
 
 done:
     /* Release resources */
-    if(heap && H5HL_unprotect(f, dxpl_id, heap, stab->heap_addr) < 0)
+    if(heap && H5HL_unprotect(heap) < 0)
         HDONE_ERROR(H5E_SYM, H5E_PROTECT, FAIL, "unable to unprotect symbol table heap")
 
     FUNC_LEAVE_NOAPI(ret_value)
@@ -220,12 +259,13 @@ done:
  *-------------------------------------------------------------------------
  */
 herr_t
-H5G_stab_insert_real(H5F_t *f, H5O_stab_t *stab, const char *name,
-    H5O_link_t *obj_lnk, hid_t dxpl_id)
+H5G_stab_insert_real(H5F_t *f, const H5O_stab_t *stab, const char *name,
+    H5O_link_t *obj_lnk, H5O_type_t obj_type, const void *crt_info,
+    hid_t dxpl_id)
 {
-    H5HL_t      *heap = NULL;           /* Pointer to local heap */
-    H5G_bt_ins_t	udata;		/* Data to pass through B-tree	*/
-    herr_t              ret_value = SUCCEED;       /* Return value */
+    H5HL_t       *heap = NULL;          /* Pointer to local heap */
+    H5G_bt_ins_t udata;		        /* Data to pass through B-tree	*/
+    herr_t       ret_value = SUCCEED;   /* Return value */
 
     FUNC_ENTER_NOAPI(H5G_stab_insert_real, FAIL)
 
@@ -243,6 +283,8 @@ H5G_stab_insert_real(H5F_t *f, H5O_stab_t *stab, const char *name,
     udata.common.name = name;
     udata.common.heap = heap;
     udata.lnk = obj_lnk;
+    udata.obj_type = obj_type;
+    udata.crt_info = crt_info;
 
     /* Insert into symbol table */
     if(H5B_insert(f, dxpl_id, H5B_SNODE, stab->btree_addr, &udata) < 0)
@@ -250,7 +292,7 @@ H5G_stab_insert_real(H5F_t *f, H5O_stab_t *stab, const char *name,
 
 done:
     /* Release resources */
-    if(heap && H5HL_unprotect(f, dxpl_id, heap, stab->heap_addr) < 0)
+    if(heap && H5HL_unprotect(heap) < 0)
         HDONE_ERROR(H5E_SYM, H5E_PROTECT, FAIL, "unable to unprotect symbol table heap")
 
     FUNC_LEAVE_NOAPI(ret_value)
@@ -273,7 +315,8 @@ done:
  *-------------------------------------------------------------------------
  */
 herr_t
-H5G_stab_insert(const H5O_loc_t *grp_oloc, const char *name, H5O_link_t *obj_lnk,
+H5G_stab_insert(const H5O_loc_t *grp_oloc, const char *name,
+    H5O_link_t *obj_lnk, H5O_type_t obj_type, const void *crt_info,
     hid_t dxpl_id)
 {
     H5O_stab_t		stab;		/* Symbol table message		*/
@@ -290,7 +333,8 @@ H5G_stab_insert(const H5O_loc_t *grp_oloc, const char *name, H5O_link_t *obj_lnk
     if(NULL == H5O_msg_read(grp_oloc, H5O_STAB_ID, &stab, dxpl_id))
         HGOTO_ERROR(H5E_SYM, H5E_BADMESG, FAIL, "not a symbol table")
 
-    if(H5G_stab_insert_real(grp_oloc->file, &stab, name, obj_lnk, dxpl_id) < 0)
+    if(H5G_stab_insert_real(grp_oloc->file, &stab, name, obj_lnk, obj_type,
+            crt_info, dxpl_id) < 0)
         HGOTO_ERROR(H5E_DATATYPE, H5E_CANTINIT, H5_ITER_ERROR, "unable to insert the name")
 
 done:
@@ -315,9 +359,9 @@ H5G_stab_remove(H5O_loc_t *loc, hid_t dxpl_id, H5RS_str_t *grp_full_path_r,
     const char *name)
 {
     H5HL_t      *heap = NULL;           /* Pointer to local heap */
-    H5O_stab_t		stab;		/*symbol table message		*/
-    H5G_bt_rm_t		udata;		/*data to pass through B-tree	*/
-    herr_t              ret_value = SUCCEED;       /* Return value */
+    H5O_stab_t	stab;		        /*symbol table message		*/
+    H5G_bt_rm_t	udata;		        /*data to pass through B-tree	*/
+    herr_t      ret_value = SUCCEED;    /* Return value */
 
     FUNC_ENTER_NOAPI(H5G_stab_remove, FAIL)
 
@@ -343,7 +387,7 @@ H5G_stab_remove(H5O_loc_t *loc, hid_t dxpl_id, H5RS_str_t *grp_full_path_r,
 
 done:
     /* Release resources */
-    if(heap && H5HL_unprotect(loc->file, dxpl_id, heap, stab.heap_addr) < 0)
+    if(heap && H5HL_unprotect(heap) < 0)
         HDONE_ERROR(H5E_SYM, H5E_PROTECT, FAIL, "unable to unprotect symbol table heap")
 
     FUNC_LEAVE_NOAPI(ret_value)
@@ -367,11 +411,11 @@ H5G_stab_remove_by_idx(H5O_loc_t *grp_oloc, hid_t dxpl_id, H5RS_str_t *grp_full_
     H5_iter_order_t order, hsize_t n)
 {
     H5HL_t      *heap = NULL;           /* Pointer to local heap */
-    H5O_stab_t		stab;		/* Symbol table message		*/
-    H5G_bt_rm_t	udata;		/* Data to pass through B-tree	*/
-    H5O_link_t          obj_lnk;        /* Object's link within group */
-    hbool_t             lnk_copied = FALSE;         /* Whether the link was copied */
-    herr_t              ret_value = SUCCEED;       /* Return value */
+    H5O_stab_t	stab;		        /* Symbol table message		*/
+    H5G_bt_rm_t udata;		        /* Data to pass through B-tree	*/
+    H5O_link_t  obj_lnk;                /* Object's link within group */
+    hbool_t     lnk_copied = FALSE;     /* Whether the link was copied */
+    herr_t      ret_value = SUCCEED;    /* Return value */
 
     FUNC_ENTER_NOAPI(H5G_stab_remove_by_idx, FAIL)
 
@@ -401,7 +445,7 @@ H5G_stab_remove_by_idx(H5O_loc_t *grp_oloc, hid_t dxpl_id, H5RS_str_t *grp_full_
 
 done:
     /* Release resources */
-    if(heap && H5HL_unprotect(grp_oloc->file, dxpl_id, heap, stab.heap_addr) < 0)
+    if(heap && H5HL_unprotect(heap) < 0)
         HDONE_ERROR(H5E_SYM, H5E_PROTECT, FAIL, "unable to unprotect symbol table heap")
 
     /* Reset the link information, if we have a copy */
@@ -429,9 +473,9 @@ done:
 herr_t
 H5G_stab_delete(H5F_t *f, hid_t dxpl_id, const H5O_stab_t *stab)
 {
-    H5HL_t      *heap = NULL;           /* Pointer to local heap */
-    H5G_bt_rm_t	udata;		/*data to pass through B-tree	*/
-    herr_t  ret_value = SUCCEED;
+    H5HL_t *heap = NULL;                /* Pointer to local heap */
+    H5G_bt_rm_t	udata;		        /*data to pass through B-tree	*/
+    herr_t ret_value = SUCCEED;         /* Return value */
 
     FUNC_ENTER_NOAPI(H5G_stab_delete, FAIL)
 
@@ -453,7 +497,7 @@ H5G_stab_delete(H5F_t *f, hid_t dxpl_id, const H5O_stab_t *stab)
         HGOTO_ERROR(H5E_SYM, H5E_CANTDELETE, FAIL, "unable to delete symbol table B-tree")
 
     /* Release resources */
-    if(H5HL_unprotect(f, dxpl_id, heap, stab->heap_addr) < 0)
+    if(H5HL_unprotect(heap) < 0)
         HGOTO_ERROR(H5E_SYM, H5E_PROTECT, FAIL, "unable to unprotect symbol table heap")
     heap = NULL;
 
@@ -463,7 +507,7 @@ H5G_stab_delete(H5F_t *f, hid_t dxpl_id, const H5O_stab_t *stab)
 
 done:
     /* Release resources */
-    if(heap && H5HL_unprotect(f, dxpl_id, heap, stab->heap_addr) < 0)
+    if(heap && H5HL_unprotect(heap) < 0)
         HDONE_ERROR(H5E_SYM, H5E_PROTECT, FAIL, "unable to unprotect symbol table heap")
 
     FUNC_LEAVE_NOAPI(ret_value)
@@ -486,10 +530,10 @@ herr_t
 H5G_stab_iterate(const H5O_loc_t *oloc, hid_t dxpl_id, H5_iter_order_t order,
     hsize_t skip, hsize_t *last_lnk, H5G_lib_iterate_t op, void *op_data)
 {
-    H5HL_t              *heap = NULL;           /* Local heap for group */
-    H5O_stab_t		stab;		        /* Info about symbol table */
+    H5HL_t *heap = NULL;                        /* Local heap for group */
+    H5O_stab_t stab;		                /* Info about symbol table */
     H5G_link_table_t ltable = {0, NULL};        /* Link table */
-    herr_t		ret_value;
+    herr_t ret_value;                           /* Return value */
 
     FUNC_ENTER_NOAPI(H5G_stab_iterate, FAIL)
 
@@ -518,7 +562,7 @@ H5G_stab_iterate(const H5O_loc_t *oloc, hid_t dxpl_id, H5_iter_order_t order,
         udata.op_data = op_data;
 
         /* Iterate over the group members */
-        if((ret_value = H5B_iterate(oloc->file, dxpl_id, H5B_SNODE, H5G_node_iterate, stab.btree_addr, &udata)) < 0)
+        if((ret_value = H5B_iterate(oloc->file, dxpl_id, H5B_SNODE, stab.btree_addr, H5G_node_iterate, &udata)) < 0)
             HERROR(H5E_SYM, H5E_CANTNEXT, "iteration operator failed");
 
         /* Check for too high of a starting index (ex post facto :-) */
@@ -535,8 +579,7 @@ H5G_stab_iterate(const H5O_loc_t *oloc, hid_t dxpl_id, H5_iter_order_t order,
         udata.ltable = &ltable;
 
         /* Iterate over the group members */
-        if(H5B_iterate(oloc->file, dxpl_id, H5B_SNODE, H5G_node_build_table,
-                stab.btree_addr, &udata) < 0)
+        if(H5B_iterate(oloc->file, dxpl_id, H5B_SNODE, stab.btree_addr, H5G_node_build_table, &udata) < 0)
             HGOTO_ERROR(H5E_SYM, H5E_NOTFOUND, FAIL, "unable to build link table")
 
         /* Check for skipping out of bounds */
@@ -554,7 +597,7 @@ H5G_stab_iterate(const H5O_loc_t *oloc, hid_t dxpl_id, H5_iter_order_t order,
 
 done:
     /* Release resources */
-    if(heap && H5HL_unprotect(oloc->file, dxpl_id, heap, stab.heap_addr) < 0)
+    if(heap && H5HL_unprotect(heap) < 0)
         HDONE_ERROR(H5E_SYM, H5E_PROTECT, FAIL, "unable to unprotect symbol table heap")
     if(ltable.lnks && H5G_link_release_table(&ltable) < 0)
         HDONE_ERROR(H5E_SYM, H5E_CANTFREE, FAIL, "unable to release link table")
@@ -595,7 +638,7 @@ H5G_stab_count(H5O_loc_t *oloc, hsize_t *num_objs, hid_t dxpl_id)
 	HGOTO_ERROR(H5E_SYM, H5E_NOTFOUND, FAIL, "unable to determine local heap address")
 
     /* Iterate over the group members */
-    if(H5B_iterate(oloc->file, dxpl_id, H5B_SNODE, H5G_node_sumup, stab.btree_addr, num_objs) < 0)
+    if(H5B_iterate(oloc->file, dxpl_id, H5B_SNODE, stab.btree_addr, H5G_node_sumup, num_objs) < 0)
         HGOTO_ERROR(H5E_SYM, H5E_CANTINIT, FAIL, "iteration operator failed")
 
 done:
@@ -616,10 +659,12 @@ done:
  *-------------------------------------------------------------------------
  */
 herr_t
-H5G_stab_bh_size(H5F_t *f, hid_t dxpl_id, const H5O_stab_t *stab, H5_ih_info_t *bh_info)
+H5G_stab_bh_size(H5F_t *f, hid_t dxpl_id, const H5O_stab_t *stab,
+    H5_ih_info_t *bh_info)
 {
-    H5B_info_ud_t       bh_udata;               /* User-data for B-tree callbacks */
-    herr_t              ret_value = SUCCEED;
+    hsize_t     snode_size;             /* Symbol table node size */
+    H5B_info_t  bt_info;                /* B-tree node info */
+    herr_t      ret_value = SUCCEED;
 
     FUNC_ENTER_NOAPI(H5G_stab_bh_size, FAIL)
 
@@ -628,14 +673,16 @@ H5G_stab_bh_size(H5F_t *f, hid_t dxpl_id, const H5O_stab_t *stab, H5_ih_info_t *
     HDassert(stab);
     HDassert(bh_info);
 
-    /* Set up user data for B-tree callback */
-    bh_udata.udata = NULL;
-    bh_udata.btree_size = &(bh_info->index_size);
+    /* Set up user data for B-tree iteration */
+    snode_size = 0;
 
     /* Get the B-tree & symbol table node size info */
-    if(H5B_iterate_size(f, dxpl_id, H5B_SNODE, H5G_node_iterate_size, stab->btree_addr, &bh_udata) < 0)
+    if(H5B_get_info(f, dxpl_id, H5B_SNODE, stab->btree_addr, &bt_info, H5G_node_iterate_size, &snode_size) < 0)
         HGOTO_ERROR(H5E_BTREE, H5E_CANTINIT, FAIL, "iteration operator failed")
 
+    /* Add symbol table & B-tree node sizes to index info */
+    bh_info->index_size += snode_size + bt_info.size;
+
     /* Get the size of the local heap for the group */
     if(H5HL_heapsize(f, dxpl_id, stab->heap_addr, &(bh_info->heap_size)) < 0)
         HGOTO_ERROR(H5E_HEAP, H5E_CANTINIT, FAIL, "iteration operator failed")
@@ -648,7 +695,7 @@ done:
 /*-------------------------------------------------------------------------
  * Function:	H5G_stab_get_name_by_idx_cb
  *
- * Purpose:     Callback for B-tree iteration 'by index' info query to 
+ * Purpose:     Callback for B-tree iteration 'by index' info query to
  *              retrieve the name of a link
  *
  * Return:	Success:        Non-negative
@@ -674,7 +721,7 @@ H5G_stab_get_name_by_idx_cb(const H5G_entry_t *ent, void *_udata)
 
     /* Get name offset in heap */
     name_off = ent->name_off;
-    name = H5HL_offset_into(udata->common.f, udata->heap, name_off);
+    name = (const char *)H5HL_offset_into(udata->heap, name_off);
     HDassert(name);
     udata->name = H5MM_strdup(name);
     HDassert(udata->name);
@@ -700,10 +747,14 @@ ssize_t
 H5G_stab_get_name_by_idx(H5O_loc_t *oloc, H5_iter_order_t order, hsize_t n,
     char* name, size_t size, hid_t dxpl_id)
 {
-    H5HL_t      *heap = NULL;           /* Pointer to local heap */
-    H5O_stab_t		stab;	        /* Info about local heap & B-tree */
-    H5G_bt_it_gnbi_t	udata;          /* Iteration information */
-    ssize_t		ret_value;      /* Return value */
+    H5HL_t *heap = NULL;        /* Pointer to local heap */
+    H5O_stab_t	stab;	        /* Info about local heap & B-tree */
+    H5G_bt_it_gnbi_t udata;     /* Iteration information */
+    hbool_t udata_valid = FALSE;        /* Whether iteration information is valid */
+    ssize_t ret_value;          /* Return value */
+
+    /* Portably clear udata struct (before FUNC_ENTER) */
+    HDmemset(&udata, 0, sizeof(udata));
 
     FUNC_ENTER_NOAPI(H5G_stab_get_name_by_idx, FAIL)
 
@@ -723,7 +774,7 @@ H5G_stab_get_name_by_idx(H5O_loc_t *oloc, H5_iter_order_t order, hsize_t n,
         hsize_t nlinks = 0;           /* Number of links in group */
 
         /* Iterate over the symbol table nodes, to count the links */
-        if(H5B_iterate(oloc->file, dxpl_id, H5B_SNODE, H5G_node_sumup, stab.btree_addr, &nlinks) < 0)
+        if(H5B_iterate(oloc->file, dxpl_id, H5B_SNODE, stab.btree_addr, H5G_node_sumup, &nlinks) < 0)
             HGOTO_ERROR(H5E_SYM, H5E_CANTINIT, FAIL, "iteration operator failed")
 
         /* Map decreasing iteration order index to increasing iteration order index */
@@ -731,15 +782,15 @@ H5G_stab_get_name_by_idx(H5O_loc_t *oloc, H5_iter_order_t order, hsize_t n,
     } /* end if */
 
     /* Set iteration information */
-    udata.common.f = oloc->file;
     udata.common.idx = n;
     udata.common.num_objs = 0;
     udata.common.op = H5G_stab_get_name_by_idx_cb;
     udata.heap = heap;
     udata.name = NULL;
+    udata_valid = TRUE;
 
     /* Iterate over the group members */
-    if(H5B_iterate(oloc->file, dxpl_id, H5B_SNODE, H5G_node_by_idx, stab.btree_addr, &udata) < 0)
+    if(H5B_iterate(oloc->file, dxpl_id, H5B_SNODE, stab.btree_addr, H5G_node_by_idx, &udata) < 0)
 	HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "iteration operator failed")
 
     /* If we don't know the name now, we almost certainly went out of bounds */
@@ -758,11 +809,11 @@ H5G_stab_get_name_by_idx(H5O_loc_t *oloc, H5_iter_order_t order, hsize_t n,
 
 done:
     /* Release resources */
-    if(heap && H5HL_unprotect(oloc->file, dxpl_id, heap, stab.heap_addr) < 0)
+    if(heap && H5HL_unprotect(heap) < 0)
         HDONE_ERROR(H5E_SYM, H5E_PROTECT, FAIL, "unable to unprotect symbol table heap")
 
     /* Free the duplicated name */
-    if(udata.name != NULL)
+    if(udata_valid && udata.name != NULL)
         H5MM_xfree(udata.name);
 
     FUNC_LEAVE_NOAPI(ret_value)
@@ -794,7 +845,7 @@ H5G_stab_lookup_cb(const H5G_entry_t *ent, void *_udata)
     /* Check for setting link info */
     if(udata->lnk)
         /* Convert the entry to a link */
-        if(H5G_ent_to_link(udata->file, udata->lnk, udata->heap, ent, udata->name) < 0)
+        if(H5G_ent_to_link(udata->lnk, udata->heap, ent, udata->name) < 0)
             HGOTO_ERROR(H5E_SYM, H5E_CANTCONVERT, FAIL, "unable to convert symbol table entry to link")
 
 done:
@@ -807,7 +858,7 @@ done:
  *
  * Purpose:	Look up an object relative to a group, using symbol table
  *
- * Return:	Non-negative on success/Negative on failure
+ * Return:	Non-negative (TRUE/FALSE) on success/Negative on failure
  *
  * Programmer:	Quincey Koziol
  *		koziol@ncsa.uiuc.edu
@@ -815,15 +866,15 @@ done:
  *
  *-------------------------------------------------------------------------
  */
-herr_t
+htri_t
 H5G_stab_lookup(H5O_loc_t *grp_oloc, const char *name, H5O_link_t *lnk,
     hid_t dxpl_id)
 {
-    H5HL_t      *heap = NULL;   /* Pointer to local heap */
-    H5G_bt_lkp_t bt_udata;      /* Data to pass through B-tree	*/
-    H5G_stab_fnd_ud_t udata;    /* 'User data' to give to callback */
-    H5O_stab_t stab;		/* Symbol table message		*/
-    herr_t     ret_value = SUCCEED;     /* Return value */
+    H5HL_t *heap = NULL;                /* Pointer to local heap */
+    H5G_bt_lkp_t bt_udata;              /* Data to pass through B-tree	*/
+    H5G_stab_fnd_ud_t udata;            /* 'User data' to give to callback */
+    H5O_stab_t stab;		        /* Symbol table message		*/
+    htri_t     ret_value;       /* Return value */
 
     FUNC_ENTER_NOAPI(H5G_stab_lookup, FAIL)
 
@@ -841,7 +892,6 @@ H5G_stab_lookup(H5O_loc_t *grp_oloc, const char *name, H5O_link_t *lnk,
         HGOTO_ERROR(H5E_SYM, H5E_PROTECT, FAIL, "unable to protect symbol table heap")
 
     /* Set up user data to pass to 'find' operation callback */
-    udata.file = grp_oloc->file;
     udata.name = name;
     udata.lnk = lnk;
     udata.heap = heap;
@@ -853,12 +903,12 @@ H5G_stab_lookup(H5O_loc_t *grp_oloc, const char *name, H5O_link_t *lnk,
     bt_udata.op_data = &udata;
 
     /* Search the B-tree */
-    if(H5B_find(grp_oloc->file, dxpl_id, H5B_SNODE, stab.btree_addr, &bt_udata) < 0)
+    if((ret_value = H5B_find(grp_oloc->file, dxpl_id, H5B_SNODE, stab.btree_addr, &bt_udata)) < 0)
         HGOTO_ERROR(H5E_SYM, H5E_NOTFOUND, FAIL, "not found")
 
 done:
     /* Release resources */
-    if(heap && H5HL_unprotect(grp_oloc->file, dxpl_id, heap, stab.heap_addr) < 0)
+    if(heap && H5HL_unprotect(heap) < 0)
         HDONE_ERROR(H5E_SYM, H5E_PROTECT, FAIL, "unable to unprotect symbol table heap")
 
     FUNC_LEAVE_NOAPI(ret_value)
@@ -868,7 +918,7 @@ done:
 /*-------------------------------------------------------------------------
  * Function:	H5G_stab_lookup_by_idx_cb
  *
- * Purpose:     Callback for B-tree iteration 'by index' info query to 
+ * Purpose:     Callback for B-tree iteration 'by index' info query to
  *              retrieve the link
  *
  * Return:	Success:        Non-negative
@@ -893,11 +943,11 @@ H5G_stab_lookup_by_idx_cb(const H5G_entry_t *ent, void *_udata)
     HDassert(udata && udata->heap);
 
     /* Get a pointer to the link name */
-    name = H5HL_offset_into(udata->common.f, udata->heap, ent->name_off);
+    name = (const char *)H5HL_offset_into(udata->heap, ent->name_off);
     HDassert(name);
 
     /* Convert the entry to a link */
-    if(H5G_ent_to_link(udata->common.f, udata->lnk, udata->heap, ent, name) < 0)
+    if(H5G_ent_to_link(udata->lnk, udata->heap, ent, name) < 0)
         HGOTO_ERROR(H5E_SYM, H5E_CANTCONVERT, FAIL, "unable to convert symbol table entry to link")
     udata->found = TRUE;
 
@@ -923,10 +973,10 @@ herr_t
 H5G_stab_lookup_by_idx(H5O_loc_t *grp_oloc, H5_iter_order_t order, hsize_t n,
     H5O_link_t *lnk, hid_t dxpl_id)
 {
-    H5HL_t      *heap = NULL;   /* Pointer to local heap */
-    H5G_bt_it_lbi_t udata;             /* Iteration information */
+    H5HL_t *heap = NULL;                /* Pointer to local heap */
+    H5G_bt_it_lbi_t udata;              /* Iteration information */
     H5O_stab_t stab;		        /* Symbol table message */
-    herr_t     ret_value = SUCCEED;     /* Return value */
+    herr_t ret_value = SUCCEED;         /* Return value */
 
     FUNC_ENTER_NOAPI(H5G_stab_lookup_by_idx, FAIL)
 
@@ -947,7 +997,7 @@ H5G_stab_lookup_by_idx(H5O_loc_t *grp_oloc, H5_iter_order_t order, hsize_t n,
         hsize_t nlinks = 0;           /* Number of links in group */
 
         /* Iterate over the symbol table nodes, to count the links */
-        if(H5B_iterate(grp_oloc->file, dxpl_id, H5B_SNODE, H5G_node_sumup, stab.btree_addr, &nlinks) < 0)
+        if(H5B_iterate(grp_oloc->file, dxpl_id, H5B_SNODE, stab.btree_addr, H5G_node_sumup, &nlinks) < 0)
             HGOTO_ERROR(H5E_SYM, H5E_CANTINIT, FAIL, "iteration operator failed")
 
         /* Map decreasing iteration order index to increasing iteration order index */
@@ -955,7 +1005,6 @@ H5G_stab_lookup_by_idx(H5O_loc_t *grp_oloc, H5_iter_order_t order, hsize_t n,
     } /* end if */
 
     /* Set iteration information */
-    udata.common.f = grp_oloc->file;
     udata.common.idx = n;
     udata.common.num_objs = 0;
     udata.common.op = H5G_stab_lookup_by_idx_cb;
@@ -964,7 +1013,7 @@ H5G_stab_lookup_by_idx(H5O_loc_t *grp_oloc, H5_iter_order_t order, hsize_t n,
     udata.found = FALSE;
 
     /* Iterate over the group members */
-    if(H5B_iterate(grp_oloc->file, dxpl_id, H5B_SNODE, H5G_node_by_idx, stab.btree_addr, &udata) < 0)
+    if(H5B_iterate(grp_oloc->file, dxpl_id, H5B_SNODE, stab.btree_addr, H5G_node_by_idx, &udata) < 0)
 	HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "iteration operator failed")
 
     /* If we didn't find the link, we almost certainly went out of bounds */
@@ -973,18 +1022,100 @@ H5G_stab_lookup_by_idx(H5O_loc_t *grp_oloc, H5_iter_order_t order, hsize_t n,
 
 done:
     /* Release resources */
-    if(heap && H5HL_unprotect(grp_oloc->file, dxpl_id, heap, stab.heap_addr) < 0)
+    if(heap && H5HL_unprotect(heap) < 0)
         HDONE_ERROR(H5E_SYM, H5E_PROTECT, FAIL, "unable to unprotect symbol table heap")
 
     FUNC_LEAVE_NOAPI(ret_value)
 } /* end H5G_stab_lookup_by_idx() */
 
+#ifndef H5_STRICT_FORMAT_CHECKS
+
+/*-------------------------------------------------------------------------
+ * Function:	H5G_stab_valid
+ *
+ * Purpose:	Verify that a group's symbol table message is valid.  If
+ *              provided, the addresses in alt_stab will be tried if the
+ *              addresses in the group's stab message are invalid, and
+ *              the stab message will be updated if necessary.
+ *
+ *              NOTE: This function is only called when strict format
+ *              checks are disabled.  This is so that, when strict
+ *              format checks are enabled,  errors in the symbol table
+ *              messages are not fixed by this function and are instead
+ *              reported by the library.
+ *
+ * Return:	Non-negative on success/Negative on failure
+ *
+ * Programmer:	Neil Fortner
+ *		nfortne2@hdfgroup.org
+ *		Mar 17, 2009
+ *
+ *-------------------------------------------------------------------------
+ */
+herr_t
+H5G_stab_valid(H5O_loc_t *grp_oloc, hid_t dxpl_id, H5O_stab_t *alt_stab)
+{
+    H5O_stab_t  stab;                   /* Current symbol table */
+    H5HL_t      *heap = NULL;           /* Pointer to local heap */
+    hbool_t     changed = FALSE;        /* Whether stab has been modified */
+    herr_t      ret_value = SUCCEED;    /* Return value */
+
+    FUNC_ENTER_NOAPI(H5G_stab_valid, FAIL)
+
+    /* Read the symbol table message */
+    if(NULL == H5O_msg_read(grp_oloc, H5O_STAB_ID, &stab, dxpl_id))
+        HGOTO_ERROR(H5E_SYM, H5E_BADMESG, FAIL, "unable to read symbol table message");
+
+    /* Check if the symbol table message's b-tree address is valid */
+    if(H5B_valid(grp_oloc->file, dxpl_id, H5B_SNODE, stab.btree_addr) < 0) {
+        /* Address is invalid, try the b-tree address in the alternate symbol
+         * table message */
+        if(!alt_stab || H5B_valid(grp_oloc->file, dxpl_id, H5B_SNODE, alt_stab->btree_addr) < 0)
+            HGOTO_ERROR(H5E_BTREE, H5E_NOTFOUND, FAIL, "unable to locate b-tree")
+        else {
+            /* The alternate symbol table's b-tree address is valid.  Adjust the
+             * symbol table message in the group. */
+            stab.btree_addr = alt_stab->btree_addr;
+            changed = TRUE;
+        } /* end else */
+    } /* end if */
+
+    /* Check if the symbol table message's heap address is valid */
+    if(NULL == (heap = H5HL_protect(grp_oloc->file, dxpl_id, stab.heap_addr, H5AC_READ))) {
+        /* Address is invalid, try the heap address in the alternate symbol
+         * table message */
+        if(!alt_stab || NULL == (heap = H5HL_protect(grp_oloc->file, dxpl_id, alt_stab->heap_addr, H5AC_READ)))
+            HGOTO_ERROR(H5E_HEAP, H5E_NOTFOUND, FAIL, "unable to locate heap")
+        else {
+            /* The alternate symbol table's heap address is valid.  Adjust the
+             * symbol table message in the group. */
+            stab.heap_addr = alt_stab->heap_addr;
+            changed = TRUE;
+        } /* end else */
+    } /* end if */
+
+    /* Update the symbol table message and clear errors if necessary */
+    if(changed) {
+        H5E_clear_stack(NULL);
+        if(H5O_msg_write(grp_oloc, H5O_STAB_ID, 0, H5O_UPDATE_TIME | H5O_UPDATE_FORCE, &stab, dxpl_id) < 0)
+            HGOTO_ERROR(H5E_SYM, H5E_CANTINIT, FAIL, "unable to correct symbol table message")
+    } /* end if */
+
+done:
+    /* Release resources */
+    if(heap && H5HL_unprotect(heap) < 0)
+        HDONE_ERROR(H5E_SYM, H5E_PROTECT, FAIL, "unable to unprotect symbol table heap")
+
+    FUNC_LEAVE_NOAPI(ret_value)
+} /* end H5G_stab_valid */
+#endif /* H5_STRICT_FORMAT_CHECKS */
+
 #ifndef H5_NO_DEPRECATED_SYMBOLS
 
 /*-------------------------------------------------------------------------
  * Function:	H5G_stab_get_type_by_idx_cb
  *
- * Purpose:     Callback for B-tree iteration 'by index' info query to 
+ * Purpose:     Callback for B-tree iteration 'by index' info query to
  *              retrieve the type of an object
  *
  * Return:	Success:        Non-negative
@@ -1013,13 +1144,17 @@ H5G_stab_get_type_by_idx_cb(const H5G_entry_t *ent, void *_udata)
             udata->type = H5G_LINK;
             break;
 
+        case H5G_CACHED_ERROR:
+        case H5G_NOTHING_CACHED:
+        case H5G_CACHED_STAB:
+        case H5G_NCACHED:
         default:
             {
                 H5O_loc_t tmp_oloc;             /* Temporary object location */
                 H5O_type_t obj_type;            /* Type of object at location */
 
                 /* Build temporary object location */
-                tmp_oloc.file = udata->common.f;
+                tmp_oloc.file = udata->f;
                 HDassert(H5F_addr_defined(ent->header));
                 tmp_oloc.addr = ent->header;
 
@@ -1068,15 +1203,15 @@ H5G_stab_get_type_by_idx(H5O_loc_t *oloc, hsize_t idx, hid_t dxpl_id)
 	HGOTO_ERROR(H5E_SYM, H5E_NOTFOUND, H5G_UNKNOWN, "unable to determine local heap address")
 
     /* Set iteration information */
-    udata.common.f = oloc->file;
     udata.common.idx = idx;
     udata.common.num_objs = 0;
     udata.common.op = H5G_stab_get_type_by_idx_cb;
+    udata.f = oloc->file;
     udata.dxpl_id = dxpl_id;
     udata.type = H5G_UNKNOWN;
 
     /* Iterate over the group members */
-    if(H5B_iterate(oloc->file, dxpl_id, H5B_SNODE, H5G_node_by_idx, stab.btree_addr, &udata) < 0)
+    if(H5B_iterate(oloc->file, dxpl_id, H5B_SNODE, stab.btree_addr, H5G_node_by_idx, &udata) < 0)
 	HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, H5G_UNKNOWN, "iteration operator failed")
 
     /* If we don't know the type now, we almost certainly went out of bounds */
