@@ -43,6 +43,7 @@
 /* Local Macros */
 /****************/
 
+#ifndef NDEBUG
 /* Limit on the size of the max. direct block size */
 /* (This is limited to 32-bits currently, because I think it's unlikely to
  *      need to be larger, the 32-bit limit for H5V_log2_of2(n), and
@@ -55,6 +56,8 @@
  *      need to be larger, and its encoded with a maxiumum of 16-bits  - QAK)
  */
 #define H5HF_WIDTH_LIMIT (64 * 1024)
+#endif /* NDEBUG */
+
 
 /******************/
 /* Local Typedefs */
@@ -76,7 +79,7 @@
 /*********************/
 
 /* Declare a free list to manage the H5HF_hdr_t struct */
-H5FL_DEFINE(H5HF_hdr_t);
+H5FL_DEFINE_STATIC(H5HF_hdr_t);
 
 
 /*****************************/
@@ -107,7 +110,7 @@ H5HF_hdr_t *
 H5HF_hdr_alloc(H5F_t *f)
 {
     H5HF_hdr_t *hdr = NULL;          /* Shared fractal heap header */
-    H5HF_hdr_t *ret_value = NULL;   /* Return value */
+    H5HF_hdr_t *ret_value;              /* Return value */
 
     FUNC_ENTER_NOAPI_NOINIT(H5HF_hdr_alloc)
 
@@ -118,7 +121,7 @@ H5HF_hdr_alloc(H5F_t *f)
 
     /* Allocate space for the shared information */
     if(NULL == (hdr = H5FL_CALLOC(H5HF_hdr_t)))
-	HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, NULL, "memory allocation failed for fractal heap shared header")
+	HGOTO_ERROR(H5E_HEAP, H5E_CANTALLOC, NULL, "allocation failed for fractal heap shared header")
 
     /* Set the internal parameters for the heap */
     hdr->f = f;
@@ -129,10 +132,6 @@ H5HF_hdr_alloc(H5F_t *f)
     ret_value = hdr;
 
 done:
-    if(!ret_value)
-        if(hdr)
-            (void)H5HF_cache_hdr_dest(f, hdr);
-
     FUNC_LEAVE_NOAPI(ret_value)
 } /* end H5HF_hdr_alloc() */
 
@@ -219,7 +218,7 @@ H5HF_hdr_finish_init_phase1(H5HF_hdr_t *hdr)
     HDassert(hdr);
 
     /* Compute/cache some values */
-    hdr->heap_off_size = H5HF_SIZEOF_OFFSET_BITS(hdr->man_dtable.cparam.max_index);
+    hdr->heap_off_size = (uint8_t)H5HF_SIZEOF_OFFSET_BITS(hdr->man_dtable.cparam.max_index);
     if(H5HF_dtable_init(&hdr->man_dtable) < 0)
 	HGOTO_ERROR(H5E_HEAP, H5E_CANTINIT, FAIL, "can't initialize doubling table info")
 
@@ -268,12 +267,6 @@ H5HF_hdr_finish_init_phase2(H5HF_hdr_t *hdr)
         else
             if(H5HF_hdr_compute_free_space(hdr, u) < 0)
                 HGOTO_ERROR(H5E_HEAP, H5E_CANTINIT, FAIL, "can't initialize direct block free space for indirect block")
-#ifdef QAK
-HDfprintf(stderr, "%s: row_block_size[%Zu] = %Hu\n", FUNC, u, hdr->man_dtable.row_block_size[u]);
-HDfprintf(stderr, "%s: row_block_off[%Zu] = %Hu\n", FUNC, u, hdr->man_dtable.row_block_off[u]);
-HDfprintf(stderr, "%s: row_tot_dblock_free[%Zu] = %Hu\n", FUNC, u, hdr->man_dtable.row_tot_dblock_free[u]);
-HDfprintf(stderr, "%s: row_max_dblock_free[%Zu] = %Zu\n", FUNC, u, hdr->man_dtable.row_max_dblock_free[u]);
-#endif /* QAK */
     } /* end for */
 
     /* Initialize the block iterator for searching for free space */
@@ -406,9 +399,6 @@ H5HF_hdr_create(H5F_t *f, hid_t dxpl_id, const H5HF_create_t *cparam)
     /* Set "huge" object tracker v2 B-tree address to indicate that there aren't any yet */
     hdr->huge_bt2_addr = HADDR_UNDEF;
 
-    /* Note that the shared info is dirty (it's not written to the file yet) */
-    hdr->dirty = TRUE;
-
     /* First phase of header final initialization */
     /* (doesn't need ID length set up) */
     if(H5HF_hdr_finish_init_phase1(hdr) < 0)
@@ -420,6 +410,17 @@ H5HF_hdr_create(H5F_t *f, hid_t dxpl_id, const H5HF_create_t *cparam)
      *  length is already set in that case (its stored in the header on disk))
      */
     if(cparam->pline.nused > 0) {
+        /* Check if the filters in the DCPL can be applied to this dataset */
+        if(H5Z_can_apply_direct(&(cparam->pline)) < 0)
+            HGOTO_ERROR(H5E_HEAP, H5E_CANTINIT, HADDR_UNDEF, "I/O filters can't operate on this heap")
+
+        /* Mark the filters as checked */
+        hdr->checked_filters = TRUE;
+
+        /* Make the "set local" filter callbacks for this dataset */
+        if(H5Z_set_local_direct(&(cparam->pline)) < 0)
+            HGOTO_ERROR(H5E_HEAP, H5E_CANTINIT, HADDR_UNDEF, "unable to set local filter parameters")
+
         /* Copy the I/O filter pipeline from the creation parameters to the header */
         if(NULL == H5O_msg_copy(H5O_PLINE_ID, &(cparam->pline), &(hdr->pline)))
             HGOTO_ERROR(H5E_HEAP, H5E_CANTCOPY, HADDR_UNDEF, "can't copy I/O filter pipeline")
@@ -427,15 +428,12 @@ H5HF_hdr_create(H5F_t *f, hid_t dxpl_id, const H5HF_create_t *cparam)
         /* Pay attention to the latest version flag for the file */
         if(H5F_USE_LATEST_FORMAT(hdr->f))
             /* Set the latest version for the I/O pipeline message */
-            if(H5Z_set_latest_version(&(hdr->pline)) < 0)
+            if(H5O_pline_set_latest_version(&(hdr->pline)) < 0)
                 HGOTO_ERROR(H5E_HEAP, H5E_CANTSET, HADDR_UNDEF, "can't set latest version of I/O filter pipeline")
 
         /* Compute the I/O filters' encoded size */
         if(0 == (hdr->filter_len = H5O_msg_raw_size(hdr->f, H5O_PLINE_ID, FALSE, &(hdr->pline))))
             HGOTO_ERROR(H5E_HEAP, H5E_CANTGETSIZE, HADDR_UNDEF, "can't get I/O filter pipeline size")
-#ifdef QAK
-HDfprintf(stderr, "%s: hdr->filter_len = %u\n", FUNC, hdr->filter_len);
-#endif /* QAK */
 
         /* Compute size of header on disk */
         hdr->heap_size = H5HF_HEADER_SIZE(hdr)  /* Base header size */
@@ -443,10 +441,14 @@ HDfprintf(stderr, "%s: hdr->filter_len = %u\n", FUNC, hdr->filter_len);
             + 4                                 /* Size of filter mask for filtered root direct block */
             + hdr->filter_len;                  /* Size of encoded I/O filter info */
     } /* end if */
-    else
+    else {
         /* Set size of header on disk */
         hdr->heap_size = H5HF_HEADER_SIZE(hdr);
 
+        /* Mark filters as checked, for performance reasons */
+        hdr->checked_filters = TRUE;
+    } /* end else */
+
     /* Set the length of IDs in the heap */
     /* (This code is not in the "finish init phase" routines because those
      *  routines are also called from the cache 'load' callback, and the ID
@@ -454,7 +456,7 @@ HDfprintf(stderr, "%s: hdr->filter_len = %u\n", FUNC, hdr->filter_len);
      */
     switch(cparam->id_len) {
         case 0: /* Set the length of heap IDs to just enough to hold the offset & length of 'normal' objects in the heap */
-            hdr->id_len = 1 + hdr->heap_off_size + hdr->heap_len_size;
+            hdr->id_len = (unsigned)1 + hdr->heap_off_size + hdr->heap_len_size;
             break;
 
         case 1: /* Set the length of heap IDs to just enough to hold the information needed to directly access 'huge' objects in the heap */
@@ -481,9 +483,6 @@ HDfprintf(stderr, "%s: hdr->filter_len = %u\n", FUNC, hdr->filter_len);
             hdr->id_len = cparam->id_len;
             break;
     } /* end switch */
-#ifdef QAK
-HDfprintf(stderr, "%s: hdr->id_len = %Zu\n", FUNC, hdr->id_len);
-#endif /* QAK */
 
     /* Second phase of header final initialization */
     /* (needs ID and filter lengths set up) */
@@ -501,20 +500,68 @@ HDfprintf(stderr, "%s: hdr->id_len = %Zu\n", FUNC, hdr->id_len);
 	HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, HADDR_UNDEF, "file allocation failed for fractal heap header")
 
     /* Cache the new fractal heap header */
-    if(H5AC_set(f, dxpl_id, H5AC_FHEAP_HDR, hdr->heap_addr, hdr, H5AC__NO_FLAGS_SET) < 0)
-	HGOTO_ERROR(H5E_HEAP, H5E_CANTINIT, HADDR_UNDEF, "can't add fractal heap header to cache")
+    if(H5AC_insert_entry(f, dxpl_id, H5AC_FHEAP_HDR, hdr->heap_addr, hdr, H5AC__NO_FLAGS_SET) < 0)
+	HGOTO_ERROR(H5E_HEAP, H5E_CANTINSERT, HADDR_UNDEF, "can't add fractal heap header to cache")
 
     /* Set address of heap header to return */
     ret_value = hdr->heap_addr;
 
 done:
-    if(!H5F_addr_defined(ret_value))
-        if(hdr)
-            (void)H5HF_cache_hdr_dest(NULL, hdr);
+    if(!H5F_addr_defined(ret_value) && hdr)
+        if(H5HF_hdr_free(hdr) < 0)
+            HDONE_ERROR(H5E_HEAP, H5E_CANTRELEASE, HADDR_UNDEF, "unable to release fractal heap header")
 
     FUNC_LEAVE_NOAPI(ret_value)
 } /* end H5HF_hdr_create() */
 
+
+/*-------------------------------------------------------------------------
+ * Function:	H5HF_hdr_protect
+ *
+ * Purpose:	Convenience wrapper around H5AC_protect on an indirect block
+ *
+ * Return:	Pointer to indirect block on success, NULL on failure
+ *
+ * Programmer:	Quincey Koziol
+ *		koziol@hdfgroup.org
+ *		May  5 2010
+ *
+ *-------------------------------------------------------------------------
+ */
+H5HF_hdr_t *
+H5HF_hdr_protect(H5F_t *f, hid_t dxpl_id, haddr_t addr, H5AC_protect_t rw)
+{
+    H5HF_hdr_cache_ud_t cache_udata;    /* User-data for callback */
+    H5HF_hdr_t *hdr;                    /* Fractal heap header */
+    H5HF_hdr_t *ret_value;              /* Return value */
+
+    FUNC_ENTER_NOAPI_NOINIT(H5HF_hdr_protect)
+
+    /* Check arguments */
+    HDassert(f);
+    HDassert(H5F_addr_defined(addr));
+
+    /* Set up userdata for protect call */
+    cache_udata.f = f;
+    cache_udata.dxpl_id = dxpl_id;
+
+    /* Lock the heap header into memory */
+    if(NULL == (hdr = (H5HF_hdr_t *)H5AC_protect(f, dxpl_id, H5AC_FHEAP_HDR, addr, &cache_udata, rw)))
+        HGOTO_ERROR(H5E_HEAP, H5E_CANTPROTECT, NULL, "unable to protect fractal heap header")
+
+    /* Set the header's address */
+    hdr->heap_addr = addr;
+
+    /* Update header's file pointer */
+    hdr->f = f;
+
+    /* Set the return value */
+    ret_value = hdr;
+
+done:
+    FUNC_LEAVE_NOAPI(ret_value)
+} /* end H5HF_hdr_protect() */
+
 
 /*-------------------------------------------------------------------------
  * Function:	H5HF_hdr_incr
@@ -541,7 +588,7 @@ H5HF_hdr_incr(H5HF_hdr_t *hdr)
 
     /* Mark header as un-evictable when a block is depending on it */
     if(hdr->rc == 0)
-        if(H5AC_pin_protected_entry(hdr->f, hdr) < 0)
+        if(H5AC_pin_protected_entry(hdr) < 0)
             HGOTO_ERROR(H5E_HEAP, H5E_CANTPIN, FAIL, "unable to pin fractal heap header")
 
     /* Increment reference count on shared header */
@@ -582,7 +629,7 @@ H5HF_hdr_decr(H5HF_hdr_t *hdr)
     /* Mark header as evictable again when no child blocks depend on it */
     if(hdr->rc == 0) {
         HDassert(hdr->file_rc == 0);
-        if(H5AC_unpin_entry(hdr->f, hdr) < 0)
+        if(H5AC_unpin_entry(hdr) < 0)
             HGOTO_ERROR(H5E_HEAP, H5E_CANTUNPIN, FAIL, "unable to unpin fractal heap header")
     } /* end if */
 
@@ -667,19 +714,19 @@ H5HF_hdr_dirty(H5HF_hdr_t *hdr)
     herr_t ret_value = SUCCEED;         /* Return value */
 
     FUNC_ENTER_NOAPI_NOINIT(H5HF_hdr_dirty)
-#ifdef QAK
-HDfprintf(stderr, "%s: Marking heap header as dirty\n", FUNC);
-#endif /* QAK */
 
     /* Sanity check */
     HDassert(hdr);
 
-    /* Mark header as dirty in cache */
-    if(H5AC_mark_pinned_or_protected_entry_dirty(hdr->f, hdr) < 0)
-        HGOTO_ERROR(H5E_HEAP, H5E_CANTMARKDIRTY, FAIL, "unable to mark fractal heap header as dirty")
+    /* Resize pinned header in cache if I/O filter is present. */
+    if(hdr->filter_len > 0) {
+        if(H5AC_resize_entry(hdr, (size_t)hdr->heap_size) < 0)
+            HGOTO_ERROR(H5E_HEAP, H5E_CANTRESIZE, FAIL, "unable to resize fractal heap header")
+    } /* end if */
 
-    /* Set the dirty flags for the heap header */
-    hdr->dirty = TRUE;
+    /* Mark header as dirty in cache */
+    if(H5AC_mark_entry_dirty(hdr) < 0)
+        HGOTO_ERROR(H5E_HEAP, H5E_CANTMARKDIRTY, FAIL, "unable to mark fractal heap header as dirty")
 
 done:
     FUNC_LEAVE_NOAPI(ret_value)
@@ -705,9 +752,6 @@ H5HF_hdr_adj_free(H5HF_hdr_t *hdr, ssize_t amt)
     herr_t ret_value = SUCCEED;         /* Return value */
 
     FUNC_ENTER_NOAPI_NOINIT(H5HF_hdr_adj_free)
-#ifdef QAK
-HDfprintf(stderr, "%s: amt = %Zd\n", FUNC, amt);
-#endif /* QAK */
 
     /*
      * Check arguments.
@@ -717,9 +761,6 @@ HDfprintf(stderr, "%s: amt = %Zd\n", FUNC, amt);
     /* Update heap header */
     HDassert(amt > 0 || hdr->total_man_free >= (hsize_t)-amt);
     hdr->total_man_free += amt;
-#ifdef QAK
-HDfprintf(stderr, "%s: hdr->total_man_free = %Hu\n", FUNC, hdr->total_man_free);
-#endif /* QAK */
 
     /* Mark heap header as modified */
     if(H5HF_hdr_dirty(hdr) < 0)
@@ -754,12 +795,6 @@ H5HF_hdr_adjust_heap(H5HF_hdr_t *hdr, hsize_t new_size, hssize_t extra_free)
      * Check arguments.
      */
     HDassert(hdr);
-#ifdef QAK
-HDfprintf(stderr, "%s; new_size = %Hu, extra_free = %Hd\n", FUNC, new_size, extra_free);
-HDfprintf(stderr, "%s; hdr->total_size = %Hu\n", FUNC, hdr->total_size);
-HDfprintf(stderr, "%s; hdr->man_size = %Hu\n", FUNC, hdr->man_size);
-HDfprintf(stderr, "%s; hdr->total_man_free = %Hu\n", FUNC, hdr->total_man_free);
-#endif /* QAK */
 
     /* Set the total managed space in heap */
     hdr->man_size = new_size;
@@ -802,9 +837,6 @@ H5HF_hdr_inc_alloc(H5HF_hdr_t *hdr, size_t alloc_size)
 
     /* Update the "allocated" size within the heap */
     hdr->man_alloc_size += alloc_size;
-#ifdef QAK
-HDfprintf(stderr, "%s: hdr->man_alloc_size = %Hu\n", "H5HF_hdr_inc_alloc", hdr->man_alloc_size);
-#endif /* QAK */
 
     FUNC_LEAVE_NOAPI(SUCCEED)
 } /* end H5HF_hdr_inc_alloc() */
@@ -878,9 +910,6 @@ H5HF_hdr_reset_iter(H5HF_hdr_t *hdr, hsize_t curr_off)
         HGOTO_ERROR(H5E_HEAP, H5E_CANTRELEASE, FAIL, "can't reset block iterator")
 
     /* Set the offset of the iterator in the heap */
-#ifdef QAK
-HDfprintf(stderr, "%s: hdr->man_iter_off = %Hu, curr_off = %Hu\n", FUNC, hdr->man_iter_off, curr_off);
-#endif /* QAK */
     hdr->man_iter_off = curr_off;
 
 done:
@@ -910,9 +939,6 @@ H5HF_hdr_skip_blocks(H5HF_hdr_t *hdr, hid_t dxpl_id, H5HF_indirect_t *iblock,
     herr_t ret_value = SUCCEED;         /* Return value */
 
     FUNC_ENTER_NOAPI_NOINIT(H5HF_hdr_skip_blocks)
-#ifdef QAK
-HDfprintf(stderr, "%s: start_entry = %u, nentries = %u\n", FUNC, start_entry, nentries);
-#endif /* QAK */
 
     /*
      * Check arguments.
@@ -925,17 +951,11 @@ HDfprintf(stderr, "%s: start_entry = %u, nentries = %u\n", FUNC, start_entry, ne
     row = start_entry / hdr->man_dtable.cparam.width;
     col = start_entry % hdr->man_dtable.cparam.width;
     sect_size = H5HF_dtable_span_size(&hdr->man_dtable, row, col, nentries);
-#ifdef QAK
-HDfprintf(stderr, "%s: Check 1.0 - hdr->man_iter_off = %Hu, sect_size = %Hu\n", FUNC, hdr->man_iter_off, sect_size);
-#endif /* QAK */
     HDassert(sect_size > 0);
 
     /* Advance the new block iterator */
     if(H5HF_hdr_inc_iter(hdr, sect_size, nentries) < 0)
         HGOTO_ERROR(H5E_HEAP, H5E_CANTRELEASE, FAIL, "can't increase allocated heap size")
-#ifdef QAK
-HDfprintf(stderr, "%s: Check 2.0 - hdr->man_iter_off = %Hu\n", FUNC, hdr->man_iter_off);
-#endif /* QAK */
 
     /* Add 'indirect' section for blocks skipped in this row */
     if(H5HF_sect_indirect_add(hdr, dxpl_id, iblock, start_entry, nentries) < 0)
@@ -968,9 +988,6 @@ H5HF_hdr_update_iter(H5HF_hdr_t *hdr, hid_t dxpl_id, size_t min_dblock_size)
     herr_t ret_value = SUCCEED;         /* Return value */
 
     FUNC_ENTER_NOAPI_NOINIT(H5HF_hdr_update_iter)
-#ifdef QAK
-HDfprintf(stderr, "%s: min_dblock_size = %Zu\n", FUNC, min_dblock_size);
-#endif /* QAK */
 
     /*
      * Check arguments.
@@ -980,9 +997,6 @@ HDfprintf(stderr, "%s: min_dblock_size = %Zu\n", FUNC, min_dblock_size);
 
     /* Check for creating first indirect block */
     if(hdr->man_dtable.curr_root_rows == 0) {
-#ifdef QAK
-HDfprintf(stderr, "%s: Creating root direct block\n", FUNC);
-#endif /* QAK */
         if(H5HF_man_iblock_root_create(hdr, dxpl_id, min_dblock_size) < 0)
             HGOTO_ERROR(H5E_HEAP, H5E_CANTEXTEND, FAIL, "unable to create root indirect block")
     } /* end if */
@@ -993,21 +1007,11 @@ HDfprintf(stderr, "%s: Creating root direct block\n", FUNC);
         unsigned next_entry;            /* Iterator's next block entry */
         unsigned min_dblock_row;        /* Minimum row for direct block size request */
 
-#ifdef QAK
-HDfprintf(stderr, "%s: searching root indirect block\n", FUNC);
-#endif /* QAK */
         /* Compute min. row for direct block requested */
         min_dblock_row = H5HF_dtable_size_to_row(&hdr->man_dtable, min_dblock_size);
-#ifdef QAK
-HDfprintf(stderr, "%s: min_dblock_size = %Zu, min_dblock_row = %u\n", FUNC, min_dblock_size, min_dblock_row);
-HDfprintf(stderr, "%s: hdr->man_iter_off = %Hu\n", FUNC, hdr->man_iter_off);
-#endif /* QAK */
 
         /* Initialize block iterator, if necessary */
         if(!H5HF_man_iter_ready(&hdr->next_block)) {
-#ifdef QAK
-HDfprintf(stderr, "%s: hdr->man_iter_off = %Hu\n", FUNC, hdr->man_iter_off);
-#endif /* QAK */
             /* Start iterator with previous offset of iterator */
             if(H5HF_man_iter_start_offset(hdr, dxpl_id, &hdr->next_block, hdr->man_iter_off) < 0)
                 HGOTO_ERROR(H5E_HEAP, H5E_CANTINIT, FAIL, "unable to set block iterator location")
@@ -1017,14 +1021,6 @@ HDfprintf(stderr, "%s: hdr->man_iter_off = %Hu\n", FUNC, hdr->man_iter_off);
         if(H5HF_man_iter_curr(&hdr->next_block, &next_row, NULL, &next_entry, &iblock) < 0)
             HGOTO_ERROR(H5E_HEAP, H5E_CANTGET, FAIL, "unable to retrieve current block iterator location")
 
-#ifdef QAK
-HDfprintf(stderr, "%s: Check 1.0\n", FUNC);
-HDfprintf(stderr, "%s: iblock = %p\n", FUNC, iblock);
-HDfprintf(stderr, "%s: iblock->block_off = %Hu\n", FUNC, iblock->block_off);
-HDfprintf(stderr, "%s: iblock->nrows = %u\n", FUNC, iblock->nrows);
-HDfprintf(stderr, "%s: next_row = %u\n", FUNC, next_row);
-HDfprintf(stderr, "%s: next_entry = %u\n", FUNC, next_entry);
-#endif /* QAK */
         /* Check for skipping over blocks in the current block */
         if(min_dblock_row > next_row && next_row < iblock->nrows) {
             unsigned min_entry;         /* Min entry for direct block requested */
@@ -1036,9 +1032,6 @@ HDfprintf(stderr, "%s: next_entry = %u\n", FUNC, next_entry);
                 skip_entries = (iblock->nrows * hdr->man_dtable.cparam.width) - next_entry;
             else
                 skip_entries = min_entry - next_entry;
-#ifdef QAK
-HDfprintf(stderr, "%s: min_entry = %u, skip_entries = %u\n", FUNC, min_entry, skip_entries);
-#endif /* QAK */
 
             /* Add skipped direct blocks to heap's free space */
             if(H5HF_hdr_skip_blocks(hdr, dxpl_id, iblock, next_entry, skip_entries) < 0)
@@ -1053,33 +1046,15 @@ HDfprintf(stderr, "%s: min_entry = %u, skip_entries = %u\n", FUNC, min_entry, sk
             /* Reset conditions for leaving loop */
             walked_up = walked_down = FALSE;
 
-#ifdef QAK
-HDfprintf(stderr, "%s: Check 2.0\n", FUNC);
-HDfprintf(stderr, "%s: iblock = %p\n", FUNC, iblock);
-HDfprintf(stderr, "%s: iblock->block_off = %Hu\n", FUNC, iblock->block_off);
-HDfprintf(stderr, "%s: iblock->nrows = %u\n", FUNC, iblock->nrows);
-HDfprintf(stderr, "%s: next_row = %u\n", FUNC, next_row);
-HDfprintf(stderr, "%s: next_entry = %u\n", FUNC, next_entry);
-#endif /* QAK */
-
             /* Check for walking off end of indirect block */
             /* (walk up iterator) */
             while(next_row >= iblock->nrows) {
-#ifdef QAK
-HDfprintf(stderr, "%s: Off the end of a block, next_row = %u, iblock->nrows = %u\n", FUNC, next_row, iblock->nrows);
-#endif /* QAK */
                 /* Check for needing to expand root indirect block */
                 if(iblock->parent == NULL) {
-#ifdef QAK
-HDfprintf(stderr, "%s: Doubling root block\n", FUNC);
-#endif /* QAK */
                     if(H5HF_man_iblock_root_double(hdr, dxpl_id, min_dblock_size) < 0)
                         HGOTO_ERROR(H5E_HEAP, H5E_CANTEXTEND, FAIL, "unable to double root indirect block")
                 } /* end if */
                 else {
-#ifdef QAK
-HDfprintf(stderr, "%s: Walking up a level\n", FUNC);
-#endif /* QAK */
                     /* Move iterator up one level */
                     if(H5HF_man_iter_up(&hdr->next_block) < 0)
                         HGOTO_ERROR(H5E_HEAP, H5E_CANTNEXT, FAIL, "unable to advance current block iterator location")
@@ -1096,36 +1071,16 @@ HDfprintf(stderr, "%s: Walking up a level\n", FUNC);
                 /* Indicate that we walked up */
                 walked_up = TRUE;
             } /* end while */
-#ifdef QAK
-HDfprintf(stderr, "%s: Check 3.0\n", FUNC);
-HDfprintf(stderr, "%s: iblock = %p\n", FUNC, iblock);
-HDfprintf(stderr, "%s: iblock->nrows = %u\n", FUNC, iblock->nrows);
-HDfprintf(stderr, "%s: next_row = %u\n", FUNC, next_row);
-HDfprintf(stderr, "%s: next_entry = %u\n", FUNC, next_entry);
-#endif /* QAK */
 
             /* Check for walking into child indirect block */
             /* (walk down iterator) */
             if(next_row >= hdr->man_dtable.max_direct_rows) {
                 unsigned child_nrows;   /* Number of rows in new indirect block */
 
-#ifdef QAK
-HDfprintf(stderr, "%s: Walking down into child indirect block\n", FUNC);
-#endif /* QAK */
-#ifdef QAK
-HDfprintf(stderr, "%s: Check 3.1\n", FUNC);
-HDfprintf(stderr, "%s: iblock = %p\n", FUNC, iblock);
-HDfprintf(stderr, "%s: iblock->nrows = %u\n", FUNC, iblock->nrows);
-HDfprintf(stderr, "%s: next_row = %u\n", FUNC, next_row);
-HDfprintf(stderr, "%s: next_entry = %u\n", FUNC, next_entry);
-#endif /* QAK */
                 HDassert(!H5F_addr_defined(iblock->ents[next_entry].addr));
 
                 /* Compute # of rows in next child indirect block to use */
                 child_nrows = H5HF_dtable_size_to_rows(&hdr->man_dtable, hdr->man_dtable.row_block_size[next_row]);
-#ifdef QAK
-HDfprintf(stderr, "%s: child_nrows = %u\n", FUNC, child_nrows);
-#endif /* QAK */
 
                 /* Check for skipping over indirect blocks */
                 /* (that don't have direct blocks large enough to hold direct block size requested) */
@@ -1133,19 +1088,12 @@ HDfprintf(stderr, "%s: child_nrows = %u\n", FUNC, child_nrows);
                     unsigned child_rows_needed;     /* Number of rows needed to hold direct block */
                     unsigned child_entry;           /* Entry of child indirect block */
 
-#ifdef QAK
-HDfprintf(stderr, "%s: Skipping indirect block row that is too small\n", FUNC);
-#endif /* QAK */
                     /* Compute # of rows needed in child indirect block */
                     child_rows_needed = (H5V_log2_of2((uint32_t)min_dblock_size) - H5V_log2_of2((uint32_t)hdr->man_dtable.cparam.start_block_size)) + 2;
                     HDassert(child_rows_needed > child_nrows);
                     child_entry = (next_row + (child_rows_needed - child_nrows)) * hdr->man_dtable.cparam.width;
                     if(child_entry > (iblock->nrows * hdr->man_dtable.cparam.width))
                         child_entry = iblock->nrows * hdr->man_dtable.cparam.width;
-#ifdef QAK
-HDfprintf(stderr, "%s: child_rows_needed = %u\n", FUNC, child_rows_needed);
-HDfprintf(stderr, "%s: child_entry = %u\n", FUNC, child_entry);
-#endif /* QAK */
 
                     /* Add skipped indirect blocks to heap's free space */
                     if(H5HF_hdr_skip_blocks(hdr, dxpl_id, iblock, next_entry, (child_entry - next_entry)) < 0)
@@ -1156,9 +1104,6 @@ HDfprintf(stderr, "%s: child_entry = %u\n", FUNC, child_entry);
                     hbool_t did_protect;            /* Whether we protected the indirect block or not */
                     haddr_t new_iblock_addr;        /* New indirect block's address */
 
-#ifdef QAK
-HDfprintf(stderr, "%s: Allocating new child indirect block\n", FUNC);
-#endif /* QAK */
                     /* Allocate new indirect block */
                     if(H5HF_man_iblock_create(hdr, dxpl_id, iblock, next_entry, child_nrows, child_nrows, &new_iblock_addr) < 0)
                         HGOTO_ERROR(H5E_HEAP, H5E_CANTALLOC, FAIL, "can't allocate fractal heap indirect block")
@@ -1177,9 +1122,6 @@ HDfprintf(stderr, "%s: Allocating new child indirect block\n", FUNC);
 
                         /* Compute entry for direct block size requested */
                         new_entry = hdr->man_dtable.cparam.width * min_dblock_row;
-#ifdef QAK
-HDfprintf(stderr, "%s: Skipping rows in new child indirect block - new_entry = %u\n", FUNC, new_entry);
-#endif /* QAK */
 
                         /* Add skipped blocks to heap's free space */
                         if(H5HF_hdr_skip_blocks(hdr, dxpl_id, new_iblock, 0, new_entry) < 0)
@@ -1231,9 +1173,6 @@ H5HF_hdr_inc_iter(H5HF_hdr_t *hdr, hsize_t adv_size, unsigned nentries)
      */
     HDassert(hdr);
     HDassert(nentries);
-#ifdef QAK
-HDfprintf(stderr, "%s: hdr->man_iter_off = %Hu, adv_size = %Hu\n", FUNC, hdr->man_iter_off, adv_size);
-#endif /* QAK */
 
     /* Advance the iterator for the current location within the indirect block */
     if(hdr->next_block.curr)
@@ -1289,11 +1228,6 @@ H5HF_hdr_reverse_iter(H5HF_hdr_t *hdr, hid_t dxpl_id, haddr_t dblock_addr)
     /* Get information about current iterator location */
     if(H5HF_man_iter_curr(&hdr->next_block, NULL, NULL, &curr_entry, &iblock) < 0)
         HGOTO_ERROR(H5E_HEAP, H5E_CANTGET, FAIL, "unable to retrieve current block iterator information")
-#ifdef QAK
-HDfprintf(stderr, "%s: iblock->nchildren = %u\n", FUNC, iblock->nchildren);
-HDfprintf(stderr, "%s: iblock->parent = %p\n", FUNC, iblock->parent);
-HDfprintf(stderr, "%s: curr_entry = %u\n", FUNC, curr_entry);
-#endif /* QAK */
 
     /* Move current iterator position backwards once */
     curr_entry--;
@@ -1309,23 +1243,14 @@ HDfprintf(stderr, "%s: curr_entry = %u\n", FUNC, curr_entry);
         /* Walk backwards through entries, until we find one that has a child */
         /* (Skip direct block that will be deleted, if we find it) */
         tmp_entry = curr_entry;
-#ifdef QAK
-HDfprintf(stderr, "%s: tmp_entry = %d\n", FUNC, tmp_entry);
-#endif /* QAK */
         while(tmp_entry >= 0 &&
                 (H5F_addr_eq(iblock->ents[tmp_entry].addr, dblock_addr) ||
                     !H5F_addr_defined(iblock->ents[tmp_entry].addr)))
             tmp_entry--;
-#ifdef QAK
-HDfprintf(stderr, "%s: check 2.0 - tmp_entry = %d\n", FUNC, tmp_entry);
-#endif /* QAK */
         /* Check for no earlier blocks in this indirect block */
         if(tmp_entry < 0) {
             /* Check for parent of current indirect block */
             if(iblock->parent) {
-#ifdef QAK
-HDfprintf(stderr, "%s: Walking up a block\n", FUNC);
-#endif /* QAK */
                 /* Move iterator to parent of current block */
                 if(H5HF_man_iter_up(&hdr->next_block) < 0)
                     HGOTO_ERROR(H5E_HEAP, H5E_CANTNEXT, FAIL, "unable to move current block iterator location up")
@@ -1333,11 +1258,6 @@ HDfprintf(stderr, "%s: Walking up a block\n", FUNC);
                 /* Get information about current iterator location */
                 if(H5HF_man_iter_curr(&hdr->next_block, NULL, NULL, &curr_entry, &iblock) < 0)
                     HGOTO_ERROR(H5E_HEAP, H5E_CANTGET, FAIL, "unable to retrieve current block iterator information")
-#ifdef QAK
-HDfprintf(stderr, "%s: iblock->nchildren = %u\n", FUNC, iblock->nchildren);
-HDfprintf(stderr, "%s: iblock->parent = %p\n", FUNC, iblock->parent);
-HDfprintf(stderr, "%s: curr_entry = %u\n", FUNC, curr_entry);
-#endif /* QAK */
 
                 /* Move current iterator position backwards once */
                 curr_entry--;
@@ -1346,9 +1266,6 @@ HDfprintf(stderr, "%s: curr_entry = %u\n", FUNC, curr_entry);
                 walked_up = TRUE;
             } /* end if */
             else {
-#ifdef QAK
-HDfprintf(stderr, "%s: Heap empty\n", FUNC);
-#endif /* QAK */
                 /* Reset iterator offset */
                 hdr->man_iter_off = 0;
 
@@ -1364,14 +1281,7 @@ HDfprintf(stderr, "%s: Heap empty\n", FUNC);
 
             /* Check if entry is for a direct block */
             row = curr_entry / hdr->man_dtable.cparam.width;
-#ifdef QAK
-HDfprintf(stderr, "%s: curr_entry = %u\n", FUNC, curr_entry);
-HDfprintf(stderr, "%s: row = %u\n", FUNC, row);
-#endif /* QAK */
             if(row < hdr->man_dtable.max_direct_rows) {
-#ifdef QAK
-HDfprintf(stderr, "%s: Found direct block\n", FUNC);
-#endif /* QAK */
                 /* Increment entry to empty location */
                 curr_entry++;
 
@@ -1383,19 +1293,12 @@ HDfprintf(stderr, "%s: Found direct block\n", FUNC);
                 hdr->man_iter_off = iblock->block_off;
                 hdr->man_iter_off += hdr->man_dtable.row_block_off[curr_entry / hdr->man_dtable.cparam.width];
                 hdr->man_iter_off += hdr->man_dtable.row_block_size[curr_entry / hdr->man_dtable.cparam.width] * (curr_entry % hdr->man_dtable.cparam.width);
-#ifdef QAK
-HDfprintf(stderr, "%s: hdr->man_iter_off = %Hu\n", FUNC, hdr->man_iter_off);
-HDfprintf(stderr, "%s: curr_entry = %u\n", FUNC, curr_entry);
-#endif /* QAK */
             } /* end if */
             else {
                 H5HF_indirect_t *child_iblock;  /* Pointer to child indirect block */
                 hbool_t did_protect;            /* Whether we protected the indirect block or not */
                 unsigned child_nrows;           /* # of rows in child block */
 
-#ifdef QAK
-HDfprintf(stderr, "%s: Walking down into child block\n", FUNC);
-#endif /* QAK */
                 /* Compute # of rows in next child indirect block to use */
                 child_nrows = H5HF_dtable_size_to_rows(&hdr->man_dtable, hdr->man_dtable.row_block_size[row]);
 
@@ -1414,11 +1317,6 @@ HDfprintf(stderr, "%s: Walking down into child block\n", FUNC);
                 /* Update iterator location */
                 iblock = child_iblock;
                 curr_entry = (child_iblock->nrows * hdr->man_dtable.cparam.width) - 1;
-#ifdef QAK
-HDfprintf(stderr, "%s: iblock->nchildren = %u\n", FUNC, iblock->nchildren);
-HDfprintf(stderr, "%s: iblock->parent = %p\n", FUNC, iblock->parent);
-HDfprintf(stderr, "%s: curr_entry = %u\n", FUNC, curr_entry);
-#endif /* QAK */
 
                 /* Unprotect child indirect block */
                 if(H5HF_man_iblock_unprotect(child_iblock, dxpl_id, H5AC__NO_FLAGS_SET, did_protect) < 0)
@@ -1454,18 +1352,12 @@ H5HF_hdr_empty(H5HF_hdr_t *hdr)
     herr_t ret_value = SUCCEED;         /* Return value */
 
     FUNC_ENTER_NOAPI_NOINIT(H5HF_hdr_empty)
-#ifdef QAK
-HDfprintf(stderr, "%s: Resetting heap header to empty\n", FUNC);
-#endif /* QAK */
 
     /* Sanity check */
     HDassert(hdr);
 
     /* Reset block iterator, if necessary */
     if(H5HF_man_iter_ready(&hdr->next_block)) {
-#ifdef QAK
-HDfprintf(stderr, "%s: 'next block' iterator is ready\n", FUNC);
-#endif /* QAK */
         if(H5HF_man_iter_reset(&hdr->next_block) < 0)
             HGOTO_ERROR(H5E_HEAP, H5E_CANTRELEASE, FAIL, "can't reset block iterator")
     } /* end if */
@@ -1488,6 +1380,48 @@ done:
     FUNC_LEAVE_NOAPI(ret_value)
 } /* end H5HF_hdr_empty() */
 
+
+/*-------------------------------------------------------------------------
+ * Function:	H5HF_hdr_free
+ *
+ * Purpose:	Free shared fractal heap header
+ *
+ * Return:	Non-negative on success/Negative on failure
+ *
+ * Programmer:	Quincey Koziol
+ *		koziol@hdfgroup.org
+ *		Oct 27 2009
+ *
+ *-------------------------------------------------------------------------
+ */
+herr_t
+H5HF_hdr_free(H5HF_hdr_t *hdr)
+{
+    herr_t      ret_value = SUCCEED;    /* Return value */
+
+    FUNC_ENTER_NOAPI_NOINIT(H5HF_hdr_free)
+
+    /*
+     * Check arguments.
+     */
+    HDassert(hdr);
+
+    /* Free the block size lookup table for the doubling table */
+    if(H5HF_dtable_dest(&hdr->man_dtable) < 0)
+        HGOTO_ERROR(H5E_HEAP, H5E_CANTFREE, FAIL, "unable to destroy fractal heap doubling table")
+
+    /* Release any I/O pipeline filter information */
+    if(hdr->pline.nused)
+        if(H5O_msg_reset(H5O_PLINE_ID, &(hdr->pline)) < 0)
+            HGOTO_ERROR(H5E_HEAP, H5E_CANTFREE, FAIL, "unable to reset I/O pipeline message")
+
+    /* Free the shared info itself */
+    hdr = H5FL_FREE(H5HF_hdr_t, hdr);
+
+done:
+    FUNC_LEAVE_NOAPI(ret_value)
+} /* end H5HF_hdr_free() */
+
 
 /*-------------------------------------------------------------------------
  * Function:	H5HF_hdr_delete
@@ -1505,8 +1439,8 @@ done:
 herr_t
 H5HF_hdr_delete(H5HF_hdr_t *hdr, hid_t dxpl_id)
 {
-    unsigned cache_flags = H5AC__NO_FLAGS_SET;      /* Flags for unprotecting heap header */
-    herr_t ret_value = SUCCEED;
+    unsigned cache_flags = H5AC__NO_FLAGS_SET;  /* Flags for unprotecting heap header */
+    herr_t ret_value = SUCCEED;                 /* Return value */
 
     FUNC_ENTER_NOAPI(H5HF_hdr_delete, FAIL)
 
@@ -1535,9 +1469,6 @@ H5HF_hdr_delete(H5HF_hdr_t *hdr, hid_t dxpl_id)
      *  will get unpinned)
      */
     if(H5F_addr_defined(hdr->fs_addr)) {
-#ifdef QAK
-HDfprintf(stderr, "%s: hdr->fs_addr = %a\n", FUNC, hdr->fs_addr);
-#endif /* QAK */
         /* Delete free space manager for heap */
         if(H5HF_space_delete(hdr, dxpl_id) < 0)
             HGOTO_ERROR(H5E_HEAP, H5E_CANTFREE, FAIL, "unable to release fractal heap free space manager")
@@ -1545,18 +1476,12 @@ HDfprintf(stderr, "%s: hdr->fs_addr = %a\n", FUNC, hdr->fs_addr);
 
     /* Check for root direct/indirect block */
     if(H5F_addr_defined(hdr->man_dtable.table_addr)) {
-#ifdef QAK
-HDfprintf(stderr, "%s: hdr->man_dtable.table_addr = %a\n", FUNC, hdr->man_dtable.table_addr);
-#endif /* QAK */
         if(hdr->man_dtable.curr_root_rows == 0) {
             hsize_t dblock_size;        /* Size of direct block */
 
             /* Check for I/O filters on this heap */
             if(hdr->filter_len > 0) {
                 dblock_size = (hsize_t)hdr->pline_root_direct_size;
-#ifdef QAK
-HDfprintf(stderr, "%s: hdr->pline_root_direct_size = %Zu\n", FUNC, hdr->pline_root_direct_size);
-#endif /* QAK */
 
                 /* Reset the header's pipeline information */
                 hdr->pline_root_direct_size = 0;
@@ -1578,9 +1503,6 @@ HDfprintf(stderr, "%s: hdr->pline_root_direct_size = %Zu\n", FUNC, hdr->pline_ro
 
     /* Check for 'huge' objects in heap */
     if(H5F_addr_defined(hdr->huge_bt2_addr)) {
-#ifdef QAK
-HDfprintf(stderr, "%s: hdr->huge_bt2_addr = %a\n", FUNC, hdr->huge_bt2_addr);
-#endif /* QAK */
         /* Delete huge objects in heap and their tracker */
         if(H5HF_huge_delete(hdr, dxpl_id) < 0)
             HGOTO_ERROR(H5E_HEAP, H5E_CANTFREE, FAIL, "unable to release fractal heap 'huge' objects and tracker")
@@ -1591,7 +1513,7 @@ HDfprintf(stderr, "%s: hdr->huge_bt2_addr = %a\n", FUNC, hdr->huge_bt2_addr);
 
 done:
     /* Unprotect the header with appropriate flags */
-    if(hdr && H5AC_unprotect(hdr->f, dxpl_id, H5AC_FHEAP_HDR, hdr->heap_addr, hdr, cache_flags) < 0)
+    if(H5AC_unprotect(hdr->f, dxpl_id, H5AC_FHEAP_HDR, hdr->heap_addr, hdr, cache_flags) < 0)
         HDONE_ERROR(H5E_HEAP, H5E_CANTUNPROTECT, FAIL, "unable to release fractal heap header")
 
     FUNC_LEAVE_NOAPI(ret_value)
