@@ -42,6 +42,7 @@
 
 
 #include "H5private.h"		/* Generic Functions			*/
+#include "H5ACprivate.h"        /* Metadata cache                       */
 #include "H5Eprivate.h"		/* Error handling		  	*/
 #include "H5FLprivate.h"	/* Free Lists                           */
 #include "H5Ipkg.h"		/* IDs			  		*/
@@ -61,6 +62,10 @@
 
 /* Local Macros */
 
+/* Define the maximum number of returned ID structures to keep around
+   for re-use. */
+#define MAX_FREE_ID_STRUCTS 1000
+
 /*
  * Define the following macro for fast hash calculations (but limited
  * hash sizes)
@@ -91,19 +96,22 @@ typedef struct H5I_id_info_t {
     hid_t	id;		/* ID for this info			    */
     unsigned	count;		/* ref. count for this atom		    */
     unsigned    app_count;      /* ref. count of application visible atoms  */
-    void	*obj_ptr;	/* pointer associated with the atom	    */
+    const void	*obj_ptr;	/* pointer associated with the atom	    */
     struct H5I_id_info_t *next;	/* link to next atom (in case of hash-clash)*/
 } H5I_id_info_t;
 
 /* ID type structure used */
 typedef struct {
     unsigned	count;		/*# of times this type has been initialized*/
+    unsigned    free_count; /* # of available ID structures awaiting recycling */
     unsigned	reserved;	/*# of IDs to reserve for constant IDs	    */
     unsigned	wrapped;	/*whether the id count has wrapped around   */
     size_t	hash_size;	/*sizeof the hash table to store the IDs in */
     unsigned	ids;		/*current number of IDs held		    */
     unsigned	nextid;		/*ID to use for the next atom		    */
     H5I_free_t	free_func;	/*release object method	    		    */
+    unsigned    reuse_ids;  /* whether to reuse returned IDs for this type */
+    H5I_id_info_t * next_id_ptr; /* pointer to head of available ID list */
     H5I_id_info_t **id_list;	/*pointer to an array of ptrs to IDs	    */
 } H5I_id_type_t;
 
@@ -164,9 +172,7 @@ H5I_init_interface(void)
  *
  * 		Failure:	Negative.
  *
- * Programmer:
- *
- * Modifications:
+ * Programmer:	Unknown
  *
  *-------------------------------------------------------------------------
  */
@@ -216,15 +222,12 @@ H5I_term_interface(void)
  *		the type.
  *
  * Return:	Success:	Type ID of the new type
- *
  *		Failure:	H5I_BADID
  *
  * Programmers:	Nathaniel Furrer
- *				James Laird
+ *		James Laird
  *		Friday, April 30, 2004
  *
- * Modifications:
- *
  *-------------------------------------------------------------------------
  */
 H5I_type_t
@@ -262,22 +265,9 @@ done:
  *		Failure:	H5I_BADID
  *
  * Programmers:	Nathaniel Furrer
- *				James Laird
+ *		James Laird
  *		Friday, April 30, 2004
  *
- * Modifications: The initialization section of this function was formerly
- *					H5I_init_type, programmed by Robb Matzke on February 19,
- *					1999.
- *
- * 		Bill Wendling, 2000-05-05
- * 		Instead of the ugly test of whether hash_size is a power of
- * 		two, I placed it in a macro POWER_OF_TWO which uses the fact
- * 		that a number that is a power of two has only 1 bit set.
- *
- * 		Bill Wendling, 2000-05-09
- * 		Changed POWER_OF_TWO macro to allow 1 as a valid power of two.
- * 		Changed test below accordingly.
- *
  *-------------------------------------------------------------------------
  */
 H5I_type_t
@@ -348,9 +338,20 @@ H5I_register_type(H5I_type_t type_id, size_t hash_size, unsigned reserved,
         type_ptr->ids = 0;
         type_ptr->nextid = reserved;
         type_ptr->free_func = free_func;
+        type_ptr->next_id_ptr = NULL;
         type_ptr->id_list = (H5I_id_info_t **)H5MM_calloc(hash_size * sizeof(H5I_id_info_t *));
         if(NULL == type_ptr->id_list)
             HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, H5I_BADID, "memory allocation failed")
+
+        /* Don't re-use IDs for property lists, as this causes problems
+         * with some virtual file drivers. Also, open datatypes are not 
+         * getting reduced to zero before file close in some situations,
+         * resulting in memory leak, so skip them for now as well. */
+        if (type_id == H5I_GENPROP_LST || type_id == H5I_DATATYPE)
+            type_ptr->reuse_ids = FALSE;
+        else
+            type_ptr->reuse_ids = TRUE;
+
     } /* end if */
 
     /* Increment the count of the times this type has been initialized */
@@ -375,15 +376,12 @@ done:
  *              currently registered with the library.
  *
  * Return:	Success:        1 if the type is registered, 0 if it is not
- *
  *		Failure:	Negative
  *
  * Programmer:	James Laird
  *		Nathaniel Furrer
  *              Tuesday, June 29, 2004
  *
- * Modifications:
- *
  *-------------------------------------------------------------------------
  */
 htri_t
@@ -414,19 +412,12 @@ done:
  *              private interface, which will just return 0.
  *
  * Return:	Success:	Zero
- *
  *		Failure:	Negative
  *
  * Programmer:	James Laird
  *		Nathaniel Furrer
  *              Friday, April 23, 2004
  *
- * Modifications:
- *              June 29, 2004
- *              Nat Furrer and James Laird
- *              Changed function signature to return the number of members
- *              by reference.
- *
  *-------------------------------------------------------------------------
  */
 herr_t
@@ -476,8 +467,6 @@ done:
  * Programmer:	Robb Matzke
  *              Wednesday, March 24, 1999
  *
- * Modifications:
- *
  *-------------------------------------------------------------------------
  */
 int
@@ -509,14 +498,12 @@ done:
  *		Public interface to H5I_clear_type.
  *
  * Return:	Success:	Non-negative
- *
  *		Failure:	negative
  *
  * Programmer:	James Laird
- *				Nathaniel Furrer
+ *		Nathaniel Furrer
  *              Friday, April 23, 2004
  *
- * Modifications:
  *-------------------------------------------------------------------------
  */
 herr_t
@@ -544,7 +531,6 @@ done:
  *		function for each object regardless of the reference count.
  *
  * Return:	Success:	Non-negative
- *
  *		Failure:	negative
  *
  * Programmer:	Robb Matzke
@@ -609,7 +595,8 @@ H5I_clear_type(H5I_type_t type, hbool_t force, hbool_t app_ref)
             } /* end if */
 
             /* Check for a 'free' function and call it, if it exists */
-            if(type_ptr->free_func && (type_ptr->free_func)(cur->obj_ptr) < 0) {
+            /* (Casting away const OK -QAK) */
+            if(type_ptr->free_func && (type_ptr->free_func)((void *)cur->obj_ptr) < 0) {
                 if(force) {
 #ifdef H5I_DEBUG
                     if(H5DEBUG(I)) {
@@ -669,7 +656,7 @@ H5I_clear_type(H5I_type_t type, hbool_t force, hbool_t app_ref)
                 } /* end else */
 
                 /* Free the node */
-                (void)H5FL_FREE(H5I_id_info_t, cur);
+                cur = H5FL_FREE(H5I_id_info_t, cur);
             } /* end if */
             else {
                 /* Advance to next node */
@@ -678,6 +665,16 @@ H5I_clear_type(H5I_type_t type, hbool_t force, hbool_t app_ref)
         } /* end for */
     } /* end for */
 
+    /* Also free any ID structures being retained for potential re-use */
+    while(type_ptr->next_id_ptr) {
+        H5I_id_info_t *tmp_id_ptr;          /* temp ptr to next atom */
+
+        tmp_id_ptr = type_ptr->next_id_ptr->next;
+        (void)H5FL_FREE(H5I_id_info_t, type_ptr->next_id_ptr);
+        type_ptr->next_id_ptr = tmp_id_ptr;
+    } /* end while */
+    type_ptr->free_count = 0;
+
 done:
     FUNC_LEAVE_NOAPI(ret_value)
 } /* end H5I_clear_type() */
@@ -697,8 +694,6 @@ done:
  * Programmer:	Nathaniel Furrer
  *		James Laird
  *
- * Modifications:
- *
  *-------------------------------------------------------------------------
  */
 herr_t
@@ -732,8 +727,6 @@ done:
  * Programmer:	Nathaniel Furrer
  *		James Laird
  *
- * Modifications:
- *
  *-------------------------------------------------------------------------
  */
 herr_t
@@ -771,18 +764,15 @@ done:
  * Purpose:	Public interface to H5I_register.
  *
  * Return:	Success:	New object id.
- *
  *		Failure:	Negative
  *
  * Programmer:	Nathaniel Furrer
- *				James Laird
- *
- * Modifications:
+ *		James Laird
  *
  *-------------------------------------------------------------------------
  */
 hid_t
-H5Iregister(H5I_type_t type, void *object)
+H5Iregister(H5I_type_t type, const void *object)
 {
     hid_t ret_value;                      /* Return value */
 
@@ -811,7 +801,6 @@ done:
  *		the ID which is returned to the user.
  *
  * Return:	Success:	New object id.
- *
  *		Failure:	Negative
  *
  * Programmer:	Unknown
@@ -826,11 +815,10 @@ done:
  *-------------------------------------------------------------------------
  */
 hid_t
-H5I_register(H5I_type_t type, void *object, hbool_t app_ref)
+H5I_register(H5I_type_t type, const void *object, hbool_t app_ref)
 {
     H5I_id_type_t	*type_ptr;	/*ptr to the type		*/
     H5I_id_info_t	*id_ptr;	/*ptr to the new ID information */
-    hid_t		new_id;		/*new ID			*/
     unsigned		hash_loc;	/*new item's hash table location*/
     hid_t		next_id;	/*next ID to check		*/
     H5I_id_info_t	*curr_id;	/*ptr to the current atom	*/
@@ -845,26 +833,46 @@ H5I_register(H5I_type_t type, void *object, hbool_t app_ref)
     type_ptr = H5I_id_type_list_g[type];
     if(NULL == type_ptr || type_ptr->count <= 0)
 	HGOTO_ERROR(H5E_ATOM, H5E_BADGROUP, FAIL, "invalid type")
-    if(NULL == (id_ptr = H5FL_MALLOC(H5I_id_info_t)))
-        HGOTO_ERROR(H5E_ATOM, H5E_NOSPACE, FAIL, "memory allocation failed")
 
-    /* Create the struct & it's ID */
-    new_id = H5I_MAKE(type, type_ptr->nextid);
-    id_ptr->id = new_id;
+    /* If there is an available ID structure, use it. */
+    if(type_ptr->next_id_ptr) {
+        /* Use existing available ID struct */
+        id_ptr = type_ptr->next_id_ptr;
+    
+        /* Remove struct from list of available ones */
+        type_ptr->next_id_ptr = type_ptr->next_id_ptr->next;
+
+        /* Decrease count of available ID structures */
+        type_ptr->free_count--;
+    } /* end if */
+    /* If no available ID structure, then create a new id for use, and
+     * allocate a new struct to house it. */
+    else {
+        /* Allocate new ID struct */
+        if(NULL == (id_ptr = H5FL_MALLOC(H5I_id_info_t)))
+            HGOTO_ERROR(H5E_ATOM, H5E_NOSPACE, FAIL, "memory allocation failed")
+
+        /* Make a new ID */
+        id_ptr->id = H5I_MAKE(type, type_ptr->nextid);
+
+        /* Increment nextid value */
+        type_ptr->nextid++;
+    } /* end if */
+
+    /* Fill in remaining fields of ID struct */
     id_ptr->count = 1; /*initial reference count*/
     id_ptr->app_count = !!app_ref;
     id_ptr->obj_ptr = object;
     id_ptr->next = NULL;
 
     /* hash bucket already full, prepend to front of chain */
-    hash_loc = type_ptr->nextid % (unsigned)type_ptr->hash_size;
+    hash_loc = id_ptr->id % (unsigned)type_ptr->hash_size;
     if(type_ptr->id_list[hash_loc] != NULL)
 	id_ptr->next = type_ptr->id_list[hash_loc];
 
     /* Insert into the type */
     type_ptr->id_list[hash_loc] = id_ptr;
     type_ptr->ids++;
-    type_ptr->nextid++;
 
     /*
      * This next section of code checks for the 'nextid' getting too large and
@@ -894,7 +902,7 @@ H5I_register(H5I_type_t type, void *object, hbool_t app_ref)
 
 	    /* new ID to check for */
 	    next_id = H5I_MAKE(type, type_ptr->nextid);
-	    hash_loc = H5I_LOC(type_ptr->nextid, type_ptr->hash_size);
+	    hash_loc = (unsigned)H5I_LOC(type_ptr->nextid, type_ptr->hash_size);
 	    curr_id = type_ptr->id_list[hash_loc];
 	    if(curr_id == NULL)
                 break; /* Ha! this is not likely... */
@@ -915,12 +923,50 @@ H5I_register(H5I_type_t type, void *object, hbool_t app_ref)
     } /* end if */
 
     /* Set return value */
-    ret_value = new_id;
+    ret_value = id_ptr->id;
 
 done:
     FUNC_LEAVE_NOAPI(ret_value)
 } /* end H5I_register() */
 
+
+/*-------------------------------------------------------------------------
+ * Function:	H5I_subst
+ *
+ * Purpose:	Substitute a new object pointer for the specified ID.
+ *
+ * Return:	Success:	Non-null previous object pointer associated
+ *				with the specified ID.
+ *		Failure:	NULL
+ *
+ * Programmer:	Quincey Koziol
+ *		Saturday, February 27, 2010
+ *
+ *-------------------------------------------------------------------------
+ */
+void *
+H5I_subst(hid_t id, const void *new_object)
+{
+    H5I_id_info_t	*id_ptr;	/* Ptr to the atom	*/
+    void		*ret_value;	/* Return value		*/
+
+    FUNC_ENTER_NOAPI(H5I_subst, NULL)
+
+    /* General lookup of the ID */
+    if(NULL == (id_ptr = H5I_find_id(id)))
+        HGOTO_ERROR(H5E_ATOM, H5E_NOTFOUND, NULL, "can't get ID ref count")
+
+    /* Get the old object pointer to return */
+    /* (Casting away const OK -QAK) */
+    ret_value = (void *)id_ptr->obj_ptr;
+
+    /* Set the new object pointer for the ID */
+    id_ptr->obj_ptr = new_object;
+
+done:
+    FUNC_LEAVE_NOAPI(ret_value)
+} /* end if */
+
 
 /*-------------------------------------------------------------------------
  * Function:	H5I_object
@@ -929,12 +975,9 @@ done:
  *
  * Return:	Success:	Non-null object pointer associated with the
  *				specified ID.
- *
  *		Failure:	NULL
  *
- * Programmer:
- *
- * Modifications:
+ * Programmer:	Unknown
  *
  *-------------------------------------------------------------------------
  */
@@ -949,7 +992,8 @@ H5I_object(hid_t id)
     /* General lookup of the ID */
     if(NULL != (id_ptr = H5I_find_id(id))) {
         /* Get the object pointer to return */
-        ret_value = id_ptr->obj_ptr;
+        /* (Casting away const OK -QAK) */
+        ret_value = (void *)id_ptr->obj_ptr;
     } /* end if */
 
 done:
@@ -966,15 +1010,12 @@ done:
  *
  * Return:	Success:	Non-null object pointer associated with the
  *				specified ID.
- *
  *		Failure:	NULL
  *
  * Programmer:	Nathaniel Furrer
  *		James Laird
  *		Friday, April 23, 2004
  *
- * Modifications:
- *
  *-------------------------------------------------------------------------
  */
 void *
@@ -1005,14 +1046,11 @@ done:
  *
  * Return:	Success:	Non-null object pointer associated with the
  *				specified ID.
- *
  *		Failure:	NULL
  *
  * Programmer:	Quincey Koziol
  *		Wednesday, July 31, 2002
  *
- * Modifications:
- *
  *-------------------------------------------------------------------------
  */
 void *
@@ -1028,7 +1066,8 @@ H5I_object_verify(hid_t id, H5I_type_t id_type)
     /* Verify that the type of the ID is correct & lookup the ID */
     if(id_type == H5I_TYPE(id) && NULL != (id_ptr = H5I_find_id(id))) {
         /* Get the object pointer to return */
-        ret_value = id_ptr->obj_ptr;
+        /* (Casting away const OK -QAK) */
+        ret_value = (void *)id_ptr->obj_ptr;
     } /* end if */
 
 done:
@@ -1045,14 +1084,11 @@ done:
  *		in the object ID.
  *
  * Return:	Success:	A valid type number
- *
  *		Failure:	H5I_BADID, a negative value.
  *
  * Programmer:	Robb Matzke
  *		Friday, February 19, 1999
  *
- * Modifications:
- *
  *-------------------------------------------------------------------------
  */
 H5I_type_t
@@ -1081,15 +1117,10 @@ done:
  *		encoded as part of the ID.
  *
  * Return:	Success:	Type number
- *
  *		Failure:	H5I_BADID, a negative value
  *
- * Programmer:
+ * Programmer:	Unknown
  *
- * Modifications:
- *		Robb Matzke, 1999-08-23
- *		Also fails if the ID has a valid type but no longer exists
- *		in the ID tables.
  *-------------------------------------------------------------------------
  */
 H5I_type_t
@@ -1120,13 +1151,10 @@ done:
  * Return:	Success:	A pointer to the object that was removed, the
  *				same pointer which would have been found by
  *				calling H5I_object().
- *
  *		Failure:	NULL
  *
  * Programmer:	James Laird
- *				Nathaniel Furrer
- *
- * Modifications:
+ *		Nathaniel Furrer
  *
  *-------------------------------------------------------------------------
  */
@@ -1157,13 +1185,10 @@ done:
  * Return:	Success:	A pointer to the object that was removed, the
  *				same pointer which would have been found by
  *				calling H5I_object().
- *
  *		Failure:	NULL
  *
  * Programmer:	James Laird
- *				Nat Furrer
- *
- * Modifications:
+ *		Nat Furrer
  *
  *-------------------------------------------------------------------------
  */
@@ -1194,12 +1219,9 @@ done:
  * Return:	Success:	A pointer to the object that was removed, the
  *				same pointer which would have been found by
  *				calling H5I_object().
- *
  *		Failure:	NULL
  *
- * Programmer:
- *
- * Modifications:
+ * Programmer:	Unknown
  *
  *-------------------------------------------------------------------------
  */
@@ -1209,6 +1231,7 @@ H5I_remove(hid_t id)
     H5I_id_type_t	*type_ptr;	/*ptr to the atomic type	*/
     H5I_id_info_t	*curr_id;	/*ptr to the current atom	*/
     H5I_id_info_t	*last_id;	/*ptr to the last atom		*/
+    H5I_id_info_t   *tmp_id_ptr; /*temp ptr to next atom     */
     H5I_type_t		type;		/*atom's atomic type		*/
     unsigned		hash_loc;	/*atom's hash table location	*/
     void *	      ret_value = NULL;	/*return value			*/
@@ -1244,9 +1267,21 @@ H5I_remove(hid_t id)
         } else {
             last_id->next = curr_id->next;
         }
-        ret_value = curr_id->obj_ptr;
-        (void)H5FL_FREE(H5I_id_info_t, curr_id);
-    } else {
+        /* (Casting away const OK -QAK) */
+        ret_value = (void *)curr_id->obj_ptr;
+        
+        /* If there's room, and we can save IDs of this type, then 
+           save the struct (and its ID) for future re-use */
+        if((type_ptr->reuse_ids)&&(type_ptr->free_count < MAX_FREE_ID_STRUCTS)) {
+            curr_id->next = type_ptr->next_id_ptr;
+            type_ptr->next_id_ptr = curr_id;
+            type_ptr->free_count++;
+        } /* end if */
+        /* Otherwise, just toss it. */
+        else
+            curr_id = H5FL_FREE(H5I_id_info_t, curr_id);
+    } /* end if */
+    else {
         /* couldn't find the ID in the proper place */
 	HGOTO_ERROR(H5E_ATOM, H5E_BADATOM, NULL, "invalid ID")
     }
@@ -1254,6 +1289,20 @@ H5I_remove(hid_t id)
     /* Decrement the number of IDs in the type */
     (type_ptr->ids)--;
 
+    /* If there are no more IDs of this type, then we can free all available
+       ID strutures, and reset starting typeid and wrapped status. */
+    if(type_ptr->ids == 0) {
+        while(type_ptr->next_id_ptr) {
+            tmp_id_ptr = type_ptr->next_id_ptr->next;
+            (void)H5FL_FREE(H5I_id_info_t, type_ptr->next_id_ptr);
+            type_ptr->next_id_ptr = tmp_id_ptr;
+        } /* end while */
+        type_ptr->free_count = 0;
+
+        type_ptr->nextid = type_ptr->reserved;
+        type_ptr->wrapped = FALSE;
+    } /* end if */
+
 done:
     FUNC_LEAVE_NOAPI(ret_value)
 } /* end H5I_remove() */
@@ -1272,8 +1321,6 @@ done:
  * Programmer:  Quincey Koziol
  *              Dec  7, 2003
  *
- * Modifications:
- *
  *-------------------------------------------------------------------------
  */
 int
@@ -1289,7 +1336,7 @@ H5Idec_ref(hid_t id)
 	HGOTO_ERROR(H5E_ATOM, H5E_BADATOM, FAIL, "invalid ID")
 
     /* Do actual decrement operation */
-    if((ret_value = H5I_dec_ref(id, TRUE)) < 0)
+    if((ret_value = H5I_dec_app_ref(id)) < 0)
         HGOTO_ERROR(H5E_ATOM, H5E_CANTDEC, FAIL, "can't decrement ID ref count")
 
 done:
@@ -1312,36 +1359,10 @@ done:
  *
  * Programmer:	Unknown
  *
- * Modifications:
- *
- *	Robb Matzke, 19 Feb 1998
- *	It is no longer an error when the reference count of an item reaches
- *	zero and no `free' function has been defined.  The object is still
- *	removed from the list.
- *
- *	Robb Matzke, 30 Dec 1998
- *	Fixed a bug where the return value was always zero instead of the new
- *	reference count.
- *
- *	Robb Matzke, 19 Feb 1999
- *	If the free method is defined and fails then the object is not
- *	removed from the type and its reference count is not decremented.
- *	The type number is now passed to the free method.
- *
- *	Raymond, 11 Dec 2001
- *	If the freeing function fails, return failure instead of reference
- *	count 1.  This feature is needed by file close with H5F_CLOSE_SEMI
- *	value.
- *
- *  Neil Fortner, 7 Aug 2008
- *  Added app_ref parameter and support for the app_count field, to
- *  distiguish between reference count from the library and from the
- *  application.
- *
  *-------------------------------------------------------------------------
  */
 int
-H5I_dec_ref(hid_t id, hbool_t app_ref)
+H5I_dec_ref(hid_t id)
 {
     H5I_type_t		type;		/*type the object is in*/
     H5I_id_type_t	*type_ptr;	/*ptr to the type	*/
@@ -1362,7 +1383,7 @@ H5I_dec_ref(hid_t id, hbool_t app_ref)
 	HGOTO_ERROR(H5E_ARGS, H5E_BADRANGE, FAIL, "invalid type number")
 
     /* General lookup of the ID */
-    if(NULL == (id_ptr=H5I_find_id(id)))
+    if(NULL == (id_ptr = H5I_find_id(id)))
 	HGOTO_ERROR(H5E_ATOM, H5E_BADATOM, FAIL, "can't locate ID")
 
     /*
@@ -1374,26 +1395,123 @@ H5I_dec_ref(hid_t id, hbool_t app_ref)
      * reference count without calling the free method.
      *
      * Beware: the free method may call other H5I functions.
+     * 
+     * If an object is closing, we can remove the ID even though the free 
+     * method might fail.  This can happen when a mandatory filter fails to
+     * write when a dataset is closed and the chunk cache is flushed to the 
+     * file.  We have to close the dataset anyway. (SLU - 2010/9/7)
      */
     if(1 == id_ptr->count) {
-        if(!type_ptr->free_func || (type_ptr->free_func)(id_ptr->obj_ptr) >= 0) {
+        /* (Casting away const OK -QAK) */
+        if(!type_ptr->free_func || (type_ptr->free_func)((void *)id_ptr->obj_ptr) >= 0) {
             H5I_remove(id);
             ret_value = 0;
-        } else {
+        } /* end if */
+        else
             ret_value = FAIL;
-        }
-    } else {
+    } /* end if */
+    else {
         --(id_ptr->count);
-        if (app_ref)
-            --(id_ptr->app_count);
-        HDassert(id_ptr->count >= id_ptr->app_count);
-        ret_value = app_ref ? id_ptr->app_count : id_ptr->count;
-    }
+        ret_value = (int)id_ptr->count;
+    } /* end else */
 
 done:
     FUNC_LEAVE_NOAPI(ret_value)
 } /* end H5I_dec_ref() */
 
+
+/*-------------------------------------------------------------------------
+ * Function:	H5I_dec_app_ref
+ *
+ * Purpose:	H5I_dec_ref wrapper for case of modifying the application ref.
+ *		count for an ID as well as normal reference count.
+ *
+ * Return:	Success:	New app. reference count.
+ *		Failure:	Negative
+ *
+ * Programmer:  Quincey Koziol
+ *              Sept 16, 2010
+ *
+ *-------------------------------------------------------------------------
+ */
+int
+H5I_dec_app_ref(hid_t id)
+{
+    H5I_id_info_t	*id_ptr;	/*ptr to the new ID	*/
+    int ret_value;                      /* Return value */
+
+    FUNC_ENTER_NOAPI(H5I_dec_app_ref, FAIL)
+
+    /* Sanity check */
+    HDassert(id >= 0);
+
+    /* Call regular decrement reference count routine */
+    if((ret_value = H5I_dec_ref(id)) < 0)
+        HGOTO_ERROR(H5E_ATOM, H5E_CANTDEC, FAIL, "can't decrement ID ref count")
+
+    /* Check if the ID still exists */
+    if(ret_value > 0) {
+        /* General lookup of the ID */
+        if(NULL == (id_ptr = H5I_find_id(id)))
+            HGOTO_ERROR(H5E_ATOM, H5E_BADATOM, FAIL, "can't locate ID")
+
+        /* Adjust app_ref */
+        --(id_ptr->app_count);
+        HDassert(id_ptr->count >= id_ptr->app_count);
+
+        /* Set return value */
+        ret_value = (int)id_ptr->app_count;
+    } /* end if */
+
+done:
+    FUNC_LEAVE_NOAPI(ret_value)
+} /* end H5I_dec_app_ref() */
+
+
+/*-------------------------------------------------------------------------
+ * Function:	H5I_dec_app_ref_always_close
+ *
+ * Purpose:	H5I_dec_app_ref wrapper for case of always closing the ID,
+ *		even when the free routine fails
+ *
+ * Return:	Success:	New app. reference count.
+ *		Failure:	Negative
+ *
+ * Programmer:  Quincey Koziol
+ *              Sept 16, 2010
+ *
+ *-------------------------------------------------------------------------
+ */
+int
+H5I_dec_app_ref_always_close(hid_t id)
+{
+    int ret_value;                      /* Return value */
+
+    FUNC_ENTER_NOAPI(H5I_dec_app_ref_always_close, FAIL)
+
+    /* Sanity check */
+    HDassert(id >= 0);
+
+    /* Call application decrement reference count routine */
+    ret_value = H5I_dec_app_ref(id);
+
+    /* Check for failure */
+    if(ret_value < 0) {
+        /*
+         * If an object is closing, we can remove the ID even though the free 
+         * method might fail.  This can happen when a mandatory filter fails to
+         * write when a dataset is closed and the chunk cache is flushed to the 
+         * file.  We have to close the dataset anyway. (SLU - 2010/9/7)
+         */
+        H5I_remove(id);
+
+        HGOTO_ERROR(H5E_ATOM, H5E_CANTDEC, FAIL, "can't decrement ID ref count")
+    } /* end if */
+
+done:
+    FUNC_LEAVE_NOAPI(ret_value)
+} /* end H5I_dec_app_ref_always_close() */
+
 
 /*-------------------------------------------------------------------------
  * Function:	H5Iinc_ref
@@ -1406,8 +1524,6 @@ done:
  * Programmer:  Quincey Koziol
  *              Dec  7, 2003
  *
- * Modifications:
- *
  *-------------------------------------------------------------------------
  */
 int
@@ -1437,7 +1553,6 @@ done:
  * Purpose:	Increment the reference count for an object.
  *
  * Return:	Success:	The new reference count.
- *
  *		Failure:	Negative
  *
  * Programmer:	Robb Matzke
@@ -1483,7 +1598,7 @@ H5I_inc_ref(hid_t id, hbool_t app_ref)
         ++(id_ptr->app_count);
 
     /* Set return value */
-    ret_value = app_ref ? id_ptr->app_count : id_ptr->count;
+    ret_value = (int)(app_ref ? id_ptr->app_count : id_ptr->count);
 
 done:
     FUNC_LEAVE_NOAPI(ret_value)
@@ -1501,8 +1616,6 @@ done:
  * Programmer:  Quincey Koziol
  *              Dec  7, 2003
  *
- * Modifications:
- *
  *-------------------------------------------------------------------------
  */
 int
@@ -1532,7 +1645,6 @@ done:
  * Purpose:	Retrieve the reference count for an object.
  *
  * Return:	Success:	The reference count.
- *
  *		Failure:	Negative
  *
  * Programmer:	Quincey Koziol
@@ -1573,7 +1685,7 @@ H5I_get_ref(hid_t id, hbool_t app_ref)
 	HGOTO_ERROR(H5E_ATOM, H5E_BADATOM, FAIL, "can't locate ID")
 
     /* Set return value */
-    ret_value = app_ref ? id_ptr->app_count : id_ptr->count;
+    ret_value = (int)(app_ref ? id_ptr->app_count : id_ptr->count);
 
 done:
     FUNC_LEAVE_NOAPI(ret_value)
@@ -1589,11 +1701,9 @@ done:
  *		Failure:	Negative
  *
  * Programmer:  Nat Furrer
- *				James Laird
+ *		James Laird
  *              April 30, 2004
  *
- * Modifications:
- *
  *-------------------------------------------------------------------------
  */
 int
@@ -1626,15 +1736,12 @@ done:
  * Purpose:	Increment the reference count for an ID type.
  *
  * Return:	Success:	The new reference count.
- *
  *		Failure:	Negative
  *
  * Programmer:	James Laird
- *				Nat Furrer
+ *		Nat Furrer
  *              Friday, April 30, 2004
  *
- * Modifications:
- *
  *-------------------------------------------------------------------------
  */
 int
@@ -1654,7 +1761,7 @@ H5I_inc_type_ref(H5I_type_t type)
 	HGOTO_ERROR(H5E_ATOM, H5E_BADGROUP, FAIL, "invalid type")
 
     /* Set return value */
-    ret_value = ++(type_ptr->count);
+    ret_value = (int)(++(type_ptr->count));
 
 done:
     FUNC_LEAVE_NOAPI(ret_value)
@@ -1679,9 +1786,7 @@ done:
  * Return:	Number of references to type on success/Negative on failure
  *
  * Programmer:	Nathaniel Furrer
- *				James Laird
- *
- * Modifications:
+ *		James Laird
  *
  *-------------------------------------------------------------------------
  */
@@ -1721,11 +1826,6 @@ done:
  *
  * Programmer:	Unknown
  *
- * Modifications:
- *
- *	Robb Matzke, 25 Feb 1998
- *	IDs are freed when a type is destroyed.
- *
  *-------------------------------------------------------------------------
  */
 herr_t
@@ -1755,7 +1855,7 @@ H5I_dec_type_ref(H5I_type_t type)
     } /* end if */
     else {
         --(type_ptr->count);
-        ret_value = type_ptr->count;
+        ret_value = (herr_t)type_ptr->count;
     } /* end else */
 
 done:
@@ -1772,11 +1872,9 @@ done:
  *		Failure:	Negative
  *
  * Programmer:  Nat Furrer
- *				James Laird
+ *		James Laird
  *              April 30, 2004
  *
- * Modifications:
- *
  *-------------------------------------------------------------------------
  */
 int
@@ -1813,11 +1911,9 @@ done:
  *		Failure:	Negative
  *
  * Programmer:  Nat Furrer
- *				James Laird
+ *		James Laird
  *              April 30, 2004
  *
- * Modifications:
- *
  *-------------------------------------------------------------------------
  */
 int
@@ -1837,7 +1933,7 @@ H5I_get_type_ref(H5I_type_t type)
         HGOTO_ERROR(H5E_ATOM, H5E_BADGROUP, FAIL, "invalid type")
 
     /* Set return value */
-    ret_value = type_ptr->count;
+    ret_value = (int)type_ptr->count;
 
 done:
     FUNC_LEAVE_NOAPI(ret_value)
@@ -1857,12 +1953,6 @@ done:
  * Programmer:  Neil Fortner
  *              Friday, October 31, 2008 (boo)
  *
- * Modifications:
- *              Raymond Lu
- *              1 April 2009 (Believe it or not!)
- *              Moved the argument check down to H5I_find_id because other
- *              caller functions may pass in some invalid IDs to H5I_find_id.
- *              It used to do assertion check.
  *-------------------------------------------------------------------------
  */
 htri_t
@@ -1943,7 +2033,6 @@ done:
  * Return:	Success:	The first object in the type for which FUNC
  *				returns non-zero. NULL if FUNC returned zero
  *				for every object in the type.
- *
  *		Failure:	NULL
  *
  * Programmer:	Robb Matzke
@@ -1982,8 +2071,10 @@ H5I_search(H5I_type_t type, H5I_search_func_t func, void *key, hbool_t app_ref)
             id_ptr = type_ptr->id_list[i];
             while(id_ptr) {
                 next_id = id_ptr->next;      /* Protect against ID being deleted in callback */
-                if((!app_ref || id_ptr->app_count) && (*func)(id_ptr->obj_ptr, id_ptr->id, key))
-                    HGOTO_DONE(id_ptr->obj_ptr);	/*found the item*/
+                /* (Casting away const OK -QAK) */
+                if((!app_ref || id_ptr->app_count) && (*func)((void *)id_ptr->obj_ptr, id_ptr->id, key))
+                    /* (Casting away const OK -QAK) */
+                    HGOTO_DONE((void *)id_ptr->obj_ptr);	/*found the item*/
                 id_ptr = next_id;
             } /* end while */
         } /* end for */
@@ -2004,12 +2095,8 @@ done:
  *
  *		Failure:	NULL
  *
- * Programmer:
+ * Programmer:	Unknown
  *
- * Modifications:
- *              Raymond Lu
- *              1 April 2009 (Believe it or not!)
- *              Added argument check, took away assertion check.
  *-------------------------------------------------------------------------
  */
 static H5I_id_info_t *
@@ -2089,13 +2176,18 @@ done:
 ssize_t
 H5Iget_name(hid_t id, char *name/*out*/, size_t size)
 {
-    ssize_t       ret_value;
+    H5G_loc_t     loc;          /* Object location */
+    ssize_t       ret_value;    /* Return value */
 
     FUNC_ENTER_API(H5Iget_name, FAIL)
     H5TRACE3("Zs", "ixz", id, name, size);
 
+    /* Get object location */
+    if(H5G_loc(id, &loc) < 0)
+	HGOTO_ERROR(H5E_ATOM, H5E_CANTGET, FAIL, "can't retrieve object location")
+
     /* Call internal group routine to retrieve object's name */
-    if((ret_value = H5G_get_name(id, name, size, H5P_DEFAULT, H5AC_ind_dxpl_id)) < 0)
+    if((ret_value = H5G_get_name(&loc, name, size, NULL, H5P_DEFAULT, H5AC_ind_dxpl_id)) < 0)
 	HGOTO_ERROR(H5E_ATOM, H5E_CANTGET, FAIL, "can't retrieve object name")
 
 done:
@@ -2116,8 +2208,6 @@ done:
  * Programmer:  Raymond Lu
  *              Oct 27, 2003
  *
- * Modifications:
- *
  *-------------------------------------------------------------------------
  */
 hid_t
@@ -2143,7 +2233,6 @@ done:
  *              ID given an object ID.
  *
  * Return:	Success:	file ID
- *
  *		Failure:	a negative value
  *
  * Programmer:  Raymond Lu
@@ -2154,7 +2243,6 @@ done:
 hid_t
 H5I_get_file_id(hid_t obj_id, hbool_t app_ref)
 {
-    H5G_loc_t loc;              /* Location of object */
     H5I_type_t type;            /* ID type */
     hid_t ret_value;            /* Return value */
 
@@ -2163,18 +2251,24 @@ H5I_get_file_id(hid_t obj_id, hbool_t app_ref)
     /* Get object type */
     type = H5I_TYPE(obj_id);
     if(type == H5I_FILE) {
-        ret_value = obj_id;
-
-        /* Increment reference count on atom. */
-        if(H5I_inc_ref(ret_value, app_ref) < 0)
+        /* Increment reference count on file ID */
+        if(H5I_inc_ref(obj_id, app_ref) < 0)
             HGOTO_ERROR(H5E_ATOM, H5E_CANTSET, FAIL, "incrementing file ID failed")
-    }
+
+        /* Set return value */
+        ret_value = obj_id;
+    } /* end if */
     else if(type == H5I_DATATYPE || type == H5I_GROUP || type == H5I_DATASET || type == H5I_ATTR) {
+        H5G_loc_t loc;              /* Location of object */
+
+        /* Get the object location information */
         if(H5G_loc(obj_id, &loc) < 0)
             HGOTO_ERROR(H5E_ATOM, H5E_CANTGET, FAIL, "can't get object location")
+
+        /* Get the file ID for the object */
         if((ret_value = H5F_get_id(loc.oloc->file, app_ref)) < 0)
             HGOTO_ERROR(H5E_ATOM, H5E_CANTGET, FAIL, "can't get file ID")
-    }
+    } /* end if */
     else
         HGOTO_ERROR(H5E_ARGS, H5E_BADRANGE, FAIL, "invalid object ID")
 
@@ -2184,21 +2278,15 @@ done:
 
 
 /*-------------------------------------------------------------------------
- * Function: H5I_debug
+ * Function:	H5I_debug
  *
- * Purpose: Dump the contents of a type to stderr for debugging.
+ * Purpose:	Dump the contents of a type to stderr for debugging.
  *
- * Return: Success: Non-negative
+ * Return:	Success: Non-negative
+ *		Failure: Negative
  *
- *   Failure: Negative
- *
- * Programmer: Robb Matzke
- *  Friday, February 19, 1999
- *
- * Modifications:
- *
- *      Pedro Vicente, <pvn@ncsa.uiuc.edu> 22 Aug 2002
- *      Added `id to name' support.
+ * Programmer:	Robb Matzke
+ *		Friday, February 19, 1999
  *
  *-------------------------------------------------------------------------
  */
