@@ -20,12 +20,13 @@
 #define H5_INTERFACE_INIT_FUNC	H5L_init_extern_interface
 
 #include "H5private.h"          /* Generic Functions                    */
+#include "H5ACprivate.h"        /* Metadata cache                       */
 #include "H5Eprivate.h"         /* Error handling                       */
 #include "H5Gpkg.h"             /* Groups                               */
 #include "H5Iprivate.h"		/* IDs					*/
 #include "H5Lpkg.h"             /* Links                                */
 #include "H5MMprivate.h"        /* Memory management                    */
-#include "H5Opublic.h"         /* File objects                         */
+#include "H5Opublic.h"          /* File objects                         */
 #include "H5Pprivate.h"         /* Property lists                       */
 
 static hid_t H5L_extern_traverse(const char UNUSED *link_name, hid_t cur_group,
@@ -52,6 +53,9 @@ const H5L_class_t H5L_EXTERN_LINK_CLASS[1] = {{
 /* Valid flags for external links */
 #define H5L_EXT_FLAGS_ALL       0
 
+/* Size of local link name buffer for traversing external links */
+#define H5L_EXT_TRAVERSE_BUF_SIZE       256
+
 
 /*--------------------------------------------------------------------------
 NAME
@@ -142,10 +146,9 @@ H5L_build_name(char *prefix, char *file_name, char **full_name/*out*/)
 
     /* Copy the prefix into the buffer */
     HDstrcpy(*full_name, prefix);
-#ifndef H5_VMS
+
     if (!CHECK_DELIMITER(prefix[prefix_len-1]))
         HDstrcat(*full_name, DIR_SEPS);
-#endif
 
     /* Add the external link's filename to the prefix supplied */
     HDstrcat(*full_name, file_name);
@@ -184,6 +187,9 @@ done:
  *		Otherwise, the file access property retrieved from H5Pget_elink_fapl()
  *		is used to H5F_open() the target file.
  *
+ *              Vailin Choi; Nov 2010
+ *              Free memory pointed to by tmp_env_prefix for HDF5_EXT_PREFIX case.
+ *
  *-------------------------------------------------------------------------
  */
 static hid_t
@@ -202,18 +208,15 @@ H5L_extern_traverse(const char UNUSED *link_name, hid_t cur_group,
     size_t      fname_len;              /* Length of external link file name */
     unsigned    intent;                 /* File access permissions */
     H5L_elink_cb_t cb_info;             /* Callback info struct */
-    const char  *parent_file_name = NULL; /* Parent file name */
-    const char  *parent_group_name = NULL; /* Parent group name */
     hid_t       fapl_id = -1;           /* File access property list for external link's file */
     hid_t       ext_obj = -1;           /* ID for external link's object */
-    hid_t       ret_value;              /* Return value */
-
-    char        *tempname=NULL, *ptr=NULL, *extpath=NULL;
-    char        *env_prefix=NULL, *tmp_env_prefix=NULL;
-    char        *out_prefix_name=NULL, *pp=NULL;
-
-    H5P_genplist_t 	*fa_plist;      /* File access property list pointer */
+    char        *parent_group_name = NULL;/* Temporary pointer to group name */
+    char        local_group_name[H5L_EXT_TRAVERSE_BUF_SIZE];  /* Local buffer to hold group name */
+    char        *temp_file_name = NULL; /* Temporary pointer to file name */
+    char        *actual_file_name = NULL; /* Parent file's actual name */
+    H5P_genplist_t  *fa_plist;          /* File access property list pointer */
     H5F_close_degree_t 	fc_degree = H5F_CLOSE_WEAK;  /* File close degree for target file */
+    hid_t       ret_value;              /* Return value */
 
     FUNC_ENTER_NOAPI(H5L_extern_traverse, FAIL)
 
@@ -236,7 +239,7 @@ H5L_extern_traverse(const char UNUSED *link_name, hid_t cur_group,
     if(NULL == (plist = H5P_object_verify(lapl_id, H5P_LINK_ACCESS)))
         HGOTO_ERROR(H5E_ATOM, H5E_BADATOM, FAIL, "can't find object for ID")
 
-    /* get the fapl_id set for lapl_id if any */
+    /* Get the fapl_id set for lapl_id if any */
     if(H5P_get(plist, H5L_ACS_ELINK_FAPL_NAME, &fapl_id) < 0)
         HGOTO_ERROR(H5E_PLIST, H5E_CANTGET, FAIL, "can't get fapl for links")
 
@@ -266,40 +269,35 @@ H5L_extern_traverse(const char UNUSED *link_name, hid_t cur_group,
 
     /* Make callback if it exists */
     if(cb_info.func) {
+        const char  *parent_file_name;  /* Parent file name */
+        ssize_t group_name_len;         /* Length of parent group name */
+
         /* Get parent file name */
-        parent_file_name = H5F_NAME(loc.oloc->file);
+        parent_file_name = H5F_OPEN_NAME(loc.oloc->file);
+
+        /* Query length of parent group name */
+        if((group_name_len = H5G_get_name(&loc, NULL, (size_t) 0, NULL, lapl_id, H5AC_ind_dxpl_id)) < 0)
+            HGOTO_ERROR(H5E_LINK, H5E_CANTGET, FAIL, "unable to retrieve length of group name")
+
+        /* Account for null terminator */
+        group_name_len++;
+
+        /* Check if we need to allocate larger buffer */
+        if((size_t)group_name_len > sizeof(local_group_name)) {
+            if(NULL == (parent_group_name = (char *)H5MM_malloc((size_t)group_name_len)))
+                HGOTO_ERROR(H5E_LINK, H5E_CANTALLOC, FAIL, "can't allocate buffer to hold group name, group_name_len = %Zu", group_name_len)
+        } /* end if */
+        else
+            parent_group_name = local_group_name;
 
         /* Get parent group name */
-        if(loc.path->user_path_r != NULL && loc.path->obj_hidden == 0)
-            /* Use user_path_r if possible */
-            parent_group_name = H5RS_get_str(loc.path->user_path_r);
-        else {
-            /* Otherwise use H5G_get_name */
-            ssize_t group_name_len; /* Length of parent group name */
-
-            /* Get length of parent group name */
-            if((group_name_len = H5G_get_name(cur_group, NULL, (size_t) 0, lapl_id, H5AC_ind_dxpl_id)) < 0)
-                HGOTO_ERROR(H5E_LINK, H5E_CANTGET, FAIL, "unable to retrieve length of group name")
-
-            /* account for null terminator */
-            group_name_len++;
-
-            /* Copy parent group name */
-            if(NULL == (tempname = (char *) H5MM_malloc((size_t) group_name_len)))
-                HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, FAIL, "memory allocation failed")
-            if(H5G_get_name(cur_group, tempname, (size_t) group_name_len, lapl_id, H5AC_ind_dxpl_id) < 0)
-                HGOTO_ERROR(H5E_LINK, H5E_CANTGET, FAIL, "unable to retrieve group name")
-            parent_group_name = tempname;
-        } /* end else */
+        if(H5G_get_name(&loc, parent_group_name, (size_t) group_name_len, NULL, lapl_id, H5AC_ind_dxpl_id) < 0)
+            HGOTO_ERROR(H5E_LINK, H5E_CANTGET, FAIL, "unable to retrieve group name")
 
         /* Make callback */
-        if((cb_info.func)(parent_file_name, parent_group_name, file_name, obj_name,
-                &intent, fapl_id, cb_info.user_data) < 0)
+        if((cb_info.func)(parent_file_name, parent_group_name, file_name, obj_name, &intent, fapl_id, cb_info.user_data) < 0)
             HGOTO_ERROR(H5E_LINK, H5E_CALLBACK, FAIL, "traversal operator failed")
 
-        /* Free tempname */
-        tempname = (char *)H5MM_xfree(tempname);
-
         /* Check access flags */
         if((intent & H5F_ACC_TRUNC) || (intent & H5F_ACC_EXCL))
             HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, FAIL, "invalid file open flags")
@@ -312,126 +310,154 @@ H5L_extern_traverse(const char UNUSED *link_name, hid_t cur_group,
     /*
      * Start searching for the target file
      */
-    if ((tempname=H5MM_strdup(file_name)) == NULL)
+
+    /* Simplify intent flags for open calls */
+    intent = ((intent & H5F_ACC_RDWR) ? H5F_ACC_RDWR : H5F_ACC_RDONLY);
+
+    /* Copy the file name to use */
+    if(NULL == (temp_file_name = H5MM_strdup(file_name)))
         HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, FAIL, "memory allocation failed")
 
     /* target file_name is an absolute pathname: see RM for detailed description */
-    if (CHECK_ABSOLUTE(file_name) || CHECK_ABS_PATH(file_name)) {
-        if(NULL == (ext_file = H5F_open(file_name, ((intent & H5F_ACC_RDWR) ? H5F_ACC_RDWR : H5F_ACC_RDONLY),
-                                H5P_FILE_CREATE_DEFAULT, fapl_id, H5AC_dxpl_id))) {
+    if(CHECK_ABSOLUTE(file_name) || CHECK_ABS_PATH(file_name)) {
+        /* Try opening file */
+        if(NULL == (ext_file = H5F_efc_open(loc.oloc->file, file_name, intent, H5P_FILE_CREATE_DEFAULT, fapl_id, H5AC_dxpl_id))) {
+            char *ptr = NULL;
+
             H5E_clear_stack(NULL);
+
             /* get last component of file_name */
 	    GET_LAST_DELIMITER(file_name, ptr)
 	    HDassert(ptr);
-	    HDstrcpy(tempname, ++ptr);
-        }
-    } else if (CHECK_ABS_DRIVE(file_name)) {
-        if(NULL == (ext_file = H5F_open(file_name, ((intent & H5F_ACC_RDWR) ? H5F_ACC_RDWR : H5F_ACC_RDONLY),
-                                H5P_FILE_CREATE_DEFAULT, fapl_id, H5AC_dxpl_id))) {
+	    HDstrcpy(temp_file_name, ++ptr);
+        } /* end if */
+    } /* end if */
+    else if(CHECK_ABS_DRIVE(file_name)) {
+        if(NULL == (ext_file = H5F_efc_open(loc.oloc->file, file_name, intent, H5P_FILE_CREATE_DEFAULT, fapl_id, H5AC_dxpl_id))) {
             H5E_clear_stack(NULL);
 	    /* strip "<drive-letter>:" */
-	    HDstrcpy(tempname, &file_name[2]);
-	}
-    }
+	    HDstrcpy(temp_file_name, &file_name[2]);
+	} /* end if */
+    } /* end if */
 
     /* try searching from paths set in the environment variable */
-    if ((ext_file == NULL) && (env_prefix=HDgetenv("HDF5_EXT_PREFIX"))) {
+    if(ext_file == NULL) {
+        char *env_prefix;
 
-        if ((tmp_env_prefix = H5MM_strdup(env_prefix)) == NULL)
-            HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, FAIL, "memory allocation failed")
-	pp = tmp_env_prefix;
+        if(NULL != (env_prefix = HDgetenv("HDF5_EXT_PREFIX"))) {
+            char *tmp_env_prefix, *saved_env;
 
-        while ((tmp_env_prefix) && (*tmp_env_prefix)) {
-            out_prefix_name = H5L_getenv_prefix_name(&tmp_env_prefix/*in,out*/);
-            if ((out_prefix_name) && (*out_prefix_name)) {
+            if(NULL == (saved_env = tmp_env_prefix = H5MM_strdup(env_prefix)))
+                HGOTO_ERROR(H5E_LINK, H5E_NOSPACE, FAIL, "memory allocation failed")
 
-                if (H5L_build_name(out_prefix_name, tempname, &full_name/*out*/) < 0)
-                    HGOTO_ERROR(H5E_LINK, H5E_CANTGET, FAIL, "can't prepend prefix to filename")
+            while((tmp_env_prefix) && (*tmp_env_prefix)) {
+                char *out_prefix_name;
 
-                ext_file = H5F_open(full_name, ((intent & H5F_ACC_RDWR) ? H5F_ACC_RDWR : H5F_ACC_RDONLY),
-                            H5P_FILE_CREATE_DEFAULT, fapl_id, H5AC_dxpl_id);
-                if (full_name)
-                    H5MM_free(full_name);
-                if (ext_file != NULL)
-                    break;
-                H5E_clear_stack(NULL);
-            }
-        } /* end while */
-        pp = (char *)H5MM_xfree(pp);
-    }
+                out_prefix_name = H5L_getenv_prefix_name(&tmp_env_prefix/*in,out*/);
+                if(out_prefix_name && (*out_prefix_name)) {
+                    if(H5L_build_name(out_prefix_name, temp_file_name, &full_name/*out*/) < 0) {
+			saved_env = (char *)H5MM_xfree(saved_env);
+                        HGOTO_ERROR(H5E_LINK, H5E_CANTGET, FAIL, "can't prepend prefix to filename")
+		    } /* end if */
+
+                    ext_file = H5F_efc_open(loc.oloc->file, full_name, intent, H5P_FILE_CREATE_DEFAULT, fapl_id, H5AC_dxpl_id);
+                    full_name = (char *)H5MM_xfree(full_name);
+                    if(ext_file != NULL)
+                        break;
+                    H5E_clear_stack(NULL);
+                } /* end if */
+            } /* end while */
+	    saved_env = (char *)H5MM_xfree(saved_env);
+        } /* end if */
+    } /* end if */
 
     /* try searching from property list */
-    if (ext_file == NULL) {
+    if(ext_file == NULL) {
         if(H5P_get(plist, H5L_ACS_ELINK_PREFIX_NAME, &my_prefix) < 0)
             HGOTO_ERROR(H5E_PLIST, H5E_CANTGET, FAIL, "can't get external link prefix")
-        if (my_prefix) {
-            if (H5L_build_name(my_prefix, tempname, &full_name/*out*/) < 0)
+        if(my_prefix) {
+            if(H5L_build_name(my_prefix, temp_file_name, &full_name/*out*/) < 0)
                 HGOTO_ERROR(H5E_LINK, H5E_CANTGET, FAIL, "can't prepend prefix to filename")
-            if ((ext_file=H5F_open(full_name, ((intent & H5F_ACC_RDWR) ? H5F_ACC_RDWR : H5F_ACC_RDONLY),
-                        H5P_FILE_CREATE_DEFAULT, fapl_id, H5AC_dxpl_id)) == NULL)
+            if(NULL == (ext_file = H5F_efc_open(loc.oloc->file, full_name, intent, H5P_FILE_CREATE_DEFAULT, fapl_id, H5AC_dxpl_id)))
                 H5E_clear_stack(NULL);
-            if (full_name)
-                H5MM_free(full_name);
-        }
-    }
+            full_name = (char *)H5MM_xfree(full_name);
+        } /* end if */
+    } /* end if */
 
-    /* try searching from main file's "extpath":see description in H5F_open() & H5_build_extpath() */
-    if ((ext_file == NULL) && (extpath=H5F_EXTPATH(loc.oloc->file))) {
-        if (H5L_build_name(extpath, tempname, &full_name/*out*/) < 0)
-            HGOTO_ERROR(H5E_LINK, H5E_CANTGET, FAIL, "can't prepend prefix to filename")
-        if ((ext_file = H5F_open(full_name, ((intent & H5F_ACC_RDWR) ? H5F_ACC_RDWR : H5F_ACC_RDONLY),
-                    H5P_FILE_CREATE_DEFAULT, fapl_id, H5AC_dxpl_id)) == NULL)
+    /* try searching from main file's "extpath": see description in H5F_open() & H5_build_extpath() */
+    if(ext_file == NULL) {
+        char *extpath;
+
+        if(NULL != (extpath = H5F_EXTPATH(loc.oloc->file))) {
+            if(H5L_build_name(extpath, temp_file_name, &full_name/*out*/) < 0)
+                HGOTO_ERROR(H5E_LINK, H5E_CANTGET, FAIL, "can't prepend prefix to filename")
+            if(NULL == (ext_file = H5F_efc_open(loc.oloc->file, full_name, intent, H5P_FILE_CREATE_DEFAULT, fapl_id, H5AC_dxpl_id)))
+                H5E_clear_stack(NULL);
+            full_name = (char *)H5MM_xfree(full_name);
+        } /* end if */
+    } /* end if */
+
+    /* try the relative file_name stored in temp_file_name */
+    if(ext_file == NULL) {
+        if(NULL == (ext_file = H5F_efc_open(loc.oloc->file, temp_file_name, intent, H5P_FILE_CREATE_DEFAULT, fapl_id, H5AC_dxpl_id)))
             H5E_clear_stack(NULL);
-        if (full_name)
-            H5MM_free(full_name);
-    }
+    } /* end if */
 
-    /* try the relative file_name stored in tempname */
-    if (ext_file == NULL) {
-        if ((ext_file=H5F_open(tempname, ((intent & H5F_ACC_RDWR) ? H5F_ACC_RDWR : H5F_ACC_RDONLY),
-                        H5P_FILE_CREATE_DEFAULT, fapl_id, H5AC_dxpl_id)) == NULL)
-            HGOTO_ERROR(H5E_LINK, H5E_CANTOPENFILE, FAIL, "unable to open external file")
-    }
+    /* try the 'resolved' name for the parent file (i.e. the name after symlinks
+     * were resolved)
+     */
+    if(ext_file == NULL) {
+        char *ptr = NULL;
 
-    tempname = (char *)H5MM_xfree(tempname);
+        /* Copy resolved file name */
+        if(NULL == (actual_file_name = H5MM_strdup(H5F_ACTUAL_NAME(loc.oloc->file))))
+            HGOTO_ERROR(H5E_LINK, H5E_CANTALLOC, FAIL, "can't duplicate resolved file name string")
+
+        /* get last component of file_name */
+        GET_LAST_DELIMITER(actual_file_name, ptr)
+        if(!ptr)
+            HGOTO_ERROR(H5E_LINK, H5E_CANTOPENFILE, FAIL, "unable to open external file, external link file name = '%s', temp_file_name = '%s'", file_name, temp_file_name)
+
+        /* Truncate filename portion from actual file name path */
+        *ptr = '\0';
+
+        /* Build new file name for the external file */
+        if(H5L_build_name(actual_file_name, temp_file_name, &full_name/*out*/) < 0)
+            HGOTO_ERROR(H5E_LINK, H5E_CANTGET, FAIL, "can't prepend prefix to filename")
+
+        /* Try opening with the resolved name */
+        if(NULL == (ext_file = H5F_efc_open(loc.oloc->file, full_name, intent, H5P_FILE_CREATE_DEFAULT, fapl_id, H5AC_dxpl_id)))
+            HGOTO_ERROR(H5E_LINK, H5E_CANTOPENFILE, FAIL, "unable to open external file, external link file name = '%s', temp_file_name = '%s'", file_name, temp_file_name)
+        full_name = (char *)H5MM_xfree(full_name);
+    } /* end if */
 
-    /* Increment the number of open objects, to hold the file open */
-    H5F_incr_nopen_objs(ext_file);
 
     /* Retrieve the "group location" for the file's root group */
-    if(H5G_loc_root(ext_file, &root_loc) < 0)
+    if(H5G_root_loc(ext_file, &root_loc) < 0)
         HGOTO_ERROR(H5E_SYM, H5E_BADVALUE, FAIL, "unable to create location for file")
 
     /* Open the object referenced in the external file */
-    if((ext_obj = H5O_open_name(&root_loc, obj_name, lapl_id, FALSE)) < 0) {
-        H5F_decr_nopen_objs(ext_file);
+    if((ext_obj = H5O_open_name(&root_loc, obj_name, lapl_id, FALSE)) < 0)
         HGOTO_ERROR(H5E_SYM, H5E_CANTOPENOBJ, FAIL, "unable to open object")
-    } /* end if */
-
-    /* Decrement the number of open objects, to let the file close */
-    H5F_decr_nopen_objs(ext_file);
-
-    /* Close the external file */
-    if(H5F_try_close(ext_file) < 0)
-        HGOTO_ERROR(H5E_LINK, H5E_CANTCLOSEFILE, FAIL, "problem closing external file")
-    ext_file = NULL;
 
     /* Set return value */
     ret_value = ext_obj;
 
 done:
     /* Release resources */
-    if(fapl_id > 0 && H5I_dec_ref(fapl_id, FALSE) < 0)
+    if(fapl_id > 0 && H5I_dec_ref(fapl_id) < 0)
         HDONE_ERROR(H5E_ATOM, H5E_CANTRELEASE, FAIL, "unable to close atom for file access property list")
-    if(ext_file && H5F_try_close(ext_file) < 0)
+    if(ext_file && H5F_efc_close(loc.oloc->file, ext_file) < 0)
         HDONE_ERROR(H5E_LINK, H5E_CANTCLOSEFILE, FAIL, "problem closing external file")
+    if(parent_group_name && parent_group_name != local_group_name)
+        parent_group_name = (char *)H5MM_xfree(parent_group_name);
+    full_name = (char *)H5MM_xfree(full_name);
+    temp_file_name = (char *)H5MM_xfree(temp_file_name);
+    actual_file_name = (char *)H5MM_xfree(actual_file_name);
 
     if(ret_value < 0) {
-        H5MM_xfree(tempname);
-        H5MM_xfree(pp);
-
         /* Close object if it's open and something failed */
-        if(ext_obj >= 0 && H5I_dec_ref(ext_obj, FALSE) < 0)
+        if(ext_obj >= 0 && H5I_dec_ref(ext_obj) < 0)
             HDONE_ERROR(H5E_ATOM, H5E_CANTRELEASE, FAIL, "unable to close atom for external object")
     } /* end if */
 
@@ -482,7 +508,7 @@ H5L_extern_query(const char UNUSED * link_name, const void *_udata, size_t udata
     } /* end if */
 
     /* Set return value */
-    ret_value = udata_size;
+    ret_value = (ssize_t)udata_size;
 
 done:
     FUNC_LEAVE_NOAPI(ret_value)
@@ -514,6 +540,7 @@ H5Lcreate_external(const char *file_name, const char *obj_name,
     hid_t link_loc_id, const char *link_name, hid_t lcpl_id, hid_t lapl_id)
 {
     H5G_loc_t	link_loc;               /* Group location to create link */
+    char       *norm_obj_name = NULL;	/* Pointer to normalized current name */
     void       *ext_link_buf = NULL;    /* Buffer to contain external link */
     size_t      buf_size;               /* Size of buffer to hold external link */
     uint8_t    *p;                      /* Pointer into external link buffer */
@@ -533,8 +560,12 @@ H5Lcreate_external(const char *file_name, const char *obj_name,
     if(!link_name || !*link_name)
 	HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, FAIL, "no link name specified")
 
+    /* Get normalized copy of the link target */
+    if(NULL == (norm_obj_name = H5G_normalize(obj_name)))
+        HGOTO_ERROR(H5E_SYM, H5E_BADVALUE, FAIL, "can't normalize object name")
+
     /* Combine the filename and link name into a single buffer to give to the UD link */
-    buf_size = 1 + (HDstrlen(file_name) + 1) + (HDstrlen(obj_name) + 1);
+    buf_size = 1 + (HDstrlen(file_name) + 1) + (HDstrlen(norm_obj_name) + 1);
     if(NULL == (ext_link_buf = H5MM_malloc(buf_size)))
         HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, FAIL, "unable to allocate udata buffer")
 
@@ -543,15 +574,15 @@ H5Lcreate_external(const char *file_name, const char *obj_name,
     *p++ = (H5L_EXT_VERSION << 4) | H5L_EXT_FLAGS_ALL;  /* External link version & flags */
     HDstrcpy((char *)p, file_name);     /* Name of file containing external link's object */
     p += HDstrlen(file_name) + 1;
-    HDstrcpy((char *)p, obj_name);       /* External link's object */
+    HDstrcpy((char *)p, norm_obj_name);       /* External link's object */
 
     /* Create an external link */
     if(H5L_create_ud(&link_loc, link_name, ext_link_buf, buf_size, H5L_TYPE_EXTERNAL, lcpl_id, lapl_id, H5AC_dxpl_id) < 0)
         HGOTO_ERROR(H5E_LINK, H5E_CANTINIT, FAIL, "unable to create link")
 
 done:
-    if(ext_link_buf != NULL)
-        H5MM_free(ext_link_buf);
+    H5MM_xfree(ext_link_buf);
+    H5MM_xfree(norm_obj_name);
 
     FUNC_LEAVE_API(ret_value)
 } /* end H5Lcreate_external() */
