@@ -110,16 +110,15 @@ H5MM_calloc(size_t size)
  *		H5MM_realloc (NULL, 0)	  <==> NULL
  *
  * Return:	Success:	Ptr to new memory or NULL if the memory
- *				was freed.
+ *				was freed or HDrealloc couldn't allocate
+ *				memory.
  *
- *		Failure:	abort()
+ *		Failure:	NULL
  *
  * Programmer:	Robb Matzke
  *		matzke@llnl.gov
  *		Jul 10 1997
  *
- * Modifications:
- *
  *-------------------------------------------------------------------------
  */
 void *
@@ -128,44 +127,36 @@ H5MM_realloc(void *mem, size_t size)
     void *ret_value;
 
     /* Use FUNC_ENTER_NOAPI_NOINIT_NOFUNC here to avoid performance issues */
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5MM_realloc);
+    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5MM_realloc)
 
-    if (!mem) {
-	if (0 == size)
-            HGOTO_DONE(NULL);
-	mem = H5MM_malloc(size);
+    if(NULL == mem) {
+	if(0 == size)
+            ret_value = NULL;
+        else
+            ret_value = H5MM_malloc(size);
+    } /* end if */
+    else if(0 == size)
+	ret_value = H5MM_xfree(mem);
+    else
+	ret_value = HDrealloc(mem, size);
 
-    } else if (0 == size) {
-	mem = H5MM_xfree(mem);
-
-    } else {
-	mem = HDrealloc(mem, size);
-	assert(mem);
-    }
-
-    /* Set return value */
-    ret_value=mem;
-
-done:
-    FUNC_LEAVE_NOAPI(ret_value);
-}
+    FUNC_LEAVE_NOAPI(ret_value)
+} /* end H5MM_realloc() */
 
 
 /*-------------------------------------------------------------------------
- * Function:	H5MM_xstrdup
+ * Function:    H5MM_xstrdup
  *
- * Purpose:	Duplicates a string.  If the string to be duplicated is the
- *		null pointer, then return null.	 If the string to be duplicated
- *		is the empty string then return a new empty string.
+ * Purpose:     Duplicates a string, including memory allocation.
+ *              NULL is an acceptable value for the input string.
  *
- * Return:	Success:	Ptr to a new string (or null if no string).
+ * Return:      Success:    Pointer to a new string (NULL if s is NULL).
  *
- *		Failure:	abort()
- *
- * Programmer:	Robb Matzke
- *		matzke@llnl.gov
- *		Jul 10 1997
+ *              Failure:    abort()
  *
+ * Programmer:  Robb Matzke
+ *              matzke@llnl.gov
+ *              Jul 10 1997
  *-------------------------------------------------------------------------
  */
 char *
@@ -173,36 +164,35 @@ H5MM_xstrdup(const char *s)
 {
     char	*ret_value = NULL;
 
-    /* Use FUNC_ENTER_NOAPI_NOINIT_NOFUNC here to avoid performance issues */
-    FUNC_ENTER_NOAPI_NOINIT_NOFUNC(H5MM_xstrdup)
+    FUNC_ENTER_NOAPI(H5MM_xstrdup, NULL)
 
     if(s) {
-        ret_value = H5MM_malloc(HDstrlen(s) + 1);
-        HDassert(ret_value);
+        if(NULL == (ret_value = (char *)H5MM_malloc(HDstrlen(s) + 1)))
+            HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, NULL, "memory allocation failed")
         HDstrcpy(ret_value, s);
     } /* end if */
 
+done:
     FUNC_LEAVE_NOAPI(ret_value)
 } /* end H5MM_xstrdup() */
 
 
 /*-------------------------------------------------------------------------
- * Function:	H5MM_strdup
+ * Function:    H5MM_strdup
  *
- * Purpose:	Duplicates a string.  If the string to be duplicated is the
- *		null pointer, then return null.	 If the string to be duplicated
- *		is the empty string then return a new empty string.
+ * Purpose:     Duplicates a string, including memory allocation.
+ *              NULL is NOT an acceptable value for the input string.
  *
- * Return:	Success:	Ptr to a new string (or null if no string).
+ *              If the string to be duplicated is the NULL pointer, then
+ *              an error will be raised.
  *
- *		Failure:	abort()
+ * Return:      Success:    Pointer to a new string
  *
- * Programmer:	Robb Matzke
- *		matzke@llnl.gov
- *		Jul 10 1997
- *
- * Modifications:
+ *              Failure:    abort()
  *
+ * Programmer:  Robb Matzke
+ *              matzke@llnl.gov
+ *              Jul 10 1997
  *-------------------------------------------------------------------------
  */
 char *
@@ -210,17 +200,17 @@ H5MM_strdup(const char *s)
 {
     char	*ret_value;
 
-    FUNC_ENTER_NOAPI(H5MM_strdup, NULL);
+    FUNC_ENTER_NOAPI(H5MM_strdup, NULL)
 
-    if (!s)
-	HGOTO_ERROR (H5E_ARGS, H5E_BADVALUE, NULL, "null string");
-    if (NULL==(ret_value = H5MM_malloc(HDstrlen(s) + 1)))
-	HGOTO_ERROR (H5E_RESOURCE, H5E_NOSPACE, NULL, "memory allocation failed");
+    if(!s)
+	HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, NULL, "null string")
+    if(NULL == (ret_value = (char *)H5MM_malloc(HDstrlen(s) + 1)))
+	HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, NULL, "memory allocation failed")
     HDstrcpy(ret_value, s);
 
 done:
-    FUNC_LEAVE_NOAPI(ret_value);
-}
+    FUNC_LEAVE_NOAPI(ret_value)
+} /* end H5MM_strdup() */
 
 
 /*-------------------------------------------------------------------------
