@@ -35,7 +35,8 @@
 
 
 /* PRIVATE PROTOTYPES */
-static void *H5O_ainfo_decode(H5F_t *f, hid_t dxpl_id, unsigned mesg_flags, const uint8_t *p);
+static void *H5O_ainfo_decode(H5F_t *f, hid_t dxpl_id, H5O_t *open_oh,
+    unsigned mesg_flags, unsigned *ioflags, const uint8_t *p);
 static herr_t H5O_ainfo_encode(H5F_t *f, hbool_t disable_shared, uint8_t *p, const void *_mesg);
 static void *H5O_ainfo_copy(const void *_mesg, void *_dest);
 static size_t H5O_ainfo_size(const H5F_t *f, hbool_t disable_shared, const void *_mesg);
@@ -47,6 +48,9 @@ static herr_t H5O_ainfo_pre_copy_file(H5F_t *file_src, const void *mesg_src,
 static void *H5O_ainfo_copy_file(H5F_t *file_src, void *mesg_src,
     H5F_t *file_dst, hbool_t *recompute_size, H5O_copy_t *cpy_info, void *udata,
     hid_t dxpl_id);
+static herr_t H5O_ainfo_post_copy_file(const H5O_loc_t *src_oloc,
+    const void *mesg_src, H5O_loc_t *dst_oloc, void *mesg_dst, hid_t dxpl_id,
+    H5O_copy_t *cpy_info);
 static herr_t H5O_ainfo_debug(H5F_t *f, hid_t dxpl_id, const void *_mesg,
 			     FILE * stream, int indent, int fwidth);
 
@@ -68,7 +72,7 @@ const H5O_msg_class_t H5O_MSG_AINFO[1] = {{
     NULL,		    	/*can share method		*/
     H5O_ainfo_pre_copy_file,	/* pre copy native value to file */
     H5O_ainfo_copy_file,	/* copy native value to file    */
-    NULL,			/* post copy native value to file */
+    H5O_ainfo_post_copy_file,   /* post copy native value to file */
     NULL,			/* get creation index		*/
     NULL,			/* set creation index		*/
     H5O_ainfo_debug          	/*debug the message             */
@@ -101,8 +105,8 @@ H5FL_DEFINE_STATIC(H5O_ainfo_t);
  *-------------------------------------------------------------------------
  */
 static void *
-H5O_ainfo_decode(H5F_t *f, hid_t UNUSED dxpl_id, unsigned UNUSED mesg_flags,
-    const uint8_t *p)
+H5O_ainfo_decode(H5F_t *f, hid_t UNUSED dxpl_id, H5O_t UNUSED *open_oh,
+    unsigned UNUSED mesg_flags, unsigned UNUSED *ioflags, const uint8_t *p)
 {
     H5O_ainfo_t	*ainfo = NULL;  /* Attribute info */
     unsigned char flags;        /* Flags for encoding attribute info */
@@ -155,7 +159,7 @@ H5O_ainfo_decode(H5F_t *f, hid_t UNUSED dxpl_id, unsigned UNUSED mesg_flags,
 
 done:
     if(ret_value == NULL && ainfo != NULL)
-        H5FL_FREE(H5O_ainfo_t, ainfo);
+        ainfo = H5FL_FREE(H5O_ainfo_t, ainfo);
 
     FUNC_LEAVE_NOAPI(ret_value)
 } /* end H5O_ainfo_decode() */
@@ -310,7 +314,7 @@ H5O_ainfo_free(void *mesg)
 
     HDassert(mesg);
 
-    H5FL_FREE(H5O_ainfo_t, mesg);
+    mesg = H5FL_FREE(H5O_ainfo_t, mesg);
 
     FUNC_LEAVE_NOAPI(SUCCEED)
 } /* end H5O_ainfo_free() */
@@ -392,15 +396,14 @@ H5O_ainfo_pre_copy_file(H5F_t UNUSED *file_src, const void UNUSED *native_src,
  * Return:      Success:        Ptr to _DEST
  *              Failure:        NULL
  *
- * Programmer:  Quincey Koziol
- *              March 9, 2007
+ * Programmer:  Peter Cao
+ *              July 18, 2007
  *
  *-------------------------------------------------------------------------
  */
 static void *
-H5O_ainfo_copy_file(H5F_t UNUSED *file_src, void *mesg_src,
-    H5F_t UNUSED *file_dst, hbool_t UNUSED *recompute_size,
-    H5O_copy_t UNUSED *cpy_info, void UNUSED *udata, hid_t UNUSED dxpl_id)
+H5O_ainfo_copy_file(H5F_t *file_src, void *mesg_src, H5F_t *file_dst,
+    hbool_t UNUSED *recompute_size, H5O_copy_t *cpy_info, void UNUSED *udata, hid_t dxpl_id)
 {
     H5O_ainfo_t *ainfo_src = (H5O_ainfo_t *)mesg_src;
     H5O_ainfo_t *ainfo_dst = NULL;
@@ -415,10 +418,6 @@ H5O_ainfo_copy_file(H5F_t UNUSED *file_src, void *mesg_src,
     HDassert(cpy_info);
     HDassert(!cpy_info->copy_without_attr);
 
-/* XXX: Bail out for now, if the source object has densely stored attributes */
-    if(H5F_addr_defined(ainfo_src->fheap_addr))
-        HGOTO_ERROR(H5E_OHDR, H5E_UNSUPPORTED, NULL, "densely stored attributes not supported yet")
-
     /* Allocate space for the destination message */
     if(NULL == (ainfo_dst = H5FL_MALLOC(H5O_ainfo_t)))
         HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, NULL, "memory allocation failed")
@@ -426,17 +425,62 @@ H5O_ainfo_copy_file(H5F_t UNUSED *file_src, void *mesg_src,
     /* Copy the top level of the information */
     *ainfo_dst = *ainfo_src;
 
+    if(H5F_addr_defined(ainfo_src->fheap_addr)) {
+        /* copy dense attribute */
+
+        if(H5A_dense_create(file_dst, dxpl_id, ainfo_dst) < 0)
+            HGOTO_ERROR(H5E_OHDR, H5E_CANTINIT, NULL, "unable to create dense storage for attributes")
+    } /* end if */
+
     /* Set return value */
     ret_value = ainfo_dst;
 
 done:
     /* Release destination attribute information on failure */
-    if(ret_value == NULL && ainfo_dst != NULL)
-        H5FL_FREE(H5O_ainfo_t, ainfo_dst);
+    if(!ret_value && ainfo_dst)
+        ainfo_dst = H5FL_FREE(H5O_ainfo_t, ainfo_dst);
 
     FUNC_LEAVE_NOAPI(ret_value)
 } /* H5O_ainfo_copy_file() */
 
+
+/*-------------------------------------------------------------------------
+ * Function:    H5O_ainfo_post_copy_file
+ *
+ * Purpose:     Finish copying a message from between files.
+ *              We have to copy the values of a reference attribute in the
+ *              post copy because H5O_post_copy_file() fails in the case that
+ *              an object may have a reference attribute that points to the
+ *              object itself.
+ *
+ * Return:      Non-negative on success/Negative on failure
+ *
+ * Programmer:  Peter Cao
+ *              July 25, 2007
+ *
+ *-------------------------------------------------------------------------
+ */
+static herr_t
+H5O_ainfo_post_copy_file(const H5O_loc_t *src_oloc, const void *mesg_src,
+    H5O_loc_t *dst_oloc, void *mesg_dst, hid_t dxpl_id, H5O_copy_t *cpy_info)
+{
+    const H5O_ainfo_t *ainfo_src = (const H5O_ainfo_t *)mesg_src;
+    herr_t ret_value = SUCCEED;   /* Return value */
+
+    FUNC_ENTER_NOAPI_NOINIT(H5O_ainfo_post_copy_file)
+
+    HDassert(ainfo_src);
+
+    if(H5F_addr_defined(ainfo_src->fheap_addr)) {
+        if(H5A_dense_post_copy_file_all(src_oloc, ainfo_src, dst_oloc,
+                (H5O_ainfo_t *)mesg_dst, dxpl_id, cpy_info) < 0)
+        HGOTO_ERROR(H5E_ATTR, H5E_CANTCOPY, FAIL, "can't copy attribute")
+    } /* end if */
+
+done:
+    FUNC_LEAVE_NOAPI(ret_value)
+} /* H5O_ainfo_post_copy_file() */
+
 
 /*-------------------------------------------------------------------------
  * Function:    H5O_ainfo_debug
