@@ -17,6 +17,7 @@
 #define H5T_PACKAGE		/*prevent warning from including H5Tpkg   */
 
 #include "H5private.h"		/* Generic Functions			*/
+#include "H5Dprivate.h"		/* Datasets				*/
 #include "H5Eprivate.h"		/* Error handling		  	*/
 #include "H5Fprivate.h"		/* Files				*/
 #include "H5FLprivate.h"	/* Free Lists				*/
@@ -103,7 +104,7 @@ const H5O_msg_class_t H5O_MSG_DTYPE[1] = {{
     H5O_dtype_can_share,	/* can share method		*/
     H5O_dtype_pre_copy_file,	/* pre copy native value to file */
     H5O_dtype_shared_copy_file,	/* copy native value to file    */
-    NULL,			/* post copy native value to file */
+    H5O_dtype_shared_post_copy_file,	/* post copy native value to file */
     NULL,			/* get creation index		*/
     NULL,			/* set creation index		*/
     H5O_dtype_shared_debug	/* debug the message		*/
@@ -267,12 +268,13 @@ H5O_dtype_decode_helper(H5F_t *f, unsigned *ioflags/*in,out*/, const uint8_t **p
                  * Compound datatypes...
                  */
                 dt->shared->u.compnd.nmembs = flags & 0xffff;
-                HDassert(dt->shared->u.compnd.nmembs > 0);
-                dt->shared->u.compnd.packed = TRUE; /* Start off packed */
+                if(dt->shared->u.compnd.nmembs == 0)
+                    HGOTO_ERROR(H5E_DATATYPE, H5E_BADVALUE, FAIL, "invalid number of members: %u", dt->shared->u.compnd.nmembs)
                 dt->shared->u.compnd.nalloc = dt->shared->u.compnd.nmembs;
                 dt->shared->u.compnd.memb = (H5T_cmemb_t *)H5MM_calloc(dt->shared->u.compnd.nalloc * sizeof(H5T_cmemb_t));
+                dt->shared->u.compnd.memb_size = 0;
                 if(NULL == dt->shared->u.compnd.memb)
-                    HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, FAIL, "memory allocation failed")
+                    HGOTO_ERROR(H5E_DATATYPE, H5E_CANTALLOC, FAIL, "memory allocation failed")
                 for(i = 0; i < dt->shared->u.compnd.nmembs; i++) {
                     unsigned ndims = 0;     /* Number of dimensions of the array field */
                     htri_t can_upgrade;     /* Whether we can upgrade this type's version */
@@ -384,6 +386,7 @@ H5O_dtype_decode_helper(H5F_t *f, unsigned *ioflags/*in,out*/, const uint8_t **p
 
                     /* Member size */
                     dt->shared->u.compnd.memb[i].size = temp_type->shared->size;
+                    dt->shared->u.compnd.memb_size += temp_type->shared->size;
 
                     /* Set the field datatype (finally :-) */
                     dt->shared->u.compnd.memb[i].type = temp_type;
@@ -399,31 +402,11 @@ H5O_dtype_decode_helper(H5F_t *f, unsigned *ioflags/*in,out*/, const uint8_t **p
 
                     /* Update the maximum member position covered */
                     max_memb_pos = MAX(max_memb_pos, (dt->shared->u.compnd.memb[i].offset + dt->shared->u.compnd.memb[i].size));
-
-                    /* Check if the datatype stayed packed */
-                    if(dt->shared->u.compnd.packed) {
-                        /* Check if the member type is packed */
-                        if(H5T_is_packed(temp_type) > 0) {
-                            if(i == 0) {
-                                /* If the is the first member, the datatype is not packed
-                                 * if the first member isn't at offset 0
-                                 */
-                                if(dt->shared->u.compnd.memb[i].offset > 0)
-                                    dt->shared->u.compnd.packed = FALSE;
-                            } /* end if */
-                            else {
-                                /* If the is not the first member, the datatype is not
-                                 * packed if the new member isn't adjoining the previous member
-                                 */
-                                if(dt->shared->u.compnd.memb[i].offset != (dt->shared->u.compnd.memb[i - 1].offset + dt->shared->u.compnd.memb[i - 1].size))
-                                    dt->shared->u.compnd.packed = FALSE;
-                            } /* end else */
-                        } /* end if */
-                        else
-                            dt->shared->u.compnd.packed = FALSE;
-                    } /* end if */
                 } /* end for */
 
+                /* Check if the compound type is packed */
+                H5T_update_packed(dt);
+
                 /* Upgrade the compound if requested */
                 if(version < upgrade_to) {
                     version = upgrade_to;
@@ -452,8 +435,9 @@ H5O_dtype_decode_helper(H5F_t *f, unsigned *ioflags/*in,out*/, const uint8_t **p
 
             /* Set extra information for object references, so the hobj_ref_t gets swizzled correctly */
             if(dt->shared->u.atomic.u.r.rtype == H5R_OBJECT) {
-                /* This type is on disk */
-                dt->shared->u.atomic.u.r.loc = H5T_LOC_DISK;
+                /* Mark location this type as undefined for now.  The caller function should
+                 * decide the location. */
+                dt->shared->u.atomic.u.r.loc = H5T_LOC_BADLOC;
 
                 /* This type needs conversion */
                 dt->shared->force_conv = TRUE;
@@ -518,8 +502,10 @@ H5O_dtype_decode_helper(H5F_t *f, unsigned *ioflags/*in,out*/, const uint8_t **p
                 ioflags, "vlen", FAIL)
 
             dt->shared->force_conv=TRUE;
-            /* Mark this type as on disk */
-            if(H5T_set_loc(dt, f, H5T_LOC_DISK) < 0)
+
+            /* Mark location this type as undefined for now.  The caller function should
+             * decide the location. */
+            if(H5T_set_loc(dt, f, H5T_LOC_BADLOC) < 0)
                 HGOTO_ERROR(H5E_DATATYPE, H5E_CANTINIT, FAIL, "invalid datatype location")
             break;
 
@@ -567,6 +553,8 @@ H5O_dtype_decode_helper(H5F_t *f, unsigned *ioflags/*in,out*/, const uint8_t **p
                 dt->shared->force_conv = TRUE;
             break;
 
+        case H5T_NO_CLASS:
+        case H5T_NCLASSES:
         default:
             HGOTO_ERROR(H5E_DATATYPE, H5E_UNSUPPORTED, FAIL, "unknown datatype class found")
     } /* end switch */
@@ -575,8 +563,8 @@ done:
     if(ret_value < 0) {
         if(dt != NULL) {
             if(dt->shared != NULL)
-                H5FL_FREE(H5T_shared_t, dt->shared);
-            H5FL_FREE(H5T_t, dt);
+                dt->shared = H5FL_FREE(H5T_shared_t, dt->shared);
+            dt = H5FL_FREE(H5T_t, dt);
         } /* end if */
     } /* end if */
 
@@ -604,7 +592,7 @@ static herr_t
 H5O_dtype_encode_helper(const H5F_t *f, uint8_t **pp, const H5T_t *dt)
 {
     unsigned	flags = 0;
-    char	*hdr = (char *)*pp;
+    uint8_t	*hdr = (uint8_t *)*pp;
     unsigned	i;
     size_t	n, z;
     herr_t      ret_value = SUCCEED;    /* Return value */
@@ -627,9 +615,14 @@ H5O_dtype_encode_helper(const H5F_t *f, uint8_t **pp, const H5T_t *dt)
             switch (dt->shared->u.atomic.order) {
                 case H5T_ORDER_LE:
                     break;		/*nothing */
+
                 case H5T_ORDER_BE:
                     flags |= 0x01;
                     break;
+
+                case H5T_ORDER_ERROR:
+                case H5T_ORDER_VAX:
+                case H5T_ORDER_NONE:
                 default:
                     HGOTO_ERROR(H5E_DATATYPE, H5E_UNSUPPORTED, FAIL, "byte order is not supported in file format yet")
             } /* end switch */
@@ -637,9 +630,14 @@ H5O_dtype_encode_helper(const H5F_t *f, uint8_t **pp, const H5T_t *dt)
             switch (dt->shared->u.atomic.lsb_pad) {
                 case H5T_PAD_ZERO:
                     break;		/*nothing */
+
                 case H5T_PAD_ONE:
                     flags |= 0x02;
                     break;
+
+                case H5T_PAD_ERROR:
+                case H5T_PAD_BACKGROUND:
+                case H5T_NPAD:
                 default:
                     HGOTO_ERROR(H5E_DATATYPE, H5E_UNSUPPORTED, FAIL, "bit padding is not supported in file format yet")
             } /* end switch */
@@ -647,9 +645,14 @@ H5O_dtype_encode_helper(const H5F_t *f, uint8_t **pp, const H5T_t *dt)
             switch (dt->shared->u.atomic.msb_pad) {
                 case H5T_PAD_ZERO:
                     break;		/*nothing */
+
+                case H5T_PAD_ERROR:
+                case H5T_PAD_BACKGROUND:
+                case H5T_NPAD:
                 case H5T_PAD_ONE:
                     flags |= 0x04;
                     break;
+
                 default:
                     HGOTO_ERROR(H5E_DATATYPE, H5E_UNSUPPORTED, FAIL, "bit padding is not supported in file format yet")
             } /* end switch */
@@ -657,9 +660,13 @@ H5O_dtype_encode_helper(const H5F_t *f, uint8_t **pp, const H5T_t *dt)
             switch (dt->shared->u.atomic.u.i.sign) {
                 case H5T_SGN_NONE:
                     break;		/*nothing */
+
                 case H5T_SGN_2:
                     flags |= 0x08;
                     break;
+
+                case H5T_SGN_ERROR:
+                case H5T_NSGN:
                 default:
                     HGOTO_ERROR(H5E_DATATYPE, H5E_UNSUPPORTED, FAIL, "sign scheme is not supported in file format yet")
             } /* end switch */
@@ -675,13 +682,18 @@ H5O_dtype_encode_helper(const H5F_t *f, uint8_t **pp, const H5T_t *dt)
             switch (dt->shared->u.atomic.order) {
                 case H5T_ORDER_LE:
                     break;		/*nothing*/
+
                 case H5T_ORDER_BE:
                     flags |= 0x01;
                     break;
+
                 case H5T_ORDER_VAX:     /*turn on 1st and 6th (reserved before adding VAX) bits*/
                     flags |= 0x41;
                     HDassert(dt->shared->version >= H5O_DTYPE_VERSION_3);
                     break;
+
+                case H5T_ORDER_ERROR:
+                case H5T_ORDER_NONE:
                 default:
                     HGOTO_ERROR(H5E_DATATYPE, H5E_UNSUPPORTED, FAIL, "byte order is not supported in file format yet")
             } /* end switch */
@@ -689,9 +701,14 @@ H5O_dtype_encode_helper(const H5F_t *f, uint8_t **pp, const H5T_t *dt)
             switch (dt->shared->u.atomic.lsb_pad) {
                 case H5T_PAD_ZERO:
                     break;		/*nothing */
+
                 case H5T_PAD_ONE:
                     flags |= 0x02;
                     break;
+
+                case H5T_PAD_ERROR:
+                case H5T_PAD_BACKGROUND:
+                case H5T_NPAD:
                 default:
                     HGOTO_ERROR(H5E_DATATYPE, H5E_UNSUPPORTED, FAIL, "bit padding is not supported in file format yet")
             } /* end switch */
@@ -699,9 +716,14 @@ H5O_dtype_encode_helper(const H5F_t *f, uint8_t **pp, const H5T_t *dt)
             switch (dt->shared->u.atomic.msb_pad) {
                 case H5T_PAD_ZERO:
                     break;		/*nothing */
+
                 case H5T_PAD_ONE:
                     flags |= 0x04;
                     break;
+
+                case H5T_PAD_ERROR:
+                case H5T_PAD_BACKGROUND:
+                case H5T_NPAD:
                 default:
                     HGOTO_ERROR(H5E_DATATYPE, H5E_UNSUPPORTED, FAIL, "bit padding is not supported in file format yet")
             } /* end switch */
@@ -709,9 +731,14 @@ H5O_dtype_encode_helper(const H5F_t *f, uint8_t **pp, const H5T_t *dt)
             switch (dt->shared->u.atomic.u.f.pad) {
                 case H5T_PAD_ZERO:
                     break;		/*nothing */
+
                 case H5T_PAD_ONE:
                     flags |= 0x08;
                     break;
+
+                case H5T_PAD_ERROR:
+                case H5T_PAD_BACKGROUND:
+                case H5T_NPAD:
                 default:
                     HGOTO_ERROR(H5E_DATATYPE, H5E_UNSUPPORTED, FAIL, "bit padding is not supported in file format yet")
             } /* end switch */
@@ -719,12 +746,16 @@ H5O_dtype_encode_helper(const H5F_t *f, uint8_t **pp, const H5T_t *dt)
             switch (dt->shared->u.atomic.u.f.norm) {
                 case H5T_NORM_NONE:
                     break;		/*nothing */
+
                 case H5T_NORM_MSBSET:
                     flags |= 0x10;
                     break;
+
                 case H5T_NORM_IMPLIED:
                     flags |= 0x20;
                     break;
+
+                case H5T_NORM_ERROR:
                 default:
                     HGOTO_ERROR(H5E_DATATYPE, H5E_UNSUPPORTED, FAIL, "normalization scheme is not supported in file format yet")
             } /* end switch */
@@ -747,9 +778,14 @@ H5O_dtype_encode_helper(const H5F_t *f, uint8_t **pp, const H5T_t *dt)
             switch (dt->shared->u.atomic.order) {
                 case H5T_ORDER_LE:
                     break;		/*nothing */
+
                 case H5T_ORDER_BE:
                     flags |= 0x01;
                     break;
+
+                case H5T_ORDER_ERROR:
+                case H5T_ORDER_VAX:
+                case H5T_ORDER_NONE:
                 default:
                     HGOTO_ERROR(H5E_DATATYPE, H5E_UNSUPPORTED, FAIL, "byte order is not supported in file format yet")
             } /* end switch */
@@ -777,9 +813,14 @@ H5O_dtype_encode_helper(const H5F_t *f, uint8_t **pp, const H5T_t *dt)
             switch (dt->shared->u.atomic.order) {
                 case H5T_ORDER_LE:
                     break;		/*nothing */
+
                 case H5T_ORDER_BE:
                     flags |= 0x01;
                     break;
+
+                case H5T_ORDER_ERROR:
+                case H5T_ORDER_VAX:
+                case H5T_ORDER_NONE:
                 default:
                     HGOTO_ERROR(H5E_DATATYPE, H5E_UNSUPPORTED, FAIL, "byte order is not supported in file format yet")
             } /* end switch */
@@ -787,9 +828,14 @@ H5O_dtype_encode_helper(const H5F_t *f, uint8_t **pp, const H5T_t *dt)
             switch (dt->shared->u.atomic.lsb_pad) {
                 case H5T_PAD_ZERO:
                     break;		/*nothing */
+
                 case H5T_PAD_ONE:
                     flags |= 0x02;
                     break;
+
+                case H5T_PAD_ERROR:
+                case H5T_PAD_BACKGROUND:
+                case H5T_NPAD:
                 default:
                     HGOTO_ERROR(H5E_DATATYPE, H5E_UNSUPPORTED, FAIL, "bit padding is not supported in file format yet")
             } /* end switch */
@@ -797,9 +843,14 @@ H5O_dtype_encode_helper(const H5F_t *f, uint8_t **pp, const H5T_t *dt)
             switch (dt->shared->u.atomic.msb_pad) {
                 case H5T_PAD_ZERO:
                     break;		/*nothing */
+
                 case H5T_PAD_ONE:
                     flags |= 0x04;
                     break;
+
+                case H5T_PAD_ERROR:
+                case H5T_PAD_BACKGROUND:
+                case H5T_NPAD:
                 default:
                     HGOTO_ERROR(H5E_DATATYPE, H5E_UNSUPPORTED, FAIL, "bit padding is not supported in file format yet")
             } /* end switch */
@@ -946,7 +997,7 @@ H5O_dtype_encode_helper(const H5F_t *f, uint8_t **pp, const H5T_t *dt)
             flags |= (dt->shared->u.vlen.type & 0x0f);
             if(dt->shared->u.vlen.type == H5T_VLEN_STRING) {
                 flags |= (dt->shared->u.vlen.pad   & 0x0f) << 4;
-                flags |= (dt->shared->u.vlen.cset  & 0x0f) << 8;
+                flags |= ((unsigned)dt->shared->u.vlen.cset  & 0x0f) << 8;
             } /* end if */
 
             /* Encode base type of VL information */
@@ -965,7 +1016,8 @@ H5O_dtype_encode_helper(const H5F_t *f, uint8_t **pp, const H5T_t *dt)
             HDassert(dt->shared->version >= dt->shared->parent->shared->version);
 
             /* Encode the number of dimensions */
-            *(*pp)++ = dt->shared->u.array.ndims;
+            HDassert(dt->shared->u.array.ndims <= UCHAR_MAX);
+            *(*pp)++ = (uint8_t)dt->shared->u.array.ndims;
 
             /* Drop this information for Version 3 of the format */
             if(dt->shared->version < H5O_DTYPE_VERSION_3) {
@@ -991,16 +1043,18 @@ H5O_dtype_encode_helper(const H5F_t *f, uint8_t **pp, const H5T_t *dt)
                 HGOTO_ERROR(H5E_DATATYPE, H5E_CANTENCODE, FAIL, "unable to encode VL parent type")
             break;
 
+        case H5T_NO_CLASS:
+        case H5T_NCLASSES:
         default:
             /*nothing */
             break;
     } /* end switch */
 
     /* Encode the type's class, version and bit field */
-    *hdr++ = ((unsigned)(dt->shared->type) & 0x0f) | (dt->shared->version << 4);
-    *hdr++ = (flags >> 0) & 0xff;
-    *hdr++ = (flags >> 8) & 0xff;
-    *hdr++ = (flags >> 16) & 0xff;
+    *hdr++ = (uint8_t)(((unsigned)(dt->shared->type) & 0x0f) | (dt->shared->version << 4));
+    *hdr++ = (uint8_t)((flags >> 0) & 0xff);
+    *hdr++ = (uint8_t)((flags >> 8) & 0xff);
+    *hdr++ = (uint8_t)((flags >> 16) & 0xff);
 
 done:
     FUNC_LEAVE_NOAPI(ret_value)
@@ -1129,7 +1183,7 @@ H5O_dtype_copy(const void *_src, void *_dst)
     /* Was result already allocated? */
     if(_dst) {
         *((H5T_t *) _dst) = *dst;
-        (void)H5FL_FREE(H5T_t, dst);
+        dst = H5FL_FREE(H5T_t, dst);
         dst = (H5T_t *) _dst;
     } /* end if */
 
@@ -1219,7 +1273,7 @@ H5O_dtype_size(const H5F_t *f, const void *_mesg)
                     /* (starting with version 3 of the datatype message, use the minimum # of bytes required) */
                     if(dt->shared->version >= H5O_DTYPE_VERSION_3)
                         ret_value += offset_nbytes; 	/*member offset*/
-                    if(dt->shared->version >= H5O_DTYPE_VERSION_2)
+                    else if(dt->shared->version == H5O_DTYPE_VERSION_2)
                         ret_value += 4; 	/*member offset*/
                     else
                         ret_value += 4 +	/*member offset*/
@@ -1264,6 +1318,10 @@ H5O_dtype_size(const H5F_t *f, const void *_mesg)
             ret_value += H5O_dtype_size(f, dt->shared->parent);
             break;
 
+        case H5T_NO_CLASS:
+        case H5T_STRING:
+        case H5T_REFERENCE:
+        case H5T_NCLASSES:
         default:
             /*no properties */
             break;
@@ -1323,8 +1381,8 @@ H5O_dtype_free(void *mesg)
 
     HDassert(mesg);
 
-    (void)H5FL_FREE(H5T_shared_t, ((H5T_t *) mesg)->shared);
-    (void)H5FL_FREE(H5T_t, mesg);
+    ((H5T_t *) mesg)->shared = H5FL_FREE(H5T_shared_t, ((H5T_t *) mesg)->shared);
+    mesg = H5FL_FREE(H5T_t, mesg);
 
     FUNC_LEAVE_NOAPI(SUCCEED)
 } /* end H5O_dtype_free() */
@@ -1600,6 +1658,8 @@ H5O_dtype_debug(H5F_t *f, hid_t dxpl_id, const void *mesg, FILE *stream,
             s = "vlen";
             break;
 
+        case H5T_NO_CLASS:
+        case H5T_NCLASSES:
         default:
             sprintf(buf, "H5T_CLASS_%d", (int)(dt->shared->type));
             s = buf;
@@ -1661,13 +1721,34 @@ H5O_dtype_debug(H5F_t *f, hid_t dxpl_id, const void *mesg, FILE *stream,
             case H5T_CSET_ASCII:
                 s = "ASCII";
                 break;
+
             case H5T_CSET_UTF8:
                 s = "UTF-8";
                 break;
-            default:
+
+            case H5T_CSET_RESERVED_2:
+            case H5T_CSET_RESERVED_3:
+            case H5T_CSET_RESERVED_4:
+            case H5T_CSET_RESERVED_5:
+            case H5T_CSET_RESERVED_6:
+            case H5T_CSET_RESERVED_7:
+            case H5T_CSET_RESERVED_8:
+            case H5T_CSET_RESERVED_9:
+            case H5T_CSET_RESERVED_10:
+            case H5T_CSET_RESERVED_11:
+            case H5T_CSET_RESERVED_12:
+            case H5T_CSET_RESERVED_13:
+            case H5T_CSET_RESERVED_14:
+            case H5T_CSET_RESERVED_15:
                 sprintf(buf, "H5T_CSET_RESERVED_%d", (int)(dt->shared->u.atomic.u.s.cset));
                 s = buf;
                 break;
+
+            case H5T_CSET_ERROR:
+            default:
+                sprintf(buf, "Unknown character set: %d", (int)(dt->shared->u.atomic.u.s.cset));
+                s = buf;
+                break;
         } /* end switch */
         fprintf(stream, "%*s%-*s %s\n", indent, "", fwidth,
                 "Character Set:",
@@ -1677,16 +1758,37 @@ H5O_dtype_debug(H5F_t *f, hid_t dxpl_id, const void *mesg, FILE *stream,
             case H5T_STR_NULLTERM:
                 s = "NULL Terminated";
                 break;
+
             case H5T_STR_NULLPAD:
                 s = "NULL Padded";
                 break;
+
             case H5T_STR_SPACEPAD:
                 s = "Space Padded";
                 break;
-            default:
+
+            case H5T_STR_RESERVED_3:
+            case H5T_STR_RESERVED_4:
+            case H5T_STR_RESERVED_5:
+            case H5T_STR_RESERVED_6:
+            case H5T_STR_RESERVED_7:
+            case H5T_STR_RESERVED_8:
+            case H5T_STR_RESERVED_9:
+            case H5T_STR_RESERVED_10:
+            case H5T_STR_RESERVED_11:
+            case H5T_STR_RESERVED_12:
+            case H5T_STR_RESERVED_13:
+            case H5T_STR_RESERVED_14:
+            case H5T_STR_RESERVED_15:
                 sprintf(buf, "H5T_STR_RESERVED_%d", (int)(dt->shared->u.atomic.u.s.pad));
                 s = buf;
                 break;
+
+            case H5T_STR_ERROR:
+            default:
+                sprintf(buf, "Unknown string padding: %d", (int)(dt->shared->u.atomic.u.s.pad));
+                s = buf;
+                break;
         } /* end switch */
         fprintf(stream, "%*s%-*s %s\n", indent, "", fwidth,
                 "String Padding:",
@@ -1696,9 +1798,13 @@ H5O_dtype_debug(H5F_t *f, hid_t dxpl_id, const void *mesg, FILE *stream,
             case H5T_VLEN_SEQUENCE:
                 s = "sequence";
                 break;
+
             case H5T_VLEN_STRING:
                 s = "string";
                 break;
+
+            case H5T_VLEN_BADTYPE:
+            case H5T_VLEN_MAXTYPE:
             default:
                 sprintf(buf, "H5T_VLEN_%d", dt->shared->u.vlen.type);
                 s = buf;
@@ -1711,9 +1817,13 @@ H5O_dtype_debug(H5F_t *f, hid_t dxpl_id, const void *mesg, FILE *stream,
             case H5T_LOC_MEMORY:
                 s = "memory";
                 break;
+
             case H5T_LOC_DISK:
                 s = "disk";
                 break;
+
+            case H5T_LOC_BADLOC:
+            case H5T_LOC_MAXLOC:
             default:
                 sprintf(buf, "H5T_LOC_%d", (int)dt->shared->u.vlen.loc);
                 s = buf;
@@ -1728,13 +1838,34 @@ H5O_dtype_debug(H5F_t *f, hid_t dxpl_id, const void *mesg, FILE *stream,
                 case H5T_CSET_ASCII:
                     s = "ASCII";
                     break;
+
                 case H5T_CSET_UTF8:
                     s = "UTF-8";
                     break;
-                default:
+
+                case H5T_CSET_RESERVED_2:
+                case H5T_CSET_RESERVED_3:
+                case H5T_CSET_RESERVED_4:
+                case H5T_CSET_RESERVED_5:
+                case H5T_CSET_RESERVED_6:
+                case H5T_CSET_RESERVED_7:
+                case H5T_CSET_RESERVED_8:
+                case H5T_CSET_RESERVED_9:
+                case H5T_CSET_RESERVED_10:
+                case H5T_CSET_RESERVED_11:
+                case H5T_CSET_RESERVED_12:
+                case H5T_CSET_RESERVED_13:
+                case H5T_CSET_RESERVED_14:
+                case H5T_CSET_RESERVED_15:
                     sprintf(buf, "H5T_CSET_RESERVED_%d", (int)(dt->shared->u.vlen.cset));
                     s = buf;
                     break;
+
+                case H5T_CSET_ERROR:
+                default:
+                    sprintf(buf, "Unknown character set: %d", (int)(dt->shared->u.vlen.cset));
+                    s = buf;
+                    break;
             } /* end switch */
             fprintf(stream, "%*s%-*s %s\n", indent, "", fwidth,
                     "Character Set:",
@@ -1744,16 +1875,37 @@ H5O_dtype_debug(H5F_t *f, hid_t dxpl_id, const void *mesg, FILE *stream,
                 case H5T_STR_NULLTERM:
                     s = "NULL Terminated";
                     break;
+
                 case H5T_STR_NULLPAD:
                     s = "NULL Padded";
                     break;
+
                 case H5T_STR_SPACEPAD:
                     s = "Space Padded";
                     break;
-                default:
+
+                case H5T_STR_RESERVED_3:
+                case H5T_STR_RESERVED_4:
+                case H5T_STR_RESERVED_5:
+                case H5T_STR_RESERVED_6:
+                case H5T_STR_RESERVED_7:
+                case H5T_STR_RESERVED_8:
+                case H5T_STR_RESERVED_9:
+                case H5T_STR_RESERVED_10:
+                case H5T_STR_RESERVED_11:
+                case H5T_STR_RESERVED_12:
+                case H5T_STR_RESERVED_13:
+                case H5T_STR_RESERVED_14:
+                case H5T_STR_RESERVED_15:
                     sprintf(buf, "H5T_STR_RESERVED_%d", (int)(dt->shared->u.vlen.pad));
                     s = buf;
                     break;
+
+                case H5T_STR_ERROR:
+                default:
+                    sprintf(buf, "Unknown string padding: %d", (int)(dt->shared->u.vlen.pad));
+                    s = buf;
+                    break;
             } /* end switch */
             fprintf(stream, "%*s%-*s %s\n", indent, "", fwidth,
                     "String Padding:",
@@ -1774,15 +1926,20 @@ H5O_dtype_debug(H5F_t *f, hid_t dxpl_id, const void *mesg, FILE *stream,
             case H5T_ORDER_LE:
                 s = "little endian";
                 break;
+
             case H5T_ORDER_BE:
                 s = "big endian";
                 break;
+
             case H5T_ORDER_VAX:
                 s = "VAX";
                 break;
+
             case H5T_ORDER_NONE:
                 s = "none";
                 break;
+
+            case H5T_ORDER_ERROR:
             default:
                 sprintf(buf, "H5T_ORDER_%d", dt->shared->u.atomic.order);
                 s = buf;
@@ -1806,9 +1963,17 @@ H5O_dtype_debug(H5F_t *f, hid_t dxpl_id, const void *mesg, FILE *stream,
             case H5T_PAD_ZERO:
                 s = "zero";
                 break;
+
             case H5T_PAD_ONE:
                 s = "one";
                 break;
+
+            case H5T_PAD_BACKGROUND:
+                s = "background";
+                break;
+
+            case H5T_PAD_ERROR:
+            case H5T_NPAD:
             default:
                 s = "pad?";
                 break;
@@ -1820,9 +1985,17 @@ H5O_dtype_debug(H5F_t *f, hid_t dxpl_id, const void *mesg, FILE *stream,
             case H5T_PAD_ZERO:
                 s = "zero";
                 break;
+
             case H5T_PAD_ONE:
                 s = "one";
                 break;
+
+            case H5T_PAD_BACKGROUND:
+                s = "background";
+                break;
+
+            case H5T_PAD_ERROR:
+            case H5T_NPAD:
             default:
                 s = "pad?";
                 break;
@@ -1835,15 +2008,22 @@ H5O_dtype_debug(H5F_t *f, hid_t dxpl_id, const void *mesg, FILE *stream,
                 case H5T_PAD_ZERO:
                     s = "zero";
                     break;
+
                 case H5T_PAD_ONE:
                     s = "one";
                     break;
+
+                case H5T_PAD_BACKGROUND:
+                    s = "background";
+                    break;
+
+                case H5T_PAD_ERROR:
+                case H5T_NPAD:
                 default:
-                    if (dt->shared->u.atomic.u.f.pad < 0) {
+                    if (dt->shared->u.atomic.u.f.pad < 0)
                         sprintf(buf, "H5T_PAD_%d", -(dt->shared->u.atomic.u.f.pad));
-                    } else {
+                    else
                         sprintf(buf, "bit-%d", dt->shared->u.atomic.u.f.pad);
-                    }
                     s = buf;
                     break;
 	    } /* end switch */
@@ -1854,12 +2034,16 @@ H5O_dtype_debug(H5F_t *f, hid_t dxpl_id, const void *mesg, FILE *stream,
                 case H5T_NORM_IMPLIED:
                     s = "implied";
                     break;
+
                 case H5T_NORM_MSBSET:
                     s = "msb set";
                     break;
+
                 case H5T_NORM_NONE:
                     s = "none";
                     break;
+
+                case H5T_NORM_ERROR:
                 default:
                     sprintf(buf, "H5T_NORM_%d", (int) (dt->shared->u.atomic.u.f.norm));
                     s = buf;
@@ -1896,9 +2080,13 @@ H5O_dtype_debug(H5F_t *f, hid_t dxpl_id, const void *mesg, FILE *stream,
                 case H5T_SGN_NONE:
                     s = "none";
                     break;
+
                 case H5T_SGN_2:
                     s = "2's comp";
                     break;
+
+                case H5T_SGN_ERROR:
+                case H5T_NSGN:
                 default:
                     sprintf(buf, "H5T_SGN_%d", (int) (dt->shared->u.atomic.u.i.sign));
                     s = buf;
