@@ -120,7 +120,7 @@ const H5O_msg_class_t H5O_MSG_FILL[1] = {{
     NULL,		    	/*can share method		*/
     NULL,			/* pre copy native value to file	*/
     H5O_fill_shared_copy_file,	/* copy native value to file		*/
-    NULL,			/* post copy native value to file	*/
+    H5O_fill_shared_post_copy_file,	/* post copy native value to file	*/
     NULL,			/* get creation index		*/
     NULL,			/* set creation index		*/
     H5O_fill_shared_debug       /*debug the message			*/
@@ -144,7 +144,7 @@ const H5O_msg_class_t H5O_MSG_FILL_NEW[1] = {{
     NULL,		    	/*can share method		*/
     NULL,			/* pre copy native value to file	*/
     H5O_fill_new_shared_copy_file, /* copy native value to file		*/
-    NULL,			/* post copy native value to file	*/
+    H5O_fill_new_shared_post_copy_file,	/* post copy native value to file	*/
     NULL,			/* get creation index		*/
     NULL,			/* set creation index		*/
     H5O_fill_new_shared_debug	/*debug the message			*/
@@ -236,10 +236,10 @@ H5O_fill_new_decode(H5F_t UNUSED *f, hid_t UNUSED dxpl_id, H5O_t UNUSED *open_oh
             HGOTO_ERROR(H5E_OHDR, H5E_CANTLOAD, NULL, "unknown flag for fill value message")
 
         /* Space allocation time */
-        fill->alloc_time = (flags >> H5O_FILL_SHIFT_ALLOC_TIME) & H5O_FILL_MASK_ALLOC_TIME;
+        fill->alloc_time = (H5D_alloc_time_t)((flags >> H5O_FILL_SHIFT_ALLOC_TIME) & H5O_FILL_MASK_ALLOC_TIME);
 
         /* Fill value write time */
-        fill->fill_time = (flags >> H5O_FILL_SHIFT_FILL_TIME) & H5O_FILL_MASK_FILL_TIME;
+        fill->fill_time = (H5D_fill_time_t)((flags >> H5O_FILL_SHIFT_FILL_TIME) & H5O_FILL_MASK_FILL_TIME);
 
         /* Check for undefined fill value */
         if(flags & H5O_FILL_FLAG_UNDEFINED_VALUE) {
@@ -275,7 +275,7 @@ done:
     if(!ret_value && fill) {
         if(fill->buf)
             H5MM_xfree(fill->buf);
-	(void)H5FL_FREE(H5O_fill_t, fill);
+	fill = H5FL_FREE(H5O_fill_t, fill);
     } /* end if */
 
     FUNC_LEAVE_NOAPI(ret_value)
@@ -335,7 +335,7 @@ done:
     if(!ret_value && fill) {
         if(fill->buf)
             H5MM_xfree(fill->buf);
-	(void)H5FL_FREE(H5O_fill_t, fill);
+	fill = H5FL_FREE(H5O_fill_t, fill);
     } /* end if */
 
     FUNC_LEAVE_NOAPI(ret_value)
@@ -525,7 +525,7 @@ H5O_fill_copy(const void *_src, void *_dst)
 
             /* Set up type conversion function */
             if(NULL == (tpath = H5T_path_find(src->type, dst->type, NULL, NULL, H5AC_ind_dxpl_id, FALSE)))
-                HGOTO_ERROR(H5E_DATASET, H5E_UNSUPPORTED, NULL, "unable to convert between src and dst data types")
+                HGOTO_ERROR(H5E_OHDR, H5E_UNSUPPORTED, NULL, "unable to convert between src and dst data types")
 
             /* If necessary, convert fill value datatypes (which copies VL components, etc.) */
             if(!H5T_path_noop(tpath)) {
@@ -536,35 +536,35 @@ H5O_fill_copy(const void *_src, void *_dst)
                 /* Wrap copies of types to convert */
                 dst_id = H5I_register(H5I_DATATYPE, H5T_copy(dst->type, H5T_COPY_TRANSIENT), FALSE);
                 if(dst_id < 0)
-                    HGOTO_ERROR(H5E_DATATYPE, H5E_CANTINIT, NULL, "unable to copy/register datatype")
+                    HGOTO_ERROR(H5E_OHDR, H5E_CANTINIT, NULL, "unable to copy/register datatype")
                 src_id = H5I_register(H5I_DATATYPE, H5T_copy(src->type, H5T_COPY_ALL), FALSE);
                 if(src_id < 0) {
-                    H5I_dec_ref(dst_id, FALSE);
-                    HGOTO_ERROR(H5E_DATATYPE, H5E_CANTINIT, NULL, "unable to copy/register datatype")
+                    H5I_dec_ref(dst_id);
+                    HGOTO_ERROR(H5E_OHDR, H5E_CANTINIT, NULL, "unable to copy/register datatype")
                 } /* end if */
 
                 /* Allocate a background buffer */
                 bkg_size = MAX(H5T_get_size(dst->type), H5T_get_size(src->type));
                 if(H5T_path_bkg(tpath) && NULL == (bkg_buf = H5FL_BLK_CALLOC(type_conv, bkg_size))) {
-                    H5I_dec_ref(src_id, FALSE);
-                    H5I_dec_ref(dst_id, FALSE);
+                    H5I_dec_ref(src_id);
+                    H5I_dec_ref(dst_id);
                     HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, NULL, "memory allocation failed")
                 } /* end if */
 
                 /* Convert fill value */
                 if(H5T_convert(tpath, src_id, dst_id, (size_t)1, (size_t)0, (size_t)0, dst->buf, bkg_buf, H5AC_ind_dxpl_id) < 0) {
-                    H5I_dec_ref(src_id, FALSE);
-                    H5I_dec_ref(dst_id, FALSE);
+                    H5I_dec_ref(src_id);
+                    H5I_dec_ref(dst_id);
                     if(bkg_buf)
-                        (void)H5FL_BLK_FREE(type_conv, bkg_buf);
-                    HGOTO_ERROR(H5E_DATASET, H5E_CANTCONVERT, NULL, "datatype conversion failed")
+                        bkg_buf = H5FL_BLK_FREE(type_conv, bkg_buf);
+                    HGOTO_ERROR(H5E_OHDR, H5E_CANTCONVERT, NULL, "datatype conversion failed")
                 } /* end if */
 
                 /* Release the background buffer */
-                H5I_dec_ref(src_id, FALSE);
-                H5I_dec_ref(dst_id, FALSE);
+                H5I_dec_ref(src_id);
+                H5I_dec_ref(dst_id);
                 if(bkg_buf)
-                    (void)H5FL_BLK_FREE(type_conv, bkg_buf);
+                    bkg_buf = H5FL_BLK_FREE(type_conv, bkg_buf);
             } /* end if */
         } /* end if */
     } /* end if */
@@ -581,7 +581,7 @@ done:
 	if(dst->type)
             H5T_close(dst->type);
 	if(!_dst)
-            (void)H5FL_FREE(H5O_fill_t, dst);
+            dst = H5FL_FREE(H5O_fill_t, dst);
     } /* end if */
 
     FUNC_LEAVE_NOAPI(ret_value)
@@ -689,26 +689,26 @@ H5O_fill_reset_dyn(H5O_fill_t *fill)
     HDassert(fill);
 
     if(fill->buf) {
-        if(fill->type && H5T_detect_class(fill->type, H5T_VLEN) > 0) {
+        if(fill->type && H5T_detect_class(fill->type, H5T_VLEN, FALSE) > 0) {
             H5T_t *fill_type;           /* Copy of fill value datatype */
             H5S_t *fill_space;          /* Scalar dataspace for fill value element */
 
             /* Copy the fill value datatype and get an ID for it */
             if(NULL == (fill_type = H5T_copy(fill->type, H5T_COPY_TRANSIENT)))
-                HGOTO_ERROR(H5E_DATATYPE, H5E_CANTINIT, FAIL, "unable to copy fill value datatype")
+                HGOTO_ERROR(H5E_OHDR, H5E_CANTINIT, FAIL, "unable to copy fill value datatype")
             if((fill_type_id = H5I_register(H5I_DATATYPE, fill_type, FALSE)) < 0) {
                 H5T_close(fill_type);
-                HGOTO_ERROR(H5E_DATATYPE, H5E_CANTREGISTER, FAIL, "unable to register fill value datatype")
+                HGOTO_ERROR(H5E_OHDR, H5E_CANTREGISTER, FAIL, "unable to register fill value datatype")
             } /* end if */
 
             /* Create a scalar dataspace for the fill value element */
             if(NULL == (fill_space = H5S_create(H5S_SCALAR)))
-                HGOTO_ERROR(H5E_DATASPACE, H5E_CANTCREATE, FAIL, "can't create scalar dataspace")
+                HGOTO_ERROR(H5E_OHDR, H5E_CANTCREATE, FAIL, "can't create scalar dataspace")
 
             /* Reclaim any variable length components of the fill value */
             if(H5D_vlen_reclaim(fill_type_id, fill_space, H5P_DATASET_XFER_DEFAULT, fill->buf) < 0) {
                 H5S_close(fill_space);
-                HGOTO_ERROR(H5E_DATASET, H5E_BADITER, FAIL, "unable to reclaim variable-length fill value data")
+                HGOTO_ERROR(H5E_OHDR, H5E_BADITER, FAIL, "unable to reclaim variable-length fill value data")
             } /* end if */
 
             /* Release the scalar fill value dataspace */
@@ -725,8 +725,9 @@ H5O_fill_reset_dyn(H5O_fill_t *fill)
     } /* end if */
 
 done:
-    if(fill_type_id > 0)
-        H5I_dec_ref(fill_type_id, FALSE);
+    if(fill_type_id > 0 && H5I_dec_ref(fill_type_id) < 0)
+        HDONE_ERROR(H5E_OHDR, H5E_CANTDEC, FAIL, "unable to decrement ref count for temp ID")
+
     FUNC_LEAVE_NOAPI(ret_value)
 } /* end H5O_fill_reset_dyn() */
 
@@ -783,7 +784,7 @@ H5O_fill_free(void *fill)
 
     HDassert(fill);
 
-    (void)H5FL_FREE(H5O_fill_t, fill);
+    fill = H5FL_FREE(H5O_fill_t, fill);
 
     FUNC_LEAVE_NOAPI(SUCCEED)
 } /* end H5O_fill_free() */
@@ -934,13 +935,13 @@ H5O_fill_convert(H5O_fill_t *fill, H5T_t *dset_type, hbool_t *fill_changed, hid_
      * Can we convert between source and destination data types?
      */
     if(NULL == (tpath = H5T_path_find(fill->type, dset_type, NULL, NULL, dxpl_id, FALSE)))
-	HGOTO_ERROR(H5E_DATATYPE, H5E_CANTINIT, FAIL, "unable to convert between src and dst datatypes")
+	HGOTO_ERROR(H5E_OHDR, H5E_CANTINIT, FAIL, "unable to convert between src and dst datatypes")
 
     /* Don't bother doing anything if there will be no actual conversion */
     if(!H5T_path_noop(tpath)) {
         if((src_id = H5I_register(H5I_DATATYPE, H5T_copy(fill->type, H5T_COPY_ALL), FALSE)) < 0 ||
                 (dst_id = H5I_register(H5I_DATATYPE, H5T_copy(dset_type, H5T_COPY_ALL), FALSE)) < 0)
-            HGOTO_ERROR(H5E_DATATYPE, H5E_CANTINIT, FAIL, "unable to copy/register data type")
+            HGOTO_ERROR(H5E_OHDR, H5E_CANTINIT, FAIL, "unable to copy/register data type")
 
         /*
          * Datatype conversions are always done in place, so we need a buffer
@@ -961,10 +962,11 @@ H5O_fill_convert(H5O_fill_t *fill, H5T_t *dset_type, hbool_t *fill_changed, hid_
 
         /* Do the conversion */
         if(H5T_convert(tpath, src_id, dst_id, (size_t)1, (size_t)0, (size_t)0, buf, bkg, dxpl_id) < 0)
-            HGOTO_ERROR(H5E_DATATYPE, H5E_CANTINIT, FAIL, "datatype conversion failed")
+            HGOTO_ERROR(H5E_OHDR, H5E_CANTINIT, FAIL, "datatype conversion failed")
 
         /* Update the fill message */
         if(buf != fill->buf) {
+            H5T_vlen_reclaim_elmt(fill->buf, fill->type, dxpl_id);
             H5MM_xfree(fill->buf);
             fill->buf = buf;
         } /* end if */
@@ -977,10 +979,10 @@ H5O_fill_convert(H5O_fill_t *fill, H5T_t *dset_type, hbool_t *fill_changed, hid_
     } /* end if */
 
 done:
-    if(src_id >= 0)
-        H5I_dec_ref(src_id, FALSE);
-    if(dst_id >= 0)
-        H5I_dec_ref(dst_id, FALSE);
+    if(src_id >= 0 && H5I_dec_ref(src_id) < 0)
+        HDONE_ERROR(H5E_OHDR, H5E_CANTDEC, FAIL, "unable to decrement ref count for temp ID")
+    if(dst_id >= 0 && H5I_dec_ref(dst_id) < 0)
+        HDONE_ERROR(H5E_OHDR, H5E_CANTDEC, FAIL, "unable to decrement ref count for temp ID")
     if(buf != fill->buf)
         H5MM_xfree(buf);
     if(bkg)
