@@ -17,6 +17,7 @@
 #define H5S_PACKAGE		/*prevent warning from including H5Spkg.h */
 
 #include "H5private.h"		/* Generic Functions			*/
+#include "H5Dprivate.h"		/* Datasets				*/
 #include "H5Eprivate.h"		/* Error handling		  	*/
 #include "H5FLprivate.h"	/* Free lists                           */
 #include "H5Gprivate.h"		/* Groups			  	*/
@@ -76,7 +77,7 @@ const H5O_msg_class_t H5O_MSG_SDSPACE[1] = {{
     NULL,		    	/*can share method		*/
     H5O_sdspace_pre_copy_file,	/* pre copy native value to file */
     H5O_sdspace_shared_copy_file,/* copy native value to file    */
-    NULL,			/* post copy native value to file    */
+    H5O_sdspace_shared_post_copy_file,/* post copy native value to file    */
     NULL,			/* get creation index		*/
     NULL,			/* set creation index		*/
     H5O_sdspace_shared_debug	/* debug the message		    	*/
@@ -190,7 +191,7 @@ H5O_sdspace_decode(H5F_t *f, hid_t UNUSED dxpl_id, H5O_t UNUSED *open_oh,
 done:
     if(!ret_value && sdim) {
         H5S_extent_release(sdim);
-        (void)H5FL_FREE(H5S_extent_t, sdim);
+        sdim = H5FL_FREE(H5S_extent_t, sdim);
     } /* end if */
 
     FUNC_LEAVE_NOAPI(ret_value)
@@ -245,15 +246,15 @@ H5O_sdspace_encode(H5F_t *f, uint8_t *p, const void *_mesg)
     /* Version */
     HDassert(sdim->version > 0);
     HDassert(sdim->type != H5S_NULL || sdim->version >= H5O_SDSPACE_VERSION_2);
-    *p++ = sdim->version;
+    *p++ = (uint8_t)sdim->version;
 
     /* Rank */
-    *p++ = sdim->rank;
+    *p++ = (uint8_t)sdim->rank;
 
     /* Flags */
     if(sdim->max)
         flags |= H5S_VALID_MAX;
-    *p++ = flags;
+    *p++ = (uint8_t)flags;
 
     /* Dataspace type */
     if(sdim->version > H5O_SDSPACE_VERSION_1)
@@ -286,9 +287,9 @@ H5O_sdspace_encode(H5F_t *f, uint8_t *p, const void *_mesg)
  PURPOSE
     Copies a message from MESG to DEST, allocating DEST if necessary.
  USAGE
-    void *H5O_sdspace_copy(mesg, dest)
-	const void *mesg;	IN: Pointer to the source extent dimensionality struct
-	const void *dest;	IN: Pointer to the destination extent dimensionality struct
+    void *H5O_sdspace_copy(_mesg, _dest)
+	const void *_mesg;	IN: Pointer to the source extent dimensionality struct
+	const void *_dest;	IN: Pointer to the destination extent dimensionality struct
  RETURNS
     Pointer to DEST on success, NULL on failure
  DESCRIPTION
@@ -296,27 +297,31 @@ H5O_sdspace_encode(H5F_t *f, uint8_t *p, const void *_mesg)
     allocating the destination structure if necessary.
 --------------------------------------------------------------------------*/
 static void *
-H5O_sdspace_copy(const void *mesg, void *dest)
+H5O_sdspace_copy(const void *_mesg, void *_dest)
 {
-    const H5S_extent_t	   *src = (const H5S_extent_t *) mesg;
-    H5S_extent_t	   *dst = (H5S_extent_t *) dest;
+    const H5S_extent_t	   *mesg = (const H5S_extent_t *)_mesg;
+    H5S_extent_t	   *dest = (H5S_extent_t *)_dest;
     void                   *ret_value;          /* Return value */
 
     FUNC_ENTER_NOAPI_NOINIT(H5O_sdspace_copy)
 
     /* check args */
-    HDassert(src);
-    if(!dst && NULL == (dst = H5FL_MALLOC(H5S_extent_t)))
+    HDassert(mesg);
+    if(!dest && NULL == (dest = H5FL_MALLOC(H5S_extent_t)))
 	HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, NULL, "memory allocation failed")
 
     /* Copy extent information */
-    if(H5S_extent_copy(dst, src, TRUE) < 0)
+    if(H5S_extent_copy(dest, mesg, TRUE) < 0)
         HGOTO_ERROR(H5E_DATASPACE, H5E_CANTCOPY, NULL, "can't copy extent")
 
     /* Set return value */
-    ret_value = dst;
+    ret_value = dest;
 
 done:
+    if(NULL == ret_value)
+        if(dest && NULL == _dest)
+            dest = H5FL_FREE(H5S_extent_t, dest);
+
     FUNC_LEAVE_NOAPI(ret_value)
 } /* end H5O_sdspace_copy() */
 
@@ -414,7 +419,7 @@ H5O_sdspace_free(void *mesg)
 
     HDassert(mesg);
 
-    (void)H5FL_FREE(H5S_extent_t, mesg);
+    mesg = H5FL_FREE(H5S_extent_t, mesg);
 
     FUNC_LEAVE_NOAPI(SUCCEED)
 } /* end H5O_sdspace_free() */
