@@ -20,6 +20,7 @@
  * Purpose:	Tests datasets stored in external raw files.
  */
 #include "h5test.h"
+#include "H5srcdir.h"
 
 /* File for external link test.  Created with gen_udlinks.c */
 #define LINKED_FILE  "be_extlink2.h5"
@@ -58,21 +59,23 @@ same_contents (const char *name1, const char *name2)
 
     fd1 = HDopen(name1, O_RDONLY, 0666);
     fd2 = HDopen(name2, O_RDONLY, 0666);
-    assert (fd1>=0 && fd2>=0);
+    assert(fd1 >= 0 && fd2 >= 0);
 
-    while (1) {
-	n1 = HDread(fd1, buf1, sizeof(buf1));
-	n2 = HDread(fd2, buf2, sizeof(buf2));
-	assert (n1>=0 && (size_t)n1<=sizeof(buf1));
-	assert (n2>=0 && (size_t)n2<=sizeof(buf2));
-	assert (n1==n2);
+    while(1) {
+        /* Asserts will catch negative return values */
+        n1 = HDread(fd1, buf1, sizeof(buf1));
+        n2 = HDread(fd2, buf2, sizeof(buf2));
+        assert(n1 >= 0 && (size_t)n1 <= sizeof(buf1));
+        assert(n2 >= 0 && (size_t)n2 <= sizeof(buf2));
+        assert(n1 == n2);
 
-	if (n1<=0 && n2<=0) break;
-	if (memcmp (buf1, buf2, (size_t)n1)) {
-	    HDclose(fd1);
-	    HDclose(fd2);
-	    return 0;
-	}
+        if(n1 == 0 && n2 == 0)
+            break;
+        if(HDmemcmp(buf1, buf2, (size_t)n1)) {
+            HDclose(fd1);
+            HDclose(fd2);
+            return 0;
+        }
     }
     HDclose(fd1);
     HDclose(fd2);
@@ -838,10 +841,13 @@ test_3 (hid_t fapl)
 static int
 test_4 (hid_t fapl)
 {
-    hid_t       fid, gid, xid, xid2;
-    char	filename[1024];		/*file name			*/
-    char        pathname[1024];
-    char       *srcdir = getenv("srcdir"); /*where the src code is located*/
+    hid_t fid = -1;
+    hid_t gid = -1;
+    hid_t xid = -1;
+    hid_t xid2 = -1;
+    char  filename[1024];		/*file name			*/
+    char  pathname[1024];
+    char *srcdir = getenv("srcdir"); /*where the src code is located*/
 
     TESTING("opening external link twice");
 
@@ -882,7 +888,7 @@ test_4 (hid_t fapl)
     if((fid = H5Fopen(filename, H5F_ACC_RDONLY, fapl)) < 0)
         goto error;
 
-    /* Open the external link */
+    /* Open the external link which is "/ link" as created previously via H5Lcreate_external() */
     if((xid = H5Gopen2(fid, "/ link", H5P_DEFAULT)) < 0)
         goto error;
 
@@ -911,7 +917,7 @@ test_4 (hid_t fapl)
         H5Gclose(gid);
         H5Gclose(xid);
         H5Gclose(xid2);
-	H5Fclose(fid);
+        H5Fclose(fid);
     } H5E_END_TRY;
     return 1;
 }
@@ -960,6 +966,10 @@ main (void)
     nerrors += test_2(fapl);
     nerrors += test_3(fapl);
     nerrors += test_4(fapl);
+
+    /* Verify symbol table messages are cached */
+    nerrors += (h5_verify_cached_stabs(FILENAME, fapl) < 0 ? 1 : 0);
+
     if (nerrors>0) goto error;
 
     if (H5Fclose(file) < 0) goto error;
