@@ -26,7 +26,7 @@
 /*
  * Definitions for the testing structure.
  */
-#define MAXNUMOFTESTS   45
+#define MAXNUMOFTESTS   50
 #define MAXTESTNAME     16
 #define MAXTESTDESC     64
 
@@ -45,10 +45,12 @@ typedef struct TestStruct {
  * Variables used by testing framework.
  */
 static int num_errs = 0;        /* Total number of errors during testing */
-static int Verbosity = VERBO_DEF;       /* Default Verbosity is Low */
+int TestVerbosity = VERBO_DEF;       /* Default Verbosity is Low */
 static int Summary = 0;		/* Show test summary. Default is no. */
 static int CleanUp = 1;		/* Do cleanup or not. Default is yes. */
 static int TestExpress = -1;	/* Do TestExpress or not. -1 means not set yet. */
+static H5E_auto_t	*PrintErrorStackFunc;
+static void		**PrintErrorStackData;
 static TestStruct Test[MAXNUMOFTESTS];
 static int    Index = 0;
 static const void *Test_parameters = NULL;
@@ -130,12 +132,14 @@ void TestInit(const char *ProgName, void (*private_usage)(void), int (*private_p
     setbuf(stdout, NULL);
 #endif
 
+    /* Save error printing settings */
+    H5Eget_auto2(H5E_DEFAULT, PrintErrorStackFunc, PrintErrorStackData);
     /*
      * Turn off automatic error reporting since we do it ourselves.  Besides,
      * half the functions this test calls are private, so automatic error
      * reporting wouldn't do much good since it's triggered at the API layer.
      */
-    H5Eset_auto2(H5E_DEFAULT, NULL, NULL);
+    PrintErrorStackOff();
 
     /*
      * Record the program name and private routines if provided.
@@ -361,7 +365,7 @@ void TestCleanup(void)
  */
 int GetTestVerbosity(void)
 {
-    return(Verbosity);
+    return(TestVerbosity);
 }
 
 /*
@@ -372,8 +376,8 @@ int SetTestVerbosity(int newval)
 {
     int oldval;
 
-    oldval = Verbosity;
-    Verbosity = newval;
+    oldval = TestVerbosity;
+    TestVerbosity = newval;
     return(oldval);
 }
 
@@ -588,3 +592,37 @@ void SetTest(const char *testname, int action)
 	    break;
     }
 }
+
+
+/*
+ * Enable alarm on test execution, configurable by environment variable
+ */
+void TestAlarmOn(void)
+{
+    char * env_val = HDgetenv("HDF5_ALARM_SECONDS");    /* Alarm environment */
+    unsigned long alarm_sec = H5_ALARM_SEC;     /* Number of seconds before alarm goes off */
+
+    /* Get the alarm value from the environment variable, if set */
+    if(env_val != NULL)
+        alarm_sec = (unsigned)HDstrtoul(env_val, (char **)NULL, 10);
+
+    /* Set the number of seconds before alarm goes off */
+    HDalarm((unsigned)alarm_sec);
+}
+
+
+/*
+ * Enable error stack printing when errors occur.
+ */
+void PrintErrorStackOn(void)
+{
+    H5Eset_auto2(H5E_DEFAULT, PrintErrorStackFunc, PrintErrorStackData);
+}
+
+/*
+ * Disable error stack printing when errors occur.
+ */
+void PrintErrorStackOff(void)
+{
+    H5Eset_auto2(H5E_DEFAULT, NULL, NULL);
+}
