@@ -23,6 +23,7 @@
 
 #include "hdf5.h"
 #include "testhdf5.h"
+#include "H5srcdir.h"
 
 #include "H5Bprivate.h"
 #include "H5Pprivate.h"
@@ -35,12 +36,21 @@
 #define H5F_TESTING
 #include "H5Fpkg.h"		/* File access	 			*/
 
+#define BAD_USERBLOCK_SIZE1  (hsize_t)1
+#define BAD_USERBLOCK_SIZE2  (hsize_t)2
+#define BAD_USERBLOCK_SIZE3  (hsize_t)3
+#define BAD_USERBLOCK_SIZE4  (hsize_t)64
+#define BAD_USERBLOCK_SIZE5  (hsize_t)511
+#define BAD_USERBLOCK_SIZE6  (hsize_t)513
+#define BAD_USERBLOCK_SIZE7  (hsize_t)6144
+
 #define F1_USERBLOCK_SIZE  (hsize_t)0
 #define F1_OFFSET_SIZE	   sizeof(haddr_t)
 #define F1_LENGTH_SIZE	   sizeof(hsize_t)
 #define F1_SYM_LEAF_K	   4
 #define F1_SYM_INTERN_K	   16
 #define FILE1	"tfile1.h5"
+#define SFILE1	"sys_file1"
 
 #define F2_USERBLOCK_SIZE  (hsize_t)512
 #define F2_OFFSET_SIZE	   8
@@ -89,6 +99,10 @@
 
 #define USERBLOCK_SIZE      ((hsize_t) 512)
 
+/* Declaration for test_libver_macros2() */
+#define FILE5		"tfile5.h5"	/* Test file */
+
+
 static void
 create_objects(hid_t, hid_t, hid_t *, hid_t *, hid_t *, hid_t *);
 static void
@@ -208,6 +222,36 @@ test_file_create(void)
     tmpl1 = H5Pcreate(H5P_FILE_CREATE);
     CHECK(tmpl1, FAIL, "H5Pcreate");
 
+    /* Try setting some bad userblock sizes */
+    H5E_BEGIN_TRY {
+        ret = H5Pset_userblock(tmpl1, BAD_USERBLOCK_SIZE1);
+    } H5E_END_TRY;
+    VERIFY(ret, FAIL, "H5Pset_userblock");
+    H5E_BEGIN_TRY {
+        ret = H5Pset_userblock(tmpl1, BAD_USERBLOCK_SIZE2);
+    } H5E_END_TRY;
+    VERIFY(ret, FAIL, "H5Pset_userblock");
+    H5E_BEGIN_TRY {
+        ret = H5Pset_userblock(tmpl1, BAD_USERBLOCK_SIZE3);
+    } H5E_END_TRY;
+    VERIFY(ret, FAIL, "H5Pset_userblock");
+    H5E_BEGIN_TRY {
+        ret = H5Pset_userblock(tmpl1, BAD_USERBLOCK_SIZE4);
+    } H5E_END_TRY;
+    VERIFY(ret, FAIL, "H5Pset_userblock");
+    H5E_BEGIN_TRY {
+        ret = H5Pset_userblock(tmpl1, BAD_USERBLOCK_SIZE5);
+    } H5E_END_TRY;
+    VERIFY(ret, FAIL, "H5Pset_userblock");
+    H5E_BEGIN_TRY {
+        ret = H5Pset_userblock(tmpl1, BAD_USERBLOCK_SIZE6);
+    } H5E_END_TRY;
+    VERIFY(ret, FAIL, "H5Pset_userblock");
+    H5E_BEGIN_TRY {
+        ret = H5Pset_userblock(tmpl1, BAD_USERBLOCK_SIZE7);
+    } H5E_END_TRY;
+    VERIFY(ret, FAIL, "H5Pset_userblock");
+
     /* Set the new file-creation parameters */
     ret = H5Pset_userblock(tmpl1, F2_USERBLOCK_SIZE);
     CHECK(ret, FAIL, "H5Pset_userblock");
@@ -233,7 +277,7 @@ test_file_create(void)
     {
        hid_t       dataset_id, dataspace_id;  /* identifiers */
        hsize_t     dims[F2_RANK];
-       int         data[F2_DIM0][F2_DIM1];
+       unsigned    data[F2_DIM0][F2_DIM1];
        unsigned i,j;
 
        /* Create the data space for the dataset. */
@@ -243,7 +287,7 @@ test_file_create(void)
        CHECK(dataspace_id, FAIL, "H5Screate_simple");
 
        /* Create the dataset. */
-       dataset_id = H5Dcreate2(fid2, F2_DSET, H5T_NATIVE_INT, dataspace_id, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
+       dataset_id = H5Dcreate2(fid2, F2_DSET, H5T_NATIVE_UINT, dataspace_id, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
        CHECK(dataset_id, FAIL, "H5Dcreate2");
 
        for(i = 0; i < F2_DIM0; i++)
@@ -251,7 +295,7 @@ test_file_create(void)
                data[i][j] = i * 10 + j;
 
        /* Write data to the new dataset */
-       ret = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, data);
+       ret = H5Dwrite(dataset_id, H5T_NATIVE_UINT, H5S_ALL, H5S_ALL, H5P_DEFAULT, data);
        CHECK(ret, FAIL, "H5Dwrite");
 
        /* End access to the dataset and release resources used by it. */
@@ -804,7 +848,7 @@ create_objects(hid_t fid1, hid_t fid2, hid_t *ret_did, hid_t *ret_gid1,
     {
        hid_t       dataset_id, dataspace_id;  /* identifiers */
        hsize_t     dims[F2_RANK];
-       int         data[F2_DIM0][F2_DIM1];
+       unsigned    data[F2_DIM0][F2_DIM1];
        unsigned    i,j;
 
        /* Create the data space for the dataset. */
@@ -814,7 +858,7 @@ create_objects(hid_t fid1, hid_t fid2, hid_t *ret_did, hid_t *ret_gid1,
        CHECK(dataspace_id, FAIL, "H5Screate_simple");
 
        /* Create the dataset. */
-       dataset_id = H5Dcreate2(fid1, "/dset", H5T_NATIVE_INT, dataspace_id, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
+       dataset_id = H5Dcreate2(fid1, "/dset", H5T_NATIVE_UINT, dataspace_id, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
        CHECK(dataset_id, FAIL, "H5Dcreate2");
 
        for(i = 0; i < F2_DIM0; i++)
@@ -822,8 +866,7 @@ create_objects(hid_t fid1, hid_t fid2, hid_t *ret_did, hid_t *ret_gid1,
                data[i][j] = i * 10 + j;
 
        /* Write data to the new dataset */
-       ret = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL,
-                H5P_DEFAULT, data);
+       ret = H5Dwrite(dataset_id, H5T_NATIVE_UINT, H5S_ALL, H5S_ALL, H5P_DEFAULT, data);
        CHECK(ret, FAIL, "H5Dwrite");
 
        if(ret_did != NULL)
@@ -1081,46 +1124,46 @@ test_obj_count_and_id(hid_t fid1, hid_t fid2, hid_t did, hid_t gid1,
     oid_count = H5Fget_obj_count(H5F_OBJ_ALL, H5F_OBJ_ALL);
     CHECK(oid_count, FAIL, "H5Fget_obj_count");
     VERIFY(oid_count, OBJ_ID_COUNT_8, "H5Fget_obj_count");
+ 
+    if(oid_count > 0) {
+        hid_t *oid_list;
 
-    {
-        hid_t      *oid_list;
-        int   i;
-        H5I_type_t id_type;
-
-        oid_list = (hid_t*)calloc((size_t)oid_count, sizeof(hid_t));
+        oid_list = (hid_t *)HDcalloc((size_t)oid_count, sizeof(hid_t));
         if(oid_list != NULL) {
+            int   i;
+
 	    ret_count = H5Fget_obj_ids(H5F_OBJ_ALL, H5F_OBJ_ALL, (size_t)oid_count, oid_list);
 	    CHECK(ret_count, FAIL, "H5Fget_obj_ids");
-        }
 
-        for(i=0; i<oid_count; i++) {
-	    id_type = H5Iget_type(oid_list[i]);
-	    switch(id_type) {
-	        case H5I_FILE:
-		    if(oid_list[i]!=fid1 && oid_list[i]!=fid2 &&
-			oid_list[i]!=fid3 && oid_list[i]!=fid4) {
-			ret = FAIL;
-			CHECK(ret, FAIL, "H5Fget_obj_ids");
-		    }
-		    break;
-	        case H5I_GROUP:
-                    if(oid_list[i]!=gid1 && oid_list[i]!=gid2 &&
-                        oid_list[i]!=gid3) {
-			ret = FAIL;
-                        CHECK(ret, FAIL, "H5Fget_obj_ids");
-                    }
-		    break;
-	        case H5I_DATASET:
-	 	    VERIFY(oid_list[i], did, "H5Fget_obj_ids");
-		    break;
-		default:
-		    ret = FAIL;
-                    CHECK(ret, FAIL, "H5Fget_obj_ids");
-	    }
-        }
+            for(i = 0; i < oid_count; i++) {
+                H5I_type_t id_type;
 
-        free(oid_list);
-    }
+                id_type = H5Iget_type(oid_list[i]);
+                switch(id_type) {
+                    case H5I_FILE:
+                        if(oid_list[i] != fid1 && oid_list[i] != fid2
+                                && oid_list[i] != fid3 && oid_list[i] != fid4)
+                            ERROR("H5Fget_obj_ids");
+                        break;
+
+                    case H5I_GROUP:
+                        if(oid_list[i] != gid1 && oid_list[i] != gid2
+                                && oid_list[i] != gid3)
+                            ERROR("H5Fget_obj_ids");
+                        break;
+
+                    case H5I_DATASET:
+                        VERIFY(oid_list[i], did, "H5Fget_obj_ids");
+                        break;
+
+                    default:
+                        ERROR("H5Fget_obj_ids");
+                } /* end switch */
+            } /* end for */
+
+            HDfree(oid_list);
+        } /* end if */
+    } /* end if */
 
     /* close the two new files */
     ret = H5Fclose(fid3);
@@ -1352,7 +1395,7 @@ test_file_ishdf5(void)
 
     /* Create non-HDF5 file and check it */
     fd=HDopen(FILE1, O_RDWR|O_CREAT|O_TRUNC, 0666);
-    CHECK(ret, FAIL, "HDopen");
+    CHECK(fd, FAIL, "HDopen");
 
     /* Initialize information to write */
     for(u=0; u<1024; u++)
@@ -1950,7 +1993,7 @@ test_userblock_file_size(void)
     CHECK(ret, FAIL, "H5Fget_filesize");
 
     /* Verify that the file sizes differ exactly by the userblock size */
-    VERIFY(filesize2, filesize1 + USERBLOCK_SIZE, "H5Fget_filesize");
+    VERIFY_TYPE((unsigned long long)filesize2, (unsigned long long)(filesize1 + USERBLOCK_SIZE), unsigned long long, "%llu", "H5Fget_filesize");
 
     /* Close files */
     ret = H5Fclose(file1_id);
@@ -2014,7 +2057,7 @@ test_cached_stab_info(void)
 
     /* Reopen file */
     file_id = H5Fopen(FILE1, H5F_ACC_RDONLY, H5P_DEFAULT);
-    CHECK(ret, FAIL, "H5Fopen");
+    CHECK(file_id, FAIL, "H5Fopen");
 
     /* Verify the cached symbol table information */
     ret = H5F_check_cached_stab_test(file_id);
@@ -2025,6 +2068,669 @@ test_cached_stab_info(void)
     CHECK(ret, FAIL, "H5Fclose");
 } /* end test_cached_stab_info() */
 
+/****************************************************************
+**
+**  test_rw_noupdate(): low-level file test routine.
+**      This test checks to ensure that opening and closing a file
+**      with read/write permissions does not write anything to the
+**      file if the file does not change.
+**
+**  Programmer: Mike McGreevy
+**              mamcgree@hdfgroup.org
+**              June 29, 2009
+**
+*****************************************************************/
+static void
+test_rw_noupdate(void)
+{
+    int fd;             /* File Descriptor */
+    h5_stat_t sb1, sb2; /* Info from 'stat' call */
+    double diff;        /* Difference in modification times */
+    herr_t ret;         /* Generic return value */
+
+    /* Output message about test being performed */
+    MESSAGE(5, ("Testing to verify that nothing is written if nothing is changed.\n"));
+
+    /* First make sure the stat function behaves as we expect - the modification time
+     * is the time that the file was modified last time. */
+    fd = HDopen(SFILE1, O_RDWR | O_CREAT | O_TRUNC, 0666);
+    CHECK(fd, FAIL, "HDopen");
+    ret = HDclose(fd);
+    CHECK(ret, FAIL, "HDclose");
+
+    /* Determine File's Initial Timestamp */
+    ret = HDstat(SFILE1, &sb1);
+    VERIFY(ret, 0, "HDstat");
+
+    /* Wait for 2 seconds */
+    /* (This ensures a system time difference between the two file accesses) */
+    HDsleep(2);
+
+    fd = HDopen(SFILE1, O_RDWR, 0666);
+    CHECK(fd, FAIL, "HDopen");
+    ret = HDclose(fd);
+    CHECK(ret, FAIL, "HDclose");
+
+    /* Determine File's New Timestamp */
+    ret = HDstat(SFILE1, &sb2);
+    VERIFY(ret, 0, "HDstat");
+
+    /* Get difference between timestamps */
+    diff = HDdifftime(sb2.st_mtime, sb1.st_mtime);
+
+    /* Check That Timestamps Are Equal */
+    if(diff > 0.0) {
+        /* Output message about test being performed */
+        MESSAGE(1, ("Testing to verify that nothing is written if nothing is changed: This test is skipped on this system because the modification time from stat is the same as the last access time (We know OpenVMS behaves in this way).\n"));
+    } /* end if */
+    else {
+        hid_t file_id;      /* HDF5 File ID */
+
+        /* Create and Close a HDF5 File */
+        file_id = H5Fcreate(FILE1, H5F_ACC_TRUNC, H5P_DEFAULT, H5P_DEFAULT);
+        CHECK(file_id, FAIL, "H5Fcreate");
+        ret = H5Fclose(file_id);
+        CHECK(ret, FAIL, "H5Fclose");
+
+        /* Determine File's Initial Timestamp */
+        ret = HDstat(FILE1, &sb1);
+        VERIFY(ret, 0, "HDfstat");
+
+        /* Wait for 2 seconds */
+        /* (This ensures a system time difference between the two file accesses) */
+        HDsleep(2);
+
+        /* Open and Close File With Read/Write Permission */
+        file_id = H5Fopen(FILE1, H5F_ACC_RDWR, H5P_DEFAULT);
+        CHECK(file_id, FAIL, "H5Fopen");
+        ret = H5Fclose(file_id);
+        CHECK(ret, FAIL, "H5Fclose");
+
+        /* Determine File's New Timestamp */
+        ret = HDstat(FILE1, &sb2);
+        VERIFY(ret, 0, "HDstat");
+
+        /* Ensure That Timestamps Are Equal */
+        diff = HDdifftime(sb2.st_mtime, sb1.st_mtime);
+        ret = (diff > 0.0);
+        VERIFY(ret, 0, "Timestamp");
+    } /* end else */
+} /* end test_rw_noupdate() */
+
+/****************************************************************
+**
+**  test_userblock_alignment_helper1(): helper routine for
+**      test_userblock_alignment() test, to handle common testing
+**
+**  Programmer: Quincey Koziol
+**              koziol@hdfgroup.org
+**              Septmber 10, 2009
+**
+*****************************************************************/
+static int
+test_userblock_alignment_helper1(hid_t fcpl, hid_t fapl)
+{
+    hid_t fid;          /* File ID */
+    int curr_num_errs = GetTestNumErrs();       /* Retrieve the current # of errors */
+    herr_t ret;         /* Generic return value */
+
+    /* Create a file with FAPL & FCPL */
+    fid = H5Fcreate(FILE1, H5F_ACC_TRUNC, fcpl, fapl);
+    CHECK(fid, FAIL, "H5Fcreate");
+
+    /* Only proceed further if file ID is OK */
+    if(fid > 0) {
+        hid_t gid;      /* Group ID */
+        hid_t sid;      /* Dataspace ID */
+        hid_t did;      /* Dataset ID */
+        int val = 2;    /* Dataset value */
+
+        /* Create a group */
+        gid = H5Gcreate2(fid, "group1", H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
+        CHECK(gid, FAIL, "H5Gcreate2");
+
+        /* Create a dataset */
+        sid = H5Screate(H5S_SCALAR);
+        CHECK(sid, FAIL, "H5Screate");
+        did = H5Dcreate2(gid, "dataset", H5T_NATIVE_INT, sid, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
+        CHECK(did, FAIL, "H5Dcreate2");
+
+        /* Close dataspace */
+        ret = H5Sclose(sid);
+        CHECK(ret, FAIL, "H5Sclose");
+
+        /* Write value to dataset */
+        ret = H5Dwrite(did, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, &val);
+        CHECK(ret, FAIL, "H5Dwrite");
+
+        /* Close dataset */
+        ret = H5Dclose(did);
+        CHECK(ret, FAIL, "H5Dclose");
+
+        /* Close group */
+        ret = H5Gclose(gid);
+        CHECK(ret, FAIL, "H5Gclose");
+
+        /* Close file */
+        ret = H5Fclose(fid);
+        CHECK(ret, FAIL, "H5Fclose");
+    } /* end if */
+
+    return((GetTestNumErrs() == curr_num_errs) ? 0 : -1);
+} /* end test_userblock_alignment_helper1() */
+
+/****************************************************************
+**
+**  test_userblock_alignment_helper2(): helper routine for
+**      test_userblock_alignment() test, to handle common testing
+**
+**  Programmer: Quincey Koziol
+**              koziol@hdfgroup.org
+**              Septmber 10, 2009
+**
+*****************************************************************/
+static int
+test_userblock_alignment_helper2(hid_t fapl, hbool_t open_rw)
+{
+    hid_t fid;          /* File ID */
+    int curr_num_errs = GetTestNumErrs();       /* Retrieve the current # of errors */
+    herr_t ret;         /* Generic return value */
+
+    /* Re-open file */
+    fid = H5Fopen(FILE1, (open_rw ? H5F_ACC_RDWR : H5F_ACC_RDONLY), fapl);
+    CHECK(fid, FAIL, "H5Fopen");
+
+    /* Only proceed further if file ID is OK */
+    if(fid > 0) {
+        hid_t gid;      /* Group ID */
+        hid_t did;      /* Dataset ID */
+        int val = -1;   /* Dataset value */
+
+        /* Open group */
+        gid = H5Gopen2(fid, "group1", H5P_DEFAULT);
+        CHECK(gid, FAIL, "H5Gopen2");
+
+        /* Open dataset */
+        did = H5Dopen2(gid, "dataset", H5P_DEFAULT);
+        CHECK(did, FAIL, "H5Dopen2");
+
+        /* Read value from dataset */
+        ret = H5Dread(did, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, &val);
+        CHECK(ret, FAIL, "H5Dread");
+        VERIFY(val, 2, "H5Dread");
+
+        /* Close dataset */
+        ret = H5Dclose(did);
+        CHECK(ret, FAIL, "H5Dclose");
+
+        /* Only create new objects if file is open R/W */
+        if(open_rw) {
+            hid_t gid2;        /* Group ID */
+
+            /* Create a new group */
+            gid2 = H5Gcreate2(gid, "group2", H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
+            CHECK(gid, FAIL, "H5Gcreate2");
+
+            /* Close new group */
+            ret = H5Gclose(gid2);
+            CHECK(ret, FAIL, "H5Gclose");
+        } /* end if */
+
+        /* Close group */
+        ret = H5Gclose(gid);
+        CHECK(ret, FAIL, "H5Gclose");
+
+        /* Close file */
+        ret = H5Fclose(fid);
+        CHECK(ret, FAIL, "H5Fclose");
+    } /* end if */
+
+    return((GetTestNumErrs() == curr_num_errs) ? 0 : -1);
+} /* end test_userblock_alignment_helper2() */
+
+/****************************************************************
+**
+**  test_userblock_alignment(): low-level file test routine.
+**      This test checks to ensure that files with both a userblock and a
+**      object [allocation] alignment size set interact properly.
+**
+**  Programmer: Quincey Koziol
+**              koziol@hdfgroup.org
+**              Septmber 8, 2009
+**
+*****************************************************************/
+static void
+test_userblock_alignment(void)
+{
+    hid_t fid;          /* File ID */
+    hid_t fcpl;         /* File creation property list ID */
+    hid_t fapl;         /* File access property list ID */
+    herr_t ret;         /* Generic return value */
+
+    /* Output message about test being performed */
+    MESSAGE(5, ("Testing that non-zero userblocks and object alignment interact correctly.\n"));
+
+    /* Case 1:
+     *  Userblock size = 0, alignment != 0
+     * Outcome:
+     *  Should succeed
+     */
+    /* Create file creation property list with user block */
+    fcpl = H5Pcreate(H5P_FILE_CREATE);
+    CHECK(fcpl, FAIL, "H5Pcreate");
+    ret = H5Pset_userblock(fcpl, (hsize_t)0);
+    CHECK(ret, FAIL, "H5Pset_userblock");
+
+    /* Create file access property list with alignment */
+    fapl = H5Pcreate(H5P_FILE_ACCESS);
+    CHECK(fapl, FAIL, "H5Pcreate");
+    ret = H5Pset_alignment(fapl, (hsize_t)1, (hsize_t)3);
+    CHECK(ret, FAIL, "H5Pset_alignment");
+
+    /* Call helper routines to perform file manipulations */
+    ret = test_userblock_alignment_helper1(fcpl, fapl);
+    CHECK(ret, FAIL, "test_userblock_alignment_helper1");
+    ret = test_userblock_alignment_helper2(fapl, TRUE);
+    CHECK(ret, FAIL, "test_userblock_alignment_helper2");
+
+    /* Release property lists */
+    ret = H5Pclose(fcpl);
+    CHECK(ret, FAIL, "H5Pclose");
+    ret = H5Pclose(fapl);
+    CHECK(ret, FAIL, "H5Pclose");
+
+
+    /* Case 2:
+     *  Userblock size = 512, alignment = 16
+     *  (userblock is integral mult. of alignment)
+     * Outcome:
+     *  Should succeed
+     */
+    /* Create file creation property list with user block */
+    fcpl = H5Pcreate(H5P_FILE_CREATE);
+    CHECK(fcpl, FAIL, "H5Pcreate");
+    ret = H5Pset_userblock(fcpl, (hsize_t)512);
+    CHECK(ret, FAIL, "H5Pset_userblock");
+
+    /* Create file access property list with alignment */
+    fapl = H5Pcreate(H5P_FILE_ACCESS);
+    CHECK(fapl, FAIL, "H5Pcreate");
+    ret = H5Pset_alignment(fapl, (hsize_t)1, (hsize_t)16);
+    CHECK(ret, FAIL, "H5Pset_alignment");
+
+    /* Call helper routines to perform file manipulations */
+    ret = test_userblock_alignment_helper1(fcpl, fapl);
+    CHECK(ret, FAIL, "test_userblock_alignment_helper1");
+    ret = test_userblock_alignment_helper2(fapl, TRUE);
+    CHECK(ret, FAIL, "test_userblock_alignment_helper2");
+
+    /* Release property lists */
+    ret = H5Pclose(fcpl);
+    CHECK(ret, FAIL, "H5Pclose");
+    ret = H5Pclose(fapl);
+    CHECK(ret, FAIL, "H5Pclose");
+
+
+    /* Case 3:
+     *  Userblock size = 512, alignment = 512
+     *  (userblock is equal to alignment)
+     * Outcome:
+     *  Should succeed
+     */
+    /* Create file creation property list with user block */
+    fcpl = H5Pcreate(H5P_FILE_CREATE);
+    CHECK(fcpl, FAIL, "H5Pcreate");
+    ret = H5Pset_userblock(fcpl, (hsize_t)512);
+    CHECK(ret, FAIL, "H5Pset_userblock");
+
+    /* Create file access property list with alignment */
+    fapl = H5Pcreate(H5P_FILE_ACCESS);
+    CHECK(fapl, FAIL, "H5Pcreate");
+    ret = H5Pset_alignment(fapl, (hsize_t)1, (hsize_t)512);
+    CHECK(ret, FAIL, "H5Pset_alignment");
+
+    /* Call helper routines to perform file manipulations */
+    ret = test_userblock_alignment_helper1(fcpl, fapl);
+    CHECK(ret, FAIL, "test_userblock_alignment_helper1");
+    ret = test_userblock_alignment_helper2(fapl, TRUE);
+    CHECK(ret, FAIL, "test_userblock_alignment_helper2");
+
+    /* Release property lists */
+    ret = H5Pclose(fcpl);
+    CHECK(ret, FAIL, "H5Pclose");
+    ret = H5Pclose(fapl);
+    CHECK(ret, FAIL, "H5Pclose");
+
+
+    /* Case 4:
+     *  Userblock size = 512, alignment = 3
+     *  (userblock & alignment each individually valid, but userblock is
+     *          non-integral multiple of alignment)
+     * Outcome:
+     *  Should fail at file creation
+     */
+    /* Create file creation property list with user block */
+    fcpl = H5Pcreate(H5P_FILE_CREATE);
+    CHECK(fcpl, FAIL, "H5Pcreate");
+    ret = H5Pset_userblock(fcpl, (hsize_t)512);
+    CHECK(ret, FAIL, "H5Pset_userblock");
+
+    /* Create file access property list with alignment */
+    fapl = H5Pcreate(H5P_FILE_ACCESS);
+    CHECK(fapl, FAIL, "H5Pcreate");
+    ret = H5Pset_alignment(fapl, (hsize_t)1, (hsize_t)3);
+    CHECK(ret, FAIL, "H5Pset_alignment");
+
+    /* Create a file with FAPL & FCPL */
+    H5E_BEGIN_TRY {
+        fid = H5Fcreate(FILE1, H5F_ACC_TRUNC, fcpl, fapl);
+    } H5E_END_TRY;
+    VERIFY(fid, FAIL, "H5Fcreate");
+
+    /* Release property lists */
+    ret = H5Pclose(fcpl);
+    CHECK(ret, FAIL, "H5Pclose");
+    ret = H5Pclose(fapl);
+    CHECK(ret, FAIL, "H5Pclose");
+
+
+    /* Case 5:
+     *  Userblock size = 512, alignment = 1024
+     *  (userblock & alignment each individually valid, but userblock is
+     *          less than alignment)
+     * Outcome:
+     *  Should fail at file creation
+     */
+    /* Create file creation property list with user block */
+    fcpl = H5Pcreate(H5P_FILE_CREATE);
+    CHECK(fcpl, FAIL, "H5Pcreate");
+    ret = H5Pset_userblock(fcpl, (hsize_t)512);
+    CHECK(ret, FAIL, "H5Pset_userblock");
+
+    /* Create file access property list with alignment */
+    fapl = H5Pcreate(H5P_FILE_ACCESS);
+    CHECK(fapl, FAIL, "H5Pcreate");
+    ret = H5Pset_alignment(fapl, (hsize_t)1, (hsize_t)1024);
+    CHECK(ret, FAIL, "H5Pset_alignment");
+
+    /* Create a file with FAPL & FCPL */
+    H5E_BEGIN_TRY {
+        fid = H5Fcreate(FILE1, H5F_ACC_TRUNC, fcpl, fapl);
+    } H5E_END_TRY;
+    VERIFY(fid, FAIL, "H5Fcreate");
+
+    /* Release property lists */
+    ret = H5Pclose(fcpl);
+    CHECK(ret, FAIL, "H5Pclose");
+    ret = H5Pclose(fapl);
+    CHECK(ret, FAIL, "H5Pclose");
+
+
+    /* Case 6:
+     *  File created with:
+     *          Userblock size = 512, alignment = 512
+     *  File re-opened for read-only & read-write access with:
+     *          Userblock size = 512, alignment = 1024
+     * Outcome:
+     *  Should succeed
+     */
+    /* Create file creation property list with user block */
+    fcpl = H5Pcreate(H5P_FILE_CREATE);
+    CHECK(fcpl, FAIL, "H5Pcreate");
+    ret = H5Pset_userblock(fcpl, (hsize_t)512);
+    CHECK(ret, FAIL, "H5Pset_userblock");
+
+    /* Create file access property list with alignment */
+    fapl = H5Pcreate(H5P_FILE_ACCESS);
+    CHECK(fapl, FAIL, "H5Pcreate");
+    ret = H5Pset_alignment(fapl, (hsize_t)1, (hsize_t)512);
+    CHECK(ret, FAIL, "H5Pset_alignment");
+
+    /* Call helper routines to perform file manipulations */
+    ret = test_userblock_alignment_helper1(fcpl, fapl);
+    CHECK(ret, FAIL, "test_userblock_alignment_helper1");
+
+    /* Change alignment in FAPL */
+    ret = H5Pset_alignment(fapl, (hsize_t)1, (hsize_t)1024);
+    CHECK(ret, FAIL, "H5Pset_alignment");
+
+    /* Call helper routines to perform file manipulations */
+    ret = test_userblock_alignment_helper2(fapl, FALSE);
+    CHECK(ret, FAIL, "test_userblock_alignment_helper2");
+    ret = test_userblock_alignment_helper2(fapl, TRUE);
+    CHECK(ret, FAIL, "test_userblock_alignment_helper2");
+
+    /* Release property lists */
+    ret = H5Pclose(fcpl);
+    CHECK(ret, FAIL, "H5Pclose");
+    ret = H5Pclose(fapl);
+    CHECK(ret, FAIL, "H5Pclose");
+} /* end test_userblock_alignment() */
+
+/****************************************************************
+**
+**  test_libver_bounds_real():
+**      Verify that a file created and modified with the
+**      specified libver bounds has the specified object header
+**      versions for the right objects.
+**
+****************************************************************/
+static void
+test_libver_bounds_real(H5F_libver_t libver_create, unsigned oh_vers_create,
+    H5F_libver_t libver_mod, unsigned oh_vers_mod)
+{
+    hid_t       file, group;            /* Handles */
+    hid_t       fapl;                   /* File access property list */
+    H5O_info_t  oinfo;                  /* Object info */
+    herr_t      ret;                    /* Return value */
+
+    /*
+     * Create a new file using the creation properties.
+     */
+    fapl = H5Pcreate(H5P_FILE_ACCESS);
+    CHECK(fapl, FAIL, "H5Pcreate");
+
+    ret = H5Pset_libver_bounds(fapl, libver_create, H5F_LIBVER_LATEST);
+    CHECK(ret, FAIL, "H5Pset_libver_bounds");
+
+    file = H5Fcreate("tfile5.h5", H5F_ACC_TRUNC, H5P_DEFAULT, fapl);
+    CHECK(file, FAIL, "H5Fcreate");
+
+    /*
+     * Make sure the root group has the correct object header version
+     */
+    ret = H5Oget_info_by_name(file, "/", &oinfo, H5P_DEFAULT);
+    CHECK(ret, FAIL, "H5Oget_info_by_name");
+    VERIFY(oinfo.hdr.version, oh_vers_create, "H5Oget_info_by_name");
+
+    /*
+     * Reopen the file and make sure the root group still has the correct version
+     */
+    ret = H5Fclose(file);
+    CHECK(ret, FAIL, "H5Fclose");
+
+    ret = H5Pset_libver_bounds(fapl, libver_mod, H5F_LIBVER_LATEST);
+    CHECK(ret, FAIL, "H5Pset_libver_bounds");
+
+    file = H5Fopen("tfile5.h5", H5F_ACC_RDWR, fapl);
+    CHECK(file, FAIL, "H5Fopen");
+
+    ret = H5Oget_info_by_name(file, "/", &oinfo, H5P_DEFAULT);
+    CHECK(ret, FAIL, "H5Oget_info_by_name");
+    VERIFY(oinfo.hdr.version, oh_vers_create, "H5Oget_info_by_name");
+
+    /*
+     * Create a group named "G1" in the file, and make sure it has the correct
+     * object header version
+     */
+    group = H5Gcreate2(file, "/G1", H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
+    CHECK(group, FAIL, "H5Gcreate");
+
+    ret = H5Oget_info(group, &oinfo);
+    CHECK(ret, FAIL, "H5Oget_info_by_name");
+    VERIFY(oinfo.hdr.version, oh_vers_mod, "H5Oget_info_by_name");
+
+    ret = H5Gclose(group);
+    CHECK(ret, FAIL, "H5Gclose");
+
+    /*
+     * Create a group named "/G1/G3" in the file, and make sure it has the
+     * correct object header version
+     */
+    group = H5Gcreate2(file, "/G1/G3", H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
+    CHECK(group, FAIL, "H5Gcreate");
+
+    ret = H5Oget_info(group, &oinfo);
+    CHECK(ret, FAIL, "H5Oget_info_by_name");
+    VERIFY(oinfo.hdr.version, oh_vers_mod, "H5Oget_info_by_name");
+
+    ret = H5Gclose(group);
+    CHECK(ret, FAIL, "H5Gclose");
+
+    /*
+     * Make sure the root group still has the correct object header version
+     */
+    ret = H5Oget_info_by_name(file, "/", &oinfo, H5P_DEFAULT);
+    CHECK(ret, FAIL, "H5Oget_info_by_name");
+    VERIFY(oinfo.hdr.version, oh_vers_create, "H5Oget_info_by_name");
+
+    ret = H5Fclose(file);
+    CHECK(ret, FAIL, "H5Fclose");
+} /* end test_libver_bounds_real() */
+
+/****************************************************************
+**
+**  test_libver_bounds():
+**	Verify that a file created and modified with various
+**      libver bounds is handled correctly.  (Further testing
+**      welcome)
+**
+****************************************************************/
+static void
+test_libver_bounds(void)
+{
+    /* Output message about test being performed */
+    MESSAGE(5, ("Testing setting library version bounds\n"));
+
+    /* Run the tests */
+    test_libver_bounds_real(H5F_LIBVER_EARLIEST, 1, H5F_LIBVER_LATEST, 2);
+    test_libver_bounds_real(H5F_LIBVER_EARLIEST, 1, H5F_LIBVER_18, 2);
+    test_libver_bounds_real(H5F_LIBVER_LATEST, 2, H5F_LIBVER_EARLIEST, 1);
+    test_libver_bounds_real(H5F_LIBVER_18, 2, H5F_LIBVER_EARLIEST, 1);
+} /* end test_libver_bounds() */
+
+/****************************************************************
+**
+**  test_libver_macros():
+**	Verify that H5_VERSION_GE and H5_VERSION_LE work correactly.
+**
+****************************************************************/
+static void
+test_libver_macros(void)
+{
+    unsigned	major = H5_VERS_MAJOR;
+    unsigned	minor = H5_VERS_MINOR;
+    unsigned	release = H5_VERS_RELEASE;
+    herr_t	ret;                    /* Return value */
+
+    /* Output message about test being performed */
+    MESSAGE(5, ("Testing macros for library version comparison\n"));
+
+    VERIFY(H5_VERSION_GE(major,minor,release), TRUE, "H5_VERSION_GE");
+    VERIFY(H5_VERSION_GE(major-1,minor,release), TRUE, "H5_VERSION_GE");
+    VERIFY(H5_VERSION_GE(major-1,minor+1,release), TRUE, "H5_VERSION_GE");
+    VERIFY(H5_VERSION_GE(major-1,minor,release+1), TRUE, "H5_VERSION_GE");
+    VERIFY(H5_VERSION_GE(major,minor-1,release), TRUE, "H5_VERSION_GE");
+    VERIFY(H5_VERSION_GE(major,minor-1,release+1), TRUE, "H5_VERSION_GE");
+    VERIFY(H5_VERSION_GE(major,minor,release-1), TRUE, "H5_VERSION_GE");
+
+    VERIFY(H5_VERSION_GE(major+1,minor,release), FALSE, "H5_VERSION_GE");
+    VERIFY(H5_VERSION_GE(major+1,minor-1,release), FALSE, "H5_VERSION_GE");
+    VERIFY(H5_VERSION_GE(major+1,minor-1,release-1), FALSE, "H5_VERSION_GE");
+    VERIFY(H5_VERSION_GE(major,minor+1,release), FALSE, "H5_VERSION_GE");
+    VERIFY(H5_VERSION_GE(major,minor+1,release-1), FALSE, "H5_VERSION_GE");
+    VERIFY(H5_VERSION_GE(major,minor,release+1), FALSE, "H5_VERSION_GE");
+
+    VERIFY(H5_VERSION_LE(major,minor,release), TRUE, "H5_VERSION_LE");
+    VERIFY(H5_VERSION_LE(major+1,minor,release), TRUE, "H5_VERSION_LE");
+    VERIFY(H5_VERSION_LE(major+1,minor-1,release), TRUE, "H5_VERSION_LE");
+    VERIFY(H5_VERSION_LE(major+1,minor-1,release-1), TRUE, "H5_VERSION_LE");
+    VERIFY(H5_VERSION_LE(major,minor+1,release), TRUE, "H5_VERSION_LE");
+    VERIFY(H5_VERSION_LE(major,minor+1,release-1), TRUE, "H5_VERSION_LE");
+    VERIFY(H5_VERSION_LE(major,minor,release+1), TRUE, "H5_VERSION_LE");
+
+    VERIFY(H5_VERSION_LE(major-1,minor,release), FALSE, "H5_VERSION_LE");
+    VERIFY(H5_VERSION_LE(major-1,minor+1,release), FALSE, "H5_VERSION_LE");
+    VERIFY(H5_VERSION_LE(major-1,minor+1,release+1), FALSE, "H5_VERSION_LE");
+    VERIFY(H5_VERSION_LE(major,minor-1,release), FALSE, "H5_VERSION_LE");
+    VERIFY(H5_VERSION_LE(major,minor-1,release+1), FALSE, "H5_VERSION_LE");
+    VERIFY(H5_VERSION_LE(major,minor,release-1), FALSE, "H5_VERSION_LE");
+} /* test_libver_macros() */
+
+/****************************************************************
+**
+**  test_libver_macros2():
+**	Verify that H5_VERSION_GE works correactly and show how
+**      to use it.
+**
+****************************************************************/
+static void
+test_libver_macros2(void)
+{
+    hid_t    file;
+    hid_t    grp;
+    htri_t   status;
+    herr_t   ret;                    /* Return value */
+
+    /* Output message about test being performed */
+    MESSAGE(5, ("Testing macros for library version comparison with a file\n"));
+
+    /*
+     * Create a file.
+     */
+    file = H5Fcreate(FILE5, H5F_ACC_TRUNC, H5P_DEFAULT, H5P_DEFAULT);
+    CHECK(file, FAIL, "H5Fcreate");
+
+    /*
+     * Create a group in the file.
+     */
+    grp = H5Gcreate2(file, "Group", H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
+    CHECK(file, FAIL, "H5Gcreate");
+
+    /*
+     * Close the group
+     */
+    ret = H5Gclose(grp);
+    CHECK(ret, FAIL, "H5Gclose");
+
+    /* 
+     * Delete the group using different function based on the library version.
+     *  And verify the action. 
+     */
+#if H5_VERSION_GE(1,8,0)
+    ret = H5Ldelete(file, "Group", H5P_DEFAULT);
+    CHECK(ret, FAIL, "H5Lunlink");
+
+    status = H5Lexists(file, "Group", H5P_DEFAULT);
+    VERIFY(status, FALSE, "H5Lexists");
+#else
+    ret = H5Gunlink(file, "Group");
+    CHECK(ret, FAIL, "H5Gunlink");
+
+    H5E_BEGIN_TRY {
+        grp = H5Gopen(file, "Group");
+    } H5E_END_TRY;
+    VERIFY(grp, FAIL, "H5Gopen");
+#endif
+
+    /*
+     * Close the file.
+     */
+    ret = H5Fclose(file);
+    CHECK(ret, FAIL, "H5Fclose");
+
+} /* test_libver_macros2() */
+
 /****************************************************************
 **
 **  test_file(): Main low-level file I/O test routine.
@@ -2058,7 +2764,12 @@ test_file(void)
 #endif /*H5_CANNOT_OPEN_TWICE*/
     test_userblock_file_size(); /* Tests that files created with a userblock have the correct size */
     test_cached_stab_info();    /* Tests that files are created with cached stab info in the superblock */
-}				/* test_file() */
+    test_rw_noupdate();         /* Test to ensure that RW permissions don't write the file unless dirtied */
+    test_userblock_alignment(); /* Tests that files created with a userblock and alignment interact properly */
+    test_libver_bounds();       /* Test compatibility for file space management */
+    test_libver_macros();       /* Test the macros for library version comparison */
+    test_libver_macros2();      /* Show the use of the macros for library version comparison */
+} /* test_file() */
 
 
 /*-------------------------------------------------------------------------
@@ -2078,6 +2789,7 @@ test_file(void)
 void
 cleanup_file(void)
 {
+    HDremove(SFILE1);
     HDremove(FILE1);
     HDremove(FILE2);
     HDremove(FILE3);
