@@ -71,7 +71,7 @@ test_skiplist_init(void)
 
     /* Initialize random number seed */
     curr_time = HDtime(NULL);
-    HDsrandom((unsigned long)curr_time);
+    HDsrandom((unsigned)curr_time);
 
     /* Create randomized set of numbers */
     for(u=0; u<NUM_ELEMS; u++) {
@@ -121,7 +121,7 @@ test_skiplist_create(void)
     MESSAGE(6, ("Testing Creating & Closing Skip Lists\n"));
 
     /* Try creating a skip list */
-    slist = H5SL_create(H5SL_TYPE_INT, 0.5, (size_t)16);
+    slist = H5SL_create(H5SL_TYPE_INT);
     CHECK(slist, NULL, "H5SL_create");
 
     /* Try closing the skip list */
@@ -151,7 +151,7 @@ test_skiplist_insert(void)
     MESSAGE(7, ("Testing Insertion Into Skip List\n"));
 
     /* Create a skip list */
-    slist = H5SL_create(H5SL_TYPE_INT, 0.5, (size_t)16);
+    slist = H5SL_create(H5SL_TYPE_INT);
     CHECK(slist, NULL, "H5SL_create");
 
     /* Check that the skip list has no elements */
@@ -212,7 +212,7 @@ test_skiplist_insert_many(void)
     MESSAGE(7, ("Testing Insertion of Many Items Into Skip List\n"));
 
     /* Create a skip list */
-    slist = H5SL_create(H5SL_TYPE_INT, 0.5, (size_t)16);
+    slist = H5SL_create(H5SL_TYPE_INT);
     CHECK(slist, NULL, "H5SL_create");
 
     /* Check that the skip list has no elements */
@@ -286,7 +286,7 @@ test_skiplist_remove(void)
     MESSAGE(7, ("Testing Removal From Skip List\n"));
 
     /* Create a skip list */
-    slist = H5SL_create(H5SL_TYPE_INT, 0.5, (size_t)16);
+    slist = H5SL_create(H5SL_TYPE_INT);
     CHECK(slist, NULL, "H5SL_create");
 
     /* Check that the skip list has no elements */
@@ -365,7 +365,7 @@ test_skiplist_remove_many(void)
     MESSAGE(7, ("Testing Removal of Many Items From Skip List\n"));
 
     /* Create a skip list */
-    slist = H5SL_create(H5SL_TYPE_INT, 0.5, (size_t)16);
+    slist = H5SL_create(H5SL_TYPE_INT);
     CHECK(slist, NULL, "H5SL_create");
 
     /* Check that the skip list has no elements */
@@ -465,7 +465,7 @@ test_skiplist_firstnext(void)
     MESSAGE(7, ("Testing Iterating Over Skip List With First/Next\n"));
 
     /* Create a skip list */
-    slist = H5SL_create(H5SL_TYPE_INT, 0.5, (size_t)16);
+    slist = H5SL_create(H5SL_TYPE_INT);
     CHECK(slist, NULL, "H5SL_create");
 
     /* Check that the skip list has no elements */
@@ -558,7 +558,7 @@ test_skiplist_string(void)
     MESSAGE(7, ("Testing Skip List With String Keys\n"));
 
     /* Create a skip list */
-    slist = H5SL_create(H5SL_TYPE_STR, 0.5, (size_t)16);
+    slist = H5SL_create(H5SL_TYPE_STR);
     CHECK(slist, NULL, "H5SL_create");
 
     /* Check that the skip list has no elements */
@@ -620,7 +620,7 @@ test_skiplist_iterate(void)
     MESSAGE(7, ("Testing Iterating Over Skip List\n"));
 
     /* Create a skip list */
-    slist = H5SL_create(H5SL_TYPE_INT, 0.5, (size_t)16);
+    slist = H5SL_create(H5SL_TYPE_INT);
     CHECK(slist, NULL, "H5SL_create");
 
     /* Check that the skip list has no elements */
@@ -670,7 +670,7 @@ test_skiplist_hsize(void)
     MESSAGE(7, ("Testing Skip List With hsize_t Keys\n"));
 
     /* Create a skip list */
-    slist = H5SL_create(H5SL_TYPE_HSIZE, 0.5, (size_t)16);
+    slist = H5SL_create(H5SL_TYPE_HSIZE);
     CHECK(slist, NULL, "H5SL_create");
 
     /* Check that the skip list has no elements */
@@ -725,7 +725,7 @@ test_skiplist_unsigned(void)
     MESSAGE(7, ("Testing Skip List With unsigned Keys\n"));
 
     /* Create a skip list */
-    slist = H5SL_create(H5SL_TYPE_UNSIGNED, 0.5, (size_t)16);
+    slist = H5SL_create(H5SL_TYPE_UNSIGNED);
     CHECK(slist, NULL, "H5SL_create");
 
     /* Check that the skip list has no elements */
@@ -778,7 +778,7 @@ test_skiplist_lastprev(void)
     MESSAGE(7, ("Testing Iterating Over Skip List With Last/Prev\n"));
 
     /* Create a skip list */
-    slist = H5SL_create(H5SL_TYPE_INT, 0.5, (size_t)16);
+    slist = H5SL_create(H5SL_TYPE_INT);
     CHECK(slist, NULL, "H5SL_create");
 
     /* Check that the skip list has no elements */
@@ -846,7 +846,7 @@ test_skiplist_find(void)
     MESSAGE(7, ("Testing Skip List 'Find' Operation\n"));
 
     /* Create a skip list */
-    slist = H5SL_create(H5SL_TYPE_UNSIGNED, 0.5, (size_t)16);
+    slist = H5SL_create(H5SL_TYPE_UNSIGNED);
     CHECK(slist, NULL, "H5SL_create");
 
     /* Insert objects into the skip list */
@@ -902,7 +902,7 @@ test_skiplist_add(void)
     MESSAGE(7, ("Testing Skip List 'Add' Operation\n"));
 
     /* Create a skip list */
-    slist = H5SL_create(H5SL_TYPE_UNSIGNED, 0.5, (size_t)16);
+    slist = H5SL_create(H5SL_TYPE_UNSIGNED);
     CHECK(slist, NULL, "H5SL_create");
 
     /* Insert objects into the skip list */
@@ -964,7 +964,7 @@ test_skiplist_destroy(void)
     MESSAGE(7, ("Testing Skip List 'Destroy' Operation\n"));
 
     /* Create a skip list */
-    slist = H5SL_create(H5SL_TYPE_INT, 0.5, (size_t)16);
+    slist = H5SL_create(H5SL_TYPE_INT);
     CHECK(slist, NULL, "H5SL_create");
 
     /* Insert objects into the skip list */
@@ -1000,7 +1000,7 @@ test_skiplist_free(void)
     MESSAGE(7, ("Testing Skip List 'Free' Operation\n"));
 
     /* Create a skip list */
-    slist = H5SL_create(H5SL_TYPE_INT, 0.5, (size_t)16);
+    slist = H5SL_create(H5SL_TYPE_INT);
     CHECK(slist, NULL, "H5SL_create");
 
     /* Insert objects into the skip list */
@@ -1056,7 +1056,7 @@ test_skiplist_less(void)
     MESSAGE(7, ("Testing Skip List 'Less' Operation\n"));
 
     /* Create a skip list */
-    slist = H5SL_create(H5SL_TYPE_UNSIGNED, 0.5, (size_t)16);
+    slist = H5SL_create(H5SL_TYPE_UNSIGNED);
     CHECK(slist, NULL, "H5SL_create");
 
     /* Insert objects into the skip list */
@@ -1120,7 +1120,7 @@ test_skiplist_greater(void)
     MESSAGE(7, ("Testing Skip List 'Greater' Operation\n"));
 
     /* Create a skip list */
-    slist = H5SL_create(H5SL_TYPE_UNSIGNED, 0.5, (size_t)16);
+    slist = H5SL_create(H5SL_TYPE_UNSIGNED);
     CHECK(slist, NULL, "H5SL_create");
 
     /* Insert objects into the skip list */
@@ -1163,6 +1163,164 @@ test_skiplist_greater(void)
 
 } /* end test_skiplist_greater() */
 
+/****************************************************************
+**
+**  test_skiplist_below(): Test H5SL (skip list) code.
+**      Tests 'below' operation in skip lists.
+**
+****************************************************************/
+static void
+test_skiplist_below(void)
+{
+    H5SL_t *slist;      /* Skip list created */
+    H5SL_node_t *node;  /* Skip list node */
+    size_t u;           /* Local index variable */
+    unsigned data[10]={ 10, 20, 15, 5, 50, 30, 31, 32, 80, 90};
+    /* unsigned sorted_data[10]={ 5, 10, 15, 20, 30, 31, 32, 50, 80, 90}; */
+    unsigned *found_item;    /* Item found in skip list */
+    unsigned find_item;  /* Item to add to skip list */
+    herr_t ret;         /* Generic return value */
+
+    /* Output message about test being performed */
+    MESSAGE(7, ("Testing Skip List 'Below' Operation\n"));
+
+    /* Create a skip list */
+    slist = H5SL_create(H5SL_TYPE_UNSIGNED);
+    CHECK(slist, NULL, "H5SL_create");
+
+    /* Insert objects into the skip list */
+    for(u = 0; u < 10; u++) {
+        ret = H5SL_insert(slist, &data[u], &data[u]);
+        CHECK(ret, FAIL, "H5SL_insert");
+    } /* end for */
+
+    /* Check for exact match of items in various positions */
+    find_item = 20;
+    node = H5SL_below(slist, &find_item);
+    CHECK(node, NULL, "H5SL_below");
+    found_item = H5SL_item(node);
+    VERIFY(*found_item, find_item, "H5SL_below");
+    find_item = 90;
+    node = H5SL_below(slist, &find_item);
+    CHECK(node, NULL, "H5SL_below");
+    found_item = H5SL_item(node);
+    VERIFY(*found_item, find_item, "H5SL_below");
+    find_item = 5;
+    node = H5SL_below(slist, &find_item);
+    CHECK(node, NULL, "H5SL_below");
+    found_item = H5SL_item(node);
+    VERIFY(*found_item, find_item, "H5SL_below");
+
+    /* Find item less than a missing key, in various positions */
+    find_item = 19;
+    node = H5SL_below(slist, &find_item);
+    CHECK(node, NULL, "H5SL_below");
+    found_item = H5SL_item(node);
+    VERIFY(*found_item, 15, "H5SL_below");
+    find_item = 89;
+    node = H5SL_below(slist, &find_item);
+    CHECK(node, NULL, "H5SL_below");
+    found_item = H5SL_item(node);
+    VERIFY(*found_item, 80, "H5SL_below");
+    find_item = 100;
+    node = H5SL_below(slist, &find_item);
+    CHECK(node, NULL, "H5SL_below");
+    found_item = H5SL_item(node);
+    VERIFY(*found_item, 90, "H5SL_below");
+    find_item = 9;
+    node = H5SL_below(slist, &find_item);
+    CHECK(node, NULL, "H5SL_below");
+    found_item = H5SL_item(node);
+    VERIFY(*found_item, 5, "H5SL_below");
+    find_item = 4;
+    node = H5SL_less(slist, &find_item);
+    VERIFY(node, NULL, "H5SL_below");
+
+    /* Close the skip list */
+    ret = H5SL_close(slist);
+    CHECK(ret, FAIL, "H5SL_close");
+
+} /* end test_skiplist_below() */
+
+/****************************************************************
+**
+**  test_skiplist_above(): Test H5SL (skip list) code.
+**      Tests 'above' operation in skip lists.
+**
+****************************************************************/
+static void
+test_skiplist_above(void)
+{
+    H5SL_t *slist;      /* Skip list created */
+    H5SL_node_t *node;  /* Skip list node */
+    size_t u;           /* Local index variable */
+    unsigned data[10]={ 10, 20, 15, 5, 50, 30, 31, 32, 80, 90};
+    /* unsigned sorted_data[10]={ 5, 10, 15, 20, 30, 31, 32, 50, 80, 90}; */
+    unsigned *found_item;    /* Item found in skip list */
+    unsigned find_item;  /* Item to add to skip list */
+    herr_t ret;         /* Generic return value */
+
+    /* Output message about test being performed */
+    MESSAGE(7, ("Testing Skip List 'Above' Operation\n"));
+
+    /* Create a skip list */
+    slist = H5SL_create(H5SL_TYPE_UNSIGNED);
+    CHECK(slist, NULL, "H5SL_create");
+
+    /* Insert objects into the skip list */
+    for(u = 0; u < 10; u++) {
+        ret = H5SL_insert(slist, &data[u], &data[u]);
+        CHECK(ret, FAIL, "H5SL_insert");
+    } /* end for */
+
+    /* Check for exact match of items in various positions */
+    find_item = 20;
+    node = H5SL_above(slist, &find_item);
+    CHECK(node, NULL, "H5SL_above");
+    found_item = H5SL_item(node);
+    VERIFY(*found_item, find_item, "H5SL_above");
+    find_item = 90;
+    node = H5SL_above(slist, &find_item);
+    CHECK(node, NULL, "H5SL_above");
+    found_item = H5SL_item(node);
+    VERIFY(*found_item, find_item, "H5SL_above");
+    find_item = 5;
+    node = H5SL_above(slist, &find_item);
+    CHECK(node, NULL, "H5SL_above");
+    found_item = H5SL_item(node);
+    VERIFY(*found_item, find_item, "H5SL_above");
+
+    /* Find item greater than a missing key, in various positions */
+    find_item = 19;
+    node = H5SL_above(slist, &find_item);
+    CHECK(node, NULL, "H5SL_above");
+    found_item = H5SL_item(node);
+    VERIFY(*found_item, 20, "H5SL_above");
+    find_item = 89;
+    node = H5SL_above(slist, &find_item);
+    CHECK(node, NULL, "H5SL_above");
+    found_item = H5SL_item(node);
+    VERIFY(*found_item, 90, "H5SL_above");
+    find_item = 100;
+    node = H5SL_above(slist, &find_item);
+    VERIFY(node, NULL, "H5SL_above");
+    find_item = 6;
+    node = H5SL_above(slist, &find_item);
+    CHECK(node, NULL, "H5SL_above");
+    found_item = H5SL_item(node);
+    VERIFY(*found_item, 10, "H5SL_above");
+    find_item = 4;
+    node = H5SL_above(slist, &find_item);
+    CHECK(node, NULL, "H5SL_above");
+    found_item = H5SL_item(node);
+    VERIFY(*found_item, 5, "H5SL_above");
+
+    /* Close the skip list */
+    ret = H5SL_close(slist);
+    CHECK(ret, FAIL, "H5SL_close");
+
+} /* end test_skiplist_above() */
+
 /****************************************************************
 **
 **  test_skiplist_remote_first(): Test H5SL (skip list) code.
@@ -1180,10 +1338,10 @@ test_skiplist_remove_first(void)
     herr_t ret;         /* Generic return value */
 
     /* Output message about test being performed */
-    MESSAGE(7, ("Testing Skip List 'Greater' Operation\n"));
+    MESSAGE(7, ("Testing Skip List 'Remove First' Operation\n"));
 
     /* Create a skip list */
-    slist = H5SL_create(H5SL_TYPE_UNSIGNED, 0.5, (size_t)16);
+    slist = H5SL_create(H5SL_TYPE_UNSIGNED);
     CHECK(slist, NULL, "H5SL_create");
 
     /* Insert objects into the skip list */
@@ -1208,6 +1366,51 @@ test_skiplist_remove_first(void)
 
 } /* end test_skiplist_remove_first() */
 
+/****************************************************************
+**
+**  test_skiplist_remote_first_many(): Test H5SL (skip list) code.
+**      Tests 'remove first' operation in large skip lists.
+**
+****************************************************************/
+static void
+test_skiplist_remove_first_many(void)
+{
+    H5SL_t  *slist;         /* Skip list created */
+    size_t  u;              /* Local index variable */
+    int     *found_item;    /* Item found in skip list */
+    int     prev_item = INT_MIN; /* Previously found item in skip list */
+    herr_t  ret;            /* Generic return value */
+
+    /* Output message about test being performed */
+    MESSAGE(7, ("Testing Skip List 'Remove First' Operation\n"));
+
+    /* Create a skip list */
+    slist = H5SL_create(H5SL_TYPE_INT);
+    CHECK(slist, NULL, "H5SL_create");
+
+    /* Insert objects into the skip list */
+    for(u = 0; u < NUM_ELEMS; u++) {
+        ret = H5SL_insert(slist, &rand_num[u], &rand_num[u]);
+        CHECK(ret, FAIL, "H5SL_insert");
+    } /* end for */
+
+    /* Remove objects from the skip list */
+    for(u = 0; u < NUM_ELEMS; u++) {
+        found_item = (int *)H5SL_remove_first(slist);
+        VERIFY(*found_item > prev_item, TRUE, "H5SL_remove_first");
+        prev_item = *found_item;
+    } /* end for */
+
+    /* Check for removing object from empty list */
+    found_item = (int *)H5SL_remove_first(slist);
+    VERIFY(found_item, NULL, "H5SL_remove_first");
+
+    /* Close the skip list */
+    ret = H5SL_close(slist);
+    CHECK(ret, FAIL, "H5SL_close");
+
+} /* end test_skiplist_remove_first() */
+
 /****************************************************************
 **
 **  test_skiplist(): Main H5SL testing routine.
@@ -1240,7 +1443,10 @@ test_skiplist(void)
     test_skiplist_free();       /* Test 'free' operation */
     test_skiplist_less();       /* Test 'less' operation */
     test_skiplist_greater();    /* Test 'greater' operation */
-    test_skiplist_remove_first();       /* Test 'remove first' operation */
+    test_skiplist_below();      /* Test 'below' operation */
+    test_skiplist_above();      /* Test 'above' operation */
+    test_skiplist_remove_first();   /* Test 'remove first' operation */
+    test_skiplist_remove_first_many();  /* Test 'remove first' operation on large skip lists */
 
 }   /* end test_skiplist() */
 
