@@ -44,8 +44,6 @@ const char *FILENAME[] = {
     NULL
 };
 
-#define THE_OBJECT	"/foo"
-
 /* Macros for test_create_unlink() & test_filespace */
 #define GROUPNAME       "group"
 #define GROUP2NAME      "group2"
@@ -527,16 +525,6 @@ test_filespace(hid_t fapl)
     size_t rdcc_nelmts;
     size_t rdcc_nbytes;
     double rdcc_w0;
-    const char *envval = NULL;
-    int    ExpressMode;
-
-    /* Don't run some tests for some drivers */
-    envval = HDgetenv("HDF5_DRIVER");
-    if(envval == NULL)
-        envval = "nomatch";
-
-    /* See if some tests can be skipped */
-    ExpressMode = GetTestExpress();
 
     puts("Testing file space gets reused:");
 
@@ -564,7 +552,7 @@ test_filespace(hid_t fapl)
     if(H5Fclose(file) < 0) TEST_ERROR
 
     /* Get the size of an empty file */
-    if((empty_size=h5_get_file_size(filename))<0) TEST_ERROR
+    if((empty_size = h5_get_file_size(filename, fapl)) < 0) TEST_ERROR
 
 /* Create common objects for datasets */
 
@@ -602,7 +590,7 @@ test_filespace(hid_t fapl)
     if((space = H5Screate_simple(FILESPACE_NDIMS, dims, NULL)) < 0) TEST_ERROR
 
     /* Create buffer for writing dataset */
-    if((data = HDmalloc(sizeof(int)*FILESPACE_DIM0*FILESPACE_DIM1*FILESPACE_DIM2))==NULL) TEST_ERROR
+    if(NULL == (data = (int *)HDmalloc(sizeof(int) * FILESPACE_DIM0 * FILESPACE_DIM1 * FILESPACE_DIM2))) TEST_ERROR
 
 
 /* Create single dataset (with contiguous storage & late allocation), remove it & verify file size */
@@ -622,7 +610,7 @@ test_filespace(hid_t fapl)
     if(H5Fclose(file) < 0) FAIL_STACK_ERROR
 
     /* Get the size of the file */
-    if((file_size = h5_get_file_size(filename)) < 0) TEST_ERROR
+    if((file_size = h5_get_file_size(filename, fapl)) < 0) TEST_ERROR
 
     /* Verify the file is correct size */
     if(file_size != empty_size) TEST_ERROR
@@ -647,7 +635,7 @@ test_filespace(hid_t fapl)
     if(H5Fclose(file) < 0) FAIL_STACK_ERROR
 
     /* Get the size of the file */
-    if((file_size = h5_get_file_size(filename))<0) TEST_ERROR
+    if((file_size = h5_get_file_size(filename, fapl)) < 0) TEST_ERROR
 
     /* Verify the file is correct size */
     if(file_size != empty_size) TEST_ERROR
@@ -672,7 +660,7 @@ test_filespace(hid_t fapl)
     if(H5Fclose(file) < 0) FAIL_STACK_ERROR
 
     /* Get the size of the file */
-    if((file_size = h5_get_file_size(filename))<0) TEST_ERROR
+    if((file_size = h5_get_file_size(filename, fapl)) < 0) TEST_ERROR
 
     /* Verify the file is correct size */
     if(file_size != empty_size) TEST_ERROR
@@ -697,7 +685,7 @@ test_filespace(hid_t fapl)
     if(H5Fclose(file) < 0) FAIL_STACK_ERROR
 
     /* Get the size of the file */
-    if((file_size = h5_get_file_size(filename)) < 0) TEST_ERROR
+    if((file_size = h5_get_file_size(filename, fapl)) < 0) TEST_ERROR
 
     /* Verify the file is correct size */
     if(file_size != empty_size) TEST_ERROR
@@ -722,7 +710,7 @@ test_filespace(hid_t fapl)
     if(H5Fclose(file) < 0) FAIL_STACK_ERROR
 
     /* Get the size of the file */
-    if((file_size = h5_get_file_size(filename)) < 0) TEST_ERROR
+    if((file_size = h5_get_file_size(filename, fapl)) < 0) TEST_ERROR
 
     /* Verify the file is correct size */
     if(file_size != empty_size) TEST_ERROR
@@ -746,7 +734,7 @@ test_filespace(hid_t fapl)
     for(u = 0; u < FILESPACE_REWRITE; u++) {
         /* Set buffer to some compressible values */
         for(v = 0, tmp_data = data; v < (FILESPACE_DIM0 * FILESPACE_DIM1 * FILESPACE_DIM2); v++)
-            *tmp_data++ = v * u;
+            *tmp_data++ = (int)(v * u);
 
         /* Write the buffer to the dataset */
         if(H5Dwrite(dataset, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, data) < 0) FAIL_STACK_ERROR
@@ -769,7 +757,7 @@ test_filespace(hid_t fapl)
     if(H5Fclose(file) < 0) FAIL_STACK_ERROR
 
     /* Get the size of the file */
-    if((file_size = h5_get_file_size(filename)) < 0) TEST_ERROR
+    if((file_size = h5_get_file_size(filename, fapl_nocache)) < 0) TEST_ERROR
 
     /* Verify the file is correct size */
     if(file_size != empty_size) TEST_ERROR
@@ -794,7 +782,7 @@ test_filespace(hid_t fapl)
     if(H5Fclose(file) < 0) FAIL_STACK_ERROR
 
     /* Get the size of the file */
-    if((file_size = h5_get_file_size(filename)) < 0) TEST_ERROR
+    if((file_size = h5_get_file_size(filename, fapl)) < 0) TEST_ERROR
 
     /* Verify the file is correct size */
     if(file_size != empty_size) TEST_ERROR
@@ -858,7 +846,7 @@ test_filespace(hid_t fapl)
     if(H5Fclose(file) < 0) FAIL_STACK_ERROR
 
     /* Get the size of the file */
-    if((file_size = h5_get_file_size(filename)) < 0) TEST_ERROR
+    if((file_size = h5_get_file_size(filename, fapl)) < 0) TEST_ERROR
 
     /* Verify the file is correct size */
     if(file_size != empty_size) TEST_ERROR
@@ -886,7 +874,7 @@ test_filespace(hid_t fapl)
     if(H5Fclose(file) < 0) FAIL_STACK_ERROR
 
     /* Get the size of the file */
-    if((file_size = h5_get_file_size(filename)) < 0) TEST_ERROR
+    if((file_size = h5_get_file_size(filename, fapl)) < 0) TEST_ERROR
 
     /* Verify the file is correct size */
     if(file_size != empty_size) TEST_ERROR
@@ -911,7 +899,7 @@ test_filespace(hid_t fapl)
     if(H5Fclose(file) < 0) FAIL_STACK_ERROR
 
     /* Get the size of the file */
-    if((file_size = h5_get_file_size(filename)) < 0) TEST_ERROR
+    if((file_size = h5_get_file_size(filename, fapl)) < 0) TEST_ERROR
 
     /* Verify the file is correct size */
     if(file_size != empty_size) TEST_ERROR
@@ -943,7 +931,7 @@ test_filespace(hid_t fapl)
     if(H5Fclose(file) < 0) FAIL_STACK_ERROR
 
     /* Get the size of the file */
-    if((file_size = h5_get_file_size(filename)) < 0) TEST_ERROR
+    if((file_size = h5_get_file_size(filename, fapl)) < 0) TEST_ERROR
 
     /* Verify the file is correct size */
     if(file_size != empty_size) TEST_ERROR
@@ -973,7 +961,7 @@ test_filespace(hid_t fapl)
     if(H5Fclose(file) < 0) FAIL_STACK_ERROR
 
     /* Get the size of the file */
-    if((file_size = h5_get_file_size(filename)) < 0) TEST_ERROR
+    if((file_size = h5_get_file_size(filename, fapl)) < 0) TEST_ERROR
 
     /* Verify the file is correct size */
     if(file_size != empty_size) TEST_ERROR
@@ -984,92 +972,83 @@ test_filespace(hid_t fapl)
 /* Create complex group hiearchy, remove it & verify file size */
     TESTING("    complex group hierarchy");
 
-    if (ExpressMode > 1 && !HDstrcmp(envval, "direct")) {
-        /* This test case with Direct driver has a poor performance on
-         * NCSA copper, though it works.  Skip it for now and worry
-         * about the performance later.
-         */
-        SKIPPED();
-    } else {
+    /* Create file */
+    if((file = H5Fcreate(filename, H5F_ACC_TRUNC, H5P_DEFAULT, fapl)) < 0) FAIL_STACK_ERROR
 
-        /* Create file */
-        if((file = H5Fcreate(filename, H5F_ACC_TRUNC, H5P_DEFAULT, fapl)) < 0) FAIL_STACK_ERROR
+    /* Create a complex group hierarchy to remove */
+    for(u = 0; u < FILESPACE_TOP_GROUPS; u++) {
+        /* Create group */
+        sprintf(objname,"%s %u",GROUPNAME,u);
+        if((group = H5Gcreate2(file, objname, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT)) < 0) FAIL_STACK_ERROR
 
-        /* Create a complex group hierarchy to remove */
-        for(u = 0; u < FILESPACE_TOP_GROUPS; u++) {
+        /* Create nested groups inside top groups */
+        for(v = 0; v < FILESPACE_NESTED_GROUPS; v++) {
             /* Create group */
-            sprintf(objname,"%s %u",GROUPNAME,u);
-            if((group = H5Gcreate2(file, objname, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT)) < 0) FAIL_STACK_ERROR
+            sprintf(objname, "%s %u", GROUP2NAME, v);
+            if((group2 = H5Gcreate2(group, objname, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT)) < 0) FAIL_STACK_ERROR
 
-            /* Create nested groups inside top groups */
-            for(v = 0; v < FILESPACE_NESTED_GROUPS; v++) {
-                /* Create group */
-                sprintf(objname, "%s %u", GROUP2NAME, v);
-                if((group2 = H5Gcreate2(group, objname, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT)) < 0) FAIL_STACK_ERROR
-
-                /* Create datasets inside nested groups */
-                for(w = 0; w < FILESPACE_NDATASETS; w++) {
-                    /* Create & close a dataset */
-                    sprintf(objname, "%s %u", DATASETNAME, w);
-                    if((dataset = H5Dcreate2(group2, objname, H5T_NATIVE_INT, space, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT)) < 0) FAIL_STACK_ERROR
-                    if(H5Dclose(dataset) < 0) FAIL_STACK_ERROR
-                } /* end for */
-
-                /* Close nested group */
-                if(H5Gclose(group2) < 0) FAIL_STACK_ERROR
+            /* Create datasets inside nested groups */
+            for(w = 0; w < FILESPACE_NDATASETS; w++) {
+                /* Create & close a dataset */
+                sprintf(objname, "%s %u", DATASETNAME, w);
+                if((dataset = H5Dcreate2(group2, objname, H5T_NATIVE_INT, space, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT)) < 0) FAIL_STACK_ERROR
+                if(H5Dclose(dataset) < 0) FAIL_STACK_ERROR
             } /* end for */
 
-            /* Close top group */
-            if(H5Gclose(group) < 0) FAIL_STACK_ERROR
+            /* Close nested group */
+            if(H5Gclose(group2) < 0) FAIL_STACK_ERROR
         } /* end for */
 
-        /* Remove complex group hierarchy */
-        /* (Remove them in reverse order just to make file size calculation easier -QAK) */
-        for(u = FILESPACE_TOP_GROUPS; u > 0; u--) {
-            /* Open group */
-            sprintf(objname, "%s %u", GROUPNAME, (u - 1));
-            if((group = H5Gopen2(file, objname, H5P_DEFAULT)) < 0) FAIL_STACK_ERROR
+        /* Close top group */
+        if(H5Gclose(group) < 0) FAIL_STACK_ERROR
+    } /* end for */
 
-            /* Open nested groups inside top groups */
-            for(v = 0; v < FILESPACE_NESTED_GROUPS; v++) {
-                /* Create group */
-                sprintf(objname, "%s %u", GROUP2NAME, v);
-                if((group2 = H5Gopen2(group, objname, H5P_DEFAULT)) < 0) FAIL_STACK_ERROR
+    /* Remove complex group hierarchy */
+    /* (Remove them in reverse order just to make file size calculation easier -QAK) */
+    for(u = FILESPACE_TOP_GROUPS; u > 0; u--) {
+        /* Open group */
+        sprintf(objname, "%s %u", GROUPNAME, (u - 1));
+        if((group = H5Gopen2(file, objname, H5P_DEFAULT)) < 0) FAIL_STACK_ERROR
 
-                /* Remove datasets inside nested groups */
-                for(w = 0; w < FILESPACE_NDATASETS; w++) {
-                    /* Remove dataset */
-                    sprintf(objname, "%s %u", DATASETNAME, w);
-                    if(H5Ldelete(group2, objname, H5P_DEFAULT) < 0) FAIL_STACK_ERROR
-                } /* end for */
+        /* Open nested groups inside top groups */
+        for(v = 0; v < FILESPACE_NESTED_GROUPS; v++) {
+            /* Create group */
+            sprintf(objname, "%s %u", GROUP2NAME, v);
+            if((group2 = H5Gopen2(group, objname, H5P_DEFAULT)) < 0) FAIL_STACK_ERROR
 
-                /* Close nested group */
-                if(H5Gclose(group2) < 0) FAIL_STACK_ERROR
-
-                /* Remove nested group */
-                sprintf(objname, "%s %u",GROUP2NAME, v);
-                if(H5Ldelete(group, objname, H5P_DEFAULT) < 0) FAIL_STACK_ERROR
+            /* Remove datasets inside nested groups */
+            for(w = 0; w < FILESPACE_NDATASETS; w++) {
+                /* Remove dataset */
+                sprintf(objname, "%s %u", DATASETNAME, w);
+                if(H5Ldelete(group2, objname, H5P_DEFAULT) < 0) FAIL_STACK_ERROR
             } /* end for */
 
-            /* Close top group */
-            if(H5Gclose(group) < 0) FAIL_STACK_ERROR
+            /* Close nested group */
+            if(H5Gclose(group2) < 0) FAIL_STACK_ERROR
 
-            /* Remove top group */
-            sprintf(objname, "%s %u", GROUPNAME, (u - 1));
-            if(H5Ldelete(file, objname, H5P_DEFAULT) < 0) FAIL_STACK_ERROR
+            /* Remove nested group */
+            sprintf(objname, "%s %u",GROUP2NAME, v);
+            if(H5Ldelete(group, objname, H5P_DEFAULT) < 0) FAIL_STACK_ERROR
         } /* end for */
 
-        /* Close file */
-        if(H5Fclose(file) < 0) FAIL_STACK_ERROR
+        /* Close top group */
+        if(H5Gclose(group) < 0) FAIL_STACK_ERROR
 
-        /* Get the size of the file */
-        if((file_size = h5_get_file_size(filename)) < 0) TEST_ERROR
+        /* Remove top group */
+        sprintf(objname, "%s %u", GROUPNAME, (u - 1));
+        if(H5Ldelete(file, objname, H5P_DEFAULT) < 0) FAIL_STACK_ERROR
+    } /* end for */
 
-        /* Verify the file is correct size */
-        if(file_size != empty_size) TEST_ERROR
+    /* Close file */
+    if(H5Fclose(file) < 0) FAIL_STACK_ERROR
 
-        PASSED();
-    }
+    /* Get the size of the file */
+    if((file_size = h5_get_file_size(filename, fapl)) < 0) TEST_ERROR
+
+    /* Verify the file is correct size */
+    if(file_size != empty_size) TEST_ERROR
+
+    PASSED();
 
 
 /* Create dataset and duplicate dataset, remove original & verify file size */
@@ -1098,7 +1077,7 @@ test_filespace(hid_t fapl)
     if(H5Fclose(file) < 0) FAIL_STACK_ERROR
 
     /* Get the size of the file */
-    if((file_size = h5_get_file_size(filename))<0) TEST_ERROR
+    if((file_size = h5_get_file_size(filename, fapl)) < 0) TEST_ERROR
 
     /* Verify the file is correct size */
     if(file_size != empty_size) TEST_ERROR
@@ -1132,7 +1111,7 @@ test_filespace(hid_t fapl)
     if(H5Fclose(file) < 0) FAIL_STACK_ERROR
 
     /* Get the size of the file */
-    if((file_size = h5_get_file_size(filename)) < 0) TEST_ERROR
+    if((file_size = h5_get_file_size(filename, fapl)) < 0) TEST_ERROR
 
     /* Verify the file is correct size */
     if(file_size != empty_size) TEST_ERROR
@@ -1170,7 +1149,7 @@ test_filespace(hid_t fapl)
     if(H5Fclose(file) < 0) FAIL_STACK_ERROR
 
     /* Get the size of the file */
-    if((file_size = h5_get_file_size(filename)) < 0) TEST_ERROR
+    if((file_size = h5_get_file_size(filename, fapl)) < 0) TEST_ERROR
 
     /* Verify the file is correct size */
     if(file_size != empty_size) TEST_ERROR
@@ -1220,7 +1199,7 @@ test_filespace(hid_t fapl)
     if(H5Fclose(file) < 0) FAIL_STACK_ERROR
 
     /* Get the size of the file */
-    if((file_size = h5_get_file_size(filename)) < 0) TEST_ERROR
+    if((file_size = h5_get_file_size(filename, fapl)) < 0) TEST_ERROR
 
     /* Verify the file is correct size */
     if(file_size != empty_size) TEST_ERROR
@@ -1508,8 +1487,8 @@ delete_node(hid_t pid, hid_t id)
 static int
 test_unlink_rightleaf(hid_t fid)
 {
-    hid_t rootid,       /* Group ID for root group */
-        *gids;          /* Array of IDs for groups created */
+    hid_t rootid = -1,  /* Group ID for root group */
+        *gids = NULL;   /* Array of IDs for groups created */
     int n,              /* Local index variable */
         ngroups = 150;  /* Number of groups to create */
     char name[256];     /* Name of object to create */
@@ -1517,7 +1496,7 @@ test_unlink_rightleaf(hid_t fid)
     TESTING("deleting right-most child in non-leaf B-tree node");
 
     /* Allocate space for the group IDs */
-    if(NULL == (gids = (hid_t *)HDmalloc(ngroups * sizeof(hid_t)))) TEST_ERROR
+    if(NULL == (gids = (hid_t *)HDcalloc((size_t)ngroups, sizeof(hid_t)))) TEST_ERROR
 
     if((rootid = H5Gopen2(fid, "/", H5P_DEFAULT)) < 0) FAIL_STACK_ERROR
 
@@ -1535,8 +1514,10 @@ test_unlink_rightleaf(hid_t fid)
     } /* end for */
 
     /* Close all the groups */
-    for (n = 0; n < ngroups; n++)
+    for (n = 0; n < ngroups; n++) {
         if(H5Gclose(gids[n]) < 0) TEST_ERROR
+        gids[n] = 0;
+    } /* end for */
 
     /* Close root group ID */
     if(H5Gclose(rootid) < 0) TEST_ERROR
@@ -1548,6 +1529,20 @@ test_unlink_rightleaf(hid_t fid)
     return 0;
 
 error:
+    if(gids) {
+        /* Close any open groups */
+        for (n = 0; n < ngroups; n++)
+            if(gids[n]) {
+                H5E_BEGIN_TRY {
+                    H5Gclose(gids[n]);
+                } H5E_END_TRY;
+            } /* end if */
+        HDfree(gids);
+    } /* end if */
+    H5E_BEGIN_TRY {
+        H5Gclose(rootid);
+    } H5E_END_TRY;
+
     return 1;
 } /* end test_unlink_rightleaf() */
 
@@ -1571,8 +1566,8 @@ error:
 static int
 test_unlink_rightnode(hid_t fid)
 {
-    hid_t rootid,       /* Group ID for root group */
-        *gids;          /* Array of IDs for groups created */
+    hid_t rootid = -1,       /* Group ID for root group */
+        *gids = NULL;   /* Array of IDs for groups created */
     int n,              /* Local index variable */
         ngroups = 150;  /* Number of groups to create */
     char name[256];     /* Name of object to create */
@@ -1580,7 +1575,7 @@ test_unlink_rightnode(hid_t fid)
     TESTING("deleting right-most child in non-leaf B-tree node");
 
     /* Allocate space for the group IDs */
-    if(NULL == (gids = (hid_t *)HDmalloc(ngroups * sizeof(hid_t)))) TEST_ERROR
+    if(NULL == (gids = (hid_t *)HDcalloc((size_t)ngroups, sizeof(hid_t)))) TEST_ERROR
 
     if((rootid = H5Gopen2(fid, "/", H5P_DEFAULT)) < 0) FAIL_STACK_ERROR
 
@@ -1591,8 +1586,10 @@ test_unlink_rightnode(hid_t fid)
     } /* end for */
 
     /* Close all the groups */
-    for (n = 0; n < ngroups; n++)
+    for (n = 0; n < ngroups; n++) {
         if(H5Gclose(gids[n]) < 0) FAIL_STACK_ERROR
+        gids[n] = 0;
+    } /* end for */
 
     /* Unlink specific objects to trigger deletion of right leaf in non-leaf node */
     if(H5Ldelete(fid, "/ZoneB77", H5P_DEFAULT) < 0) FAIL_STACK_ERROR
@@ -1611,6 +1608,20 @@ test_unlink_rightnode(hid_t fid)
     return 0;
 
 error:
+    if(gids) {
+        /* Close any open groups */
+        for (n = 0; n < ngroups; n++)
+            if(gids[n]) {
+                H5E_BEGIN_TRY {
+                    H5Gclose(gids[n]);
+                } H5E_END_TRY;
+            } /* end if */
+        HDfree(gids);
+    } /* end if */
+    H5E_BEGIN_TRY {
+        H5Gclose(rootid);
+    } H5E_END_TRY;
+
     return 1;
 } /* end test_unlink_rightnode() */
 
@@ -1634,8 +1645,8 @@ error:
 static int
 test_unlink_middlenode(hid_t fid)
 {
-    hid_t rootid,       /* Group ID for root group */
-        *gids;          /* Array of IDs for groups created */
+    hid_t rootid = -1,  /* Group ID for root group */
+        *gids = NULL;   /* Array of IDs for groups created */
     int n,              /* Local index variable */
         ngroups = 250;  /* Number of groups to create */
     char name[256];     /* Name of object to create */
@@ -1643,7 +1654,7 @@ test_unlink_middlenode(hid_t fid)
     TESTING("deleting right-most child in non-leaf B-tree node");
 
     /* Allocate space for the group IDs */
-    if(NULL == (gids = (hid_t *)HDmalloc(ngroups * sizeof(hid_t)))) TEST_ERROR
+    if(NULL == (gids = (hid_t *)HDcalloc((size_t)ngroups, sizeof(hid_t)))) TEST_ERROR
 
     if((rootid = H5Gopen2(fid, "/", H5P_DEFAULT)) < 0) FAIL_STACK_ERROR
 
@@ -1654,8 +1665,10 @@ test_unlink_middlenode(hid_t fid)
     } /* end for */
 
     /* Close all the groups */
-    for (n = 0; n < ngroups; n++)
+    for (n = 0; n < ngroups; n++) {
         if(H5Gclose(gids[n]) < 0) FAIL_STACK_ERROR
+        gids[n] = 0;
+    } /* end for */
 
     /* Unlink specific objects to trigger deletion of all leafs in "interior" non-leaf node */
     if(H5Ldelete(fid, "/ZoneC11", H5P_DEFAULT) < 0) FAIL_STACK_ERROR
@@ -1817,6 +1830,20 @@ test_unlink_middlenode(hid_t fid)
     return 0;
 
 error:
+    if(gids) {
+        /* Close any open groups */
+        for (n = 0; n < ngroups; n++)
+            if(gids[n]) {
+                H5E_BEGIN_TRY {
+                    H5Gclose(gids[n]);
+                } H5E_END_TRY;
+            } /* end if */
+        HDfree(gids);
+    } /* end if */
+    H5E_BEGIN_TRY {
+        H5Gclose(rootid);
+    } H5E_END_TRY;
+
     return 1;
 } /* end test_unlink_middlenode() */
 
@@ -1840,7 +1867,7 @@ error:
 static int
 test_resurrect_dataset(hid_t fapl)
 {
-    hid_t       f =-1, s =-1, d =-1;
+    hid_t       f = -1, s = -1, d = -1;
     char	filename[1024];
 
     TESTING("resurrecting dataset after deletion");
@@ -2165,7 +2192,7 @@ test_full_group_compact(hid_t fapl)
     if(H5Fclose(file_id) < 0) FAIL_STACK_ERROR
 
     /* Get the size of the file with only the objects to keep */
-    if((keep_size = h5_get_file_size(filename)) < 0) TEST_ERROR
+    if((keep_size = h5_get_file_size(filename, fapl)) < 0) TEST_ERROR
 
     /* Re-open the file */
     if((file_id = H5Fopen(filename, H5F_ACC_RDWR, fapl)) < 0) FAIL_STACK_ERROR
@@ -2229,7 +2256,7 @@ test_full_group_compact(hid_t fapl)
     if(H5Fclose(file_id) < 0) FAIL_STACK_ERROR
 
     /* Get the size of the file */
-    if((file_size = h5_get_file_size(filename)) < 0) TEST_ERROR
+    if((file_size = h5_get_file_size(filename, fapl)) < 0) TEST_ERROR
 
     /* Verify the file is correct size */
     if(file_size != keep_size) TEST_ERROR
@@ -2299,7 +2326,7 @@ test_full_group_dense(hid_t fapl)
     if(H5Fclose(file_id) < 0) FAIL_STACK_ERROR
 
     /* Get the size of the file with only the objects to keep */
-    if((keep_size = h5_get_file_size(filename)) < 0) TEST_ERROR
+    if((keep_size = h5_get_file_size(filename, fapl)) < 0) TEST_ERROR
 
     /* Re-open the file */
     if((file_id = H5Fopen(filename, H5F_ACC_RDWR, fapl)) < 0) FAIL_STACK_ERROR
@@ -2375,7 +2402,7 @@ test_full_group_dense(hid_t fapl)
     if(H5Fclose(file_id) < 0) FAIL_STACK_ERROR
 
     /* Get the size of the file */
-    if((file_size = h5_get_file_size(filename)) < 0) TEST_ERROR
+    if((file_size = h5_get_file_size(filename, fapl)) < 0) TEST_ERROR
 
     /* Verify the file is correct size */
     if(file_size != keep_size) TEST_ERROR
@@ -2413,132 +2440,126 @@ error:
 int
 main(void)
 {
-    const char *envval = NULL;
+    hid_t	fapl, fapl2, file;
+    int	nerrors = 0;
+    char	filename[1024];
+    hbool_t new_format;
 
-    /* Don't run this test using the wrong file drivers */
-    envval = HDgetenv("HDF5_DRIVER");
-    if(envval == NULL)
-        envval = "nomatch";
-    if(HDstrcmp(envval, "core") && HDstrcmp(envval, "split") && HDstrcmp(envval, "multi") && HDstrcmp(envval, "family")) {
-        hid_t	fapl, fapl2, file;
-        int	nerrors = 0;
-        char	filename[1024];
-        hbool_t new_format;
+    /* Metadata cache parameters */
+    int mdc_nelmts;
+    size_t rdcc_nelmts;
+    size_t rdcc_nbytes;
+    double rdcc_w0;
 
-	/* Metadata cache parameters */
-	int mdc_nelmts;
-	size_t rdcc_nelmts;
-	size_t rdcc_nbytes;
-	double rdcc_w0;
+    /* Set the random # seed */
+    HDsrandom((unsigned)HDtime(NULL));
 
-	/* Set the random # seed */
-	HDsrandom((unsigned long)HDtime(NULL));
+    /* Open */
+    h5_reset();
+    fapl = h5_fileaccess();
 
-	/* Open */
-	h5_reset();
-	fapl = h5_fileaccess();
+    /* Copy the file access property list */
+    if((fapl2 = H5Pcopy(fapl)) < 0) TEST_ERROR
 
-        /* Copy the file access property list */
-        if((fapl2 = H5Pcopy(fapl)) < 0) TEST_ERROR
+    /* Set the "use the latest version of the format" bounds for creating objects in the file */
+    if(H5Pset_libver_bounds(fapl2, H5F_LIBVER_LATEST, H5F_LIBVER_LATEST) < 0) TEST_ERROR
 
-        /* Set the "use the latest version of the format" bounds for creating objects in the file */
-        if(H5Pset_libver_bounds(fapl2, H5F_LIBVER_LATEST, H5F_LIBVER_LATEST) < 0) TEST_ERROR
+    /* Test with old & new format groups */
+    for(new_format = FALSE; new_format <= TRUE; new_format++) {
+        hid_t my_fapl;
 
-        /* Test with old & new format groups */
-        for(new_format = FALSE; new_format <= TRUE; new_format++) {
-            hid_t my_fapl;
+        /* Set the FAPL for the type of format */
+        if(new_format) {
+            puts("\nTesting with new group format:");
+            my_fapl = fapl2;
+        } /* end if */
+        else {
+            puts("Testing with old group format:");
+            my_fapl = fapl;
+        } /* end else */
 
-            /* Set the FAPL for the type of format */
-            if(new_format) {
-                puts("\nTesting with new group format:");
-                my_fapl = fapl2;
-            } /* end if */
-            else {
-                puts("Testing with old group format:");
-                my_fapl = fapl;
-            } /* end else */
+        h5_fixname(FILENAME[0], my_fapl, filename, sizeof filename);
+        if((file = H5Fcreate(filename, H5F_ACC_TRUNC, H5P_DEFAULT, my_fapl)) < 0) TEST_ERROR
 
-            h5_fixname(FILENAME[0], my_fapl, filename, sizeof filename);
-            if((file = H5Fcreate(filename, H5F_ACC_TRUNC, H5P_DEFAULT, my_fapl)) < 0) TEST_ERROR
+        /* Tests */
+        nerrors += test_one(file);
+        nerrors += test_many(file);
+        nerrors += test_symlink(file);
+        nerrors += test_rename(file);
 
-            /* Tests */
-            nerrors += test_one(file);
-            nerrors += test_many(file);
-            nerrors += test_symlink(file);
-            nerrors += test_rename(file);
+        nerrors += test_new_move(my_fapl);
+        nerrors += check_new_move(my_fapl);
+        nerrors += test_filespace(my_fapl);
 
-            nerrors += test_new_move(my_fapl);
-            nerrors += check_new_move(my_fapl);
-            nerrors += test_filespace(my_fapl);
+        /* Test creating & unlinking lots of objects with default FAPL */
+        nerrors += test_create_unlink("create and unlink large number of objects", my_fapl);
 
-            /* Test creating & unlinking lots of objects with default FAPL */
-            nerrors += test_create_unlink("create and unlink large number of objects", my_fapl);
+        {
+            hid_t fapl_small_mdc;
 
-            {
-                hid_t fapl_small_mdc;
+            /* Make copy of regular fapl, to turn down the elements in the metadata cache */
+            if((fapl_small_mdc = H5Pcopy(my_fapl)) < 0)
+                goto error;
 
-                /* Make copy of regular fapl, to turn down the elements in the metadata cache */
-                if((fapl_small_mdc = H5Pcopy(my_fapl)) < 0)
-                    goto error;
+            /* Get FAPL cache settings */
+            if(H5Pget_cache(fapl_small_mdc, &mdc_nelmts, &rdcc_nelmts, &rdcc_nbytes, &rdcc_w0) < 0)
+                printf("H5Pget_cache failed\n");
 
-                /* Get FAPL cache settings */
-                if(H5Pget_cache(fapl_small_mdc, &mdc_nelmts, &rdcc_nelmts, &rdcc_nbytes, &rdcc_w0) < 0)
-                    printf("H5Pget_cache failed\n");
+            /* Change FAPL cache settings */
+            mdc_nelmts=1;
+            if(H5Pset_cache(fapl_small_mdc, mdc_nelmts, rdcc_nelmts, rdcc_nbytes, rdcc_w0) < 0)
+                printf("H5Pset_cache failed\n");
 
-                /* Change FAPL cache settings */
-                mdc_nelmts=1;
-                if(H5Pset_cache(fapl_small_mdc, mdc_nelmts, rdcc_nelmts, rdcc_nbytes, rdcc_w0) < 0)
-                    printf("H5Pset_cache failed\n");
+            /* Test creating & unlinking lots of objects with a 1-element metadata cache FAPL */
+            nerrors += test_create_unlink("create and unlink large number of objects with small cache", fapl_small_mdc);
 
-                /* Test creating & unlinking lots of objects with a 1-element metadata cache FAPL */
-                nerrors += test_create_unlink("create and unlink large number of objects with small cache", fapl_small_mdc);
+            if(H5Pclose(fapl_small_mdc) < 0) TEST_ERROR
+        } /* end block */
 
-                if(H5Pclose(fapl_small_mdc) < 0) TEST_ERROR
-            } /* end block */
+        nerrors += test_link_slashes(my_fapl);
+        nerrors += test_unlink_slashes(my_fapl);
 
-            nerrors += test_link_slashes(my_fapl);
-            nerrors += test_unlink_slashes(my_fapl);
+        /* Test specific B-tree removal issues */
+        /* (only for old format groups) */
+        if(!new_format) {
+            nerrors += test_unlink_rightleaf(file);
+            nerrors += test_unlink_rightnode(file);
+            nerrors += test_unlink_middlenode(file);
+        } /* end if */
 
-            /* Test specific B-tree removal issues */
-            /* (only for old format groups) */
-            if(!new_format) {
-                nerrors += test_unlink_rightleaf(file);
-                nerrors += test_unlink_rightnode(file);
-                nerrors += test_unlink_middlenode(file);
-            } /* end if */
+        /* Test "resurrecting" objects */
+        nerrors += test_resurrect_dataset(my_fapl);
+        nerrors += test_resurrect_datatype(my_fapl);
+        nerrors += test_resurrect_group(my_fapl);
 
-            /* Test "resurrecting" objects */
-            nerrors += test_resurrect_dataset(my_fapl);
-            nerrors += test_resurrect_datatype(my_fapl);
-            nerrors += test_resurrect_group(my_fapl);
+        /* Test unlinking chunked datasets */
+        nerrors += test_unlink_chunked_dataset(my_fapl);
 
-            /* Test unlinking chunked datasets */
-            nerrors += test_unlink_chunked_dataset(my_fapl);
+        /* Test unlinked groups which still have objects in them */
+        /* (only for new format groups) */
+        if(new_format) {
+            nerrors += test_full_group_compact(my_fapl);
+            nerrors += test_full_group_dense(my_fapl);
+        } /* end if */
 
-            /* Test unlinked groups which still have objects in them */
-            /* (only for new format groups) */
-            if(new_format) {
-                nerrors += test_full_group_compact(my_fapl);
-                nerrors += test_full_group_dense(my_fapl);
-            } /* end if */
+        /* Close */
+        if(H5Fclose(file) < 0) TEST_ERROR
+    } /* end for */
 
-            /* Close */
-            if(H5Fclose(file) < 0) TEST_ERROR
-        } /* end for */
+    /* Close 2nd FAPL */
+    H5Pclose(fapl2);
 
-        /* Close 2nd FAPL */
-	H5Pclose(fapl2);
+    /* Verify symbol table messages are cached */
+    nerrors += (h5_verify_cached_stabs(FILENAME, fapl) < 0 ? 1 : 0);
 
-        if (nerrors) {
-            printf("***** %d FAILURE%s! *****\n", nerrors, 1==nerrors?"":"S");
-	    exit(1);
-	}
-
-	puts("All unlink tests passed.");
-	h5_cleanup(FILENAME, fapl);
+    if (nerrors) {
+        printf("***** %d FAILURE%s! *****\n", nerrors, 1==nerrors?"":"S");
+        exit(1);
     }
-    else
-        puts("All unlink tests skipped - Incompatible with current Virtual File Driver");
+
+    puts("All unlink tests passed.");
+
+    h5_cleanup(FILENAME, fapl);
 
     return 0;
 
