@@ -36,6 +36,8 @@ enum H5TEST_COLL_CHUNK_API {API_NONE=0,API_LINK_HARD,
 /* Constants definitions */
 #define DIM0		600 	/* Default dataset sizes. */
 #define DIM1		1200	/* Values are from a monitor pixel sizes */
+#define ROW_FACTOR	8 	/* Nominal row factor for dataset size */
+#define COL_FACTOR	16	/* Nominal column factor for dataset size */
 #define RANK		2
 #define DATASETNAME1	"Data1"
 #define DATASETNAME2	"Data2"
@@ -158,6 +160,19 @@ enum H5TEST_COLL_CHUNK_API {API_NONE=0,API_LINK_HARD,
 #define NPOINTS          4          /* Number of points that will be selected
                                                                 and overwritten */
 
+/* Definitions of the selection mode for the test_actual_io_function. */
+#define TEST_ACTUAL_IO_NO_COLLECTIVE                    0
+#define TEST_ACTUAL_IO_RESET                            1
+#define TEST_ACTUAL_IO_MULTI_CHUNK_IND                  2
+#define TEST_ACTUAL_IO_MULTI_CHUNK_COL                  3
+#define TEST_ACTUAL_IO_MULTI_CHUNK_MIX                  4
+#define TEST_ACTUAL_IO_MULTI_CHUNK_MIX_DISAGREE         5
+#define TEST_ACTUAL_IO_MULTI_CHUNK_NO_OPT_IND           6
+#define TEST_ACTUAL_IO_MULTI_CHUNK_NO_OPT_COL           7
+#define TEST_ACTUAL_IO_MULTI_CHUNK_NO_OPT_MIX_DISAGREE  8 
+#define TEST_ACTUAL_IO_LINK_CHUNK                       9
+#define TEST_ACTUAL_IO_CONTIGUOUS                       10
+
 /* Don't erase these lines, they are put here for debugging purposes */
 /*
 #define MSPACE1_RANK     1
@@ -171,9 +186,9 @@ enum H5TEST_COLL_CHUNK_API {API_NONE=0,API_LINK_HARD,
 #define MSPACE_DIM1      8
 #define MSPACE_DIM2      9
 #define NPOINTS          4
-
-
 */ /* end of debugging macro */
+
+
 /* type definitions */
 typedef struct H5Ptest_param_t  /* holds extra test parameters */
 {
@@ -184,6 +199,14 @@ typedef struct H5Ptest_param_t  /* holds extra test parameters */
 /* Dataset data type.  Int's can be easily octo dumped. */
 typedef int DATATYPE;
 
+/* Shape Same Tests Definitions */
+typedef enum {
+    IND_CONTIG,		/* Independent IO on contigous datasets */
+    COL_CONTIG,		/* Collective IO on contigous datasets */
+    IND_CHUNKED,	/* Independent IO on chunked datasets */
+    COL_CHUNKED		/* Collective IO on chunked datasets */
+} ShapeSameTestMethods;
+
 /* Shared global variables */
 extern int dim0, dim1;				/*Dataset dimensions */
 extern int chunkdim0, chunkdim1;		/*Chunk dimensions */
@@ -212,6 +235,7 @@ void dataset_readAll(void);
 void extend_readInd(void);
 void extend_readAll(void);
 void none_selection_chunk(void);
+void actual_io_mode_tests(void);
 void test_chunk_alloc(void);
 void test_filter_read(void);
 void compact_dataset(void);
@@ -235,6 +259,13 @@ void coll_irregular_simple_chunk_write(void);
 void coll_irregular_complex_chunk_read(void);
 void coll_irregular_complex_chunk_write(void);
 void io_mode_confusion(void);
+void rr_obj_hdr_flush_confusion(void);
+void rr_obj_hdr_flush_confusion_reader(MPI_Comm comm);
+void rr_obj_hdr_flush_confusion_writer(MPI_Comm comm);
+void lower_dim_size_comp_test(void);
+void link_chunk_collective_io_test(void);
+void contig_hyperslab_dr_pio_test(ShapeSameTestMethods sstest_type);
+void checker_board_hyperslab_dr_pio_test(ShapeSameTestMethods sstest_type);
 #ifdef H5_HAVE_FILTER_DEFLATE
 void compress_readAll(void);
 #endif /* H5_HAVE_FILTER_DEFLATE */
