@@ -17,19 +17,17 @@
 #ifndef H5REPACK_H__
 #define H5REPACK_H__
 
+#include "H5private.h"
 #include "hdf5.h"
 #include "h5trav.h"
-#include "H5Zprivate.h"	/* H5Z_COMMON_CD_VALUES */	
-
-
 
 #define H5FOPENERROR "unable to open file"
-
-#define PFORMAT  "%-7s %-7s %-7s\n" /*chunk info, compression info, name*/
-#define PFORMAT1 "%-7s %-7s %-7s"     /*chunk info, compression info, name*/
-
-#define MAX_NC_NAME 256 /* max length of a name */
-#define MAX_VAR_DIMS 32 /* max per variable dimensions */
+#define PFORMAT  "%-7s %-7s %-7s\n"   /* chunk info, compression info, name*/
+#define PFORMAT1 "%-7s %-7s %-7s"     /* chunk info, compression info, name*/
+#define MAX_NC_NAME 256               /* max length of a name */
+#define MAX_VAR_DIMS 32               /* max per variable dimensions */
+#define FORMAT_OBJ      " %-27s %s\n"   /* obj type, name */
+#define FORMAT_OBJ_ATTR "  %-27s %s\n"  /* obj type, name */
 
 /*-------------------------------------------------------------------------
  * data structures for command line options
@@ -53,7 +51,6 @@ typedef struct {
  H5Z_FILTER_SCALEOFFSET 6 , scaleoffset compression
 */
 
-/* #define CD_VALUES H5Z_COMMON_CD_VALUES */
 #define CD_VALUES 20
 
 typedef struct {
@@ -105,16 +102,25 @@ typedef struct {
  chunk_info_t    chunk_g;     /*global chunk INFO for the ALL case */
  H5D_layout_t    layout_g;    /*global layout information for the ALL case */
  int             verbose;     /*verbose mode */
- hsize_t         threshold;   /*minimum size to compress, in bytes */
- int             use_native;  /*use a native type in write */  
+ hsize_t         min_comp;    /*minimum size to compress, in bytes */
+ int             use_native;  /*use a native type in write */
  int             latest;      /*pack file with the latest file format */
  int             grp_compact; /* Set the maximum number of links to store as header messages in the group */
  int             grp_indexed; /* Set the minimum number of links to store in the indexed format */
- int             msg_size[8]; /* Minumum size of shared messages: dataspace, 
+ int             msg_size[8]; /* Minimum size of shared messages: dataspace,
                                  datatype, fill value, filter pipleline, attribute */
+ const char      *ublock_filename; /* user block file name */
+ hsize_t         ublock_size;      /* user block size */
+ hsize_t         threshold;        /* alignment threshold for H5Pset_alignment */
+ hsize_t         alignment ;       /* alignment for H5Pset_alignment */
 } pack_opt_t;
 
 
+typedef struct named_dt_t {
+    haddr_t             addr_in;    /* Address of the named dtype in the in file */
+    hid_t               id_out;     /* Open identifier for the dtype in the out file */
+    struct named_dt_t   *next;      /* Next dtype */
+} named_dt_t;
 
 /*-------------------------------------------------------------------------
  * public functions
@@ -131,9 +137,20 @@ int h5repack_addlayout (const char* str, pack_opt_t *options);
 int h5repack_init      (pack_opt_t *options, int verbose);
 int h5repack_end       (pack_opt_t *options);
 int h5repack_verify    (const char *fname,pack_opt_t *options);
-int h5repack_cmpdcpl   (const char *fname1,
+int h5repack_cmp_pl   (const char *fname1,
                         const char *fname2);
 
+/* Note: The below copy_named_datatype(), named_datatype_free(), copy_attr() 
+ * and struct named_dt_t were located in h5repack_copy.c as static prior to 
+ * bugfix1726. 
+ * Made shared functions as copy_attr() was needed in h5repack_refs.c. 
+ * However copy_attr() may be obsoleted when H5Acopy is available and put back
+ * others to static in h5repack_copy.c.
+ */
+hid_t copy_named_datatype(hid_t type_in, hid_t fidout, named_dt_t **named_dt_head_p, trav_table_t *travt, pack_opt_t *options);
+int named_datatype_free(named_dt_t **named_dt_head_p, int ignore_err);
+int copy_attr(hid_t loc_in, hid_t loc_out, named_dt_t **named_dt_head_p,
+              trav_table_t *travt, pack_opt_t *options);
 
 #ifdef __cplusplus
 }
@@ -159,7 +176,7 @@ int copy_objects   (const char* fnamein,
 int do_copy_refobjs(hid_t fidin,
                     hid_t fidout,
                     trav_table_t *travt,
-                    pack_opt_t *options); 
+                    pack_opt_t *options);
 
 /*-------------------------------------------------------------------------
  * filters and verify module
@@ -176,6 +193,7 @@ void init_packobject(pack_info_t *obj);
 int apply_filters(const char* name,    /* object name from traverse list */
                   int rank,            /* rank of dataset */
                   hsize_t *dims,       /* dimensions of dataset */
+                  size_t msize,        /* size of type */
                   hid_t dcpl_id,       /* dataset creation property list */
                   pack_opt_t *options, /* repack options */
                   int *has_filter);    /* (OUT) object NAME has a filter */
