@@ -14,10 +14,13 @@
  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
 #include "h5repack.h"
-#include "h5test.h"
 #include "h5tools_utils.h"
 
-extern char  *progname;
+/* number of members in an array */
+#ifndef NELMTS
+#    define NELMTS(X)		(sizeof(X)/sizeof(X[0]))
+#endif
+
 static int verify_layout(hid_t pid, pack_info_t *obj);
 static int verify_filters(hid_t pid, hid_t tid, int nfilters, filter_info_t *filter);
 
@@ -313,12 +316,12 @@ int h5repack_cmp_pl(const char *fname1,
         /* Open the files */
         if ((fid1=H5Fopen(fname1,H5F_ACC_RDONLY,H5P_DEFAULT)) < 0 )
         {
-            error_msg(progname, "<%s>: %s\n", fname1, H5FOPENERROR );
+            error_msg("<%s>: %s\n", fname1, H5FOPENERROR );
             return -1;
         }
         if ((fid2=H5Fopen(fname2,H5F_ACC_RDONLY,H5P_DEFAULT)) < 0 )
         {
-            error_msg(progname, "<%s>: %s\n", fname2, H5FOPENERROR );
+            error_msg("<%s>: %s\n", fname2, H5FOPENERROR );
             H5Fclose(fid1);
             return -1;
         }
@@ -332,7 +335,7 @@ int h5repack_cmp_pl(const char *fname1,
     trav_table_init(&trav);
     if(h5trav_gettable(fid1, trav) < 0)
         goto error;
-  
+
    /*-------------------------------------------------------------------------
     * traverse the suppplied object list
     *-------------------------------------------------------------------------
@@ -367,7 +370,7 @@ int h5repack_cmp_pl(const char *fname1,
 
             if ( crt_order_flag1 != crt_order_flag2 )
             {
-                error_msg(progname, "property lists for <%s> are different\n",trav->objs[i].name);
+                error_msg("property lists for <%s> are different\n",trav->objs[i].name);
                 goto error;
             }
 
@@ -395,7 +398,7 @@ int h5repack_cmp_pl(const char *fname1,
 
             if(ret == 0)
             {
-                error_msg(progname, "property lists for <%s> are different\n",trav->objs[i].name);
+                error_msg("property lists for <%s> are different\n",trav->objs[i].name);
                 goto error;
             }
 
@@ -472,7 +475,7 @@ error:
  *-------------------------------------------------------------------------
  */
 
-static 
+static
 int verify_filters(hid_t pid, hid_t tid, int nfilters, filter_info_t *filter)
 {
     int           nfilters_dcpl;  /* number of filters in DCPL*/
