@@ -15,9 +15,12 @@
 
 #include <stdio.h>
 #include "h5repack.h"
+#include "h5tools.h"
 #include "h5test.h"
 
 
+/* Name of tool */
+#define PROGRAMNAME "h5repack_detect_szip"
 
 /*-------------------------------------------------------------------------
  * Function: main
@@ -41,6 +44,9 @@
 
 int main(void)
 {
+    h5tools_setprogname(PROGRAMNAME);
+    h5tools_setstatus(EXIT_SUCCESS);
+
 #ifdef H5_HAVE_FILTER_SZIP
     if (h5tools_can_encode(H5Z_FILTER_SZIP) == 1) {
         printf("yes\n");
