@@ -19,27 +19,51 @@
 #include "hdf5.h"
 #include "h5trav.h"
 
+#define MAX_FILENAME 1024
+
+/*-------------------------------------------------------------------------
+ * This is used to pass multiple args into diff().
+ * Passing this instead of several each arg provides smoother extensibility 
+ * through its members along with MPI code for ph5diff
+ * as it doesn't require interface change.
+ *------------------------------------------------------------------------*/
+typedef struct {
+    h5trav_type_t   type;
+    hbool_t is_same_trgobj;
+} diff_args_t;
 /*-------------------------------------------------------------------------
  * command line options
  *-------------------------------------------------------------------------
  */
+/* linked list to keep exclude path list */
+struct exclude_path_list {
+    char  *obj_path;
+    h5trav_type_t obj_type;
+    struct exclude_path_list * next;
+};
 
 typedef struct {
-    int    m_quiet;   /* quiet mode: no output at all */
-    int    m_report;  /* report mode: print the data */
-    int    m_verbose; /* verbose mode: print the data, list of objcets, warnings */
-    int    d;         /* delta, absolute value to compare */
-    double delta;     /* delta value */
-    int    p;         /* relative error to compare*/
-    double percent;   /* relative error value */
-    int    n;         /* count, compare up to count */
-    hsize_t count;    /* count value */
-    int    err_stat;  /* an error ocurred (1, error, 0, no error) */
-    int    cmn_objs;  /* do we have common objects */
-    int    not_cmp;   /* are the objects comparable */
-    int    contents;  /* equal contents */
-    int    do_nans;   /* consider Nans while diffing floats */
-    int    m_list_not_cmp;   /* list not comparable messages */
+    int      m_quiet;               /* quiet mide: no output at all */
+    int      m_report;              /* report mode: print the data */
+    int      m_verbose;             /* verbose mode: print the data, list of objcets, warnings */
+    int      m_verbose_level;       /* control verbose details */
+    int      d;                     /* delta, absolute value to compare */
+    double   delta;                 /* delta value */
+    int      p;                     /* relative error to compare*/
+    int      use_system_epsilon;    /* flag to use system epsilon (1 or 0) */
+    double   percent;               /* relative error value */
+    int      n;                     /* count, compare up to count */
+    hsize_t  count;                 /* count value */
+    int      follow_links;          /* follow symbolic links */
+    int      no_dangle_links;       /* return error when find dangling link */
+    int      err_stat;              /* an error ocurred (1, error, 0, no error) */
+    int      cmn_objs;              /* do we have common objects */
+    int      not_cmp;               /* are the objects comparable */
+    int      contents;              /* equal contents */
+    int      do_nans;               /* consider Nans while diffing floats */
+    int      m_list_not_cmp;        /* list not comparable messages */
+    int      exclude_path;          /* exclude path to an object */
+    struct   exclude_path_list * exclude; /* keep exclude path list */
 } diff_opt_t;
 
 
@@ -52,15 +76,22 @@ typedef struct {
 extern "C" {
 #endif
 
-hsize_t  h5diff(const char *fname1,
+H5TOOLS_DLL hsize_t  h5diff(const char *fname1,
                 const char *fname2,
                 const char *objname1,
                 const char *objname2,
                 diff_opt_t *options);
 
+H5TOOLS_DLL hsize_t diff( hid_t      file1_id,
+              const char *path1,
+              hid_t      file2_id,
+              const char *path2,
+              diff_opt_t *options,
+              diff_args_t *argdata);
+
 #ifdef H5_HAVE_PARALLEL
-void phdiff_dismiss_workers(void);
-void print_manager_output(void);
+H5TOOLS_DLL void phdiff_dismiss_workers(void);
+H5TOOLS_DLL void print_manager_output(void);
 #endif
 
 #ifdef __cplusplus
@@ -87,13 +118,6 @@ hsize_t diff_datasetid( hid_t dset1_id,
                         const char *obj2_name,
                         diff_opt_t *options);
 
-hsize_t diff( hid_t      file1_id,
-              const char *path1,
-              hid_t      file2_id,
-              const char *path2,
-              diff_opt_t *options,
-              h5trav_type_t  type );
-
 hsize_t diff_compare( hid_t file1_id,
                       const char *file1_name,
                       const char *obj1_name,
@@ -104,11 +128,9 @@ hsize_t diff_compare( hid_t file1_id,
                       trav_info_t *info2,
                       diff_opt_t *options );
 
-hsize_t diff_match( hid_t file1_id,
-                    trav_info_t *info1,
-                    hid_t file2_id,
-                    trav_info_t *info2,
-                    diff_opt_t *options );
+hsize_t diff_match( hid_t file1_id, const char *grp1, trav_info_t *info1,
+                    hid_t file2_id, const char *grp2, trav_info_t *info2,
+                    trav_table_t *table, diff_opt_t *options );
 
 hsize_t diff_array( void *_mem1,
                     void *_mem2,
@@ -151,7 +173,6 @@ hsize_t diff_attr(hid_t loc1_id,
  */
 
 void        print_found(hsize_t nfound);
-void        parallel_print(const char* format, ... );
 void        print_type(hid_t type);
 const char* diff_basename(const char *name);
 const char* get_type(h5trav_type_t type);
@@ -159,9 +180,22 @@ const char* get_class(H5T_class_t tclass);
 const char* get_sign(H5T_sign_t sign);
 void        print_dimensions (int rank, hsize_t *dims);
 int         print_objname(diff_opt_t *options, hsize_t nfound);
-void        do_print_objname (const char *OBJ, const char *path1, const char *path2);
+void        do_print_objname (const char *OBJ, const char *path1, const char *path2, diff_opt_t * opts);
+void        do_print_attrname (const char *attr, const char *path1, const char *path2);
 
 
+/*-------------------------------------------------------------------------
+ * XCAO, 11/10/2010
+ * added to improve performance for compound datasets
+ */
+typedef struct mcomp_t
+{
+    int             n;      /* number of members */
+    hid_t           *ids;   /* member type id */
+    unsigned char   *flags; 
+    size_t          *offsets;   
+    struct mcomp_t  **m;     /* members */
+}mcomp_t;
 
 hsize_t diff_datum(void       *_mem1,
                    void       *_mem2,
@@ -176,7 +210,8 @@ hsize_t diff_datum(void       *_mem1,
                    const char *obj2,
                    hid_t      container1_id,
                    hid_t      container2_id, /*where the reference came from*/
-                   int        *ph);           /*print header */
+                   int        *ph,           /*print header */
+                   mcomp_t    *members);      /*compound members */
 
 
 
