@@ -39,6 +39,14 @@ int   nCols = 80;
 int         opt_err = 1;    /*get_option prints errors if this is on */
 int         opt_ind = 1;    /*token pointer                          */
 const char *opt_arg;        /*flag argument (or value)               */
+static int  h5tools_d_status = 0;
+static const char  *h5tools_progname = "h5tools";
+
+/* ``parallel_print'' variables */
+unsigned char  g_Parallel = 0;  /*0 for serial, 1 for parallel */
+char     outBuff[OUTBUFF_SIZE];
+int      outBuffOffset;
+FILE*    overflow_file = NULL;
 
 /* local functions */
 static void init_table(table_t **tbl);
@@ -47,6 +55,72 @@ static void dump_table(char* tablename, table_t *table);
 #endif  /* H5DUMP_DEBUG */
 static void add_obj(table_t *table, haddr_t objno, const char *objname, hbool_t recorded);
 
+/*-------------------------------------------------------------------------
+ * Function: parallel_print
+ *
+ * Purpose: wrapper for printf for use in parallel mode.
+ *
+ * Programmer: Leon Arber
+ *
+ * Date: December 1, 2004
+ *
+ *-------------------------------------------------------------------------
+ */
+void parallel_print(const char* format, ...)
+{
+ int  bytes_written;
+ va_list ap;
+
+ va_start(ap, format);
+
+ if(!g_Parallel)
+  vprintf(format, ap);
+ else
+ {
+
+  if(overflow_file == NULL) /*no overflow has occurred yet */
+  {
+#if 0
+   printf("calling HDvsnprintf: OUTBUFF_SIZE=%ld, outBuffOffset=%ld, ", (long)OUTBUFF_SIZE, (long)outBuffOffset);
+#endif
+   bytes_written = HDvsnprintf(outBuff+outBuffOffset, OUTBUFF_SIZE-outBuffOffset, format, ap);
+#if 0
+   printf("bytes_written=%ld\n", (long)bytes_written);
+#endif
+   va_end(ap);
+   va_start(ap, format);
+
+#if 0
+   printf("Result: bytes_written=%ld, OUTBUFF_SIZE-outBuffOffset=%ld\n", (long)bytes_written, (long)OUTBUFF_SIZE-outBuffOffset);
+#endif
+
+   if ((bytes_written < 0) ||
+#ifdef H5_VSNPRINTF_WORKS
+    (bytes_written >= (OUTBUFF_SIZE-outBuffOffset))
+#else
+    ((bytes_written+1) == (OUTBUFF_SIZE-outBuffOffset))
+#endif
+    )
+   {
+    /* Terminate the outbuff at the end of the previous output */
+    outBuff[outBuffOffset] = '\0';
+
+    overflow_file = HDtmpfile();
+    if(overflow_file == NULL)
+     fprintf(stderr, "warning: could not create overflow file.  Output may be truncated.\n");
+    else
+     bytes_written = HDvfprintf(overflow_file, format, ap);
+   }
+   else
+    outBuffOffset += bytes_written;
+  }
+  else
+   bytes_written = HDvfprintf(overflow_file, format, ap);
+
+ }
+ va_end(ap);
+}
+
 
 /*-------------------------------------------------------------------------
  * Function:    error_msg
@@ -64,13 +138,13 @@ static void add_obj(table_t *table, haddr_t objno, const char *objname, hbool_t
  *-------------------------------------------------------------------------
  */
 void
-error_msg(const char *progname, const char *fmt, ...)
+error_msg(const char *fmt, ...)
 {
     va_list ap;
 
     va_start(ap, fmt);
     HDfflush(stdout);
-    HDfprintf(stderr, "%s error: ", progname);
+    HDfprintf(stderr, "%s error: ", h5tools_getprogname());
     HDvfprintf(stderr, fmt, ap);
 
     va_end(ap);
@@ -93,22 +167,35 @@ error_msg(const char *progname, const char *fmt, ...)
  *-------------------------------------------------------------------------
  */
 void
-warn_msg(const char *progname, const char *fmt, ...)
+warn_msg(const char *fmt, ...)
 {
     va_list ap;
 
     va_start(ap, fmt);
     HDfflush(stdout);
-#ifdef _WIN32
-    HDfprintf(stdout, "%s warning: ", progname);
-    HDvfprintf(stdout, fmt, ap);
-#else /* _WIN32 */
-    HDfprintf(stderr, "%s warning: ", progname);
+    HDfprintf(stderr, "%s warning: ", h5tools_getprogname());
     HDvfprintf(stderr, fmt, ap);
-#endif /* _WIN32 */
     va_end(ap);
 }
 
+/*-------------------------------------------------------------------------
+ * Function:  help_ref_msg
+ *
+ * Purpose: Print a message to refer help page 
+ *
+ * Return:  Nothing
+ *
+ * Modifications:
+ *
+ *-------------------------------------------------------------------------
+ */
+void
+help_ref_msg(FILE *output)
+{
+    HDfprintf(output, "Try '-h' or '--help' for more information or ");
+    HDfprintf(output, "see the <%s> entry in the 'HDF5 Reference Manual'.\n",h5tools_getprogname());
+}
+
 
 /*-------------------------------------------------------------------------
  * Function:    get_option
@@ -125,7 +212,9 @@ warn_msg(const char *progname, const char *fmt, ...)
  * Programmer:  Bill Wendling
  *              Friday, 5. January 2001
  *
- * Modifications:
+ * Modifications: Pedro Vicente
+ *                October, 27 2008
+ * Wilcard "*" argument type
  *
  *-------------------------------------------------------------------------
  */
@@ -160,8 +249,11 @@ get_option(int argc, const char **argv, const char *opts, const struct long_opti
                 if (l_opts[i].has_arg != no_arg) {
                     if (arg[len] == '=') {
                         opt_arg = &arg[len + 1];
-                    } else if (opt_ind < (argc - 1) && argv[opt_ind + 1][0] != '-') {
-                        opt_arg = argv[++opt_ind];
+                    }
+                    else if (l_opts[i].has_arg != optional_arg) {
+                        if (opt_ind < (argc - 1)) 
+                            if (argv[opt_ind + 1][0] != '-')
+                                opt_arg = argv[++opt_ind];
                     } else if (l_opts[i].has_arg == require_arg) {
                         if (opt_err)
                             HDfprintf(stderr,
@@ -235,12 +327,33 @@ get_option(int argc, const char **argv, const char *opts, const struct long_opti
             }
 
             sp = 1;
-        } else {
+        }
+
+        /* wildcard argument */
+        else if (*cp == '*')
+        {
+            /* check the next argument */
+            opt_ind++;
+            /* we do have an extra argument, check if not last */
+            if ( argv[opt_ind][0] != '-' && (opt_ind+1) < argc )
+            {
+                opt_arg = argv[opt_ind++];
+            }
+            else
+            {
+                opt_arg = NULL;
+            }
+        }
+
+        else
+        {
             /* set up to look at next char in token, next time */
             if (argv[opt_ind][++sp] == '\0') {
                 /* no more in current token, so setup next token */
                 opt_ind++;
                 sp = 1;
+
+
             }
 
             opt_arg = NULL;
@@ -297,7 +410,7 @@ print_version(const char *progname)
 {
     printf("%s: Version %u.%u.%u%s%s\n",
            progname, H5_VERS_MAJOR, H5_VERS_MINOR, H5_VERS_RELEASE,
-           H5_VERS_SUBRELEASE[0] ? "-" : "", H5_VERS_SUBRELEASE);
+           ((char *)H5_VERS_SUBRELEASE)[0] ? "-" : "", H5_VERS_SUBRELEASE);
 }
 
 
@@ -318,11 +431,11 @@ print_version(const char *progname)
 static void
 init_table(table_t **tbl)
 {
-    table_t *table = HDmalloc(sizeof(table_t));
+    table_t *table = (table_t *)HDmalloc(sizeof(table_t));
 
     table->size = 20;
     table->nobjs = 0;
-    table->objs = HDmalloc(table->size * sizeof(obj_t));
+    table->objs = (obj_t *)HDmalloc(table->size * sizeof(obj_t));
 
     *tbl = table;
 }
@@ -490,21 +603,21 @@ find_objs_cb(const char *name, const H5O_info_t *oinfo, const char *already_seen
             break;
 
         case H5O_TYPE_NAMED_DATATYPE:
-        {
-            obj_t *found_obj;
+            if(NULL == already_seen) {
+                obj_t *found_obj;
 
-            if((found_obj = search_obj(info->type_table, oinfo->addr)) == NULL)
-                add_obj(info->type_table, oinfo->addr, name, TRUE);
-            else {
-                /* Use latest version of name */
-                HDfree(found_obj->objname);
-                found_obj->objname = HDstrdup(name);
+                if((found_obj = search_obj(info->type_table, oinfo->addr)) == NULL)
+                    add_obj(info->type_table, oinfo->addr, name, TRUE);
+                else {
+                    /* Use latest version of name */
+                    HDfree(found_obj->objname);
+                    found_obj->objname = HDstrdup(name);
 
-                /* Mark named datatype as having valid name */
-                found_obj->recorded = TRUE;
-            } /* end else */
+                    /* Mark named datatype as having valid name */
+                    found_obj->recorded = TRUE;
+                } /* end else */
+            } /* end if */
             break;
-        }
 
         default:
             break;
@@ -571,7 +684,7 @@ add_obj(table_t *table, haddr_t objno, const char *objname, hbool_t record)
     /* See if we need to make table larger */
     if(table->nobjs == table->size) {
         table->size *= 2;
-        table->objs = HDrealloc(table->objs, table->size * sizeof(table->objs[0]));
+        table->objs = (struct obj_t *)HDrealloc(table->objs, table->size * sizeof(table->objs[0]));
     } /* end if */
 
     /* Increment number of objects in table */
@@ -608,3 +721,171 @@ tmpfile(void)
 }
 
 #endif
+
+/*-------------------------------------------------------------------------
+ * Function: H5tools_get_symlink_info
+ *
+ * Purpose: Get symbolic link (soft, external) info and its target object type 
+            (dataset, group, named datatype) and path, if exist
+ *
+ * Patameters:
+ *  - [IN]  fileid : link file id
+ *  - [IN]  linkpath : link path
+ *  - [OUT] link_info: returning target object info (h5tool_link_info_t)
+ *
+ * Return: 
+ *   2 : given pathname is object 
+ *   1 : Succed to get link info.  
+ *   0 : Detected as a dangling link
+ *  -1 : H5 API failed.
+ *
+ * NOTE:
+ *  link_info->trg_path must be freed out of this function
+ *
+ * Programmer: Jonathan Kim
+ *
+ * Date: Feb 8, 2010
+ *-------------------------------------------------------------------------*/
+int
+H5tools_get_symlink_info(hid_t file_id, const char * linkpath, h5tool_link_info_t *link_info,
+    hbool_t get_obj_type)
+{
+    htri_t l_ret;
+    H5O_info_t trg_oinfo;
+    hid_t fapl = H5P_DEFAULT;
+    hid_t lapl = H5P_DEFAULT;
+    int ret = -1; /* init to fail */
+
+    /* init */
+    link_info->trg_type = H5O_TYPE_UNKNOWN;
+
+    /* if path is root, return group type */
+    if(!HDstrcmp(linkpath,"/"))
+    {
+        link_info->trg_type = H5O_TYPE_GROUP;
+        ret = 2;
+        goto out;
+    }
+
+    /* check if link itself exist */
+    if(H5Lexists(file_id, linkpath, H5P_DEFAULT) <= 0) {
+        if(link_info->opt.msg_mode == 1)
+            parallel_print("Warning: link <%s> doesn't exist \n",linkpath);
+        goto out;
+    } /* end if */
+
+    /* get info from link */
+    if(H5Lget_info(file_id, linkpath, &(link_info->linfo), H5P_DEFAULT) < 0) {
+        if(link_info->opt.msg_mode == 1)
+            parallel_print("Warning: unable to get link info from <%s>\n",linkpath);
+        goto out;
+    } /* end if */
+
+    /* given path is hard link (object) */
+    if(link_info->linfo.type == H5L_TYPE_HARD) {
+        ret = 2;
+        goto out;
+    } /* end if */
+
+    /* trg_path must be freed out of this function when finished using */
+    link_info->trg_path = (char*)HDcalloc(link_info->linfo.u.val_size, sizeof(char));
+    HDassert(link_info->trg_path);
+
+    /* get link value */
+    if(H5Lget_val(file_id, linkpath, (void *)link_info->trg_path, link_info->linfo.u.val_size, H5P_DEFAULT) < 0) {
+        if(link_info->opt.msg_mode == 1)
+            parallel_print("Warning: unable to get link value from <%s>\n",linkpath);
+        goto out;
+    } /* end if */
+
+    /*-----------------------------------------------------
+     * if link type is external link use different lapl to 
+     * follow object in other file
+     */
+    if(link_info->linfo.type == H5L_TYPE_EXTERNAL) {
+        if((fapl = H5Pcreate(H5P_FILE_ACCESS)) < 0)
+            goto out;
+        if(H5Pset_fapl_sec2(fapl) < 0)
+            goto out;
+        if((lapl = H5Pcreate(H5P_LINK_ACCESS)) < 0)
+            goto out;
+        if(H5Pset_elink_fapl(lapl, fapl) < 0)
+            goto out;
+    } /* end if */
+
+    /* Check for retrieving object info */
+    if(get_obj_type) {
+        /*--------------------------------------------------------------
+         * if link's target object exist, get type
+         */
+         /* check if target object exist */
+        l_ret = H5Oexists_by_name(file_id, linkpath, lapl);
+        
+        /* detect dangling link */
+        if(l_ret == FALSE) {
+            ret = 0;
+            goto out;
+        } /* end if */
+        /* function failed */
+        else if(l_ret < 0)
+            goto out;    
+
+        /* get target object info */
+        if(H5Oget_info_by_name(file_id, linkpath, &trg_oinfo, lapl) < 0) {
+            if(link_info->opt.msg_mode == 1)
+                parallel_print("Warning: unable to get object information for <%s>\n", linkpath);
+            goto out;
+        } /* end if */
+
+        /* check unknown type */
+        if(trg_oinfo.type < H5O_TYPE_GROUP || trg_oinfo.type >=H5O_TYPE_NTYPES) {
+            if(link_info->opt.msg_mode == 1)
+                parallel_print("Warning: target object of <%s> is unknown type\n", linkpath);
+            goto out;
+        }  /* end if */
+
+        /* set target obj type to return */
+        link_info->trg_type = trg_oinfo.type;
+    } /* end if */
+    else
+        link_info->trg_type = H5O_TYPE_UNKNOWN;
+
+    /* succeed */
+    ret = 1;
+
+out:
+    if(fapl != H5P_DEFAULT)
+        H5Pclose(fapl);
+    if(lapl != H5P_DEFAULT)
+        H5Pclose(lapl);
+
+    return ret;
+} /* end H5tools_get_symlink_info() */
+
+/*-------------------------------------------------------------------------
+ * Audience:    Public
+ * Chapter:     H5Tools Library
+ * Purpose:     Initialize the name and operation status of the H5 Tools library
+ * Description:
+ *      These are utility functions to set/get the program name and operation status.
+ *-------------------------------------------------------------------------
+ */
+void h5tools_setprogname(const char *Progname)
+{
+    h5tools_progname = Progname;
+}
+
+void h5tools_setstatus(int D_status)
+{
+    h5tools_d_status = D_status;
+}
+
+const char*h5tools_getprogname(void)
+{
+   return h5tools_progname;
+}
+
+int h5tools_getstatus(void)
+{
+   return h5tools_d_status;
+}
