@@ -24,16 +24,30 @@
 
 #include "hdf5.h"
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
+/* ``parallel_print'' information */
+#define PRINT_DATA_MAX_SIZE   512
+#define OUTBUFF_SIZE     (PRINT_DATA_MAX_SIZE*4)
+
+H5TOOLS_DLLVAR int  g_nTasks;
+H5TOOLS_DLLVAR unsigned char g_Parallel;
+H5TOOLS_DLLVAR char    outBuff[];
+H5TOOLS_DLLVAR int  outBuffOffset;
+H5TOOLS_DLLVAR FILE *   overflow_file;
+
 /*
  * begin get_option section
  */
-extern int         opt_err;     /* getoption prints errors if this is on    */
-extern int         opt_ind;     /* token pointer                            */
-extern const char *opt_arg;     /* flag argument (or value)                 */
+H5TOOLS_DLLVAR int         opt_err;     /* getoption prints errors if this is on    */
+H5TOOLS_DLLVAR int         opt_ind;     /* token pointer                            */
+H5TOOLS_DLLVAR const char *opt_arg;     /* flag argument (or value)                 */
 
 enum {
     no_arg = 0,         /* doesn't take an argument     */
-    require_arg,        /* requires an argument         */
+    require_arg,        /* requires an argument          */
     optional_arg        /* argument is optional         */
 };
 
@@ -44,23 +58,23 @@ enum {
  * the option. The long options are specified in the following way:
  *
  * struct long_options foo[] = {
- *  { "filename", require_arg, 'f' },
- *  { "append", no_arg, 'a' },
- *  { "width", require_arg, 'w' },
- *  { NULL, 0, 0 }
+ *   { "filename", require_arg, 'f' },
+ *   { "append", no_arg, 'a' },
+ *   { "width", require_arg, 'w' },
+ *   { NULL, 0, 0 }
  * };
  *
  * Long named options can have arguments specified as either:
  *
- *  ``--param=arg'' or ``--param arg''
+ *   ``--param=arg'' or ``--param arg''
  *
  * Short named options can have arguments specified as either:
  *
- *  ``-w80'' or ``-w 80''
+ *   ``-w80'' or ``-w 80''
  *
  * and can have more than one short named option specified at one time:
  *
- *  -aw80
+ *   -aw80
  *
  * in which case those options which expect an argument need to come at the
  * end.
@@ -72,7 +86,7 @@ typedef struct long_options {
                                  * this gets returned from get_option   */
 } long_options;
 
-extern int    get_option(int argc, const char **argv, const char *opt,
+H5TOOLS_DLL int    get_option(int argc, const char **argv, const char *opt,
                          const struct long_options *l_opt);
 /*
  * end get_option section
@@ -95,25 +109,69 @@ typedef struct table_t {
 
 /*this struct stores the information that is passed to the find_objs function*/
 typedef struct find_objs_t {
-    char *prefix;
+    hid_t fid;
     table_t *group_table;
     table_t *type_table;
     table_t *dset_table;
 } find_objs_t;
 
-extern int     nCols;               /*max number of columns for outputting  */
+H5TOOLS_DLLVAR int     nCols;               /*max number of columns for outputting  */
 
 /* Definitions of useful routines */
-extern void     indentation(int);
-extern void     print_version(const char *progname);
-extern void     error_msg(const char *progname, const char *fmt, ...);
-extern void     warn_msg(const char *progname, const char *fmt, ...);
-extern void     free_table(table_t *table);
+H5TOOLS_DLL void     indentation(int);
+H5TOOLS_DLL void     print_version(const char *progname);
+H5TOOLS_DLL void     parallel_print(const char* format, ... );
+H5TOOLS_DLL void     error_msg(const char *fmt, ...);
+H5TOOLS_DLL void     warn_msg(const char *fmt, ...);
+H5TOOLS_DLL void     help_ref_msg(FILE *output);
+H5TOOLS_DLL void     free_table(table_t *table);
 #ifdef H5DUMP_DEBUG
-extern void     dump_tables(find_objs_t *info)
+H5TOOLS_DLL void     dump_tables(find_objs_t *info)
 #endif  /* H5DUMP_DEBUG */
-extern herr_t init_objs(hid_t fid, find_objs_t *info, table_t **group_table,
+H5TOOLS_DLL herr_t init_objs(hid_t fid, find_objs_t *info, table_t **group_table,
     table_t **dset_table, table_t **type_table);
-extern obj_t   *search_obj(table_t *temp, unsigned long *objno);
+H5TOOLS_DLL obj_t   *search_obj(table_t *temp, haddr_t objno);
+#ifndef H5_HAVE_TMPFILE
+H5TOOLS_DLL FILE *  tmpfile(void);
+#endif
+
+/*************************************************************
+ *
+ * candidate functions to be public
+ *
+ *************************************************************/
+
+/* This code is layout for common code among tools */
+typedef enum toolname_t {
+    TOOL_H5DIFF, TOOL_H5LS, TOOL__H5DUMP /* add as necessary */
+} h5tool_toolname_t;
+
+/* this struct can be used to differntiate among tools */
+typedef struct {
+    h5tool_toolname_t toolname;
+    int msg_mode;
+} h5tool_opt_t;
+
+/* obtain link info from H5tools_get_symlink_info() */
+typedef struct {
+    H5O_type_t  trg_type;  /* OUT: target type */
+    const char *trg_path;  /* OUT: target obj path. This must be freed 
+                            *      when used with H5tools_get_symlink_info() */
+    H5L_info_t linfo;      /* OUT: link info */
+    h5tool_opt_t opt;      /* IN: options */
+} h5tool_link_info_t;
+
+
+/* Definitions of routines */
+H5TOOLS_DLL int H5tools_get_symlink_info(hid_t file_id, const char * linkpath,
+    h5tool_link_info_t *link_info, hbool_t get_obj_type);
+H5TOOLS_DLL const char *h5tools_getprogname(void);
+H5TOOLS_DLL void     h5tools_setprogname(const char*progname);
+H5TOOLS_DLL int      h5tools_getstatus(void);
+H5TOOLS_DLL void     h5tools_setstatus(int d_status);
+
+#ifdef __cplusplus
+}
+#endif
 
 #endif  /* H5TOOLS_UTILS_H__ */
