@@ -39,6 +39,9 @@ typedef struct {
 typedef struct {
     trav_addr_t *seen;              /* List of addresses seen already */
     const trav_visitor_t *visitor;  /* Information for visiting each link/object */
+    hbool_t is_absolute;            /* Whether the traversal has absolute paths */
+    const char *base_grp_name;      /* Name of the group that serves as the base
+                                     * for iteration */
 } trav_ud_traverse_t;
 
 typedef struct {
@@ -139,21 +142,36 @@ traverse_cb(hid_t loc_id, const char *path, const H5L_info_t *linfo,
     void *_udata)
 {
     trav_ud_traverse_t *udata = (trav_ud_traverse_t *)_udata;     /* User data */
-    char *full_name;
+    char *new_name = NULL;
+    const char *full_name;
     const char *already_visited = NULL; /* Whether the link/object was already visited */
 
     /* Create the full path name for the link */
-    full_name = HDmalloc(HDstrlen(path) + 2);
-    *full_name = '/';
-    HDstrcpy(full_name + 1, path);
+    if(udata->is_absolute) {
+        size_t base_len = HDstrlen(udata->base_grp_name);
+        size_t add_slash = base_len ? ((udata->base_grp_name)[base_len-1] != '/') : 1;
+
+        if(NULL == (new_name = HDmalloc(base_len + add_slash + HDstrlen(path) + 1)))
+            return(H5_ITER_ERROR);
+        HDstrcpy(new_name, udata->base_grp_name);
+        if (add_slash)
+            new_name[base_len] = '/';
+        HDstrcpy(new_name + base_len + add_slash, path);
+        full_name = new_name;
+    } /* end if */
+    else
+        full_name = path;
 
     /* Perform the correct action for different types of links */
     if(linfo->type == H5L_TYPE_HARD) {
         H5O_info_t oinfo;
 
         /* Get information about the object */
-        if(H5Oget_info_by_name(loc_id, path, &oinfo, H5P_DEFAULT) < 0)
+        if(H5Oget_info_by_name(loc_id, path, &oinfo, H5P_DEFAULT) < 0) {
+            if(new_name)
+                HDfree(new_name);
             return(H5_ITER_ERROR);
+        }
 
         /* If the object has multiple links, add it to the list of addresses
          *  already visited, if it isn't there already
@@ -164,15 +182,24 @@ traverse_cb(hid_t loc_id, const char *path, const H5L_info_t *linfo,
 
         /* Make 'visit object' callback */
         if(udata->visitor->visit_obj)
-            (*udata->visitor->visit_obj)(full_name, &oinfo, already_visited, udata->visitor->udata);
+            if((*udata->visitor->visit_obj)(full_name, &oinfo, already_visited, udata->visitor->udata) < 0) {
+                if(new_name)
+                    HDfree(new_name);
+                return(H5_ITER_ERROR);
+            }
     } /* end if */
     else {
         /* Make 'visit link' callback */
         if(udata->visitor->visit_lnk)
-            (*udata->visitor->visit_lnk)(full_name, linfo, udata->visitor->udata);
+            if((*udata->visitor->visit_lnk)(full_name, linfo, udata->visitor->udata) < 0) {
+                if(new_name)
+                    HDfree(new_name);
+                return(H5_ITER_ERROR);
+            }
     } /* end else */
 
-    HDfree(full_name);
+    if(new_name)
+        HDfree(new_name);
 
     return(H5_ITER_CONT);
 } /* end traverse_cb() */
@@ -222,6 +249,8 @@ traverse(hid_t file_id, const char *grp_name, hbool_t visit_start,
         /* Set up user data structure */
         udata.seen = &seen;
         udata.visitor = visitor;
+        udata.is_absolute = (*grp_name == '/');
+        udata.base_grp_name = grp_name;
 
         /* Check for iteration of links vs. visiting all links recursively */
         if(recurse) {
@@ -263,7 +292,7 @@ traverse(hid_t file_id, const char *grp_name, hbool_t visit_start,
  *
  *-------------------------------------------------------------------------
  */
-static void
+void
 trav_info_add(trav_info_t *info, const char *path, h5trav_type_t obj_type)
 {
     size_t idx;         /* Index of address to use  */
@@ -278,6 +307,8 @@ trav_info_add(trav_info_t *info, const char *path, h5trav_type_t obj_type)
     idx = info->nused++;
     info->paths[idx].path = HDstrdup(path);
     info->paths[idx].type = obj_type;
+    info->paths[idx].fileno = 0;
+    info->paths[idx].objno = HADDR_UNDEF;
 } /* end trav_info_add() */
 
 
@@ -294,14 +325,22 @@ trav_info_add(trav_info_t *info, const char *path, h5trav_type_t obj_type)
  *
  *-------------------------------------------------------------------------
  */
-static int
+int
 trav_info_visit_obj(const char *path, const H5O_info_t *oinfo,
     const char UNUSED *already_visited, void *udata)
 {
+    size_t idx;
+    trav_info_t *info_p;
     /* Add the object to the 'info' struct */
     /* (object types map directly to "traversal" types) */
     trav_info_add((trav_info_t *)udata, path, (h5trav_type_t)oinfo->type);
 
+    /* set object addr and fileno. These are for checking same object */
+    info_p = (trav_info_t *) udata;
+    idx = info_p->nused - 1;
+    info_p->paths[idx].objno = oinfo->addr;
+    info_p->paths[idx].fileno = oinfo->fileno;
+
     return(0);
 } /* end trav_info_visit_obj() */
 
@@ -319,7 +358,7 @@ trav_info_visit_obj(const char *path, const H5O_info_t *oinfo,
  *
  *-------------------------------------------------------------------------
  */
-static int
+int
 trav_info_visit_lnk(const char *path, const H5L_info_t *linfo, void *udata)
 {
     /* Add the link to the 'info' struct */
@@ -409,14 +448,21 @@ h5trav_getindex(const trav_info_t *info, const char *obj)
  */
 
 void
-trav_info_init(trav_info_t **_info)
+trav_info_init(const char *filename, hid_t fileid, trav_info_t **_info)
 {
     trav_info_t *info = (trav_info_t *)HDmalloc(sizeof(trav_info_t));
 
     /* Init info structure */
     info->nused = info->nalloc = 0;
     info->paths = NULL;
+    info->fname = filename;
+    info->fid = fileid;
 
+    /* Initialize list of visited symbolic links */
+    info->symlink_visited.nused = 0;
+    info->symlink_visited.nalloc = 0;
+    info->symlink_visited.objs = NULL;
+    info->symlink_visited.dangle_link = FALSE;
     *_info = info;
 } /* end trav_info_init() */
 
@@ -435,6 +481,16 @@ trav_info_free(trav_info_t *info)
     size_t u;           /* Local index variable */
 
     if(info) {
+        /* Free visited symbolic links path and file (if alloc) */
+        for(u=0; u < info->symlink_visited.nused; u++) 
+        {
+            if (info->symlink_visited.objs[u].file)
+                HDfree(info->symlink_visited.objs[u].file);
+
+            HDfree(info->symlink_visited.objs[u].path);
+        }
+        HDfree(info->symlink_visited.objs);
+
         /* Free path names */
         for(u = 0; u < info->nused; u++)
             HDfree(info->paths[u].path);
@@ -581,32 +637,6 @@ h5trav_getindext(const char *name, const trav_table_t *table)
     return -1;
 }
 
-/*-------------------------------------------------------------------------
- * Function: trav_table_search
- *
- * Purpose: Search in the table for OBJNO
- *
- * Return: index of object in table
- *
- * Programmer: Pedro Vicente, pvn@ncsa.uiuc.edu
- *
- * Date: November 4, 2002
- *
- *-------------------------------------------------------------------------
- */
-
-static size_t
-trav_table_search(const trav_table_t *table, haddr_t objno)
-{
-    size_t i;
-
-    for(i = 0; i < table->nobjs; i++)
-        if(table->objs[i].objno == objno)
-            return(i);
-    return(i);
-}
-
-
 /*-------------------------------------------------------------------------
  * Function: trav_table_add
  *
@@ -636,6 +666,7 @@ trav_table_add(trav_table_t *table,
     new = table->nobjs++;
     table->objs[new].objno = oinfo ? oinfo->addr : HADDR_UNDEF;
     table->objs[new].flags[0] = table->objs[new].flags[1] = 0;
+    table->objs[new].is_same_trgobj = 0;
     table->objs[new].name = (char *)HDstrdup(path);
     table->objs[new].type = oinfo ? (h5trav_type_t)oinfo->type : H5TRAV_TYPE_LINK;
     table->objs[new].nlinks = 0;
@@ -719,6 +750,7 @@ void trav_table_addflags(unsigned *flags,
     table->objs[new].objno = 0;
     table->objs[new].flags[0] = flags[0];
     table->objs[new].flags[1] = flags[1];
+    table->objs[new].is_same_trgobj = 0;
     table->objs[new].name = (char *)HDstrdup(name);
     table->objs[new].type = type;
     table->objs[new].nlinks = 0;
@@ -970,3 +1002,119 @@ h5trav_visit(hid_t fid, const char *grp_name, hbool_t visit_start,
     return 0;
 }
 
+/*-------------------------------------------------------------------------
+ * Function: symlink_visit_add
+ *
+ * Purpose: Add an symbolic link to visited data structure
+ *
+ * Return: 0 on success, -1 on failure
+ *
+ * Programmer: Neil Fortner, nfortne2@hdfgroup.org
+ *             Adapted from trav_addr_add in h5trav.c by Quincey Koziol
+ *
+ * Date: September 5, 2008
+ *
+ * Modified: 
+ *  Jonathan Kim
+ *   - Moved from h5ls.c to share among tools.  (Sep 16, 2010)
+ *   - Renamed from elink_trav_add to symlink_visit_add for both soft and 
+ *     external links.   (May 25, 2010)
+ *   - Add type parameter to distingush between soft and external link for 
+ *     sure, which prevent from mixing up visited link when the target names
+ *     are same between the soft and external link, as code marks with the
+ *     target name.  (May 25,2010) 
+ *
+ *-------------------------------------------------------------------------
+ */
+herr_t
+symlink_visit_add(symlink_trav_t *visited, H5L_type_t type, const char *file, const char *path)
+{
+    size_t  idx;         /* Index of address to use */
+    void    *tmp_ptr;
+
+    /* Allocate space if necessary */
+    if(visited->nused == visited->nalloc) 
+    {
+        visited->nalloc = MAX(1, visited->nalloc * 2);
+        if(NULL == (tmp_ptr = HDrealloc(visited->objs, visited->nalloc * sizeof(visited->objs[0]))))
+            return -1;
+        visited->objs = tmp_ptr;
+    } /* end if */
+
+    /* Append it */
+    idx = visited->nused++;
+
+    visited->objs[idx].type = type;
+    visited->objs[idx].file = NULL;
+    visited->objs[idx].path = NULL;
+
+    if (type == H5L_TYPE_EXTERNAL)
+    {
+        if(NULL == (visited->objs[idx].file = HDstrdup(file))) 
+        {
+            visited->nused--;
+            return -1;
+        }
+    }
+
+    if(NULL == (visited->objs[idx].path = HDstrdup(path))) 
+    {
+        visited->nused--;
+        if (visited->objs[idx].file)
+            HDfree (visited->objs[idx].file);
+        return -1;
+    }
+
+    return 0;
+} /* end symlink_visit_add() */
+
+
+/*-------------------------------------------------------------------------
+ * Function: symlink_is_visited
+ *
+ * Purpose: Check if an symbolic link has already been visited
+ *
+ * Return: TRUE/FALSE
+ *
+ * Programmer: Neil Fortner, nfortne2@hdfgroup.org
+ *             Adapted from trav_addr_visited in h5trav.c by Quincey Koziol
+ *
+ * Date: September 5, 2008
+ *
+ * Modified: 
+ *  Jonathan Kim
+ *   - Moved from h5ls.c to share among tools.  (Sep 16, 2010)
+ *   - Renamed from elink_trav_visited to symlink_is_visited for both soft and 
+ *     external links.  (May 25, 2010)
+ *   - Add type parameter to distingush between soft and external link for 
+ *     sure, which prevent from mixing up visited link when the target names
+ *     are same between the soft and external link, as code marks with the
+ *     target name.  (May 25,2010) 
+ *
+ *-------------------------------------------------------------------------
+ */
+hbool_t
+symlink_is_visited(symlink_trav_t *visited, H5L_type_t type, const char *file, const char *path)
+{
+    size_t u;  /* Local index variable */
+
+    /* Look for symlink */
+    for(u = 0; u < visited->nused; u++)
+    {
+        /* Check for symlink values already in array */
+        /* check type and path pair to distingush between symbolic links */
+        if ((visited->objs[u].type == type) && !HDstrcmp(visited->objs[u].path, path))
+        {
+            /* if external link, file need to be matched as well */
+            if (visited->objs[u].type == H5L_TYPE_EXTERNAL)
+            {
+                if (!HDstrcmp(visited->objs[u].file, file))
+                    return (TRUE);
+            }
+            return (TRUE);
+        }
+    }
+    /* Didn't find symlink */
+    return(FALSE);
+} /* end symlink_is_visited() */
+
