@@ -29,7 +29,6 @@
 #define H5B2_PACKAGE		/*suppress error about including H5B2pkg  */
 #define H5B2_TESTING		/*suppress warning about H5B2 testing funcs*/
 #define H5F_PACKAGE		/*suppress error about including H5Fpkg	  */
-#define H5FS_PACKAGE		/*suppress error about including H5FSpkg  */
 #define H5G_PACKAGE		/*suppress error about including H5Gpkg	  */
 #define H5HF_PACKAGE		/*suppress error about including H5HFpkg  */
 #define H5O_PACKAGE		/*suppress error about including H5Opkg	  */
@@ -37,19 +36,16 @@
 
 #include "H5private.h"		/* Generic Functions			*/
 #include "H5Apkg.h"		/* Attributes				*/
-#include "H5Bprivate.h"		/* B-trees				*/
 #include "H5B2pkg.h"		/* v2 B-trees				*/
 #include "H5Dprivate.h"		/* Datasets				*/
 #include "H5Eprivate.h"		/* Error handling		  	*/
 #include "H5Fpkg.h"             /* File access				*/
-#include "H5FSpkg.h"		/* File free space			*/
+#include "H5FSprivate.h"	/* Free space manager			*/
 #include "H5Gpkg.h"		/* Groups				*/
 #include "H5HFpkg.h"		/* Fractal heaps			*/
 #include "H5HGprivate.h"	/* Global Heaps				*/
-#include "H5HLprivate.h"	/* Local Heaps				*/
 #include "H5Iprivate.h"		/* IDs			  		*/
 #include "H5Opkg.h"             /* Object headers			*/
-#include "H5Pprivate.h"		/* Property lists			*/
 #include "H5SMpkg.h"		/* Implicitly shared messages		*/
 
 /* File drivers */
@@ -58,6 +54,77 @@
 #define INDENT  3
 #define VCOL    50
 
+
+/*-------------------------------------------------------------------------
+ * Function:    get_H5B2_class
+ *
+ * Purpose:	Determine the v2 B-tree class from the buffer read in.
+ *              B-trees are debugged through the B-tree subclass.  The subclass
+ *              identifier is two bytes after the B-tree signature.
+ *
+ * Return:	Non-NULL on success/NULL on failure
+ *
+ * Programmer:	Quincey Koziol
+ *		koziol@hdfgroup.org
+ *		Sep 11 2008
+ *
+ *-------------------------------------------------------------------------
+ */
+static const H5B2_class_t *
+get_H5B2_class(const uint8_t *sig)
+{
+    H5B2_subid_t subtype = (H5B2_subid_t)sig[H5_SIZEOF_MAGIC + 1];
+    const H5B2_class_t *cls;
+
+    switch(subtype) {
+        case H5B2_TEST_ID:
+            cls = H5B2_TEST;
+            break;
+
+        case H5B2_FHEAP_HUGE_INDIR_ID:
+            cls = H5HF_HUGE_BT2_INDIR;
+            break;
+
+        case H5B2_FHEAP_HUGE_FILT_INDIR_ID:
+            cls = H5HF_HUGE_BT2_FILT_INDIR;
+            break;
+
+        case H5B2_FHEAP_HUGE_DIR_ID:
+            cls = H5HF_HUGE_BT2_DIR;
+            break;
+
+        case H5B2_FHEAP_HUGE_FILT_DIR_ID:
+            cls = H5HF_HUGE_BT2_FILT_DIR;
+            break;
+
+        case H5B2_GRP_DENSE_NAME_ID:
+            cls = H5G_BT2_NAME;
+            break;
+
+        case H5B2_GRP_DENSE_CORDER_ID:
+            cls = H5G_BT2_CORDER;
+            break;
+
+        case H5B2_SOHM_INDEX_ID:
+            cls = H5SM_INDEX;
+            break;
+
+        case H5B2_ATTR_DENSE_NAME_ID:
+            cls = H5A_BT2_NAME;
+            break;
+
+        case H5B2_ATTR_DENSE_CORDER_ID:
+            cls = H5A_BT2_CORDER;
+            break;
+
+        default:
+            fprintf(stderr, "Unknown B-tree subtype %u\n", (unsigned)(subtype));
+            HDexit(4);
+    } /* end switch */
+
+    return(cls);
+} /* end get_H5B2_class() */
+
 
 /*-------------------------------------------------------------------------
  * Function:    main
@@ -79,7 +146,7 @@ main(int argc, char *argv[])
 {
     hid_t	fid, fapl, dxpl;
     H5F_t       *f;
-    haddr_t     addr = 0, extra = 0, extra2 = 0, extra3 = 0;
+    haddr_t     addr = 0, extra = 0, extra2 = 0, extra3 = 0, extra4 = 0;
     uint8_t     sig[H5F_SIGNATURE_LEN];
     size_t      u;
     herr_t      status = SUCCEED;
@@ -128,6 +195,8 @@ main(int argc, char *argv[])
         extra2 = HDstrtoll(argv[4], NULL, 0);
     if(argc > 5)
         extra3 = HDstrtoll(argv[5], NULL, 0);
+    if(argc > 6)
+        extra4 = (haddr_t)HDstrtoll(argv[6], NULL, 0);
 
     /*
      * Read the signature at the specified file position.
@@ -143,19 +212,19 @@ main(int argc, char *argv[])
          */
         status = H5F_debug(f, stdout, 0, VCOL);
 
-    } else if(!HDmemcmp(sig, H5HL_MAGIC, (size_t)H5HL_SIZEOF_MAGIC)) {
+    } else if(!HDmemcmp(sig, H5HL_MAGIC, (size_t)H5_SIZEOF_MAGIC)) {
         /*
          * Debug a local heap.
          */
         status = H5HL_debug(f, H5P_DATASET_XFER_DEFAULT, addr, stdout, 0, VCOL);
 
-    } else if(!HDmemcmp (sig, H5HG_MAGIC, (size_t)H5HG_SIZEOF_MAGIC)) {
+    } else if(!HDmemcmp (sig, H5HG_MAGIC, (size_t)H5_SIZEOF_MAGIC)) {
 	/*
 	 * Debug a global heap collection.
 	 */
 	status = H5HG_debug (f, H5P_DATASET_XFER_DEFAULT, addr, stdout, 0, VCOL);
 
-    } else if(!HDmemcmp(sig, H5G_NODE_MAGIC, (size_t)H5G_NODE_SIZEOF_MAGIC)) {
+    } else if(!HDmemcmp(sig, H5G_NODE_MAGIC, (size_t)H5_SIZEOF_MAGIC)) {
         /*
          * Debug a symbol table node.
          */
@@ -169,13 +238,13 @@ main(int argc, char *argv[])
 
         status = H5G_node_debug(f, H5P_DATASET_XFER_DEFAULT, addr, stdout, 0, VCOL, extra);
 
-    } else if(!HDmemcmp(sig, H5B_MAGIC, (size_t)H5B_SIZEOF_MAGIC)) {
+    } else if(!HDmemcmp(sig, H5B_MAGIC, (size_t)H5_SIZEOF_MAGIC)) {
         /*
          * Debug a B-tree.  B-trees are debugged through the B-tree
          * subclass.  The subclass identifier is the byte immediately
          * after the B-tree signature.
          */
-        H5B_subid_t subtype = (H5B_subid_t)sig[H5B_SIZEOF_MAGIC];
+        H5B_subid_t subtype = (H5B_subid_t)sig[H5_SIZEOF_MAGIC];
         unsigned    ndims;
 
         switch(subtype) {
@@ -190,7 +259,7 @@ main(int argc, char *argv[])
                 status = H5G_node_debug(f, H5P_DATASET_XFER_DEFAULT, addr, stdout, 0, VCOL, extra);
                 break;
 
-            case H5B_ISTORE_ID:
+            case H5B_CHUNK_ID:
                 /* Check for extra parameters */
                 if(extra == 0) {
                     fprintf(stderr, "ERROR: Need number of dimensions of chunk in order to dump chunk B-tree node\n");
@@ -200,7 +269,7 @@ main(int argc, char *argv[])
                 } /* end if */
 
                 ndims = (unsigned)extra;
-                status = H5D_istore_debug (f, H5P_DATASET_XFER_DEFAULT, addr, stdout, 0, VCOL, ndims);
+                status = H5D_btree_debug(f, H5P_DATASET_XFER_DEFAULT, addr, stdout, 0, VCOL, ndims);
                 break;
 
             default:
@@ -208,67 +277,21 @@ main(int argc, char *argv[])
                 HDexit(4);
         }
 
-    } else if(!HDmemcmp(sig, H5B2_HDR_MAGIC, (size_t)H5B2_SIZEOF_MAGIC)) {
+    } else if(!HDmemcmp(sig, H5B2_HDR_MAGIC, (size_t)H5_SIZEOF_MAGIC)) {
         /*
-         * Debug a v2 B-tree.  B-trees are debugged through the B-tree
-         * subclass.  The subclass identifier is two bytes after the
-         * B-tree signature.
+         * Debug a v2 B-tree header.
          */
-        H5B2_subid_t subtype = (H5B2_subid_t)sig[H5B2_SIZEOF_MAGIC+1];
+        const H5B2_class_t *cls = get_H5B2_class(sig);
+        HDassert(cls);
 
-        switch(subtype) {
-            case H5B2_TEST_ID:
-                status = H5B2_hdr_debug(f, H5P_DATASET_XFER_DEFAULT, addr, stdout, 0, VCOL, H5B2_TEST);
-                break;
+        status = H5B2_hdr_debug(f, H5P_DATASET_XFER_DEFAULT, addr, stdout, 0, VCOL, cls, (haddr_t)extra);
 
-            case H5B2_FHEAP_HUGE_INDIR_ID:
-                status = H5B2_hdr_debug(f, H5P_DATASET_XFER_DEFAULT, addr, stdout, 0, VCOL, H5HF_BT2_INDIR);
-                break;
-
-            case H5B2_FHEAP_HUGE_FILT_INDIR_ID:
-                status = H5B2_hdr_debug(f, H5P_DATASET_XFER_DEFAULT, addr, stdout, 0, VCOL, H5HF_BT2_FILT_INDIR);
-                break;
-
-            case H5B2_FHEAP_HUGE_DIR_ID:
-                status = H5B2_hdr_debug(f, H5P_DATASET_XFER_DEFAULT, addr, stdout, 0, VCOL, H5HF_BT2_DIR);
-                break;
-
-            case H5B2_FHEAP_HUGE_FILT_DIR_ID:
-                status = H5B2_hdr_debug(f, H5P_DATASET_XFER_DEFAULT, addr, stdout, 0, VCOL, H5HF_BT2_FILT_DIR);
-                break;
-
-            case H5B2_GRP_DENSE_NAME_ID:
-                status = H5B2_hdr_debug(f, H5P_DATASET_XFER_DEFAULT, addr, stdout, 0, VCOL, H5G_BT2_NAME);
-                break;
-
-            case H5B2_GRP_DENSE_CORDER_ID:
-                status = H5B2_hdr_debug(f, H5P_DATASET_XFER_DEFAULT, addr, stdout, 0, VCOL, H5G_BT2_CORDER);
-                break;
-
-            case H5B2_SOHM_INDEX_ID:
-                status = H5B2_hdr_debug(f, H5P_DATASET_XFER_DEFAULT, addr, stdout, 0, VCOL, H5SM_INDEX);
-                break;
-
-            case H5B2_ATTR_DENSE_NAME_ID:
-                status = H5B2_hdr_debug(f, H5P_DATASET_XFER_DEFAULT, addr, stdout, 0, VCOL, H5A_BT2_NAME);
-                break;
-
-            case H5B2_ATTR_DENSE_CORDER_ID:
-                status = H5B2_hdr_debug(f, H5P_DATASET_XFER_DEFAULT, addr, stdout, 0, VCOL, H5A_BT2_CORDER);
-                break;
-
-            default:
-                fprintf(stderr, "Unknown B-tree subtype %u\n", (unsigned)(subtype));
-                HDexit(4);
-        } /* end switch */
-
-    } else if(!HDmemcmp(sig, H5B2_INT_MAGIC, (size_t)H5B2_SIZEOF_MAGIC)) {
+    } else if(!HDmemcmp(sig, H5B2_INT_MAGIC, (size_t)H5_SIZEOF_MAGIC)) {
         /*
-         * Debug a v2 B-tree.  B-trees are debugged through the B-tree
-         * subclass.  The subclass identifier is the byte after the
-         * B-tree signature.
+         * Debug a v2 B-tree internal node.
          */
-        H5B2_subid_t subtype = (H5B2_subid_t)sig[H5B2_SIZEOF_MAGIC + 1];
+        const H5B2_class_t *cls = get_H5B2_class(sig);
+        HDassert(cls);
 
         /* Check for enough valid parameters */
         if(extra == 0 || extra2 == 0 || extra3 == 0) {
@@ -279,59 +302,14 @@ main(int argc, char *argv[])
             HDexit(4);
         } /* end if */
 
-        switch(subtype) {
-            case H5B2_TEST_ID:
-                status = H5B2_int_debug(f, H5P_DATASET_XFER_DEFAULT, addr, stdout, 0, VCOL, H5B2_TEST, extra, (unsigned)extra2, (unsigned)extra3);
-                break;
+        status = H5B2_int_debug(f, H5P_DATASET_XFER_DEFAULT, addr, stdout, 0, VCOL, cls, extra, (unsigned)extra2, (unsigned)extra3, (haddr_t)extra4);
 
-            case H5B2_FHEAP_HUGE_INDIR_ID:
-                status = H5B2_int_debug(f, H5P_DATASET_XFER_DEFAULT, addr, stdout, 0, VCOL, H5HF_BT2_INDIR, extra, (unsigned)extra2, (unsigned)extra3);
-                break;
-
-            case H5B2_FHEAP_HUGE_FILT_INDIR_ID:
-                status = H5B2_int_debug(f, H5P_DATASET_XFER_DEFAULT, addr, stdout, 0, VCOL, H5HF_BT2_FILT_INDIR, extra, (unsigned)extra2, (unsigned)extra3);
-                break;
-
-            case H5B2_FHEAP_HUGE_DIR_ID:
-                status = H5B2_int_debug(f, H5P_DATASET_XFER_DEFAULT, addr, stdout, 0, VCOL, H5HF_BT2_DIR, extra, (unsigned)extra2, (unsigned)extra3);
-                break;
-
-            case H5B2_FHEAP_HUGE_FILT_DIR_ID:
-                status = H5B2_int_debug(f, H5P_DATASET_XFER_DEFAULT, addr, stdout, 0, VCOL, H5HF_BT2_FILT_DIR, extra, (unsigned)extra2, (unsigned)extra3);
-                break;
-
-            case H5B2_GRP_DENSE_NAME_ID:
-                status = H5B2_int_debug(f, H5P_DATASET_XFER_DEFAULT, addr, stdout, 0, VCOL, H5G_BT2_NAME, extra, (unsigned)extra2, (unsigned)extra3);
-                break;
-
-            case H5B2_GRP_DENSE_CORDER_ID:
-                status = H5B2_int_debug(f, H5P_DATASET_XFER_DEFAULT, addr, stdout, 0, VCOL, H5G_BT2_CORDER, extra, (unsigned)extra2, (unsigned)extra3);
-                break;
-
-            case H5B2_SOHM_INDEX_ID:
-                status = H5B2_int_debug(f, H5P_DATASET_XFER_DEFAULT, addr, stdout, 0, VCOL, H5SM_INDEX, extra, (unsigned)extra2, (unsigned)extra3);
-                break;
-
-            case H5B2_ATTR_DENSE_NAME_ID:
-                status = H5B2_int_debug(f, H5P_DATASET_XFER_DEFAULT, addr, stdout, 0, VCOL, H5A_BT2_NAME, extra, (unsigned)extra2, (unsigned)extra3);
-                break;
-
-            case H5B2_ATTR_DENSE_CORDER_ID:
-                status = H5B2_int_debug(f, H5P_DATASET_XFER_DEFAULT, addr, stdout, 0, VCOL, H5A_BT2_CORDER, extra, (unsigned)extra2, (unsigned)extra3);
-                break;
-
-            default:
-                fprintf(stderr, "Unknown B-tree subtype %u\n", (unsigned)(subtype));
-                HDexit(4);
-        } /* end switch */
-
-    } else if(!HDmemcmp(sig, H5B2_LEAF_MAGIC, (size_t)H5B2_SIZEOF_MAGIC)) {
+    } else if(!HDmemcmp(sig, H5B2_LEAF_MAGIC, (size_t)H5_SIZEOF_MAGIC)) {
         /*
-         * Debug a v2 B-tree.  B-trees are debugged through the B-tree
-         * subclass.  The subclass identifier is the byte after the
-         * B-tree signature.
+         * Debug a v2 B-tree leaf node.
          */
-        H5B2_subid_t subtype = (H5B2_subid_t)sig[H5B2_SIZEOF_MAGIC + 1];
+        const H5B2_class_t *cls = get_H5B2_class(sig);
+        HDassert(cls);
 
         /* Check for enough valid parameters */
         if(extra == 0 || extra2 == 0) {
@@ -341,59 +319,15 @@ main(int argc, char *argv[])
             HDexit(4);
         } /* end if */
 
-        switch(subtype) {
-            case H5B2_TEST_ID:
-                status = H5B2_leaf_debug(f, H5P_DATASET_XFER_DEFAULT, addr, stdout, 0, VCOL, H5B2_TEST, extra, (unsigned)extra2);
-                break;
+        status = H5B2_leaf_debug(f, H5P_DATASET_XFER_DEFAULT, addr, stdout, 0, VCOL, cls, extra, (unsigned)extra2, (haddr_t)extra3);
 
-            case H5B2_FHEAP_HUGE_INDIR_ID:
-                status = H5B2_leaf_debug(f, H5P_DATASET_XFER_DEFAULT, addr, stdout, 0, VCOL, H5HF_BT2_INDIR, extra, (unsigned)extra2);
-                break;
-
-            case H5B2_FHEAP_HUGE_FILT_INDIR_ID:
-                status = H5B2_leaf_debug(f, H5P_DATASET_XFER_DEFAULT, addr, stdout, 0, VCOL, H5HF_BT2_FILT_INDIR, extra, (unsigned)extra2);
-                break;
-
-            case H5B2_FHEAP_HUGE_DIR_ID:
-                status = H5B2_leaf_debug(f, H5P_DATASET_XFER_DEFAULT, addr, stdout, 0, VCOL, H5HF_BT2_DIR, extra, (unsigned)extra2);
-                break;
-
-            case H5B2_FHEAP_HUGE_FILT_DIR_ID:
-                status = H5B2_leaf_debug(f, H5P_DATASET_XFER_DEFAULT, addr, stdout, 0, VCOL, H5HF_BT2_FILT_DIR, extra, (unsigned)extra2);
-                break;
-
-            case H5B2_GRP_DENSE_NAME_ID:
-                status = H5B2_leaf_debug(f, H5P_DATASET_XFER_DEFAULT, addr, stdout, 0, VCOL, H5G_BT2_NAME, extra, (unsigned)extra2);
-                break;
-
-            case H5B2_GRP_DENSE_CORDER_ID:
-                status = H5B2_leaf_debug(f, H5P_DATASET_XFER_DEFAULT, addr, stdout, 0, VCOL, H5G_BT2_CORDER, extra, (unsigned)extra2);
-                break;
-
-            case H5B2_SOHM_INDEX_ID:
-                status = H5B2_leaf_debug(f, H5P_DATASET_XFER_DEFAULT, addr, stdout, 0, VCOL, H5SM_INDEX, extra, (unsigned)extra2);
-                break;
-
-            case H5B2_ATTR_DENSE_NAME_ID:
-                status = H5B2_leaf_debug(f, H5P_DATASET_XFER_DEFAULT, addr, stdout, 0, VCOL, H5A_BT2_NAME, extra, (unsigned)extra2);
-                break;
-
-            case H5B2_ATTR_DENSE_CORDER_ID:
-                status = H5B2_leaf_debug(f, H5P_DATASET_XFER_DEFAULT, addr, stdout, 0, VCOL, H5A_BT2_CORDER, extra, (unsigned)extra2);
-                break;
-
-            default:
-                fprintf(stderr, "Unknown B-tree subtype %u\n", (unsigned)(subtype));
-                HDexit(4);
-        } /* end switch */
-
-    } else if(!HDmemcmp(sig, H5HF_HDR_MAGIC, (size_t)H5HF_SIZEOF_MAGIC)) {
+    } else if(!HDmemcmp(sig, H5HF_HDR_MAGIC, (size_t)H5_SIZEOF_MAGIC)) {
         /*
          * Debug a fractal heap header.
          */
         status = H5HF_hdr_debug(f, H5P_DATASET_XFER_DEFAULT, addr, stdout, 0, VCOL);
 
-    } else if(!HDmemcmp(sig, H5HF_DBLOCK_MAGIC, (size_t)H5HF_SIZEOF_MAGIC)) {
+    } else if(!HDmemcmp(sig, H5HF_DBLOCK_MAGIC, (size_t)H5_SIZEOF_MAGIC)) {
         /*
          * Debug a fractal heap direct block.
          */
@@ -408,7 +342,7 @@ main(int argc, char *argv[])
 
         status = H5HF_dblock_debug(f, H5P_DATASET_XFER_DEFAULT, addr, stdout, 0, VCOL, extra, (size_t)extra2);
 
-    } else if(!HDmemcmp(sig, H5HF_IBLOCK_MAGIC, (size_t)H5HF_SIZEOF_MAGIC)) {
+    } else if(!HDmemcmp(sig, H5HF_IBLOCK_MAGIC, (size_t)H5_SIZEOF_MAGIC)) {
         /*
          * Debug a fractal heap indirect block.
          */
@@ -423,14 +357,14 @@ main(int argc, char *argv[])
 
         status = H5HF_iblock_debug(f, H5P_DATASET_XFER_DEFAULT, addr, stdout, 0, VCOL, extra, (unsigned)extra2);
 
-    } else if(!HDmemcmp(sig, H5FS_HDR_MAGIC, (size_t)H5FS_SIZEOF_MAGIC)) {
+    } else if(!HDmemcmp(sig, H5FS_HDR_MAGIC, (size_t)H5_SIZEOF_MAGIC)) {
         /*
          * Debug a free space header.
          */
 
         status = H5FS_debug(f, H5P_DATASET_XFER_DEFAULT, addr, stdout, 0, VCOL);
 
-    } else if(!HDmemcmp(sig, H5FS_SINFO_MAGIC, (size_t)H5FS_SIZEOF_MAGIC)) {
+    } else if(!HDmemcmp(sig, H5FS_SINFO_MAGIC, (size_t)H5_SIZEOF_MAGIC)) {
         /*
          * Debug free space serialized sections.
          */
@@ -445,14 +379,14 @@ main(int argc, char *argv[])
 
         status = H5FS_sects_debug(f, H5P_DATASET_XFER_DEFAULT, addr, stdout, 0, VCOL, extra, extra2);
 
-    } else if(!HDmemcmp(sig, H5SM_TABLE_MAGIC, (size_t)H5SM_SIZEOF_MAGIC)) {
+    } else if(!HDmemcmp(sig, H5SM_TABLE_MAGIC, (size_t)H5_SIZEOF_MAGIC)) {
         /*
          * Debug shared message master table.
          */
 
         status = H5SM_table_debug(f, H5P_DATASET_XFER_DEFAULT, addr, stdout, 0, VCOL, (unsigned) UFAIL, (unsigned) UFAIL);
 
-    } else if(!HDmemcmp(sig, H5SM_LIST_MAGIC, (size_t)H5SM_SIZEOF_MAGIC)) {
+    } else if(!HDmemcmp(sig, H5SM_LIST_MAGIC, (size_t)H5_SIZEOF_MAGIC)) {
         /*
          * Debug shared message list index.
          */
@@ -467,7 +401,7 @@ main(int argc, char *argv[])
 
         status = H5SM_list_debug(f, H5P_DATASET_XFER_DEFAULT, addr, stdout, 0, VCOL, (unsigned) extra, (size_t) extra2);
 
-    } else if(!HDmemcmp(sig, H5O_HDR_MAGIC, (size_t)H5O_SIZEOF_MAGIC)) {
+    } else if(!HDmemcmp(sig, H5O_HDR_MAGIC, (size_t)H5_SIZEOF_MAGIC)) {
         /*
          * Debug v2 object header (which have signatures).
          */
